@@ -19,26 +19,30 @@
 // headers
 // ----------------------------------------------------------------------------
 
+#undef WXINTL_NO_GETTEXT_MACRO
+
 // For compilers that support precompilation, includes "wx.h".
 #include <wx/wxprec.h>
 
 #include "FileDialog.h"
 
 #ifndef WX_PRECOMP
-    #include "wx/msgdlg.h"
-    #include "wx/app.h"
-    #include "wx/sizer.h"
-    #include "wx/stattext.h"
-    #include "wx/choice.h"
+    #include <wx/msgdlg.h>
+    #include <wx/app.h>
+    #include <wx/sizer.h>
+    #include <wx/stattext.h>
+    #include <wx/choice.h>
 #endif
 
-#include "wx/filename.h"
-#include "wx/tokenzr.h"
-#include "wx/evtloop.h"
+#include <wx/clipbrd.h>
+#include <wx/filectrl.h>
+#include <wx/filename.h>
+#include <wx/tokenzr.h>
+#include <wx/evtloop.h>
 
-#include "wx/osx/private.h"
-#include "wx/sysopt.h"
-#include "wx/modalhook.h"
+#include <wx/osx/private.h>
+#include <wx/sysopt.h>
+#include <wx/modalhook.h>
 
 #include <mach-o/dyld.h>
 
@@ -46,6 +50,10 @@
 // implementation
 // ============================================================================
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_6
+   #define wxOSX_10_6_AND_LATER( x ) x
+#endif
+
 @interface OSPanelDelegate : NSObject wxOSX_10_6_AND_LATER(<NSOpenSavePanelDelegate>)
 {
     FileDialog* _dialog;
@@ -152,8 +160,8 @@ void FileDialog::Create(
 {
     FileDialogBase::Create(parent, message, defaultDir, defaultFileName, wildCard, style, pos, sz, name);
 
-    m_sheetDelegate = [[ModalDialogDelegate alloc] init];
-    [(ModalDialogDelegate*)m_sheetDelegate setImplementation: this];
+//    m_sheetDelegate = [[ModalDialogDelegate alloc] init];
+//    [(ModalDialogDelegate*)m_sheetDelegate setImplementation: this];
 }
 
 FileDialog::~FileDialog()
@@ -309,7 +317,7 @@ void FileDialog::ShowWindowModal()
         
         NSWindow* nativeParent = parentWindow->GetWXWindow();
         [sPanel beginSheetForDirectory:dir.AsNSString() file:file.AsNSString()
-            modalForWindow: nativeParent modalDelegate: m_sheetDelegate
+            modalForWindow: nativeParent modalDelegate: nil // m_sheetDelegate
             didEndSelector: @selector(sheetDidEnd:returnCode:contextInfo:)
             contextInfo: nil];
     }
@@ -329,7 +337,7 @@ void FileDialog::ShowWindowModal()
         NSWindow* nativeParent = parentWindow->GetWXWindow();
         [oPanel beginSheetForDirectory:dir.AsNSString() file:file.AsNSString()
             types: types modalForWindow: nativeParent
-            modalDelegate: m_sheetDelegate
+            modalDelegate: nil // m_sheetDelegate
             didEndSelector: @selector(sheetDidEnd:returnCode:contextInfo:)
             contextInfo: nil];
     }
@@ -607,6 +615,47 @@ int FileDialog::ShowModal()
 
         SetupExtraControls(sPanel);
 
+        // PRL:
+        // Hack for bug 1300:  intercept key down events, implement a
+        // Command+V handler, but it's a bit crude.  It always pastes
+        // the entire text field, ignoring the insertion cursor, and ignoring
+        // which control really has the focus.
+        id handler;
+        if (wxTheClipboard->IsSupported(wxDF_TEXT)) {
+           handler = [
+              NSEvent addLocalMonitorForEventsMatchingMask:NSKeyDownMask
+              handler:^NSEvent *(NSEvent *event)
+              {
+                 if ([event modifierFlags] & NSCommandKeyMask)
+                 {
+                    auto chars = [event charactersIgnoringModifiers];
+                    auto character = [chars characterAtIndex:0];
+                    if (character == 'v')
+                    {
+                       if (wxTheClipboard->Open()) {
+                          wxTextDataObject data;
+                          wxTheClipboard->GetData(data);
+                          wxTheClipboard->Close();
+                          wxString text = data.GetText();
+                          auto rawText = text.utf8_str();
+                          auto length = text.Length();
+                          NSString *myString = [[NSString alloc]
+                             initWithBytes:rawText.data()
+                              length: rawText.length()
+                              encoding: NSUTF8StringEncoding
+                          ];
+                          [sPanel setNameFieldStringValue:myString];
+                          [myString release];
+                          return nil;
+                       }
+                    }
+                 }
+
+                 return event;
+              }
+           ];
+        }
+
         // makes things more convenient:
         [sPanel setCanCreateDirectories:YES];
         [sPanel setMessage:cf.AsNSString()];
@@ -636,6 +685,9 @@ int FileDialog::ShowModal()
 
         returnCode = [sPanel runModalForDirectory: m_dir.IsEmpty() ? nil : dir.AsNSString() file:file.AsNSString() ];
         ModalFinishedCallback(sPanel, returnCode);
+
+        if (wxTheClipboard->IsSupported(wxDF_TEXT))
+           [NSEvent removeMonitor:handler];
     }
     else
     {
