@@ -408,7 +408,7 @@ table_type sound_to_table(sound_type s)
 
     if (len >= max_table_len) {
         char emsg[100];
-        sprintf(emsg, "maximum table size (%d) exceeded", max_table_len);
+        sprintf(emsg, "maximum table size (%d) exceeded", max_table_len - 1);
         xlcerror("use truncated sound for table", emsg, NIL);
     } else if (len == 0) {
         xlabort("table size must be greater than 0");
@@ -515,35 +515,38 @@ void snd_list_terminate(snd_list)
 
 void snd_list_unref(snd_list_type list)
 {
-    void (*freefunc)();
+   if (list == NULL) {
+      nyquist_printf("why did snd_list_unref get %p?\n", list);
+      return;
+   }
 
-    if (list == NULL || list == zero_snd_list) {
-        if (list == NULL)
-           nyquist_printf("why did snd_list_unref get %p?\n", list);
-        return;
-    }
-    list->refcnt--;
-/*    nyquist_printf("snd_list_unref "); print_snd_list_type(list); stdputstr("\n"); */
-    if (list->refcnt == 0) {
-        if (list->block && list->block != zero_block) {
-            /* there is a next snd_list */
-/*          stdputstr("["); */
-            sample_block_unref(list->block);
-/*          stdputstr("]"); */
-            snd_list_unref(list->u.next);
-        }
-        else if (list->block == NULL) { /* the next thing is the susp */
-            /* free suspension structure */
-            /* nyquist_printf("freeing susp@%p\n", list->u.susp); */
-            freefunc = list->u.susp->free;
-            (*freefunc)(list->u.susp);
-        }
-        /* nyquist_printf("freeing snd_list@%p\n", list); */
-        //DBY
-        if (list == list_watch) printf("freeing watched snd_list %p\n", list);
-        //DBY
-        ffree_snd_list(list, "snd_list_unref");
-    }
+   while (list && (list != zero_snd_list)) {
+      snd_list_type next;
+
+      list->refcnt--;
+      if (list->refcnt != 0) {
+         break; // the rest of the list is shared, nothing more to free
+      }
+
+      next = NULL;
+      // list nodes either point to a block of samples or this is the
+      // last list node (list->block == NULL) which points to a suspension
+      // lists can also terminate at the zero_block, which is an infinite
+      //     shared list (zero_block->block == zero_block) of zero samples
+      if (list->block && list->block != zero_block) {
+         /* there is a next snd_list */
+         next = list->u.next;
+         sample_block_unref(list->block);
+      } else if (list->block == NULL) { /* the next thing is the susp */
+         /* free suspension structure */
+         /* nyquist_printf("freeing susp@%p\n", list->u.susp); */
+         (*(list->u.susp->free))(list->u.susp);
+      }
+      /* if (list == list_watch)
+       printf("freeing watched snd_list %p\n", list); */
+      ffree_snd_list(list, "snd_list_unref");
+      list = next;
+   }
 }
 
 
@@ -1588,7 +1591,7 @@ double step_to_hz(double steps)
 }
 
 #ifdef WIN32
-#ifndef _MSC_VER < 1800
+#if _MSC_VER < 1800
 #define RECIP_LOG_2 1.44269504088895364453
 
 double log2(double x)
