@@ -241,7 +241,7 @@ typedef double promoted_sample_type;
  */
 #define MAX_SND_CHANNELS 24
 
-#define max_table_len 100000
+#define max_table_len 1000001
 /* Set to 4 for debugging block allocation stuff, 1012? for
    production
 */
@@ -272,12 +272,19 @@ typedef struct {
 /* forward declaration for circular type dependencies */
 typedef struct snd_list_struct *snd_list_type;
 
+struct snd_susp_struct;
+
+typedef void  (*snd_fetch_fn)(struct snd_susp_struct *, snd_list_type snd_list);
+typedef void  (*snd_free_fn)(struct snd_susp_struct *);
+typedef void  (*snd_mark_fn)(struct snd_susp_struct *);  /* marks LVAL nodes for GC */
+typedef void  (*snd_print_tree_fn)(struct snd_susp_struct *, int);
+
 typedef struct snd_susp_struct {
-    void  (*fetch)(struct snd_susp_struct *, snd_list_type snd_list);
+    snd_fetch_fn fetch;
     void  (*keep_fetch)(struct snd_susp_struct *, snd_list_type snd_list);
-    void  (*free)(struct snd_susp_struct *);
-    void  (*mark)(struct snd_susp_struct *);  /* marks LVAL nodes for GC */
-    void  (*print_tree)(struct snd_susp_struct *, int);    /* debugging */
+    snd_free_fn free;
+    snd_mark_fn mark;
+    snd_print_tree_fn print_tree; /* debugging */
     char *name;        /* string name for debugging */
     long  toss_cnt;    /* return this many zeros, then compute */
     long  current;     /* current sample number */
@@ -457,10 +464,6 @@ sound_type sound_zero(time_type t0, rate_type sr);
 double step_to_hz(double);
     /* LISP: (STEP-TO-HZ ANYNUM) */
 
-#ifdef WIN32
-double log2(double x);
-#endif WIN32
-
 /* macros for access to samples within a suspension */
 /* NOTE: assume suspension structure is named "susp" */
 
