@@ -14,6 +14,7 @@
 #include <wx/filedlg.h>
 #include <wx/font.h>
 #include <wx/fontdlg.h>
+#include <wx/menu.h>
 #include <wx/msgdlg.h>
 #include <wx/settings.h>
 #include <wx/sizer.h>
@@ -22,8 +23,8 @@
 #include <wx/textctrl.h>
 #include <wx/toolbar.h>
 
-#include "AudacityApp.h"
-#include "AudioIO.h"
+#include "AudioIOBase.h"
+#include "CommonCommandFlags.h"
 #include "LabelTrack.h"
 #include "ModuleManager.h"
 #include "Prefs.h"
@@ -32,6 +33,8 @@
 #include "effects/EffectManager.h"
 #include "effects/nyquist/Nyquist.h"
 #include "../images/AudacityLogo.xpm"
+#include "../../src/commands/CommandContext.h"
+#include "widgets/AudacityMessageBox.h"
 
 #include "NyqBench.h"
 
@@ -139,6 +142,11 @@ extern "C"
 {
    static NyqBench *gBench = NULL;
 
+   static CommandHandlerObject &findme(AudacityProject&)
+   {
+      return *NyqBench::GetBench();
+   }
+
    #ifdef _MSC_VER
       #define DLL_API _declspec(dllexport)
    #else
@@ -162,13 +170,10 @@ extern "C"
    // and ask if it has anything for the menus.
    int ModuleDispatch(ModuleDispatchTypes type){
       switch (type){
-         case AppInitialized:{
-            wxASSERT(gBench == NULL);
-            gBench = new NyqBench(NULL);
-         }
-         break;
          case AppQuiting: {
-            wxASSERT(gBench != NULL);
+            //It is perfectly OK for gBench to be NULL.
+            //Can happen if the menu item was never invoked.
+            //wxASSERT(gBench != NULL);
             if (gBench) {
                gBench->Destroy();
                gBench = NULL;
@@ -179,21 +184,24 @@ extern "C"
          case MenusRebuilt:  {
             AudacityProject *p = GetActiveProject();
             wxASSERT(p != NULL);
-            CommandManager *c = p->GetCommandManager();
+            CommandManager *c = &CommandManager::Get( *p );
             wxASSERT(c != NULL);
 
-            wxMenuBar * pBar = p->GetMenuBar();
+            wxMenuBar * pBar = GetProjectFrame( *p ).GetMenuBar();
             wxASSERT(pBar != NULL );
-            wxMenu * pMenu = pBar->GetMenu( 2 );  // Menu 2 is the View Menu.
+            wxMenu * pMenu = pBar->GetMenu( 9 );  // Menu 9 is the Tools Menu.
             wxASSERT( pMenu != NULL );
 
-            c->SetToMenu( pMenu );
+            c->SetCurrentMenu(pMenu);
             c->AddSeparator();
-            // c->BeginMenu(_("T&ools"));
-            c->SetDefaultFlags(AudioIONotBusyFlag, AudioIONotBusyFlag);
             c->AddItem(wxT("NyqBench"),
-                       _("&Nyquist Workbench..."),
-                       new ModNyqBenchCommandFunctor());
+               _("&Nyquist Workbench..."),
+               true,
+               findme,
+               static_cast<CommandFunctorPointer>(&NyqBench::ShowNyqBench),
+               AudioIONotBusyFlag);
+
+            c->ClearCurrentMenu();
          }
          break;
          default:
@@ -590,18 +598,6 @@ void NyqRedirector::AppendText()
    s.clear();
 }
 
-//----------------------------------------------------------------------------
-// ModNyqBenchCommandFunctor
-// Connects Audacity menu item to an action in this dll.
-// Only one action implemented so far.
-//----------------------------------------------------------------------------
-void ModNyqBenchCommandFunctor::operator()(int index, const wxEvent *e)
-{
-   if (gBench) {
-      gBench->Show();
-   }
-}
-
 //----------------------------------------------------------------------------
 // NyqBench
 //----------------------------------------------------------------------------
@@ -703,6 +699,16 @@ BEGIN_EVENT_TABLE(NyqBench, wxFrame)
    EVT_UPDATE_UI(ID_OUTPUT, NyqBench::OnOutputUpdate)
 END_EVENT_TABLE()
 
+/*static*/ NyqBench *NyqBench::GetBench()
+{
+   if (gBench == nullptr)
+   {
+      gBench = new NyqBench(NULL);
+   }
+
+   return gBench;
+}
+
 NyqBench::NyqBench(wxWindow * parent)
 :  wxFrame(NULL,
            wxID_ANY,
@@ -1086,7 +1092,7 @@ void NyqBench::OnSave(wxCommandEvent & e)
 
    if (!mScript->SaveFile(mPath.GetFullPath()))
    {
-      wxMessageBox(_("Script was not saved."),
+      AudacityMessageBox(_("Script was not saved."),
                    _("Warning"),
                    wxICON_EXCLAMATION,
                    this);
@@ -1116,7 +1122,7 @@ void NyqBench::OnSaveAs(wxCommandEvent & e)
 
    if (!mScript->SaveFile(mPath.GetFullPath()))
    {
-      wxMessageBox(_("Script was not saved."),
+      AudacityMessageBox(_("Script was not saved."),
                    _("Warning"),
                    wxICON_EXCLAMATION,
                    this);
@@ -1398,7 +1404,7 @@ void NyqBench::OnGo(wxCommandEvent & e)
       mRunning = true;
       UpdateWindowUI();
 
-      p->OnEffect(ID);
+      EffectManager::DoEffect(ID, CommandContext(*p), 0);
 
       mRunning = false;
       UpdateWindowUI();
@@ -1491,7 +1497,7 @@ void NyqBench::OnFindDialog(wxFindDialogEvent & e)
    }
 
    if (pos == wxString::npos) {
-      wxMessageBox(_("No matches found"),
+      AudacityMessageBox(_("No matches found"),
                    _("Nyquist Effect Workbench"),
                    wxOK | wxCENTER,
                    e.GetDialog());
@@ -1584,6 +1590,7 @@ void NyqBench::OnRunUpdate(wxUpdateUIEvent & e)
    wxToolBar *tbar = GetToolBar();
    wxMenuBar *mbar = GetMenuBar();
 
+   auto gAudioIO = AudioIOBase::Get();
    if (p && gAudioIO->IsBusy()) {
       mbar->Enable(ID_GO, false);
       mbar->Enable(ID_STOP, false);
@@ -1628,7 +1635,7 @@ bool NyqBench::Validate()
 {
    if (mScript->GetLastPosition() > 0 && mScript->IsModified()) {
       int ans;
-      ans = wxMessageBox(_("Code has been modified.  Are you sure?"),
+      ans = AudacityMessageBox(_("Code has been modified. Are you sure?"),
                          _("Warning"),
                          wxYES_NO | wxICON_QUESTION,
                          this);
@@ -1769,3 +1776,12 @@ void NyqBench::LoadFile()
 
 //   mScript->LoadFile(mPath.GetFullPath());
 }
+
+//----------------------------------------------------------------------------
+// Connects Audacity menu item to an action in this dll.
+// Only one action implemented so far.
+//----------------------------------------------------------------------------
+void NyqBench::ShowNyqBench(const CommandContext &)
+{
+   Show();
+}
