@@ -25,7 +25,10 @@ hold information about one contributor to Audacity.
 *//********************************************************************/
 
 
-#include "Audacity.h"
+#include "Audacity.h" // for USE_* macros
+#include "AboutDialog.h"
+
+#include "Experimental.h"
 
 #include <wx/dialog.h>
 #include <wx/html/htmlwin.h>
@@ -35,114 +38,145 @@ hold information about one contributor to Audacity.
 #include <wx/statbmp.h>
 #include <wx/intl.h>
 
-#include "AboutDialog.h"
 #include "FileNames.h"
-#include "Internat.h"
+#include "HelpText.h"
 #include "ShuttleGui.h"
-#include "widgets/LinkingHtmlWindow.h"
+#include "widgets/HelpSystem.h"
 
-#include <wx/listimpl.cpp>
-WX_DEFINE_LIST(AboutDialogCreditItemsList);
-
-#include "Theme.h"
 #include "AllThemeResources.h"
 
+// DA: Logo for About box.
+#ifdef EXPERIMENTAL_DA
+#include "../images/DarkAudacityLogoWithName.xpm"
+#else
 #include "../images/AudacityLogoWithName.xpm"
+#endif
+#include "RevisionIdent.h"
+
+// RevisionIdent.h may contain #defines like these ones:
+//#define REV_LONG "28864acb238cb3ca71dda190a2d93242591dd80e"
+//#define REV_TIME "Sun Apr 12 12:40:22 2015 +0100"
+
+
+#ifndef REV_TIME
+#define REV_TIME "unknown date and time"
+#endif
+
+#ifdef REV_LONG
+#define REV_IDENT wxString( "[[https://github.com/audacity/audacity/commit/" )+ REV_LONG + "|" + wxString( REV_LONG ).Left(6) + "]] of " +  REV_TIME 
+#else
+#define REV_IDENT wxT("No revision identifier was provided")
+#endif
+
+extern wxString FormatHtmlText( const wxString & Text );
 
 void AboutDialog::CreateCreditsList()
 {
    // The Audacity Team: developers and support
-   AddCredit(wxT("Richard Ash"), roleTeamDeveloper);
-   AddCredit(wxT("James Crook"), roleTeamDeveloper);
-   AddCredit(wxString(wxT("Roger Dannenberg - ")) + _("co-founder"), roleTeamDeveloper);
-   AddCredit(wxT("Benjamin Drung"), roleTeamDeveloper);
-   AddCredit(wxT("Vaughan Johnson"), roleTeamDeveloper);
-   AddCredit(wxT("Paul Licameli"), roleTeamDeveloper);
-   AddCredit(wxT("Leland Lucius"), roleTeamDeveloper);
-   AddCredit(wxT("Martyn Shaw"), roleTeamDeveloper);
-
-   AddCredit(wxString(wxT("Gale Andrews - ")) + _("quality assurance"), roleTeamSupport);
-   AddCredit(wxT("Christian Brochec"), roleTeamSupport);
-   AddCredit(wxT("Steve Daulton"), roleTeamSupport);
-   AddCredit(wxT("Greg Kozikowski"), roleTeamSupport);
-   AddCredit(wxT("Peter Sampson"), roleTeamSupport);
-   AddCredit(wxT("Bill Wharrie"), roleTeamSupport);
+   AddCredit(wxString(wxT("Arturo \"Buanzo\" Busleiman, ")) + _("system administration"), roleTeamMember);
+   AddCredit(wxString(wxT("James Crook, ")) + _("developer"), roleTeamMember);
+   AddCredit(wxString(wxT("Roger Dannenberg, ")) + _("co-founder and developer"), roleTeamMember);
+   AddCredit(wxString(wxT("Steve Daulton")), roleTeamMember);
+   AddCredit(wxString(wxT("Greg Kozikowski, ")) + _("documentation and support"), roleTeamMember);
+   AddCredit(wxString(wxT("Paul Licameli, ")) + _("developer"), roleTeamMember);
+   AddCredit(wxString(wxT("Peter Sampson")), roleTeamMember);
+   AddCredit(wxString(wxT("Bill Wharrie, ")) + _("documentation and support"), roleTeamMember);
 
    // Emeritus: people who were "lead developers" or made an
    // otherwise distinguished contribution, but who are no
    // longer active.
-   AddCredit(wxT("Matt Brubeck"), roleEmeritusDeveloper);
-   AddCredit(wxT("Michael Chinen"), roleEmeritusDeveloper);
-   AddCredit(wxT("Al Dimond"), roleEmeritusDeveloper);
-   AddCredit(wxT("Joshua Haberman"), roleEmeritusDeveloper);
-   AddCredit(wxT("Ruslan Ijbulatov"), roleEmeritusDeveloper);
-   AddCredit(wxString(wxT("Dominic Mazzoni - "))+_("co-founder"), roleEmeritusDeveloper);
-   AddCredit(wxT("Markus Meyer"), roleEmeritusDeveloper);
-   AddCredit(wxT("Monty Montgomery"), roleEmeritusDeveloper);
-   AddCredit(wxT("Shane Mueller"), roleEmeritusDeveloper);
-   AddCredit(wxT("Tony Oetzmann"), roleEmeritusSupport);
-   AddCredit(wxT("Alexandre Prokoudine"), roleEmeritusSupport);
+   AddCredit(wxString(wxT("[[https://wiki.audacityteam.org/wiki/User:Galeandrews|Gale Andrews]], ")) + _("quality assurance"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Richard Ash, ")) + _("developer"), roleEmeritusTeam); 
+   AddCredit(wxString(wxT("Christian Brochec, ")) + _("documentation and support, French"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Matt Brubeck, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Michael Chinen, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Al Dimond, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Benjamin Drung, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Joshua Haberman, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Ruslan Ijbulatov, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Vaughan Johnson, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Leland Lucius, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Dominic Mazzoni, "))+_("co-founder and developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Markus Meyer, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Monty Montgomery, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Shane Mueller, ")) + _("developer"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Tony Oetzmann, ")) + _("documentation and support"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Alexandre Prokoudine, ")) + _("documentation and support"), roleEmeritusTeam);
+   AddCredit(wxString(wxT("Martyn Shaw, ")) + _("developer"), roleEmeritusTeam);
 
-   // All other contributors
-   AddCredit(wxT("Lynn Allan"), roleContributor);
-   AddCredit(wxT("David Avery"), roleContributor);
-   AddCredit(wxT("David Bailes"), roleContributor);
-   AddCredit(wxT("William Bland"), roleContributor);
-   AddCredit(wxT("Sami Boukortt"), roleContributor);
-   AddCredit(wxT("Jeremy R. Brown"), roleContributor);
-   AddCredit(wxT("Alex S. Brown"), roleContributor);
-   AddCredit(wxT("Chris Cannam"), roleContributor);
-   AddCredit(wxT("Cory Cook"), roleContributor);
-   AddCredit(wxT("Craig DeForest"), roleContributor);
-   AddCredit(wxT("Mitch Golden"), roleContributor);
-   AddCredit(wxT("Brian Gunlogson"), roleContributor);
-   AddCredit(wxT("Andrew Hallendorff"), roleContributor);
-   AddCredit(wxT("Daniel Horgan"), roleContributor);
-   AddCredit(wxT("David Hostetler"), roleContributor);
-   AddCredit(wxT("Steve Jolly"), roleContributor);
-   AddCredit(wxT("Steven Jones"), roleContributor);
-   AddCredit(wxT("Arun Kishore"), roleContributor);
-   AddCredit(wxT("Paul Livesey"), roleContributor);
-   AddCredit(wxT("Harvey Lubin"), roleContributor);
-   AddCredit(wxT("Greg Mekkes"), roleContributor);
-   AddCredit(wxT("Abe Milde"), roleContributor);
-   AddCredit(wxT("<a href=\"http://www.paulnasca.com/\">Paul Nasca</a>"), roleContributor);
-   AddCredit(wxT("Clayton Otey"), roleContributor);
-   AddCredit(wxT("Andr\x00e9 Pinto"), roleContributor);
-   AddCredit(wxT("Mark Phillips"), roleContributor);
-   AddCredit(wxT("Jean Claude Risset"), roleContributor);
-   AddCredit(wxT("Edgar-RFT"), roleContributor);
-   AddCredit(wxT("Augustus Saunders"), roleContributor);
-   AddCredit(wxT("Benjamin Schwartz"), roleContributor);
-   AddCredit(wxT("David R. Sky"), roleContributor);
-   AddCredit(wxT("Rob Sykes"), roleContributor);
-   AddCredit(wxT("Mike Underwood"), roleContributor);
-   AddCredit(wxT("Philip Van Baren"), roleContributor);
-   AddCredit(wxT("Salvo Ventura"), roleContributor);
-   AddCredit(wxT("Jun Wan"), roleContributor);
-   AddCredit(wxT("Daniel Winzen"), roleContributor);
-   AddCredit(wxT("Tom Woodhams"), roleContributor);
-   AddCredit(wxT("Wing Yu"), roleContributor);
+   // Contributors
+   AddCredit(wxString(wxT("Lynn Allan, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Brian Armstrong, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("David Avery, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("David Bailes, ")) + _("accessibility advisor"), roleContributor);
+   AddCredit(wxString(wxT("William Bland, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Sami Boukortt, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Jeremy R. Brown, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Alex S. Brown, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Chris Cannam, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Cory Cook, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Craig DeForest, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Edgar Franke (Edgar-RFT), ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Mitch Golden, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Brian Gunlogson, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Andrew Hallendorff, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Robert H\u00E4nggi, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Daniel Horgan, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("David Hostetler, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Steve Jolly, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Steven Jones, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Henric Jungheim, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Arun Kishore, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Paul Livesey, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Harvey Lubin, ")) + _("graphic artist"), roleContributor);
+   AddCredit(wxString(wxT("Max Maisel, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Greg Mekkes, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Abe Milde, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Paul Nasca, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Clayton Otey, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Mark Phillips, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Andr\u00E9 Pinto, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Jean Claude Risset, ")) + _("composer"), roleContributor);
+   AddCredit(wxString(wxT("Augustus Saunders, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Benjamin Schwartz, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Cliff Scott, ")) + _("tester"), roleContributor);
+   AddCredit(wxString(wxT("David R. Sky, ")) + _("Nyquist plug-ins"), roleContributor);
+   AddCredit(wxString(wxT("Rob Sykes, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Mike Underwood, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Philip Van Baren, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Salvo Ventura, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Darrell Walisser, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Jun Wan, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Daniel Winzen, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Tom Woodhams, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Mark Young, ")) + _("developer"), roleContributor);
+   AddCredit(wxString(wxT("Wing Yu, ")) + _("developer"), roleContributor);
 
-   AddCredit(wxT("expat"), roleLibrary);
-   AddCredit(wxT("FLAC"), roleLibrary);
-   AddCredit(wxT("LAME"), roleLibrary);
-   AddCredit(wxT("libmad"), roleLibrary);
-   AddCredit(wxT("libsoxr, by Rob Sykes"), roleLibrary);
-   #if USE_LV2
-      AddCredit(wxT("lilv, serd, sord, and sratom, by David Robillard"), roleLibrary);
-      AddCredit(wxT("msinttypes, by Alexander Chemeris"), roleLibrary);
-   #endif
-   AddCredit(wxT("libsndfile"), roleLibrary);
-   AddCredit(wxT("Nyquist"), roleLibrary);
-   AddCredit(wxT("Ogg Vorbis"), roleLibrary);
-   AddCredit(wxT("PortAudio"), roleLibrary);
-   AddCredit(wxT("portsmf"), roleLibrary);
-   AddCredit(wxT("sbsms, by Clayton Otey"), roleLibrary);
-   AddCredit(wxT("<a href=\"http://www.surina.net/soundtouch/\">SoundTouch</a>, by Olli Parviainen"), roleLibrary);
-   AddCredit(wxT("TwoLAME"), roleLibrary);
-   AddCredit(wxT("Vamp"), roleLibrary);
-   AddCredit(wxT("wxWidgets"), roleLibrary);
+   // Website and Graphics
+   AddCredit(wxString(wxT("Shinta Carolinasari, ")) + _("web developer"), roleGraphics);
+   AddCredit(wxString(wxT("Bayu Rizaldhan Rayes, ")) + _("graphics"), roleGraphics);
+
+   // Libraries
+
+   AddCredit(wxT("[[https://libexpat.github.io/|expat]]"), roleLibrary);
+   AddCredit(wxT("[[https://xiph.org/flac/|FLAC]]"), roleLibrary);
+   AddCredit(wxT("[[http://lame.sourceforge.net/|LAME]]"), roleLibrary);
+   AddCredit(wxT("[[https://www.underbit.com/products/mad/|libmad]]"), roleLibrary);
+   AddCredit(wxT("[[http://www.mega-nerd.com/libsndfile/|libsndfile]]"), roleLibrary);
+   AddCredit(wxT("[[https://sourceforge.net/p/soxr/wiki/Home/|libsoxr]]"), roleLibrary);
+   AddCredit(wxT("[[http://lv2plug.in/|lv2]] (") + _("incorporating") + wxT(" lilv, msinttypes, serd, sord and sratom)"), roleLibrary);
+   AddCredit(wxT("[[https://www.cs.cmu.edu/~music/nyquist/|Nyquist]]"), roleLibrary);
+   AddCredit(wxT("[[https://xiph.org/vorbis/|Ogg Vorbis]]"), roleLibrary);
+   AddCredit(wxT("[[http://www.portaudio.com/|PortAudio]]"), roleLibrary);
+   AddCredit(wxT("[[http://www.portmedia.sourceforge.net/portmidi/|PortMidi]]"), roleLibrary);
+   AddCredit(wxT("[[https://sourceforge.net/p/portmedia/wiki/portsmf/|portsmf]]"), roleLibrary);
+   AddCredit(wxT("[[http://sbsms.sourceforge.net/|sbsms]]"), roleLibrary);
+   AddCredit(wxT("[[https://www.surina.net/soundtouch/|SoundTouch]]"), roleLibrary);
+   AddCredit(wxT("[[http://www.twolame.org/|TwoLAME]]"), roleLibrary);
+   AddCredit(wxT("[[http://www.vamp-plugins.org/|Vamp]]"), roleLibrary);
+   AddCredit(wxT("[[https://wxwidgets.org/|wxWidgets]]"), roleLibrary);
+
+   // Thanks
 
    AddCredit(wxT("Dave Beydler"), roleThanks);
    AddCredit(wxT("Brian Cameron"), roleThanks);
@@ -169,19 +203,32 @@ void AboutDialog::CreateCreditsList()
 
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(AboutDialog, wxDialog)
+BEGIN_EVENT_TABLE(AboutDialog, wxDialogWrapper)
    EVT_BUTTON(wxID_OK, AboutDialog::OnOK)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(AboutDialog, wxDialog)
+IMPLEMENT_CLASS(AboutDialog, wxDialogWrapper)
+
+namespace {
+   AboutDialog *sActiveInstance{};
+}
+
+AboutDialog *AboutDialog::ActiveIntance()
+{
+   return sActiveInstance;
+}
 
 AboutDialog::AboutDialog(wxWindow * parent)
-   :  wxDialog(parent, -1, _("About Audacity"),
+   :  wxDialogWrapper(parent, -1, _("About Audacity"),
                wxDefaultPosition, wxDefaultSize,
                wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
 {
+   wxASSERT(!sActiveInstance);
+   sActiveInstance = this;
+
    SetName(GetTitle());
    this->SetBackgroundColour(theTheme.Colour( clrAboutBoxBackground ));
+   //this->SetBackgroundColour(theTheme.Colour( clrMedium ));
    icon = NULL;
    ShuttleGui S( this, eIsCreating );
    S.StartNotebook();
@@ -191,12 +238,7 @@ AboutDialog::AboutDialog(wxWindow * parent)
       PopulateLicensePage( S );
    }
    S.EndNotebook();
-   /* i18n-hint: "OK... Audacious" appears on a button at the
-    * foot of the 'About Audacity' dialog box, after some text to read.
-    * In English it is slightly humorous alternative to an 'OK' button.
-    * If the humour doesn't work in your language, then just use whatever
-    * you would use for a translation for 'OK' on a button. */
-   wxButton *ok = new wxButton(S.GetParent(), wxID_OK, _("OK... Audacious!"));
+   wxButton *ok = safenew wxButton(S.GetParent(), wxID_OK, _("OK"));
    ok->SetDefault();
    S.Prop(0).AddWindow( ok );
 
@@ -208,23 +250,30 @@ AboutDialog::AboutDialog(wxWindow * parent)
 
 void AboutDialog::PopulateAudacityPage( ShuttleGui & S )
 {
-   creditItems.DeleteContents(true); // Switch on automatic deletion of list items.
    CreateCreditsList();
 
-   wxString par1Str = _(
-"Audacity is a free program written by a worldwide team of volunteer <a href=\"http://audacityteam.org/about/credits\">developers</a>. \
-Audacity is <a href=\"http://audacityteam.org/download\">available</a> for Windows, Mac, and GNU/Linux (and other Unix-like systems).");
+   wxString par1Str = 
+// DA: Says that it is a customised version.
+#ifdef EXPERIMENTAL_DA
+      wxT(
+"Audacity, which this is a customised version of, is a free program written by a worldwide team of [[https://www.audacityteam.org/about/credits|volunteers]]. \
+Audacity is [[https://www.audacityteam.org/download|available]] for Windows, Mac, and GNU/Linux (and other Unix-like systems).");
+#else
+      _(
+"Audacity is a free program written by a worldwide team of [[https://www.audacityteam.org/about/credits|volunteers]]. \
+Audacity is [[https://www.audacityteam.org/download|available]] for Windows, Mac, and GNU/Linux (and other Unix-like systems).");
+#endif
 
    // This trick here means that the English language version won't mention using
    // English, whereas all translated versions will.
    wxString par2StrUntranslated = wxT(
-"If you find a bug or have a suggestion for us, please write, in English, to our <a href=\"mailto:feedback@audacityteam.org\">feedback address</a>. \
-For help, view the tips and tricks on our <a href=\"http://wiki.audacityteam.org/\">wiki</a> or \
-visit our <a href=\"http://forum.audacityteam.org/\">forum</a>.");
+"If you find a bug or have a suggestion for us, please write, in English, to our [[https://forum.audacityteam.org/|forum]]. \
+For help, view the tips and tricks on our [[https://wiki.audacityteam.org/|wiki]] or \
+visit our [[https://forum.audacityteam.org/|forum]].");
    wxString par2Str = _(
-"If you find a bug or have a suggestion for us, please write, in English, to our <a href=\"mailto:feedback@audacityteam.org\">feedback address</a>. \
-For help, view the tips and tricks on our <a href=\"http://wiki.audacityteam.org/\">wiki</a> or \
-visit our <a href=\"http://forum.audacityteam.org/\">forum</a>.");
+"If you find a bug or have a suggestion for us, please write, in English, to our [[https://forum.audacityteam.org/|forum]]. \
+For help, view the tips and tricks on our [[https://wiki.audacityteam.org/|wiki]] or \
+visit our [[https://forum.audacityteam.org/|forum]].");
 
    if( par2Str == par2StrUntranslated )
       par2Str.Replace( wxT(", in English,"), wxT("") );
@@ -237,81 +286,109 @@ visit our <a href=\"http://forum.audacityteam.org/\">forum</a>.");
     *  For example:  "English translation by Dominic Mazzoni." */
    if (_("translator_credits") != wxString(wxT("translator_credits")))
    {
-      translatorCredits = _("translator_credits");
+      translatorCredits = _("translator_credits") + wxT("<br>");
    }
-   wxString localeStr = wxLocale::GetSystemEncodingName();
 
-   wxString creditStr =
-      wxT("<html><head><META http-equiv=\"Content-Type\" content=\"text/html; charset=") +
-         localeStr +
-         wxT("\"></head>") +
-      wxT("<body bgcolor=\"#ffffff\"><center>") +
-      wxT("<h3>Audacity ") + wxString(AUDACITY_VERSION_STRING) + wxT("</h3>")+
-      _("free, open source, cross-platform software for recording and editing sounds<br>") +
-      wxT("<a href=\"http://audacityteam.org/\">http://audacityteam.org/</a>") +
-      wxT("<p><br>") + par1Str +
-      wxT("<p>") + par2Str +
-      wxT("<h3>") + _("Credits") + wxT("</h3>") +
-      wxT("<p>") + translatorCredits +
+   wxString creditStr = FormatHtmlText(
+      wxString( wxT("<center>") ) +
+// DA: Description and provenance in About box
+#ifdef EXPERIMENTAL_DA
+      #undef _
+      #define _(s) wxGetTranslation((s))
+      wxT("<h3>DarkAudacity ") + wxString(AUDACITY_VERSION_STRING) + wxT("</center></h3>") +
+      wxT("Customised version of the Audacity free, open source, cross-platform software " ) +
+      wxT("for recording and editing sounds.") +
+      wxT("<p><br>&nbsp; &nbsp; <b>Audacity<sup>&reg;</sup></b> software is copyright &copy; 1999-2018 Audacity Team.<br>") +
+      wxT("&nbsp; &nbsp; The name <b>Audacity</b> is a registered trademark of Dominic Mazzoni.<br><br>") +
 
-      wxT("<p><b>") + wxString::Format(_("Audacity Developers")) + wxT("</b><br>") +
-      GetCreditsByRole(roleTeamDeveloper) +
+#else
+      _("<h3>Audacity ") + wxString(AUDACITY_VERSION_STRING) + wxT("</center></h3>") +
+      _("Audacity the free, open source, cross-platform software for recording and editing sounds.") +
+#endif
 
-      wxT("<p><b>") + wxString::Format(_("Audacity Support Team")) + wxT("</b><br>") +
-      GetCreditsByRole(roleTeamSupport) +
+      //wxT("<p><br>") + par1Str +
+      //wxT("<p>") + par2Str +
+      wxT("<h3>") + _("Credits") + wxT("</h3>") + wxT("<p>") +
 
-      wxT("<p><b>") + _("Emeritus Developers") + wxT("</b><br>") +
-      GetCreditsByRole(roleEmeritusDeveloper) +
+// DA: Customisation credit
+#ifdef EXPERIMENTAL_DA
+      wxT("<p><b>") + wxString::Format(_("DarkAudacity Customisation")) + wxT("</b><br>") +
+      wxT("James Crook, art, coding &amp; design<br>") +
+#endif
 
-      wxT("<p><b>") + _(" Emeritus Team Members") + wxT("</b><br>") +
-      GetCreditsByRole(roleEmeritusSupport) +
+      wxT("<p><b>") + _("Audacity Team Members") + wxT("</b><br>") +
+      GetCreditsByRole(roleTeamMember) +
 
-      wxT("<p><b>") + _("Other Contributors") + wxT("</b><br>") +
+      wxT("<p><b> ") + _("Emeritus:") + wxT("</b><br>") +
+      _("Distinguished Audacity Team members, not currently active") + wxT("<br><br>") +
+      GetCreditsByRole(roleEmeritusTeam) +
+
+      wxT("<p><b>") + _("Contributors") + wxT("</b><br>") +
       GetCreditsByRole(roleContributor) +
 
-      wxT("<p><b>") +  _("Audacity is based on code from the following projects:") + wxT("</b><br>") +
+      wxT("<p><b>") + _("Website and Graphics") + wxT("</b><br>") +
+      GetCreditsByRole(roleGraphics) +
+
+      (translatorCredits.empty()
+         ? wxT("")
+         : (wxT("<p><b>") + _("Translators") + wxT("</b><br>") +
+            translatorCredits)) +
+
+      wxT("<p><b>") +  _("Libraries") + wxT("</b><br>") +
+      _("Audacity includes code from the following projects:") + wxT("<br><br>") +
       GetCreditsByRole(roleLibrary) +
 
       wxT("<p><b>") +  _("Special thanks:") + wxT("</b><br>") +
       GetCreditsByRole(roleThanks) +
 
-      wxT("<p><br>") + _("<b>Audacity&reg;</b> software is copyright")+
-      wxT("&copy; 1999-2015 Audacity Team.<br>") +
-      _("The name <b>Audacity&reg;</b> is a registered trademark of Dominic Mazzoni.") +
-      wxT("</center></font></body></html>");
+      wxT("<p><br>") + _("Audacity website: ") + wxT("[[https://www.audacityteam.org/|https://www.audacityteam.org/]]") +
 
+// DA: Link for DA url too
+#ifdef EXPERIMENTAL_DA
+      wxT("<br>DarkAudacity website: [[http://www.darkaudacity.com/|https://www.darkaudacity.com/]]") +
+#else
+      _("<p><br>&nbsp; &nbsp; <b>Audacity<sup>&reg;</sup></b> software is copyright &copy; 1999-2018 Audacity Team.<br>")
 
-   this->SetBackgroundColour(theTheme.Colour( clrAboutBoxBackground ));
+      +
 
+      _("&nbsp; &nbsp; The name <b>Audacity</b> is a registered trademark of Dominic Mazzoni.<br><br>") +
+#endif
 
-   // New way to add to About box....
-   S.StartNotebookPage( wxT("Audacity") );
+      wxT("</center>")
+   );
+
+   auto pPage = S.StartNotebookPage( _("Audacity") );
    S.StartVerticalLay(1);
+   {
+      //v For now, change to AudacityLogoWithName via old-fashioned way, not Theme.
+      wxBitmap logo(AudacityLogoWithName_xpm); //v
 
-   //v For now, change to AudacityLogoWithName via old-fashioned way, not Theme.
-   logo = new wxBitmap((const char **) AudacityLogoWithName_xpm); //v
+      // JKC: Resize to 50% of size.  Later we may use a smaller xpm as
+      // our source, but this allows us to tweak the size - if we want to.
+      // It also makes it easier to revert to full size if we decide to.
+      const float fScale = 0.5f;// smaller size.
+      wxImage RescaledImage(logo.ConvertToImage());
+      wxColour MainColour( 
+         RescaledImage.GetRed(1,1), 
+         RescaledImage.GetGreen(1,1), 
+         RescaledImage.GetBlue(1,1));
+      pPage->SetBackgroundColour(MainColour);
+      // wxIMAGE_QUALITY_HIGH not supported by wxWidgets 2.6.1, or we would use it here.
+      RescaledImage.Rescale((int)(LOGOWITHNAME_WIDTH * fScale), (int)(LOGOWITHNAME_HEIGHT *fScale));
+      wxBitmap RescaledBitmap(RescaledImage);
 
-   // JKC: Resize to 50% of size.  Later we may use a smaller xpm as
-   // our source, but this allows us to tweak the size - if we want to.
-   // It also makes it easier to revert to full size if we decide to.
-   const float fScale=0.5f;// smaller size.
-   wxImage RescaledImage( logo->ConvertToImage() );
-   // wxIMAGE_QUALITY_HIGH not supported by wxWidgets 2.6.1, or we would use it here.
-   RescaledImage.Rescale( int(LOGOWITHNAME_WIDTH * fScale), int(LOGOWITHNAME_HEIGHT *fScale) );
-   wxBitmap RescaledBitmap( RescaledImage );
-
-   icon =
-       new wxStaticBitmap(S.GetParent(), -1,
-                          //*logo, //v
-                          //v theTheme.Bitmap(bmpAudacityLogo), wxPoint(93, 10), wxSize(215, 190));
-                          //v theTheme.Bitmap(bmpAudacityLogoWithName),
-                          RescaledBitmap,
-                          wxDefaultPosition,
-                          wxSize(int(LOGOWITHNAME_WIDTH*fScale), int(LOGOWITHNAME_HEIGHT*fScale)));
-   delete logo;
+      icon =
+         safenew wxStaticBitmap(S.GetParent(), -1,
+         //*logo, //v
+         //v theTheme.Bitmap(bmpAudacityLogo), wxPoint(93, 10), wxSize(215, 190));
+         //v theTheme.Bitmap(bmpAudacityLogoWithName),
+         RescaledBitmap,
+         wxDefaultPosition,
+         wxSize((int)(LOGOWITHNAME_WIDTH*fScale), (int)(LOGOWITHNAME_HEIGHT*fScale)));
+   }
    S.Prop(0).AddWindow( icon );
 
-   HtmlWindow *html = new LinkingHtmlWindow(S.GetParent(), -1,
+   HtmlWindow *html = safenew LinkingHtmlWindow(S.GetParent(), -1,
                                          wxDefaultPosition,
                                          wxSize(ABOUT_DIALOG_WIDTH, 359),
                                          wxHW_SCROLLBAR_AUTO | wxSUNKEN_BORDER);
@@ -335,19 +412,20 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
    wxString informationStr;   // string to build up list of information in
    S.StartNotebookPage( _("Build Information") );  // start the tab
    S.StartVerticalLay(2);  // create the window
-   HtmlWindow *html = new LinkingHtmlWindow(S.GetParent(), -1, wxDefaultPosition,
+   HtmlWindow *html = safenew LinkingHtmlWindow(S.GetParent(), -1, wxDefaultPosition,
                            wxSize(ABOUT_DIALOG_WIDTH, 264),
                            wxHW_SCROLLBAR_AUTO | wxSUNKEN_BORDER);
    // create a html pane in it to put the content in.
    wxString enabled = _("Enabled");
    wxString disabled = _("Disabled");
-   wxString blank = wxT("");
+   wxString blank;
 
    /* this builds up the list of information to go in the window in the string
     * informationStr */
    informationStr = wxT("<h2><center>");
    informationStr += _("Build Information");
    informationStr += wxT("</center></h2>\n");
+   informationStr += VerCheckHtml();
    // top level heading
    informationStr += wxT("<h3>");
    informationStr += _("File Format Support");
@@ -441,6 +519,12 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
    informationStr += _("Features");
    informationStr += wxT("</h3>\n<table>");  // start table of features
 
+#ifdef EXPERIMENTAL_DA
+   AddBuildinfoRow(&informationStr, wxT("Theme"), _("Dark Theme Extras"), enabled);
+#else
+   AddBuildinfoRow(&informationStr, wxT("Theme"), _("Dark Theme Extras"), disabled);
+#endif
+
    # if USE_NYQUIST
    AddBuildinfoRow(&informationStr, wxT("Nyquist"), _("Plug-in support"),
          enabled);
@@ -522,19 +606,32 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
 
    // Current date
    AddBuildinfoRow(&informationStr, _("Program build date: "), __TDATE__);
+   AddBuildinfoRow(&informationStr, _("Commit Id:"), REV_IDENT );
 
-// Uncomment the next two lines to test hyperlinks work from here.
-//   AddBuildinfoRow(&informationStr, wxT("Link Test:"), 
-//      wxT("<a href=\"https:web.audacityteam.org\">Click bait</a>") );
-
-   AddBuildinfoRow(&informationStr, _("Commit Id:"),
-#include "RevisionIdent.h"
-);
+   // Not translated in 2.3.1.
+   wxString bits = (sizeof(void*) == 8) ? ", 64 bits" : "";
 
 #ifdef __WXDEBUG__
-   AddBuildinfoRow(&informationStr, _("Build type:"), _("Debug build"));
+   AddBuildinfoRow(&informationStr, _("Build type:"), wxString(_("Debug build"))+bits );
 #else
-   AddBuildinfoRow(&informationStr, _("Build type:"), _("Release build"));
+   AddBuildinfoRow(&informationStr, _("Build type:"), wxString(_("Release build"))+bits);
+#endif
+
+#ifdef _MSC_FULL_VER
+   AddBuildinfoRow(&informationStr, _("Compiler:"),
+	   wxString::Format(wxT("MSVC %02d.%02d.%05d.%02d"), _MSC_VER / 100, _MSC_VER % 100, _MSC_FULL_VER % 100000, _MSC_BUILD));
+#endif
+
+#ifdef __GNUC_PATCHLEVEL__
+#ifdef __MINGW32__
+   AddBuildinfoRow(&informationStr, _("Compiler:"), wxT("MinGW ") wxMAKE_VERSION_DOT_STRING_T(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__));
+#else
+   AddBuildinfoRow(&informationStr, _("Compiler:"), wxT("GCC ") wxMAKE_VERSION_DOT_STRING_T(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__));
+#endif
+#endif
+
+#ifdef __clang_version__
+   AddBuildinfoRow(&informationStr, _("Compiler:"), wxT("clang ") __clang_version__);
 #endif
 
    // Install prefix
@@ -550,6 +647,8 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
    // end of table
    informationStr += wxT("</table>\n");
 
+   informationStr = FormatHtmlText( informationStr );
+
    html->SetPage(informationStr);   // push the page into the html renderer
    S.Prop(2).AddWindow( html, wxEXPAND ); // make it fill the page
    // I think the 2 here goes with the StartVerticalLay() call above?
@@ -562,7 +661,7 @@ void AboutDialog::PopulateLicensePage( ShuttleGui & S )
 {
    S.StartNotebookPage( _("GPL License") );
    S.StartVerticalLay(1);
-   HtmlWindow *html = new LinkingHtmlWindow(S.GetParent(), -1,
+   HtmlWindow *html = safenew LinkingHtmlWindow(S.GetParent(), -1,
                                          wxDefaultPosition,
                                          wxSize(ABOUT_DIALOG_WIDTH, 264),
                                          wxHW_SCROLLBAR_AUTO | wxSUNKEN_BORDER);
@@ -573,7 +672,7 @@ void AboutDialog::PopulateLicensePage( ShuttleGui & S )
 // better proportionally spaced.
 //
 // The GPL is not to be translated....
-   wxString PageText=
+   wxString PageText= FormatHtmlText(
 wxT("		    <center>GNU GENERAL PUBLIC LICENSE\n</center>")
 wxT("		       <center>Version 2, June 1991\n</center>")
 wxT("<p><p>")
@@ -855,7 +954,7 @@ wxT("OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n
 wxT("TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY\n")
 wxT("YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER\n")
 wxT("PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE\n")
-wxT("POSSIBILITY OF SUCH DAMAGES.\n");
+wxT("POSSIBILITY OF SUCH DAMAGES.\n"));
 
    html->SetPage( PageText );
 
@@ -865,31 +964,27 @@ wxT("POSSIBILITY OF SUCH DAMAGES.\n");
    S.EndNotebookPage();
 }
 
-void AboutDialog::AddCredit(const wxString& description, Role role)
+void AboutDialog::AddCredit(wxString &&description, Role role)
 {
-   AboutDialogCreditItem* item = new AboutDialogCreditItem();
-   item->description = description;
-   item->role = role;
-   creditItems.Append(item);
+   creditItems.emplace_back(std::move(description), role);
 }
 
 wxString AboutDialog::GetCreditsByRole(AboutDialog::Role role)
 {
    wxString s;
 
-   for (AboutDialogCreditItemsList::compatibility_iterator p=creditItems.GetFirst(); p; p = p->GetNext())
+   for (const auto &item : creditItems)
    {
-      AboutDialogCreditItem* item = p->GetData();
-      if (item->role == role)
+      if (item.role == role)
       {
-         s += item->description;
+         s += item.description;
          s += wxT("<br>");
       }
    }
 
    // Strip last <br>, if any
    if (s.Right(4) == wxT("<br>"))
-      s = s.Left(s.Length() - 4);
+      s = s.Left(s.length() - 4);
 
    return s;
 }
@@ -898,7 +993,7 @@ wxString AboutDialog::GetCreditsByRole(AboutDialog::Role role)
  *
  * Used when creating the build information tab to show if each optional
  * library is enabled or not, and what it does */
-void AboutDialog::AddBuildinfoRow( wxString* htmlstring, const wxChar * libname, const wxChar * libdesc, wxString status)
+void AboutDialog::AddBuildinfoRow( wxString* htmlstring, const wxChar * libname, const wxChar * libdesc, const wxString &status)
 {
    *htmlstring += wxT("<tr><td>");
    *htmlstring += libname;
@@ -922,14 +1017,16 @@ void AboutDialog::AddBuildinfoRow( wxString* htmlstring, const wxChar * libname,
    *htmlstring += wxT("</td></tr>");
 }
 
-
 AboutDialog::~AboutDialog()
 {
-   delete icon;
-//   delete logo;
+   sActiveInstance = {};
 }
 
 void AboutDialog::OnOK(wxCommandEvent & WXUNUSED(event))
 {
+#ifdef __WXMAC__
+   Destroy();
+#else
    EndModal(wxID_OK);
+#endif
 }
