@@ -17,16 +17,19 @@ License: GPL v2.  See License.txt.
 // Store function pointers here when including FFmpeg.h
 #define DEFINE_FFMPEG_POINTERS
 
-#include "Audacity.h"   // for config*.h
+#include "Audacity.h"   // for config*.h // for USE_* macros
 #include "FFmpeg.h"
-#include "AudacityApp.h"
+
 #include "FileNames.h"
-#include "Internat.h"
 #include "widgets/HelpSystem.h"
+#include "widgets/AudacityMessageBox.h"
 
+#include <wx/checkbox.h>
+#include <wx/dynlib.h>
 #include <wx/file.h>
-
-#include "Experimental.h"
+#include <wx/filedlg.h>
+#include <wx/log.h>
+#include <wx/textctrl.h>
 
 #if !defined(USE_FFMPEG)
 /// FFmpeg support may or may not be compiled in,
@@ -40,44 +43,41 @@ wxString GetFFmpegVersion(wxWindow *parent)
 
 /** This pointer to the shared object has global scope and is used to track the
  * singleton object which wraps the FFmpeg codecs */
-FFmpegLibs *FFmpegLibsInst = NULL;
+std::unique_ptr<FFmpegLibs> FFmpegLibsPtr{};
+FFmpegLibs *FFmpegLibsInst()
+{
+   return FFmpegLibsPtr.get();
+}
 
 FFmpegLibs *PickFFmpegLibs()
 {
-   if (FFmpegLibsInst != NULL)
-   {
-      FFmpegLibsInst->refcount++;
-      return FFmpegLibsInst;
-   }
+   if (FFmpegLibsPtr)
+      FFmpegLibsPtr->refcount++;
    else
-   {
-      FFmpegLibsInst = new FFmpegLibs();
-      return FFmpegLibsInst;
-   }
+      FFmpegLibsPtr = std::make_unique<FFmpegLibs>();
+
+   return FFmpegLibsPtr.get();
 }
 
 void DropFFmpegLibs()
 {
-   if (FFmpegLibsInst != NULL)
+   if (FFmpegLibsPtr)
    {
-      FFmpegLibsInst->refcount--;
-      if (FFmpegLibsInst->refcount == 0)
-      {
-         delete FFmpegLibsInst;
-         FFmpegLibsInst = NULL;
-      }
+      FFmpegLibsPtr->refcount--;
+      if (FFmpegLibsPtr->refcount == 0)
+         FFmpegLibsPtr.reset();
    }
 }
 
 bool LoadFFmpeg(bool showerror)
 {
    PickFFmpegLibs();
-   if (FFmpegLibsInst->ValidLibsLoaded())
+   if (FFmpegLibsInst()->ValidLibsLoaded())
    {
       DropFFmpegLibs();
       return true;
    }
-   if (!FFmpegLibsInst->LoadLibs(NULL,showerror))
+   if (!FFmpegLibsInst()->LoadLibs(NULL, showerror))
    {
       DropFFmpegLibs();
       gPrefs->Write(wxT("/FFmpeg/Enabled"), false);
@@ -102,7 +102,7 @@ void FFmpegStartup()
    {
       if (enabled)
       {
-         wxMessageBox(_("FFmpeg was configured in Preferences and successfully loaded before, \
+         AudacityMessageBox(_("FFmpeg was configured in Preferences and successfully loaded before, \
                         \nbut this time Audacity failed to load it at startup. \
                         \n\nYou may want to go back to Preferences > Libraries and re-configure it."),
                       _("FFmpeg startup failed"));
@@ -116,8 +116,8 @@ wxString GetFFmpegVersion(wxWindow * WXUNUSED(parent))
 
    wxString versionString = _("FFmpeg library not found");
 
-   if (FFmpegLibsInst->ValidLibsLoaded()) {
-      versionString = FFmpegLibsInst->GetLibraryVersion();
+   if (FFmpegLibsInst()->ValidLibsLoaded()) {
+      versionString = FFmpegLibsInst()->GetLibraryVersion();
    }
 
    DropFFmpegLibs();
@@ -135,18 +135,11 @@ void av_log_wx_callback(void* ptr, int level, const char* fmt, va_list vl)
    wxString printstring(wxT(""));
 
    if (avc) {
-      printstring.Append(wxString::Format(wxT("[%s @ %p] "), wxString::FromUTF8(avc->item_name(ptr)).c_str(), avc));
+      printstring.Append(wxString::Format(wxT("[%s @ %p] "), wxString::FromUTF8(avc->item_name(ptr)), avc));
    }
 
    wxString frm(fmt,wxConvLibc);
-#if defined(__WXMSW__)
-   frm.Replace(wxT("%t"),wxT("%i"),true); //TODO: on Windows vprintf won't handle %t, and probably some others. Investigate.
-#endif
-#if defined(wxUSE_UNICODE)
-   // String comes with %s format field and a value in value list is ascii char*. Thus in Unicode configurations
-   // we have to convert %s to %S.
-   frm.Replace(wxT("%s"),wxT("%S"),true);
-#endif
+
    printstring.Append(wxString::FormatV(frm,vl));
    wxString cpt;
    switch (level)
@@ -156,15 +149,7 @@ void av_log_wx_callback(void* ptr, int level, const char* fmt, va_list vl)
    case 2: cpt = wxT("Debug"); break;
    default: cpt = wxT("Log"); break;
    }
-#ifdef EXPERIMENTAL_OD_FFMPEG
-//if the decoding happens thru OD then this gets called from a non main thread, which means wxLogDebug
-//will crash.
-//TODO:find some workaround for the log.  perhaps use ODManager as a bridge. for now just print
-   if(!wxThread::IsMain())
-      printf("%s: %s\n",(char*)cpt.char_str(),(char*)printstring.char_str());
-   else
-#endif
-      wxLogDebug(wxT("%s: %s"),cpt.c_str(),printstring.c_str());
+   wxLogDebug(wxT("%s: %s"),cpt,printstring);
 }
 
 //======================= Unicode aware uri protocol for FFmpeg
@@ -178,7 +163,10 @@ static int ufile_read(void *opaque, uint8_t *buf, int size)
 
 static int ufile_write(void *opaque, uint8_t *buf, int size)
 {
-   return (int) ((wxFile *) opaque)->Write(buf, size);
+   auto bytes = (int) ((wxFile *) opaque)->Write(buf, size);
+   if (bytes != size)
+      return -ENOSPC;
+   return bytes;
 }
 
 static int64_t ufile_seek(void *opaque, int64_t pos, int whence)
@@ -209,29 +197,32 @@ static int64_t ufile_seek(void *opaque, int64_t pos, int whence)
 
 int ufile_close(AVIOContext *pb)
 {
-   wxFile *f = (wxFile *) pb->opaque;
+   std::unique_ptr<wxFile> f{ (wxFile *)pb->opaque };
 
+   bool success = true;
    if (f) {
-      f->Close();
-      delete f;
+      success = f->Flush() && f->Close();
+      pb->opaque = nullptr;
    }
 
-   return 0;
+   // We're not certain that a close error is for want of space, but we'll
+   // guess that
+   return success ? 0 : -ENOSPC;
+
+   // Implicitly destroy the wxFile object here
 }
 
 // Open a file with a (possibly) Unicode filename
-int ufile_fopen(AVIOContext **s, const wxString & name, int flags)
+int ufile_fopen(AVIOContext **s, const FilePath & name, int flags)
 {
-   wxFile *f;
    wxFile::OpenMode mode;
 
-   f = new wxFile;
+   auto f = std::make_unique<wxFile>();
    if (!f) {
       return -ENOMEM;
    }
 
    if (flags == (AVIO_FLAG_READ | AVIO_FLAG_WRITE)) {
-      delete f;
       return -EINVAL;
    } else if (flags == AVIO_FLAG_WRITE) {
       mode = wxFile::write;
@@ -240,63 +231,80 @@ int ufile_fopen(AVIOContext **s, const wxString & name, int flags)
    }
 
    if (!f->Open(name, mode)) {
-      delete f;
       return -ENOENT;
    }
 
    *s = avio_alloc_context((unsigned char*)av_malloc(32768), 32768,
                            flags & AVIO_FLAG_WRITE,
-                           /*opaque*/f,
+                           /*opaque*/f.get(),
                            ufile_read,
                            ufile_write,
                            ufile_seek);
    if (!*s) {
-      delete f;
       return -ENOMEM;
    }
 
+   f.release(); // s owns the file object now
+
    return 0;
 }
 
 
 // Detect type of input file and open it if recognized. Routine
 // based on the av_open_input_file() libavformat function.
-int ufile_fopen_input(AVFormatContext **ic_ptr, wxString & name)
+int ufile_fopen_input(std::unique_ptr<FFmpegContext> &context_ptr, FilePath & name)
 {
-   wxFileName f(name);
+   context_ptr.reset();
+   auto context = std::make_unique<FFmpegContext>();
+
+   wxFileName ff{ name };
    wxCharBuffer fname;
    const char *filename;
-   AVIOContext *pb = NULL;
    int err;
 
-   fname = f.GetFullName().mb_str();
+   fname = ff.GetFullName().mb_str();
    filename = (const char *) fname;
 
    // Open the file to prepare for probing
-   if ((err = ufile_fopen(&pb, name, AVIO_FLAG_READ)) < 0) {
+   if ((err = ufile_fopen(&context->pb, name, AVIO_FLAG_READ)) < 0) {
       goto fail;
    }
 
-   *ic_ptr = avformat_alloc_context();
-   (*ic_ptr)->pb = pb;
+   context->ic_ptr = avformat_alloc_context();
+   context->ic_ptr->pb = context->pb;
 
    // And finally, attempt to associate an input stream with the file
-   err = avformat_open_input(ic_ptr, filename, NULL, NULL);
+   err = avformat_open_input(&context->ic_ptr, filename, NULL, NULL);
    if (err) {
       goto fail;
    }
 
+   // success
+   context_ptr = std::move(context);
    return 0;
 
 fail:
 
-   if (pb) {
-      ufile_close(pb);
+   return err;
+}
+
+FFmpegContext::~FFmpegContext()
+{
+   if (FFmpegLibsInst()->ValidLibsLoaded())
+   {
+      if (ic_ptr)
+         avformat_close_input(&ic_ptr);
+      av_log_set_callback(av_log_default_callback);
    }
 
-   *ic_ptr = NULL;
-
-   return err;
+   if (pb) {
+      ufile_close(pb);
+      if (FFmpegLibsInst()->ValidLibsLoaded())
+      {
+         av_free(pb->buffer);
+         av_free(pb);
+      }
+   }
 }
 
 streamContext *import_ffmpeg_read_next_frame(AVFormatContext* formatContext,
@@ -304,14 +312,14 @@ streamContext *import_ffmpeg_read_next_frame(AVFormatContext* formatContext,
                                              unsigned int numStreams)
 {
    streamContext *sc = NULL;
-   AVPacket pkt;
+   AVPacketEx pkt;
 
-   if (av_read_frame(formatContext,&pkt) < 0)
+   if (av_read_frame(formatContext, &pkt) < 0)
    {
       return NULL;
    }
 
-   // Find a stream to which this frame belongs to
+   // Find a stream to which this frame belongs
    for (unsigned int i = 0; i < numStreams; i++)
    {
       if (streams[i]->m_stream->index == pkt.stream_index)
@@ -322,16 +330,14 @@ streamContext *import_ffmpeg_read_next_frame(AVFormatContext* formatContext,
    // When not all streams are selected for import this will happen very often.
    if (sc == NULL)
    {
-      av_free_packet(&pkt);
       return (streamContext*)1;
    }
 
    // Copy the frame to the stream context
-   memcpy(&sc->m_pkt, &pkt, sizeof(AVPacket));
+   sc->m_pkt.create(std::move(pkt));
 
-   sc->m_pktValid = 1;
-   sc->m_pktDataPtr = pkt.data;
-   sc->m_pktRemainingSiz = pkt.size;
+   sc->m_pktDataPtr = sc->m_pkt->data;
+   sc->m_pktRemainingSiz = sc->m_pkt->size;
 
    return sc;
 }
@@ -346,30 +352,29 @@ int import_ffmpeg_decode_frame(streamContext *sc, bool flushing)
 
    if (flushing)
    {
-      // If we're flushing the decoders we don't actually have any new data to decode.
+      // If we're flushing the decoders we don't actually have any NEW data to decode.
       pDecode = NULL;
       nDecodeSiz = 0;
    }
    else
    {
-      if (!sc->m_pktValid || (sc->m_pktRemainingSiz <= 0))
+      if (!sc->m_pkt || (sc->m_pktRemainingSiz <= 0))
       {
          //No more data
          return -1;
       }
    }
 
-   AVPacket avpkt;
-   av_init_packet(&avpkt);
+   AVPacketEx avpkt;
    avpkt.data = pDecode;
    avpkt.size = nDecodeSiz;
 
-   AVFrame *frame = av_frame_alloc();
+   AVFrameHolder frame{ av_frame_alloc() };
    int got_output = 0;
 
    nBytesDecoded =
       avcodec_decode_audio4(sc->m_codecCtx,
-                            frame,                                   // out
+                            frame.get(),                                   // out
                             &got_output,                             // out
                             &avpkt);                                 // in
 
@@ -380,23 +385,19 @@ int import_ffmpeg_decode_frame(streamContext *sc, bool flushing)
    }
 
    sc->m_samplefmt = sc->m_codecCtx->sample_fmt;
-   sc->m_samplesize = av_get_bytes_per_sample(sc->m_samplefmt);
+   sc->m_samplesize = static_cast<size_t>(av_get_bytes_per_sample(sc->m_samplefmt));
 
    int channels = sc->m_codecCtx->channels;
-   unsigned int newsize = sc->m_samplesize * frame->nb_samples * channels;
+   auto newsize = sc->m_samplesize * frame->nb_samples * channels;
    sc->m_decodedAudioSamplesValidSiz = newsize;
    // Reallocate the audio sample buffer if it's smaller than the frame size.
    if (newsize > sc->m_decodedAudioSamplesSiz )
    {
-      if (sc->m_decodedAudioSamples)
-      {
-         av_free(sc->m_decodedAudioSamples);
-      }
-
-      sc->m_decodedAudioSamples = (uint8_t *) av_malloc(newsize);
+      // Reallocate a bigger buffer.  But av_realloc is NOT compatible with the returns of av_malloc!
+      // So do this:
+      sc->m_decodedAudioSamples.reset(static_cast<uint8_t *>(av_malloc(newsize)));
       sc->m_decodedAudioSamplesSiz = newsize;
-
-      if (sc->m_decodedAudioSamples == NULL)
+      if (!sc->m_decodedAudioSamples)
       {
          //Can't allocate bytes
          return -1;
@@ -405,17 +406,15 @@ int import_ffmpeg_decode_frame(streamContext *sc, bool flushing)
    if (frame->data[1]) {
       for (int i = 0; i<frame->nb_samples; i++) {
          for (int ch = 0; ch<channels; ch++) {
-            memcpy(sc->m_decodedAudioSamples + sc->m_samplesize * (ch + channels*i),
+            memcpy(sc->m_decodedAudioSamples.get() + sc->m_samplesize * (ch + channels*i),
                   frame->extended_data[ch] + sc->m_samplesize*i,
                   sc->m_samplesize);
          }
       }
    } else {
-      memcpy(sc->m_decodedAudioSamples, frame->data[0], newsize);
+      memcpy(sc->m_decodedAudioSamples.get(), frame->data[0], newsize);
    }
 
-   av_frame_free(&frame);
-
    // We may not have read all of the data from this packet. If so, the user can call again.
    // Whether or not they do depends on if m_pktRemainingSiz == 0 (they can check).
    sc->m_pktDataPtr += nBytesDecoded;
@@ -445,12 +444,12 @@ class FFmpegNotFoundDialog;
 #define ID_FFMPEG_DLOAD  5001
 
 /// Allows user to locate libav* libraries
-class FindFFmpegDialog : public wxDialog
+class FindFFmpegDialog final : public wxDialogWrapper
 {
 public:
 
-   FindFFmpegDialog(wxWindow *parent, wxString path, wxString name, wxString type)
-      :  wxDialog(parent, wxID_ANY, wxString(_("Locate FFmpeg")))
+   FindFFmpegDialog(wxWindow *parent, const wxString &path, const wxString &name, const wxString &type)
+      :  wxDialogWrapper(parent, wxID_ANY, wxString(_("Locate FFmpeg")))
    {
       SetName(GetTitle());
       ShuttleGui S(this, eIsCreating);
@@ -471,13 +470,13 @@ public:
       S.SetBorder(10);
       S.StartVerticalLay(true);
       {
-         text.Printf(_("Audacity needs the file '%s' to import and export audio via FFmpeg."), mName.c_str());
+         text.Printf(_("Audacity needs the file '%s' to import and export audio via FFmpeg."), mName);
          S.AddTitle(text);
 
          S.SetBorder(3);
          S.StartHorizontalLay(wxALIGN_LEFT, true);
          {
-            text.Printf(_("Location of '%s':"), mName.c_str());
+            text.Printf(_("Location of '%s':"), mName);
             S.AddTitle(text);
          }
          S.EndHorizontalLay();
@@ -485,12 +484,12 @@ public:
          S.StartMultiColumn(2, wxEXPAND);
          S.SetStretchyCol(0);
          {
-            if (mLibPath.GetFullPath().IsEmpty()) {
-               text.Printf(_("To find '%s', click here -->"), mName.c_str());
-               mPathText = S.AddTextBox(wxT(""), text, 0);
+            if (mLibPath.GetFullPath().empty()) {
+               text.Printf(_("To find '%s', click here -->"), mName);
+               mPathText = S.AddTextBox( {}, text, 0);
             }
             else {
-               mPathText = S.AddTextBox(wxT(""), mLibPath.GetFullPath(), 0);
+               mPathText = S.AddTextBox( {}, mLibPath.GetFullPath(), 0);
             }
             S.Id(ID_FFMPEG_BROWSE).AddButton(_("Browse..."), wxALIGN_RIGHT);
             S.AddVariableText(_("To get a free copy of FFmpeg, click here -->"), true);
@@ -516,16 +515,17 @@ public:
       /* i18n-hint: It's asking for the location of a file, for
       example, "Where is lame_enc.dll?" - you could translate
       "Where would I find the file '%s'?" instead if you want. */
-      question.Printf(_("Where is '%s'?"), mName.c_str());
+      question.Printf(_("Where is '%s'?"), mName);
 
-      wxString path = FileSelector(question,
+      wxString path = FileNames::SelectFile(FileNames::Operation::_None,
+         question,
          mLibPath.GetPath(),
          mLibPath.GetName(),
          wxT(""),
          mType,
          wxFD_OPEN | wxRESIZE_BORDER,
          this);
-      if (!path.IsEmpty()) {
+      if (!path.empty()) {
          mLibPath = path;
          mPathText->SetValue(path);
       }
@@ -533,7 +533,7 @@ public:
 
    void OnDownload(wxCommandEvent & WXUNUSED(event))
    {
-      HelpSystem::ShowHelpDialog(this, wxT("FAQ:Installation_and_Plug-Ins#ffdown"));
+      HelpSystem::ShowHelp(this, wxT("FAQ:Installing_the_FFmpeg_Import_Export_Library"));
    }
 
    wxString GetLibPath()
@@ -554,7 +554,7 @@ private:
    DECLARE_EVENT_TABLE()
 };
 
-BEGIN_EVENT_TABLE(FindFFmpegDialog, wxDialog)
+BEGIN_EVENT_TABLE(FindFFmpegDialog, wxDialogWrapper)
    EVT_BUTTON(ID_FFMPEG_BROWSE, FindFFmpegDialog::OnBrowse)
    EVT_BUTTON(ID_FFMPEG_DLOAD,  FindFFmpegDialog::OnDownload)
 END_EVENT_TABLE()
@@ -564,7 +564,55 @@ END_EVENT_TABLE()
 // FFmpegNotFoundDialog
 //----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(FFmpegNotFoundDialog, wxDialog)
+FFmpegNotFoundDialog::FFmpegNotFoundDialog(wxWindow *parent)
+   :  wxDialogWrapper(parent, wxID_ANY, wxString(_("FFmpeg not found")))
+{
+   SetName(GetTitle());
+   ShuttleGui S(this, eIsCreating);
+   PopulateOrExchange(S);
+}
+
+void FFmpegNotFoundDialog::PopulateOrExchange(ShuttleGui & S)
+{
+   wxString text;
+
+   S.SetBorder(10);
+   S.StartVerticalLay(true);
+   {
+      S.AddFixedText(_(
+"Audacity attempted to use FFmpeg to import an audio file,\n\
+but the libraries were not found.\n\n\
+To use FFmpeg import, go to Edit > Preferences > Libraries\n\
+to download or locate the FFmpeg libraries."
+      ));
+
+      int dontShowDlg = 0;
+      gPrefs->Read(wxT("/FFmpeg/NotFoundDontShow"),&dontShowDlg,0);
+      mDontShow = S.AddCheckBox(_("Do not show this warning again"),dontShowDlg);
+
+      S.AddStandardButtons(eOkButton);
+   }
+   S.EndVerticalLay();
+
+   Layout();
+   Fit();
+   SetMinSize(GetSize());
+   Center();
+
+   return;
+}
+
+void FFmpegNotFoundDialog::OnOk(wxCommandEvent & WXUNUSED(event))
+{
+   if (mDontShow->GetValue())
+   {
+      gPrefs->Write(wxT("/FFmpeg/NotFoundDontShow"),1);
+      gPrefs->Flush();
+   }
+   this->EndModal(0);
+}
+
+BEGIN_EVENT_TABLE(FFmpegNotFoundDialog, wxDialogWrapper)
    EVT_BUTTON(wxID_OK, FFmpegNotFoundDialog::OnOk)
 END_EVENT_TABLE()
 
@@ -577,7 +625,6 @@ FFmpegLibs::FFmpegLibs()
 {
    mLibsLoaded = false;
    refcount = 1;
-   avformat = avcodec = avutil = NULL;
    if (gPrefs) {
       mLibAVFormatPath = gPrefs->Read(wxT("/FFmpeg/FFmpegLibPath"), wxT(""));
    }
@@ -594,18 +641,19 @@ bool FFmpegLibs::FindLibs(wxWindow *parent)
    wxString path;
    wxString name;
 
+   // If we're looking for the lib, use the standard name, as the
+   // configured name is not found.
+   name = GetLibAVFormatName();
    wxLogMessage(wxT("Looking for FFmpeg libraries..."));
-   if (!mLibAVFormatPath.IsEmpty()) {
-      wxLogMessage(wxT("mLibAVFormatPath ('%s') is not empty."), mLibAVFormatPath.c_str());
-      wxFileName fn = mLibAVFormatPath;
+   if (!mLibAVFormatPath.empty()) {
+      wxLogMessage(wxT("mLibAVFormatPath ('%s') is not empty."), mLibAVFormatPath);
+      const wxFileName fn{ mLibAVFormatPath };
       path = fn.GetPath();
-      name = fn.GetFullName();
    }
    else {
       path = GetLibAVFormatPath();
-      name = GetLibAVFormatName();
       wxLogMessage(wxT("mLibAVFormatPath is empty, starting with path '%s', name '%s'."),
-                  path.c_str(), name.c_str());
+                  path, name);
    }
 
    FindFFmpegDialog fd(parent,
@@ -620,7 +668,7 @@ bool FFmpegLibs::FindLibs(wxWindow *parent)
 
    path = fd.GetLibPath();
 
-   wxLogMessage(wxT("User-specified path = '%s'"), path.c_str());
+   wxLogMessage(wxT("User-specified path = '%s'"), path);
    if (!::wxFileExists(path)) {
       wxLogError(wxT("User-specified file does not exist. Failed to find FFmpeg libraries."));
       return false;
@@ -647,16 +695,16 @@ bool FFmpegLibs::LoadLibs(wxWindow * WXUNUSED(parent), bool showerr)
    }
 
    // First try loading it from a previously located path
-   if (!mLibAVFormatPath.IsEmpty()) {
-      wxLogMessage(wxT("mLibAVFormatPath ('%s') is not empty. Loading from it."),mLibAVFormatPath.c_str());
+   if (!mLibAVFormatPath.empty()) {
+      wxLogMessage(wxT("mLibAVFormatPath ('%s') is not empty. Loading from it."),mLibAVFormatPath);
       mLibsLoaded = InitLibs(mLibAVFormatPath,showerr);
    }
 
    // If not successful, try loading it from default path
-   if (!mLibsLoaded && !GetLibAVFormatPath().IsEmpty()) {
-      wxFileName fn(GetLibAVFormatPath(), GetLibAVFormatName());
+   if (!mLibsLoaded && !GetLibAVFormatPath().empty()) {
+      const wxFileName fn{ GetLibAVFormatPath(), GetLibAVFormatName() };
       wxString path = fn.GetFullPath();
-      wxLogMessage(wxT("Trying to load FFmpeg libraries from default path, '%s'."), path.c_str());
+      wxLogMessage(wxT("Trying to load FFmpeg libraries from default path, '%s'."), path);
       mLibsLoaded = InitLibs(path,showerr);
       if (mLibsLoaded) {
          mLibAVFormatPath = path;
@@ -665,10 +713,10 @@ bool FFmpegLibs::LoadLibs(wxWindow * WXUNUSED(parent), bool showerr)
 
 #if defined(__WXMAC__)
    // If not successful, try loading it from legacy path
-   if (!mLibsLoaded && !GetLibAVFormatPath().IsEmpty()) {
-      wxFileName fn(wxT("/usr/local/lib/audacity"), GetLibAVFormatName());
+   if (!mLibsLoaded && !GetLibAVFormatPath().empty()) {
+      const wxFileName fn{wxT("/usr/local/lib/audacity"), GetLibAVFormatName()};
       wxString path = fn.GetFullPath();
-      wxLogMessage(wxT("Trying to load FFmpeg libraries from legacy path, '%s'."), path.c_str());
+      wxLogMessage(wxT("Trying to load FFmpeg libraries from legacy path, '%s'."), path);
       mLibsLoaded = InitLibs(path,showerr);
       if (mLibsLoaded) {
          mLibAVFormatPath = path;
@@ -679,7 +727,7 @@ bool FFmpegLibs::LoadLibs(wxWindow * WXUNUSED(parent), bool showerr)
    // If not successful, try loading using system search paths
    if (!ValidLibsLoaded()) {
       wxString path = GetLibAVFormatName();
-      wxLogMessage(wxT("Trying to load FFmpeg libraries from system paths. File name is '%s'."), path.c_str());
+      wxLogMessage(wxT("Trying to load FFmpeg libraries from system paths. File name is '%s'."), path);
       mLibsLoaded = InitLibs(path,showerr);
       if (mLibsLoaded) {
          mLibAVFormatPath = path;
@@ -692,21 +740,16 @@ bool FFmpegLibs::LoadLibs(wxWindow * WXUNUSED(parent), bool showerr)
    {
       wxLogError(wxT("Failed to load libraries altogether."));
       int dontShowDlg;
-      FFmpegNotFoundDialog *dlg;
       gPrefs->Read(wxT("/FFmpeg/NotFoundDontShow"),&dontShowDlg,0);
       if ((dontShowDlg == 0) && (showerr))
-      {
-         dlg = new FFmpegNotFoundDialog(NULL);
-         dlg->ShowModal();
-         delete dlg;
-      }
+         FFmpegNotFoundDialog{nullptr}.ShowModal();
    }
    */
    // Oh well, just give up
    if (!ValidLibsLoaded()) {
       wxString msg = _("Failed to find compatible FFmpeg libraries.");
       if (showerr)
-         wxMessageBox(msg);
+         AudacityMessageBox(msg);
       wxLogError(msg);
       return false;
    }
@@ -720,7 +763,7 @@ bool FFmpegLibs::ValidLibsLoaded()
    return mLibsLoaded;
 }
 
-bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
+bool FFmpegLibs::InitLibs(const wxString &libpath_format, bool WXUNUSED(showerr))
 {
 #if !defined(DISABLE_DYNAMIC_LOADING_FFMPEG)
    FreeLibs();
@@ -733,31 +776,31 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    // First take PATH environment variable and store its content.
    if (wxGetEnv(wxT("PATH"),&syspath))
    {
-      wxLogMessage(wxT("PATH = '%s'"), syspath.c_str());
-      wxString fmtdirsc = wxPathOnly(libpath_format) + wxT(";");
-      wxString scfmtdir = wxT(";") + wxPathOnly(libpath_format);
-      wxString fmtdir = wxPathOnly(libpath_format);
-      wxLogMessage(wxT("Checking that '%s' is in PATH..."), fmtdir.c_str());
+      wxLogMessage(wxT("PATH = '%s'"), syspath);
+      const wxString &fmtdir{ wxPathOnly(libpath_format) };
+      wxString fmtdirsc = fmtdir + wxT(";");
+      wxString scfmtdir = wxT(";") + fmtdir;
+      wxLogMessage(wxT("Checking that '%s' is in PATH..."), fmtdir);
       // If the directory, where libavformat is, is not in PATH - add it
       if (!syspath.Contains(fmtdirsc) && !syspath.Contains(scfmtdir) && !syspath.Contains(fmtdir))
       {
-         wxLogWarning(wxT("FFmpeg directory '%s' is not in PATH."), fmtdir.c_str());
+         wxLogWarning(wxT("FFmpeg directory '%s' is not in PATH."), fmtdir);
          if (syspath.Left(1) == wxT(';'))
          {
-            wxLogMessage(wxT("Temporarily preending '%s' to PATH..."), fmtdir.c_str());
+            wxLogMessage(wxT("Temporarily prepending '%s' to PATH..."), fmtdir);
             syspath.Prepend(scfmtdir);
          }
          else
          {
-            wxLogMessage(wxT("Temporarily prepending '%s' to PATH..."), scfmtdir.c_str());
+            wxLogMessage(wxT("Temporarily prepending '%s' to PATH..."), scfmtdir);
             syspath.Prepend(fmtdirsc);
          }
 
-         if (wxSetEnv(wxT("PATH"),syspath.c_str()))
+         if (wxSetEnv(wxT("PATH"),syspath))
             // Remember to change PATH back to normal after we're done
             pathfix = true;
          else
-            wxLogSysError(wxT("Setting PATH via wxSetEnv('%s') failed."),syspath.c_str());
+            wxLogSysError(wxT("Setting PATH via wxSetEnv('%s') failed."),syspath);
       }
       else
       {
@@ -776,22 +819,23 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    wxDynamicLibrary *util = NULL;
    wxFileName avcodec_filename;
    wxFileName avutil_filename;
-   wxFileName name(libpath_format);
+   wxFileName name{ libpath_format };
+   wxString nameFull{name.GetFullPath()};
    bool gotError = false;
 
    // Check for a monolithic avformat
-   avformat = new wxDynamicLibrary();
-   wxLogMessage(wxT("Checking for monolithic avformat from '%s'."), name.GetFullPath().c_str());
-   gotError = !avformat->Load(name.GetFullPath(), wxDL_LAZY);
+   avformat = std::make_unique<wxDynamicLibrary>();
+   wxLogMessage(wxT("Checking for monolithic avformat from '%s'."), nameFull);
+   gotError = !avformat->Load(nameFull, wxDL_LAZY);
 
    // Verify it really is monolithic
    if (!gotError) {
       avutil_filename = FileNames::PathFromAddr(avformat->GetSymbol(wxT("avutil_version")));
       avcodec_filename = FileNames::PathFromAddr(avformat->GetSymbol(wxT("avcodec_version")));
-      if (avutil_filename.GetFullPath().IsSameAs(name.GetFullPath())) {
-         if (avcodec_filename.GetFullPath().IsSameAs(name.GetFullPath())) {
-            util = avformat;
-            codec = avformat;
+      if (avutil_filename.GetFullPath() == nameFull) {
+         if (avcodec_filename.GetFullPath() == nameFull) {
+            util = avformat.get();
+            codec = avformat.get();
          }
       }
       if (!avcodec_filename.FileExists()) {
@@ -812,22 +856,27 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
       }
    }
 
+   // The two wxFileNames don't change after this
+   const wxString avcodec_filename_full{ avcodec_filename.GetFullPath() };
+   const wxString avutil_filename_full{ avutil_filename.GetFullPath() };
+
    if (!util) {
-      avutil = util = new wxDynamicLibrary();
-      wxLogMessage(wxT("Loading avutil from '%s'."), avutil_filename.GetFullPath().c_str());
-      util->Load(avutil_filename.GetFullPath(), wxDL_LAZY);
+      util = (avutil = std::make_unique<wxDynamicLibrary>()).get();
+      wxLogMessage(wxT("Loading avutil from '%s'."), avutil_filename_full);
+      util->Load(avutil_filename_full, wxDL_LAZY);
    }
 
    if (!codec) {
-      avcodec = codec = new wxDynamicLibrary();
-      wxLogMessage(wxT("Loading avcodec from '%s'."), avcodec_filename.GetFullPath().c_str());
-      codec->Load(avcodec_filename.GetFullPath(), wxDL_LAZY);
+      codec = (avcodec = std::make_unique<wxDynamicLibrary>()).get();
+      wxLogMessage(wxT("Loading avcodec from '%s'."), avcodec_filename_full);
+      codec->Load(avcodec_filename_full, wxDL_LAZY);
    }
 
    if (!avformat->IsLoaded()) {
       name.SetFullName(libpath_format);
-      wxLogMessage(wxT("Loading avformat from '%s'."), name.GetFullPath().c_str());
-      gotError = !avformat->Load(name.GetFullPath(), wxDL_LAZY);
+      nameFull = name.GetFullPath();
+      wxLogMessage(wxT("Loading avformat from '%s'."), nameFull);
+      gotError = !avformat->Load(nameFull, wxDL_LAZY);
    }
 
 #if defined(__WXMSW__)
@@ -836,7 +885,7 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    {
       wxString oldpath = syspath.BeforeLast(wxT(';'));
       wxLogMessage(wxT("Returning PATH to previous setting..."));
-      wxSetEnv(wxT("PATH"),oldpath.c_str());
+      wxSetEnv(wxT("PATH"),oldpath);
    }
 #endif
 
@@ -849,15 +898,15 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    // Show the actual libraries loaded
    if (avutil) {
       wxLogMessage(wxT("Actual avutil path %s"),
-                 FileNames::PathFromAddr(avutil->GetSymbol(wxT("avutil_version"))).c_str());
+                 FileNames::PathFromAddr(avutil->GetSymbol(wxT("avutil_version"))));
    }
    if (avcodec) {
       wxLogMessage(wxT("Actual avcodec path %s"),
-                 FileNames::PathFromAddr(avcodec->GetSymbol(wxT("avcodec_version"))).c_str());
+                 FileNames::PathFromAddr(avcodec->GetSymbol(wxT("avcodec_version"))));
    }
    if (avformat) {
       wxLogMessage(wxT("Actual avformat path %s"),
-                 FileNames::PathFromAddr(avformat->GetSymbol(wxT("avformat_version"))).c_str());
+                 FileNames::PathFromAddr(avformat->GetSymbol(wxT("avformat_version"))));
    }
 
    wxLogMessage(wxT("Importing symbols..."));
@@ -878,12 +927,14 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    FFMPEG_INITDYN(avformat, avio_size);
    FFMPEG_INITDYN(avformat, avio_alloc_context);
    FFMPEG_INITALT(avformat, av_guess_format, avformat, guess_format);
+   FFMPEG_INITDYN(avformat, avformat_free_context);
 
    FFMPEG_INITDYN(avcodec, av_init_packet);
    FFMPEG_INITDYN(avcodec, av_free_packet);
    FFMPEG_INITDYN(avcodec, avcodec_find_encoder);
    FFMPEG_INITDYN(avcodec, avcodec_find_encoder_by_name);
    FFMPEG_INITDYN(avcodec, avcodec_find_decoder);
+   FFMPEG_INITDYN(avcodec, avcodec_get_name);
    FFMPEG_INITDYN(avcodec, avcodec_open2);
    FFMPEG_INITDYN(avcodec, avcodec_decode_audio4);
    FFMPEG_INITDYN(avcodec, avcodec_encode_audio2);
@@ -895,6 +946,7 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    FFMPEG_INITDYN(avcodec, avcodec_fill_audio_frame);
 
    FFMPEG_INITDYN(avutil, av_free);
+   FFMPEG_INITDYN(avutil, av_dict_free);
    FFMPEG_INITDYN(avutil, av_dict_get);
    FFMPEG_INITDYN(avutil, av_dict_set);
    FFMPEG_INITDYN(avutil, av_get_bytes_per_sample);
@@ -907,12 +959,13 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    FFMPEG_INITDYN(avutil, av_fifo_size);
    FFMPEG_INITDYN(avutil, av_malloc);
    FFMPEG_INITDYN(avutil, av_fifo_generic_write);
-   FFMPEG_INITDYN(avutil, av_freep);
+   // FFMPEG_INITDYN(avutil, av_freep);
    FFMPEG_INITDYN(avutil, av_rescale_q);
    FFMPEG_INITDYN(avutil, avutil_version);
    FFMPEG_INITALT(avutil, av_frame_alloc, avcodec, avcodec_alloc_frame);
    FFMPEG_INITALT(avutil, av_frame_free, avcodec, avcodec_free_frame);
    FFMPEG_INITDYN(avutil, av_samples_get_buffer_size);
+   FFMPEG_INITDYN(avutil, av_get_default_channel_layout);
 
    wxLogMessage(wxT("All symbols loaded successfully. Initializing the library."));
 #endif
@@ -930,18 +983,18 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
    mAVUtilVersion = wxString::Format(wxT("%d.%d.%d"),avuver >> 16 & 0xFF, avuver >> 8 & 0xFF, avuver & 0xFF);
 
    wxLogMessage(wxT("   AVCodec version 0x%06x - %s (built against 0x%06x - %s)"),
-                  avcver, mAVCodecVersion.c_str(), LIBAVCODEC_VERSION_INT,
-                  wxString::FromUTF8(AV_STRINGIFY(LIBAVCODEC_VERSION)).c_str());
+                  avcver, mAVCodecVersion, LIBAVCODEC_VERSION_INT,
+                  wxString::FromUTF8(AV_STRINGIFY(LIBAVCODEC_VERSION)));
    wxLogMessage(wxT("   AVFormat version 0x%06x - %s (built against 0x%06x - %s)"),
-                  avfver, mAVFormatVersion.c_str(), LIBAVFORMAT_VERSION_INT,
-                  wxString::FromUTF8(AV_STRINGIFY(LIBAVFORMAT_VERSION)).c_str());
+                  avfver, mAVFormatVersion, LIBAVFORMAT_VERSION_INT,
+                  wxString::FromUTF8(AV_STRINGIFY(LIBAVFORMAT_VERSION)));
    wxLogMessage(wxT("   AVUtil version 0x%06x - %s (built against 0x%06x - %s)"),
-                  avuver,mAVUtilVersion.c_str(), LIBAVUTIL_VERSION_INT,
-                  wxString::FromUTF8(AV_STRINGIFY(LIBAVUTIL_VERSION)).c_str());
+                  avuver,mAVUtilVersion, LIBAVUTIL_VERSION_INT,
+                  wxString::FromUTF8(AV_STRINGIFY(LIBAVUTIL_VERSION)));
 
-   int avcverdiff = (avcver >> 16 & 0xFF) - int(LIBAVCODEC_VERSION_MAJOR);
-   int avfverdiff = (avfver >> 16 & 0xFF) - int(LIBAVFORMAT_VERSION_MAJOR);
-   int avuverdiff = (avuver >> 16 & 0xFF) - int(LIBAVUTIL_VERSION_MAJOR);
+   int avcverdiff = (avcver >> 16 & 0xFF) - (int)(LIBAVCODEC_VERSION_MAJOR);
+   int avfverdiff = (avfver >> 16 & 0xFF) - (int)(LIBAVFORMAT_VERSION_MAJOR);
+   int avuverdiff = (avuver >> 16 & 0xFF) - (int)(LIBAVUTIL_VERSION_MAJOR);
    if (avcverdiff != 0)
       wxLogError(wxT("AVCodec version mismatch = %d"), avcverdiff);
    if (avfverdiff != 0)
@@ -960,23 +1013,10 @@ bool FFmpegLibs::InitLibs(wxString libpath_format, bool WXUNUSED(showerr))
 
 void FFmpegLibs::FreeLibs()
 {
-   if (avformat != NULL) {
-      delete avformat;
-      avformat = NULL;
-   }
-
-   if (avcodec != NULL) {
-      delete avcodec;
-      avcodec = NULL;
-   }
-
-   if (avutil != NULL) {
-      delete avutil;
-      avutil = NULL;
-   }
-
+   avformat.reset();
+   avcodec.reset();
+   avutil.reset();
    mLibsLoaded = false;
-
    return;
 }
 
