@@ -11,8 +11,11 @@
 #ifndef __AUDACITY_FILEIO__
 #define __AUDACITY_FILEIO__
 
-#include <wx/object.h>
-#include <wx/wfstream.h>
+#include "MemoryX.h"
+
+class wxInputStream;
+class wxOutputStream;
+class wxFFileOutputStream;
 
 class FileIO
 {
@@ -25,11 +28,13 @@ class FileIO
 
  public:
    FileIO(const wxString & name, FileIOMode mode);
+
+   // Calls Close()
    ~FileIO();
 
    bool IsOpened();
 
-   void Close();
+   bool Close();
 
    wxInputStream & Read(void *buffer, size_t size);
    wxOutputStream & Write(const void *buffer, size_t size);
@@ -37,8 +42,8 @@ class FileIO
  private:
    wxString mName;
    FileIOMode mMode;
-   wxInputStream *mInputStream;
-   wxOutputStream *mOutputStream;
+   std::unique_ptr<wxInputStream> mInputStream;
+   std::unique_ptr<wxFFileOutputStream> mOutputStream;
    bool mOpen;
 };
 
