@@ -11,25 +11,25 @@
 #ifndef __AUDACITY_HISTORY_WINDOW__
 #define __AUDACITY_HISTORY_WINDOW__
 
-#include <wx/button.h>
-#include <wx/dialog.h>
-#include <wx/event.h>
-#include <wx/frame.h>
-#include <wx/listctrl.h>
-#include <wx/spinctrl.h>
-#include <wx/textctrl.h>
+#include "widgets/wxPanelWrapper.h" // to inherit
 
+class wxButton;
+class wxListCtrl;
+class wxListEvent;
+class wxSpinCtrl;
+class wxTextCtrl;
 class AudacityProject;
 class ShuttleGui;
 class UndoManager;
 
-class HistoryWindow :public wxDialog {
+class HistoryWindow final : public wxDialogWrapper {
 
  public:
    HistoryWindow(AudacityProject * parent, UndoManager *manager);
-   ~HistoryWindow();
 
-   void UpdateDisplay();
+   void UpdateDisplay(wxEvent &e);
+   
+   bool Show( bool show = true ) override;
 
  private:
    void OnAudioIO(wxCommandEvent & evt);
@@ -38,14 +38,15 @@ class HistoryWindow :public wxDialog {
 
    void OnSize(wxSizeEvent & event);
    void OnCloseWindow(wxCloseEvent & WXUNUSED(event));
-   void OnChar(wxKeyEvent & event);
    void OnItemSelected(wxListEvent & event);
    void OnDiscard(wxCommandEvent & event);
+   void OnDiscardClipboard(wxCommandEvent & event);
 
    AudacityProject   *mProject;
    UndoManager       *mManager;
    wxListCtrl        *mList;
    wxTextCtrl        *mTotal;
+   wxTextCtrl        *mClipboard;
    wxTextCtrl        *mAvail;
    wxSpinCtrl        *mLevels;
    wxButton          *mDiscard;
