@@ -24,21 +24,25 @@
 #include <wx/filedlg.h>
 #include <wx/grid.h>
 #include <wx/intl.h>
-#include <wx/msgdlg.h>
+#include <wx/scrolbar.h>
 #include <wx/settings.h>
 #include <wx/sizer.h>
 #include <wx/stattext.h>
 #include <wx/textdlg.h>
 
 #include "ShuttleGui.h"
-#include "Internat.h"
 #include "LabelTrack.h"
 #include "Prefs.h"
 #include "Project.h"
+#include "ProjectWindow.h"
 #include "ViewInfo.h"
-#include "widgets/NumericTextCtrl.h"
+#include "tracks/labeltrack/ui/LabelTrackView.h"
+#include "widgets/AudacityMessageBox.h"
+#include "widgets/ErrorDialog.h"
+#include "widgets/Grid.h"
+#include "widgets/HelpSystem.h"
 
-#include "FileDialog.h"
+#include "FileNames.h"
 #include <limits>
 
 enum Column
@@ -47,6 +51,8 @@ enum Column
    Col_Label,
    Col_Stime,
    Col_Etime,
+   Col_Lfreq,
+   Col_Hfreq,
    Col_Max
 };
 
@@ -55,10 +61,11 @@ enum Column
 class RowData
 {
  public:
-   RowData() {};
+   RowData(int index_, const wxString &title_, const SelectedRegion &selectedRegion_)
+      : index(index_), title(title_), selectedRegion(selectedRegion_)
+   {}
 
    int index;
-
    wxString title;
    SelectedRegion selectedRegion;
 };
@@ -71,7 +78,7 @@ enum {
    ID_EXPORT
 };
 
-BEGIN_EVENT_TABLE(LabelDialog, wxDialog)
+BEGIN_EVENT_TABLE(LabelDialog, wxDialogWrapper)
    EVT_GRID_SELECT_CELL(LabelDialog::OnSelectCell)
    EVT_GRID_CELL_CHANGED(LabelDialog::OnCellChange)
    EVT_BUTTON(ID_INSERTA, LabelDialog::OnInsert)
@@ -82,61 +89,45 @@ BEGIN_EVENT_TABLE(LabelDialog, wxDialog)
    EVT_BUTTON(wxID_OK,      LabelDialog::OnOK)
    EVT_BUTTON(wxID_CANCEL,  LabelDialog::OnCancel)
    EVT_COMMAND(wxID_ANY, EVT_TIMETEXTCTRL_UPDATED, LabelDialog::OnUpdate)
+   EVT_COMMAND(wxID_ANY, EVT_FREQUENCYTEXTCTRL_UPDATED,
+               LabelDialog::OnFreqUpdate)
+   EVT_BUTTON(wxID_HELP, LabelDialog::OnHelp)
 END_EVENT_TABLE()
 
 LabelDialog::LabelDialog(wxWindow *parent,
-                         DirManager *dirmanager,
+                         TrackFactory &factory,
                          TrackList *tracks,
+                         LabelTrack *selectedTrack,
+                         int index,
                          ViewInfo &viewinfo,
                          double rate,
-                         const wxString & format)
-: wxDialog(parent,
+                         const NumericFormatSymbol & format,
+                         const NumericFormatSymbol &freqFormat)
+: wxDialogWrapper(parent,
            wxID_ANY,
            _("Edit Labels"),
            wxDefaultPosition,
            wxSize(800, 600),
            wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER),
-  mDirManager(dirmanager),
-  mTracks(tracks),
-  mViewInfo(&viewinfo),
+  mFactory(factory),
+  mTracks(tracks)
+  , mSelectedTrack(selectedTrack)
+  , mIndex(index)
+  , mViewInfo(&viewinfo),
   mRate(rate),
   mFormat(format)
+  , mFreqFormat(freqFormat)
 {
    SetName(GetTitle());
+   Populate();
+}
 
-   // Create the main sizer
-   wxBoxSizer *vs = new wxBoxSizer(wxVERTICAL);
-
-   // A little instruction
-   wxStaticText *instruct =
-      new wxStaticText(this,
-                       wxID_ANY,
-                       _("Press F2 or double click to edit cell contents."));
-   instruct->SetName(instruct->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   vs->Add(instruct,
-           0,
-           wxALIGN_LEFT | wxALL,
-           5);
-
-   // Create the main sizer
-   mGrid = new Grid(this, wxID_ANY);
-   vs->Add(mGrid, 1, wxEXPAND | wxALL, 5);
-
-   // Create the action buttons
-   wxBoxSizer *hs = new wxBoxSizer(wxHORIZONTAL);
-   hs->Add(new wxButton(this, ID_INSERTA, _("Insert &After")), 1, wxCENTER | wxALL, 5);
-   hs->Add(new wxButton(this, ID_INSERTB, _("Insert &Before")), 1, wxCENTER | wxALL, 5);
-   hs->Add(new wxButton(this, ID_REMOVE,  _("&Remove")), 1, wxCENTER | wxALL, 5);
-   hs->Add(new wxButton(this, ID_IMPORT,  _("&Import...")), 1, wxCENTER | wxALL, 5);
-   hs->Add(new wxButton(this, ID_EXPORT,  _("&Export...")), 1, wxCENTER | wxALL, 5);
-   vs->Add(hs, 0, wxEXPAND | wxCENTER | wxALL, 5);
-
-   // Create the exit buttons
-   vs->Add(CreateStdButtonSizer(this, eCancelButton|eOkButton), 0, wxEXPAND);
-
-   // Make it so
-   SetSizer(vs);
+LabelDialog::~LabelDialog()
+{
+}
 
+void LabelDialog::PopulateLabels()
+{
    // Build the initial (empty) grid
    mGrid->CreateGrid(0, Col_Max);
    mGrid->SetDefaultCellAlignment(wxALIGN_LEFT, wxALIGN_CENTER);
@@ -149,26 +140,44 @@ LabelDialog::LabelDialog(wxWindow *parent,
    mGrid->SetColLabelValue(2,_("Start Time"));
    /* i18n-hint: (noun) of a label*/
    mGrid->SetColLabelValue(3,_("End Time"));
+   /* i18n-hint: (noun) of a label*/
+   mGrid->SetColLabelValue(4,_("Low Frequency"));
+   /* i18n-hint: (noun) of a label*/
+   mGrid->SetColLabelValue(5,_("High Frequency"));
 
-   // Create and remember editors.  No need to delete these as the wxGrid will
-   // do it for us.
+   // Create and remember editors.  No need to DELETE these as the wxGrid will
+   // do it for us.  (The DecRef() that is needed after GetDefaultEditorForType
+   // becomes the duty of the wxGridCellAttr objects after we set them in the grid.)
    mChoiceEditor = (ChoiceEditor *) mGrid->GetDefaultEditorForType(GRID_VALUE_CHOICE);
-   mTimeEditor = (TimeEditor *) mGrid->GetDefaultEditorForType(GRID_VALUE_TIME);
+   mTimeEditor = static_cast<NumericEditor*>
+      (mGrid->GetDefaultEditorForType(GRID_VALUE_TIME));
+   mFrequencyEditor = static_cast<NumericEditor *>
+      (mGrid->GetDefaultEditorForType(GRID_VALUE_FREQUENCY));
 
    // Initialize and set the track name column attributes
-   wxGridCellAttr *attr = new wxGridCellAttr();
+   wxGridCellAttr *attr;
+   mGrid->SetColAttr(Col_Track, (attr = safenew wxGridCellAttr));
    attr->SetEditor(mChoiceEditor);
-   mGrid->SetColAttr(Col_Track, attr);
-   mTrackNames.Add(_("New..."));
+   mTrackNames.push_back(_("New..."));
 
    // Initialize and set the time column attributes
-   attr = new wxGridCellAttr();
+   mGrid->SetColAttr(Col_Stime, (attr = safenew wxGridCellAttr));
+   // Don't need DecRef() after this GetDefaultRendererForType.
    attr->SetRenderer(mGrid->GetDefaultRendererForType(GRID_VALUE_TIME));
    attr->SetEditor(mTimeEditor);
    attr->SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
-   mGrid->SetColAttr(Col_Stime, attr);
+
    mGrid->SetColAttr(Col_Etime, attr->Clone());
 
+   // Initialize and set the frequency column attributes
+   mGrid->SetColAttr(Col_Lfreq, (attr = safenew wxGridCellAttr));
+   // Don't need DecRef() after this GetDefaultRendererForType.
+   attr->SetRenderer(mGrid->GetDefaultRendererForType(GRID_VALUE_FREQUENCY));
+   attr->SetEditor(mFrequencyEditor);
+   attr->SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
+
+   mGrid->SetColAttr(Col_Hfreq, attr->Clone());
+   
    // Seems there's a bug in wxGrid.  Adding only 1 row does not
    // allow SetCellSize() to work properly and you will not get
    // the expected 1 row by 4 column cell.
@@ -195,45 +204,101 @@ LabelDialog::LabelDialog(wxWindow *parent,
    mGrid->SetColSize(Col_Label, wxMax(150, mGrid->GetColSize(Col_Label)));
    mGrid->SetColMinimalWidth(Col_Label, mGrid->GetColSize(Col_Label));
 
+}
+
+
+/// Creates the dialog and its contents.
+void LabelDialog::Populate()
+{
+
+   //------------------------- Main section --------------------
+   ShuttleGui S(this, eIsCreating);
+   PopulateOrExchange(S);
+   // ----------------------- End of main section --------------
+
+   // Go populate the macros list.
+   PopulateLabels();
+
    // Layout the works
    Layout();
+   //Fit();
 
    // Resize width based on width of columns and the vertical scrollbar
    wxRect r = mGrid->GetGridColLabelWindow()->GetRect();
    wxScrollBar sb(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSB_VERTICAL);
    r.width += sb.GetSize().GetWidth() + 6;
+
+   // Add the size of the right column of buttons too...
+   wxWindow * w = FindWindowById( ID_IMPORT, this );
+   wxASSERT( w );
+   if( w )
+      r.width += w->GetSize().GetWidth();
+
    SetClientSize(r.width, 300);
 
    // Make sure it doesn't go below this size
    r = GetRect();
    SetSizeHints(r.GetWidth(), r.GetHeight());
 
+   // Bug 1465
+   // There might be a saved size, in which case use that.
+   ReadSize();
+
    // Center on display
    Center();
 }
 
-LabelDialog::~LabelDialog()
+void LabelDialog::PopulateOrExchange( ShuttleGui & S )
 {
-   int cnt = mData.GetCount();
-
-   // Delete any RowData we've allocated
-   while (cnt) {
-      RowData *rd = mData[--cnt];
-      delete rd;
+   S.AddFixedText(_("Press F2 or double click to edit cell contents."));
+   S.StartHorizontalLay(wxEXPAND,1);
+   {
+      S.StartVerticalLay(wxEXPAND,1);
+      {
+         mGrid = safenew Grid(this, wxID_ANY);
+         S.Prop(1).AddWindow( mGrid );
+      }
+      S.EndVerticalLay();
+      S.StartVerticalLay(0);
+      {
+         //S.Id(ID_INSERTA).AddButton(_("&Insert"), wxALIGN_LEFT);
+         S.Id(ID_INSERTB).AddButton(_("&Insert"), wxALIGN_LEFT);
+         //S.Id(EditButtonID).AddButton(_("&Edit"), wxALIGN_LEFT);
+         S.Id(ID_REMOVE).AddButton(_("De&lete"), wxALIGN_LEFT);
+         S.Id(ID_IMPORT).AddButton(_("I&mport..."), wxALIGN_LEFT);
+         S.Id(ID_EXPORT).AddButton(_("&Export..."), wxALIGN_LEFT);
+      }
+      S.EndVerticalLay();
    }
+   S.EndHorizontalLay();
+
+   S.StartHorizontalLay(wxALIGN_RIGHT, false);
+   {
+      S.AddStandardButtons( eOkButton | eCancelButton | eHelpButton);
+   }
+   S.EndHorizontalLay();
 }
 
+void LabelDialog::OnHelp(wxCommandEvent & WXUNUSED(event))
+{
+   wxString page = GetHelpPageName();
+   HelpSystem::ShowHelp(this, page, true);
+}
+
+
 bool LabelDialog::TransferDataToWindow()
 {
-   int cnt = mData.GetCount();
+   int cnt = mData.size();
    int i;
 
    // Set the editor parameters.  Do this each time since they may change
-   // due to new tracks and change in NumericTextCtrl format.  Rate won't
+   // due to NEW tracks and change in NumericTextCtrl format.  Rate won't
    // change but might as well leave it here.
    mChoiceEditor->SetChoices(mTrackNames);
    mTimeEditor->SetFormat(mFormat);
    mTimeEditor->SetRate(mRate);
+   mFrequencyEditor->SetFormat(mFreqFormat);
+   mFrequencyEditor->SetRate(mRate);
 
    // Disable redrawing until we're done
    mGrid->BeginBatch();
@@ -248,18 +313,19 @@ bool LabelDialog::TransferDataToWindow()
 
    // Populate the rows
    for (i = 0; i < cnt; i++) {
-      RowData *rd = mData[i];
+      RowData &rd = mData[i];
 
       // Set the cell contents
-      mGrid->SetCellValue(i, Col_Track, TrackName(rd->index));
-      mGrid->SetCellValue(i, Col_Label, rd->title);
+      mGrid->SetCellValue(i, Col_Track, TrackName(rd.index));
+      mGrid->SetCellValue(i, Col_Label, rd.title);
       mGrid->SetCellValue(i, Col_Stime,
-         wxString::Format(wxT("%g"), rd->selectedRegion.t0()));
+         wxString::Format(wxT("%g"), rd.selectedRegion.t0()));
       mGrid->SetCellValue(i, Col_Etime,
-         wxString::Format(wxT("%g"), rd->selectedRegion.t1()));
-
-      // PRL: to do: -- populate future additional selection fields
-      // and write event code to update them from controls
+         wxString::Format(wxT("%g"), rd.selectedRegion.t1()));
+      mGrid->SetCellValue(i, Col_Lfreq,
+         wxString::Format(wxT("%g"), rd.selectedRegion.f0()));
+      mGrid->SetCellValue(i, Col_Hfreq,
+         wxString::Format(wxT("%g"), rd.selectedRegion.f1()));
    }
 
    // Autosize all the rows
@@ -274,6 +340,8 @@ bool LabelDialog::TransferDataToWindow()
    // Autosize the time columns and set their minimal widths
    mGrid->AutoSizeColumn(Col_Stime);
    mGrid->AutoSizeColumn(Col_Etime);
+   mGrid->AutoSizeColumn(Col_Lfreq);
+   mGrid->AutoSizeColumn(Col_Hfreq);
 
    // We're done, so allow the grid to redraw
    mGrid->EndBatch();
@@ -283,12 +351,18 @@ bool LabelDialog::TransferDataToWindow()
 
 bool LabelDialog::Show(bool show)
 {
-   bool ret = wxDialog::Show(show);
+   bool ret = wxDialogWrapper::Show(show);
+
+#if defined(__WXMAC__) || defined(__WXGTK__)
+   if (show) {
+      mGrid->SetFocus();   // Required for Linux and Mac.
+   }
+#endif
 
    // Set initial row
    // (This will not work until the grid is actually displayed)
    if (show && mInitialRow != -1) {
-      mGrid->SetGridCursor(mInitialRow, Col_Label);
+      mGrid->GoToCell(mInitialRow, Col_Label);
    }
 
    return ret;
@@ -296,55 +370,59 @@ bool LabelDialog::Show(bool show)
 
 bool LabelDialog::TransferDataFromWindow()
 {
-   int cnt = mData.GetCount();
+   int cnt = mData.size();
    int i;
-   TrackListIterator iter(mTracks);
-   Track *t;
    int tndx = 0;
 
-   // Clear all label tracks of labels
-   for (t = iter.First(); t; t = iter.Next()) {
-      if (t->GetKind() == Track::Label) {
-         LabelTrack *lt = (LabelTrack *)t;
-         tndx++;
-
+   // Clear label tracks of labels
+   for (auto lt : mTracks->Any<LabelTrack>()) {
+      ++tndx;
+      if (!mSelectedTrack) {
          for (i = lt->GetNumLabels() - 1; i >= 0 ; i--) {
             lt->DeleteLabel(i);
          }
       }
+      else if (mSelectedTrack == lt && mIndex > -1) {
+         lt->DeleteLabel(mIndex);
+      }
+      else
+         // Do nothing to the nonselected tracks
+         ;
    }
 
    // Create any added tracks
-   while (tndx < (int)mTrackNames.GetCount() - 1) {
+   while (tndx < (int)mTrackNames.size() - 1) {
 
       // Extract the name
       wxString name = mTrackNames[tndx + 1].AfterFirst(wxT('-')).Mid(1);
 
-      // Create the new track and add to track list
-      LabelTrack *newTrack = new LabelTrack(mDirManager);
+      // Create the NEW track and add to track list
+      auto newTrack = mFactory.NewLabelTrack();
       newTrack->SetName(name);
-      mTracks->Add(newTrack);
+      mTracks->Add( newTrack );
       tndx++;
    }
 
    // Repopulate with updated labels
    for (i = 0; i < cnt; i++) {
-      RowData *rd = mData[i];
+      RowData &rd = mData[i];
 
       // Look for track with matching index
       tndx = 1;
-      for (t = iter.First(); t; t = iter.Next()) {
-         if (t->GetKind() == Track::Label && rd->index == tndx++) {
+      LabelTrack *lt{};
+      for (auto t : mTracks->Any<LabelTrack>()) {
+         lt = t;
+         if (rd.index == tndx++) {
             break;
          }
       }
-      wxASSERT(t);
-      if (!t)
+      wxASSERT(lt);
+      if (!lt)
          return false;
 
       // Add the label to it
-      ((LabelTrack *) t)->AddLabel(rd->selectedRegion, rd->title);
-      ((LabelTrack *) t)->Unselect();
+      lt->AddLabel(rd.selectedRegion, rd.title);
+      LabelTrackView::Get( *lt ).SetSelectedIndex( -1 );
    }
 
    return true;
@@ -360,12 +438,12 @@ bool LabelDialog::Validate()
    return true;
 }
 
-wxString LabelDialog::TrackName(int & index, wxString dflt)
+wxString LabelDialog::TrackName(int & index, const wxString &dflt)
 {
-   // Generate a new track name if the passed index is out of range
-   if (index < 1 || index >= (int)mTrackNames.GetCount()) {
-      index = mTrackNames.GetCount();
-      mTrackNames.Add(wxString::Format(wxT("%d - %s"), index, dflt.c_str()));
+   // Generate a NEW track name if the passed index is out of range
+   if (index < 1 || index >= (int)mTrackNames.size()) {
+      index = mTrackNames.size();
+      mTrackNames.push_back(wxString::Format(wxT("%d - %s"), index, dflt));
    }
 
    // Return the track name
@@ -374,50 +452,42 @@ wxString LabelDialog::TrackName(int & index, wxString dflt)
 
 void LabelDialog::FindAllLabels()
 {
-   TrackListIterator iter(mTracks);
-   Track *t;
-
-
    // Add labels from all label tracks
-   for (t = iter.First(); t; t = iter.Next()) {
-      if (t->GetKind() == Track::Label) {
-         AddLabels((LabelTrack *) t);
-      }
+   for (auto lt : mTracks->Any<const LabelTrack>()) {
+      AddLabels(lt);
    }
 
    FindInitialRow();
 
-   if (mData.GetCount() == 0) {
+   if (mData.size() == 0) {
       wxCommandEvent e;
       OnInsert(e);
    }
 }
 
-void LabelDialog::AddLabels(LabelTrack *t)
+void LabelDialog::AddLabels(const LabelTrack *t)
 {
    wxString lab;
    int tndx = 0;
    int i;
 
-   // Add a new track name
+   // Add a NEW track name
    TrackName(tndx, t->GetName());
 
-   // Add each label in the track
-   for (i = 0; i < t->GetNumLabels(); i++) {
-      const LabelStruct *ls = t->GetLabel(i);
-      RowData *rd = new RowData();
+   // If editor was invoked for one label, add that one only, else add all.
+   if (!mSelectedTrack || mSelectedTrack == t) {
+      for (i = 0; i < t->GetNumLabels(); i++) {
+         const LabelStruct *ls = t->GetLabel(i);
 
-      rd->index = tndx;
-      rd->selectedRegion = ls->selectedRegion;
-      rd->title = ls->title;
-
-      mData.Add(rd);
+         if (mIndex < 0 || mIndex == i)
+            mData.push_back(RowData(tndx, ls->title, ls->selectedRegion));
+      }
    }
 }
 
 void LabelDialog::FindInitialRow()
 {
-   int cnt = mData.GetCount();
+   int cnt = mData.size();
    mInitialRow = -1;
 
    if (cnt == 0)
@@ -431,7 +501,7 @@ void LabelDialog::FindInitialRow()
    int i;
    for (i = 0; i < cnt; i++)
    {
-      dist = t0 - mData[i]->selectedRegion.t0();
+      dist = t0 - mData[i].selectedRegion.t0();
       if (dist >= 0.0 && dist < distMin)
       {
          mInitialRow = i;
@@ -446,10 +516,10 @@ void LabelDialog::FindInitialRow()
       double t0Min = std::numeric_limits<double>::max();
       for (i = 0; i < cnt; i++)
       {
-         if (mData[i]->selectedRegion.t0() < t0Min)
+         if (mData[i].selectedRegion.t0() < t0Min)
          {
             mInitialRow  = i;
-            t0Min = mData[i]->selectedRegion.t0();
+            t0Min = mData[i].selectedRegion.t0();
          }
       }
    }
@@ -457,8 +527,19 @@ void LabelDialog::FindInitialRow()
 
 void LabelDialog::OnUpdate(wxCommandEvent &event)
 {
-   // Remember the new format and repopulate grid
-   mFormat = event.GetString();
+   // Remember the NEW format and repopulate grid
+   mFormat = NumericConverter::LookupFormat(
+      NumericConverter::TIME, event.GetString() );
+   TransferDataToWindow();
+
+   event.Skip(false);
+}
+
+void LabelDialog::OnFreqUpdate(wxCommandEvent &event)
+{
+   // Remember the NEW format and repopulate grid
+   mFreqFormat = NumericConverter::LookupFormat(
+      NumericConverter::FREQUENCY, event.GetString() );
    TransferDataToWindow();
 
    event.Skip(false);
@@ -466,8 +547,7 @@ void LabelDialog::OnUpdate(wxCommandEvent &event)
 
 void LabelDialog::OnInsert(wxCommandEvent &event)
 {
-   RowData *rd = new RowData();
-   int cnt = mData.GetCount();
+   int cnt = mData.size();
    int row = 0;
    int index = 0;
 
@@ -480,28 +560,25 @@ void LabelDialog::OnInsert(wxCommandEvent &event)
    if (cnt > 0) {
       row = mGrid->GetGridCursorRow();
       if (row > 0 && row >= cnt) {
-         index = mTrackNames.Index(mGrid->GetCellValue(row - 1, Col_Track));
+         index = make_iterator_range( mTrackNames )
+            .index( mGrid->GetCellValue(row - 1, Col_Track) );
       }
       else {
-         index = mTrackNames.Index(mGrid->GetCellValue(row, Col_Track));
+         index = make_iterator_range( mTrackNames )
+            .index( mGrid->GetCellValue(row, Col_Track) );
       }
    }
 
-   // Initialize the new label
-   rd->index = index;
-   rd->selectedRegion = SelectedRegion();
-   rd->title = wxT("");
-
-   // Insert it before or after the current row
+   // Insert NEW label before or after the current row
    if (event.GetId() == ID_INSERTA && row < cnt) {
       row++;
    }
-   mData.Insert(rd, row);
+   mData.insert(mData.begin() + row, RowData(index, wxT(""), SelectedRegion()));
 
    // Repopulate the grid
    TransferDataToWindow();
 
-   // Reposition cursor to new row/col and put user into edit mode to
+   // Reposition cursor to NEW row/col and put user into edit mode to
    // set the label name
    mGrid->SetGridCursor(row, Col_Label);
    mGrid->EnableCellEditControl(true);
@@ -512,7 +589,7 @@ void LabelDialog::OnRemove(wxCommandEvent & WXUNUSED(event))
 {
    int row = mGrid->GetGridCursorRow();
    int col = mGrid->GetGridCursorCol();
-   int cnt = mData.GetCount();
+   int cnt = mData.size();
 
    // Don't try to remove if no labels exist
    if (cnt == 0) {
@@ -525,10 +602,8 @@ void LabelDialog::OnRemove(wxCommandEvent & WXUNUSED(event))
    }
 
    // Remove the row
-   RowData *rd = mData[row];
-   mTrackNames.RemoveAt(rd->index);
-   mData.RemoveAt(row);
-   delete rd;
+   //RowData &rd = mData[row];
+   mData.erase(mData.begin() + row);
 
    // Repopulate the grid
    TransferDataToWindow();
@@ -540,7 +615,7 @@ void LabelDialog::OnRemove(wxCommandEvent & WXUNUSED(event))
    mGrid->SetGridCursor(row, col);
 
    // Make sure focus isn't lost
-   if (mData.GetCount() == 0 && wxWindow::FindFocus() == mGrid->GetGridWindow()) {
+   if (mData.size() == 0 && wxWindow::FindFocus() == mGrid->GetGridWindow()) {
       wxWindow *ok = wxWindow::FindWindowById( wxID_OK, this);
       if (ok) {
          ok->SetFocus();
@@ -550,12 +625,11 @@ void LabelDialog::OnRemove(wxCommandEvent & WXUNUSED(event))
 
 void LabelDialog::OnImport(wxCommandEvent & WXUNUSED(event))
 {
-   wxString path = gPrefs->Read(wxT("/DefaultOpenPath"),::wxGetCwd());
-
    // Ask user for a filename
    wxString fileName =
-       FileSelector(_("Select a text file containing labels..."),
-                    path,     // Path
+       FileNames::SelectFile(FileNames::Operation::Open,
+                    _("Select a text file containing labels"),
+                    wxEmptyString,     // Path
                     wxT(""),       // Name
                     wxT(".txt"),   // Extension
                     _("Text files (*.txt)|*.txt|All files|*"),
@@ -563,29 +637,25 @@ void LabelDialog::OnImport(wxCommandEvent & WXUNUSED(event))
                     this);    // Parent
 
    // They gave us one...
-   if (fileName != wxT("")) {
-      path =::wxPathOnly(fileName);
-      gPrefs->Write(wxT("/DefaultOpenPath"), path);
-      gPrefs->Flush();
-
+   if (!fileName.empty()) {
       wxTextFile f;
 
       // Get at the data
       f.Open(fileName);
       if (!f.IsOpened()) {
-         wxMessageBox(_("Could not open file: ") + fileName);
+         AudacityMessageBox(
+            wxString::Format( _("Could not open file: %s"), fileName ));
       }
       else {
          // Create a temporary label track and load the labels
          // into it
-         LabelTrack *lt = new LabelTrack(mDirManager);
+         auto lt = mFactory.NewLabelTrack();
          lt->Import(f);
 
-         // Add the labesls to our collection
-         AddLabels(lt);
+         // Add the labels to our collection
+         AddLabels(lt.get());
 
          // Done with the temporary track
-         delete lt;
      }
 
       // Repopulate the grid
@@ -595,26 +665,27 @@ void LabelDialog::OnImport(wxCommandEvent & WXUNUSED(event))
 
 void LabelDialog::OnExport(wxCommandEvent & WXUNUSED(event))
 {
-   int cnt = mData.GetCount();
+   int cnt = mData.size();
 
    // Silly user (could just disable the button, but that's a hassle ;-))
    if (cnt == 0) {
-      wxMessageBox(_("No labels to export."));
+      AudacityMessageBox(_("No labels to export."));
       return;
    }
 
    // Extract the actual name.
-   wxString fName = mTrackNames[mTrackNames.GetCount() - 1].AfterFirst(wxT('-')).Mid(1);
+   wxString fName = mTrackNames[mTrackNames.size() - 1].AfterFirst(wxT('-')).Mid(1);
 
-   fName = FileSelector(_("Export Labels As:"),
-                        wxEmptyString,
-                        fName.c_str(),
-                        wxT("txt"),
-                        wxT("*.txt"),
-                        wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxRESIZE_BORDER,
-                        this);
+   fName = FileNames::SelectFile(FileNames::Operation::Export,
+      _("Export Labels As:"),
+      wxEmptyString,
+      fName,
+      wxT("txt"),
+      wxT("*.txt"),
+      wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxRESIZE_BORDER,
+      this);
 
-   if (fName == wxT(""))
+   if (fName.empty())
       return;
 
    // Move existing files out of the way.  Otherwise wxTextFile will
@@ -635,31 +706,29 @@ void LabelDialog::OnExport(wxCommandEvent & WXUNUSED(event))
 
    wxTextFile f(fName);
 #ifdef __WXMAC__
-   wxFile *temp = new wxFile();
-   temp->Create(fName);
-   delete temp;
+   wxFile{}.Create(fName);
 #else
    f.Create();
 #endif
    f.Open();
    if (!f.IsOpened()) {
-      wxMessageBox(_("Couldn't write to file: ") + fName);
+      AudacityMessageBox(
+         wxString::Format( _("Couldn't write to file: %s"), fName ) );
       return;
    }
 
    // Transfer our collection to a temporary label track
-   LabelTrack *lt = new LabelTrack(mDirManager);
+   auto lt = mFactory.NewLabelTrack();
    int i;
 
    for (i = 0; i < cnt; i++) {
-      RowData *rd = mData[i];
+      RowData &rd = mData[i];
 
-      lt->AddLabel(rd->selectedRegion, rd->title);
+      lt->AddLabel(rd.selectedRegion, rd.title);
    }
 
    // Export them and clean
    lt->Export(f);
-   delete lt;
 
 #ifdef __WXMAC__
    f.Write(wxTextFileType_Mac);
@@ -671,21 +740,15 @@ void LabelDialog::OnExport(wxCommandEvent & WXUNUSED(event))
 
 void LabelDialog::OnSelectCell(wxGridEvent &event)
 {
-   TrackListIterator iter(mTracks);
-   Track *t = iter.First();
-   while( t )
-   {
+   for (auto t: mTracks->Any())
       t->SetSelected( true );
-      t = iter.Next();
-   }
 
    if (!mData.empty())
    {
-      RowData *rd;
-      rd = mData[event.GetRow()];
-      mViewInfo->selectedRegion = rd->selectedRegion;
+      RowData &rd = mData[event.GetRow()];
+      mViewInfo->selectedRegion = rd.selectedRegion;
 
-      GetActiveProject()->RedrawProject();
+      ProjectWindow::Get( *GetActiveProject() ).RedrawProject();
    }
 
    event.Skip();
@@ -695,9 +758,8 @@ void LabelDialog::OnCellChange(wxGridEvent &event)
 {
    static bool guard = false;
    int row = event.GetRow();
-   RowData *rd;
 
-   // Guard against recursion which can happen when a change to the "new label" row
+   // Guard against recursion which can happen when a change to the "NEW label" row
    // is made.  When InsertRow() is done in TransferDataToWindow(), checks are made
    // within wxGrid to see if the edit control is active and since it hasn't yet
    // been marked inactive on the first time through here, we get entered again.
@@ -709,7 +771,7 @@ void LabelDialog::OnCellChange(wxGridEvent &event)
 
    // The change was to an existing label, so go process it based
    // on which column was changed.
-   rd = mData[row];
+   RowData *rd = &mData[row];
    switch (event.GetCol())
    {
       case Col_Track:
@@ -727,6 +789,14 @@ void LabelDialog::OnCellChange(wxGridEvent &event)
       case Col_Etime:
          OnChangeEtime(event, row, rd);
       break;
+
+      case Col_Lfreq:
+         OnChangeLfreq(event, row, rd);
+      break;
+
+      case Col_Hfreq:
+         OnChangeHfreq(event, row, rd);
+      break;
    }
 
    // Done...no need for protection anymore
@@ -739,9 +809,9 @@ void LabelDialog::OnChangeTrack(wxGridEvent & WXUNUSED(event), int row, RowData
 {
    wxString val = mGrid->GetCellValue(row, Col_Track);
 
-   // User selected the "New..." choice so ask for a new name
-   if (mTrackNames.Index(val) == 0) {
-      wxTextEntryDialog d(this,
+   // User selected the "New..." choice so ask for a NEW name
+   if ( make_iterator_range( mTrackNames ).index( val ) == 0 ) {
+      AudacityTextEntryDialog d(this,
                           _("New Label Track"),
                           _("Enter track name"),
                           /* i18n-hint: (noun) it's the name of a kind of track.*/
@@ -754,13 +824,13 @@ void LabelDialog::OnChangeTrack(wxGridEvent & WXUNUSED(event), int row, RowData
          return;
       }
 
-      // Force generation of a new track name
+      // Force generation of a NEW track name
       rd->index = 0;
       TrackName(rd->index, d.GetValue());
    }
    else {
       // Remember the tracks index
-      rd->index = mTrackNames.Index(val);
+      rd->index = make_iterator_range( mTrackNames ).index( val );
    }
 
    // Repopulate the grid
@@ -780,7 +850,7 @@ void LabelDialog::OnChangeLabel(wxGridEvent & WXUNUSED(event), int row, RowData
 void LabelDialog::OnChangeStime(wxGridEvent & WXUNUSED(event), int row, RowData *rd)
 {
    // Remember the value...no need to repopulate
-   double t;
+   double t {};
    mGrid->GetCellValue(row, Col_Stime).ToDouble(&t);
    rd->selectedRegion.setT0(t, false);
    mGrid->SetCellValue(row, Col_Etime, wxString::Format(wxT("%g"),
@@ -792,7 +862,7 @@ void LabelDialog::OnChangeStime(wxGridEvent & WXUNUSED(event), int row, RowData
 void LabelDialog::OnChangeEtime(wxGridEvent & WXUNUSED(event), int row, RowData *rd)
 {
    // Remember the value...no need to repopulate
-   double t;
+   double t {};
    mGrid->GetCellValue(row, Col_Etime).ToDouble(&t);
    rd->selectedRegion.setT1(t, false);
    mGrid->SetCellValue(row, Col_Stime, wxString::Format(wxT("%g"),
@@ -801,6 +871,49 @@ void LabelDialog::OnChangeEtime(wxGridEvent & WXUNUSED(event), int row, RowData
    return;
 }
 
+void LabelDialog::OnChangeLfreq(wxGridEvent & WXUNUSED(event), int row, RowData *rd)
+{
+   // Remember the value...no need to repopulate
+   double f;
+   mGrid->GetCellValue(row, Col_Lfreq).ToDouble(&f);
+   rd->selectedRegion.setF0(f, false);
+   mGrid->SetCellValue(row, Col_Hfreq, wxString::Format(wxT("%g"),
+                                                        rd->selectedRegion.f1()));
+
+   return;
+}
+
+void LabelDialog::OnChangeHfreq(wxGridEvent & WXUNUSED(event), int row, RowData *rd)
+{
+   // Remember the value...no need to repopulate
+   double f;
+   mGrid->GetCellValue(row, Col_Hfreq).ToDouble(&f);
+   rd->selectedRegion.setF1(f, false);
+   mGrid->SetCellValue(row, Col_Lfreq, wxString::Format(wxT("%g"),
+                                                        rd->selectedRegion.f0()));
+   
+   return;
+}
+
+void LabelDialog::ReadSize(){
+   wxSize sz = GetSize();
+   int prefWidth, prefHeight;
+   gPrefs->Read(wxT("/LabelEditor/Width"), &prefWidth, sz.x);
+   gPrefs->Read(wxT("/LabelEditor/Height"), &prefHeight, sz.y);
+   
+   wxRect screenRect(wxGetClientDisplayRect());
+   wxSize prefSize = wxSize(prefWidth, prefHeight);
+   prefSize.DecTo(screenRect.GetSize());
+   SetSize(prefSize);
+}
+
+void LabelDialog::WriteSize(){
+   wxSize sz = GetSize();
+   gPrefs->Write(wxT("/LabelEditor/Width"), sz.x);
+   gPrefs->Write(wxT("/LabelEditor/Height"), sz.y);
+   gPrefs->Flush();
+}
+
 void LabelDialog::OnOK(wxCommandEvent & WXUNUSED(event))
 {
    if (mGrid->IsCellEditControlShown()) {
@@ -811,6 +924,7 @@ void LabelDialog::OnOK(wxCommandEvent & WXUNUSED(event))
 
    // Standard handling
    if (Validate() && TransferDataFromWindow()) {
+      WriteSize();
       EndModal(wxID_OK);
    }
 
@@ -820,13 +934,16 @@ void LabelDialog::OnOK(wxCommandEvent & WXUNUSED(event))
 void LabelDialog::OnCancel(wxCommandEvent & WXUNUSED(event))
 {
    if (mGrid->IsCellEditControlShown()) {
-      mGrid->GetCellEditor(mGrid->GetGridCursorRow(),
-                           mGrid->GetGridCursorCol())
-                           ->Reset();
+      auto editor = mGrid->GetCellEditor(mGrid->GetGridCursorRow(),
+         mGrid->GetGridCursorCol());
+      editor->Reset();
+      // To avoid memory leak, don't forget DecRef()!
+      editor->DecRef();
       mGrid->HideCellEditControl();
       return;
    }
 
+   WriteSize();
    // Standard handling
    EndModal(wxID_CANCEL);
 
