@@ -18,18 +18,17 @@ i.e. an alternative to the usual interface, for Audacity.
 
 *//*******************************************************************/
 
-#include <wx/dynarray.h>
+#include "Audacity.h"
+#include "ModuleManager.h"
+
+#include "Experimental.h"
+
 #include <wx/dynlib.h>
-#include <wx/list.h>
 #include <wx/log.h>
-#include <wx/msgdlg.h>
 #include <wx/string.h>
 #include <wx/filename.h>
 
-#include "Audacity.h"
-#include "AudacityApp.h"
 #include "FileNames.h"
-#include "Internat.h"
 #include "PluginManager.h"
 
 #include "commands/ScriptCommandRelay.h"
@@ -42,12 +41,9 @@ i.e. an alternative to the usual interface, for Audacity.
 #include "./prefs/ModulePrefs.h"
 #endif
 
-#include "ModuleManager.h"
 #include "widgets/MultiDialog.h"
 
-#include <wx/arrimpl.cpp>
-
-#include "Experimental.h"
+#include "widgets/AudacityMessageBox.h"
 
 #define initFnName      "ExtensionModuleInit"
 #define versionFnName   "GetVersionString"
@@ -88,16 +84,15 @@ wxWindow * MakeHijackPanel()
 // starts a thread and reads script commands.
 static tpRegScriptServerFunc scriptFn;
 
-Module::Module(const wxString & name)
+Module::Module(const FilePath & name)
 {
    mName = name;
-   mLib = new wxDynamicLibrary();
+   mLib = std::make_unique<wxDynamicLibrary>();
    mDispatch = NULL;
 }
 
 Module::~Module()
 {
-   delete mLib;
 }
 
 bool Module::Load()
@@ -117,17 +112,17 @@ bool Module::Load()
    tVersionFn versionFn = (tVersionFn)(mLib->GetSymbol(wxT(versionFnName)));
    if (versionFn == NULL){
       wxString ShortName = wxFileName( mName ).GetName();
-      wxMessageBox(wxString::Format(_("The module %s does not provide a version string.\nIt will not be loaded."), ShortName.c_str()), _("Module Unsuitable"));
-      wxLogMessage(wxString::Format(_("The module %s does not provide a version string.  It will not be loaded."), mName.c_str()));
+      AudacityMessageBox(wxString::Format(_("The module %s does not provide a version string.\nIt will not be loaded."), ShortName), _("Module Unsuitable"));
+      wxLogMessage(wxString::Format(_("The module %s does not provide a version string. It will not be loaded."), mName));
       mLib->Unload();
       return false;
    }
 
    wxString moduleVersion = versionFn();
-   if( !moduleVersion.IsSameAs(AUDACITY_VERSION_STRING)) {
+   if( moduleVersion != AUDACITY_VERSION_STRING) {
       wxString ShortName = wxFileName( mName ).GetName();
-      wxMessageBox(wxString::Format(_("The module %s is matched with Audacity version %s.\n\nIt will not be loaded."), ShortName.c_str(), moduleVersion.c_str()), _("Module Unsuitable"));
-      wxLogMessage(wxString::Format(_("The module %s is matched with Audacity version %s.  It will not be loaded."), mName.c_str(), moduleVersion.c_str()));
+      AudacityMessageBox(wxString::Format(_("The module %s is matched with Audacity version %s.\n\nIt will not be loaded."), ShortName, moduleVersion), _("Module Unsuitable"));
+      wxLogMessage(wxString::Format(_("The module %s is matched with Audacity version %s. It will not be loaded."), mName, moduleVersion));
       mLib->Unload();
       return false;
    }
@@ -168,7 +163,7 @@ int Module::Dispatch(ModuleDispatchTypes type)
    return 0;
 }
 
-void * Module::GetSymbol(wxString name)
+void * Module::GetSymbol(const wxString &name)
 {
    return mLib->GetSymbol(name);
 }
@@ -180,18 +175,21 @@ void * Module::GetSymbol(wxString name)
 // ============================================================================
 
 // The one and only ModuleManager
-ModuleManager *ModuleManager::mInstance = NULL;
+std::unique_ptr<ModuleManager> ModuleManager::mInstance{};
 
 // Provide builtin modules a means to identify themselves
-static wxArrayPtrVoid *pBuiltinModuleList = NULL;
+using BuiltinModuleList = std::vector<ModuleMain>;
+namespace {
+   BuiltinModuleList &builtinModuleList()
+   {
+      static BuiltinModuleList theList;
+      return theList;
+   }
+}
+
 void RegisterBuiltinModule(ModuleMain moduleMain)
 {
-   if (pBuiltinModuleList == NULL)
-   {
-      pBuiltinModuleList = new wxArrayPtrVoid;
-   }
-
-   pBuiltinModuleList->Add((void *)moduleMain);
+   builtinModuleList().push_back(moduleMain);
 
    return;
 }
@@ -206,61 +204,42 @@ ModuleManager::ModuleManager()
 
 ModuleManager::~ModuleManager()
 {
-   size_t cnt = mModules.GetCount();
-
-   for (size_t ndx = 0; ndx < cnt; ndx++) {
-      delete (Module *) mModules[ndx];
-   }
-   mModules.Clear();
-
-   ModuleMap::iterator iter = mDynModules.begin();
-   while (iter != mDynModules.end())
-   {
-      UnloadModule(iter->second);
-
-      mDynModules.erase(iter->first);
-
-      iter = mDynModules.begin();
-   }
-
-   if (pBuiltinModuleList != NULL)
-   {
-      delete pBuiltinModuleList;
-   }
+   mDynModules.clear();
+   builtinModuleList().clear();
 }
 
 // static 
 void ModuleManager::Initialize(CommandHandler &cmdHandler)
 {
-   wxArrayString audacityPathList = wxGetApp().audacityPathList;
-   wxArrayString pathList;
-   wxArrayString files;
+   const auto &audacityPathList = FileNames::AudacityPathList();
+   FilePaths pathList;
+   FilePaths files;
    wxString pathVar;
    size_t i;
 
    // Code from LoadLadspa that might be useful in load modules.
    pathVar = wxGetenv(wxT("AUDACITY_MODULES_PATH"));
-   if (pathVar != wxT(""))
-      wxGetApp().AddMultiPathsToPathList(pathVar, pathList);
+   if (!pathVar.empty())
+      FileNames::AddMultiPathsToPathList(pathVar, pathList);
 
-   for (i = 0; i < audacityPathList.GetCount(); i++) {
+   for (i = 0; i < audacityPathList.size(); i++) {
       wxString prefix = audacityPathList[i] + wxFILE_SEP_PATH;
-      wxGetApp().AddUniquePathToPathList(prefix + wxT("modules"),
+      FileNames::AddUniquePathToPathList(prefix + wxT("modules"),
                                          pathList);
    }
 
    #if defined(__WXMSW__)
-   wxGetApp().FindFilesInPathList(wxT("*.dll"), pathList, files);
+   FileNames::FindFilesInPathList(wxT("*.dll"), pathList, files);
    #else
-   wxGetApp().FindFilesInPathList(wxT("*.so"), pathList, files);
+   FileNames::FindFilesInPathList(wxT("*.so"), pathList, files);
    #endif
 
    wxString saveOldCWD = ::wxGetCwd();
-   for (i = 0; i < files.GetCount(); i++) {
+   for (i = 0; i < files.size(); i++) {
       // As a courtesy to some modules that might be bridges to
       // open other modules, we set the current working
       // directory to be the module's directory.
-      wxString prefix = ::wxPathOnly(files[i]);
+      auto prefix = ::wxPathOnly(files[i]);
       ::wxSetWorkingDirectory(prefix);
 
 #ifdef EXPERIMENTAL_MODULE_PREFS
@@ -285,7 +264,7 @@ void ModuleManager::Initialize(CommandHandler &cmdHandler)
       {
          wxString ShortName = wxFileName( files[i] ).GetName();
          wxString msg;
-         msg.Printf(_("Module \"%s\" found."), ShortName.c_str());
+         msg.Printf(_("Module \"%s\" found."), ShortName);
          msg += _("\n\nOnly use modules from trusted sources");
          const wxChar *buttons[] = {_("Yes"), _("No"), NULL};  // could add a button here for 'yes and remember that', and put it into the cfg file.  Needs more thought.
          int action;
@@ -307,10 +286,11 @@ void ModuleManager::Initialize(CommandHandler &cmdHandler)
       ModulePrefs::SetModuleStatus( files[i], kModuleFailed );
 #endif
 
-      Module *module = new Module(files[i]);
-      if (module->Load())   // it will get rejected if there  are version problems
+      auto umodule = std::make_unique<Module>(files[i]);
+      if (umodule->Load())   // it will get rejected if there  are version problems
       {
-         Get().mModules.Add(module);
+         auto module = umodule.get();
+         Get().mModules.push_back(std::move(umodule));
          // We've loaded and initialised OK.
          // So look for special case functions:
          wxLogNull logNo; // Don't show wxWidgets errors if we can't do these. (Was: Fix bug 544.)
@@ -327,10 +307,6 @@ void ModuleManager::Initialize(CommandHandler &cmdHandler)
          ModulePrefs::SetModuleStatus( files[i], iModuleStatus);
 #endif
       }
-      else {
-         // No need to save status, as we already set kModuleFailed.
-         delete module;
-      }
    }
    ::wxSetWorkingDirectory(saveOldCWD);
 
@@ -346,11 +322,7 @@ void ModuleManager::Initialize(CommandHandler &cmdHandler)
 // static
 int ModuleManager::Dispatch(ModuleDispatchTypes type)
 {
-   size_t cnt = Get().mModules.GetCount();
-
-   for (size_t ndx = 0; ndx < cnt; ndx++) {
-      Module *module = (Module *)Get().mModules[ndx];
-
+   for (const auto &module: mModules) {
       module->Dispatch(type);
    }
    return 0;
@@ -366,51 +338,42 @@ ModuleManager & ModuleManager::Get()
 {
    if (!mInstance)
    {
-      mInstance = new ModuleManager();
+      mInstance.reset(safenew ModuleManager);
    }
 
    return *mInstance;
 }
 
-void ModuleManager::Destroy()
-{
-   if (mInstance)
-   {
-      delete mInstance;
-      mInstance = NULL;
-   }
-}
-
 bool ModuleManager::DiscoverProviders()
 {
    InitializeBuiltins();
 
-   wxArrayString provList;
-   wxArrayString pathList;
+   FilePaths provList;
+   FilePaths pathList;
 
    // Code from LoadLadspa that might be useful in load modules.
    wxString pathVar = wxString::FromUTF8(getenv("AUDACITY_MODULES_PATH"));
 
-   if (pathVar != wxT(""))
+   if (!pathVar.empty())
    {
-      wxGetApp().AddMultiPathsToPathList(pathVar, pathList);
+      FileNames::AddMultiPathsToPathList(pathVar, pathList);
    }
    else
    {
-      wxGetApp().AddUniquePathToPathList(FileNames::ModulesDir(), pathList);
+      FileNames::AddUniquePathToPathList(FileNames::ModulesDir(), pathList);
    }
 
 #if defined(__WXMSW__)
-   wxGetApp().FindFilesInPathList(wxT("*.dll"), pathList, provList);
+   FileNames::FindFilesInPathList(wxT("*.dll"), pathList, provList);
 #elif defined(__WXMAC__)
-   wxGetApp().FindFilesInPathList(wxT("*.dylib"), pathList, provList);
+   FileNames::FindFilesInPathList(wxT("*.dylib"), pathList, provList);
 #else
-   wxGetApp().FindFilesInPathList(wxT("*.so"), pathList, provList);
+   FileNames::FindFilesInPathList(wxT("*.so"), pathList, provList);
 #endif
 
    PluginManager & pm = PluginManager::Get();
 
-   for (int i = 0, cnt = provList.GetCount(); i < cnt; i++)
+   for (int i = 0, cnt = provList.size(); i < cnt; i++)
    {
       ModuleInterface *module = LoadModule(provList[i]);
       if (module)
@@ -430,27 +393,34 @@ void ModuleManager::InitializeBuiltins()
 {
    PluginManager & pm = PluginManager::Get();
 
-   for (size_t i = 0, cnt = pBuiltinModuleList->GetCount(); i < cnt; i++)
+   for (auto moduleMain : builtinModuleList())
    {
-      ModuleInterface *module = ((ModuleMain) (*pBuiltinModuleList)[i])(this, NULL);
+      ModuleInterfaceHandle module {
+         moduleMain(this, NULL), ModuleInterfaceDeleter{}
+      };
 
       if (module->Initialize())
       {
          // Register the provider
-         const PluginID & id = pm.RegisterPlugin(module);
+         ModuleInterface *pInterface = module.get();
+         const PluginID & id = pm.RegisterPlugin(pInterface);
 
          // Need to remember it 
-         mDynModules[id] = module;
+         mDynModules[id] = std::move(module);
 
          // Allow the module to auto-register children
-         module->AutoRegisterPlugins(pm);
+         pInterface->AutoRegisterPlugins(pm);
+      }
+      else
+      {
+         // Don't leak!  Destructor of module does that.
       }
    }
 }
 
-ModuleInterface *ModuleManager::LoadModule(const wxString & path)
+ModuleInterface *ModuleManager::LoadModule(const PluginPath & path)
 {
-   wxDynamicLibrary *lib = new wxDynamicLibrary();
+   auto lib = std::make_unique<wxDynamicLibrary>();
 
    if (lib->Load(path, wxDL_NOW))
    {
@@ -459,67 +429,72 @@ ModuleInterface *ModuleManager::LoadModule(const wxString & path)
                                                             &success);
       if (success && audacityMain)
       {
-         ModuleInterface *module = audacityMain(this, &path);
-         if (module)
+         ModuleInterfaceHandle handle {
+            audacityMain(this, &path), ModuleInterfaceDeleter{}
+         };
+         if (handle)
          {
-            if (module->Initialize())
+            if (handle->Initialize())
             {
 
-               mDynModules[PluginManager::GetID(module)] = module;
-               mLibs[module] = lib;
+               auto module = handle.get();
+               mDynModules[PluginManager::GetID(module)] = std::move(handle);
+               mLibs[module] = std::move(lib);
 
                return module;
             }
-            module->Terminate();
-            delete module;
          }
       }
 
       lib->Unload();
    }
 
-   delete lib;
-
    return NULL;
 }
 
-void ModuleManager::UnloadModule(ModuleInterface *module)
+void ModuleInterfaceDeleter::operator() (ModuleInterface *pInterface) const
 {
-   if (module)
+   if (pInterface)
    {
-      module->Terminate();
+      pInterface->Terminate();
 
-      if (mLibs.find(module) != mLibs.end())
-      {
-         mLibs[module]->Unload();
-         mLibs.erase(module);
-      }
+      auto &libs = ModuleManager::Get().mLibs;
 
-      delete module; //After terminating and unloading, we can safely delete the module
+      auto iter = libs.find(pInterface);
+      if (iter != libs.end())
+         libs.erase(iter); // This causes unloading in ~wxDynamicLibrary
+
+      std::unique_ptr < ModuleInterface > { pInterface }; // DELETE it
    }
 }
 
-void ModuleManager::RegisterModule(ModuleInterface *module)
+void ModuleManager::RegisterModule(ModuleInterface *inModule)
 {
-   PluginID id = PluginManager::GetID(module);
+   std::unique_ptr<ModuleInterface> module{ inModule };
+
+   PluginID id = PluginManager::GetID(module.get());
 
    if (mDynModules.find(id) != mDynModules.end())
    {
       // TODO:  Should we complain about a duplicate registeration????
+      // PRL:  Don't leak resources!
+      module->Terminate();
       return;
    }
 
-   mDynModules[id] = module;
+   mDynModules[id] = ModuleInterfaceHandle {
+      module.release(), ModuleInterfaceDeleter{}
+   };
 
-   PluginManager::Get().RegisterPlugin(module);
+   PluginManager::Get().RegisterPlugin(inModule);
 }
 
-void ModuleManager::FindAllPlugins(PluginIDList & providers, wxArrayString & paths)
+void ModuleManager::FindAllPlugins(PluginIDs & providers, PluginPaths & paths)
 {
    PluginManager & pm = PluginManager::Get();
 
    wxArrayString modIDs;
-   wxArrayString modPaths;
+   PluginPaths modPaths;
    const PluginDescriptor *plug = pm.GetFirstPlugin(PluginTypeModule);
    while (plug)
    {
@@ -528,24 +503,26 @@ void ModuleManager::FindAllPlugins(PluginIDList & providers, wxArrayString & pat
       plug = pm.GetNextPlugin(PluginTypeModule);
    }
 
-   for (size_t i = 0, cnt = modIDs.size(); i < cnt; i++)
+   for (size_t i = 0, cntIds = modIDs.size(); i < cntIds; i++)
    {
       PluginID providerID = modIDs[i];
 
-      ModuleInterface *module =
-         static_cast<ModuleInterface *>(CreateProviderInstance(providerID, modPaths[i]));
-      
-      wxArrayString newpaths = module->FindPlugins(pm);
-      for (size_t i = 0, cnt = newpaths.size(); i < cnt; i++)
+      auto module = CreateProviderInstance(providerID, modPaths[i]);
+
+      if (!module)
+         continue;
+
+      auto newpaths = module->FindPluginPaths(pm);
+      for (size_t j = 0, cntPaths = newpaths.size(); j < cntPaths; j++)
       {
          providers.push_back(providerID);
-         paths.push_back(newpaths[i]);
+         paths.push_back(newpaths[j]);
       }
    }
 }
 
-wxArrayString ModuleManager::FindPluginsForProvider(const PluginID & providerID,
-                                                    const wxString & path)
+PluginPaths ModuleManager::FindPluginsForProvider(const PluginID & providerID,
+                                                    const PluginPath & path)
 {
    // Instantiate if it hasn't already been done
    if (mDynModules.find(providerID) == mDynModules.end())
@@ -553,36 +530,39 @@ wxArrayString ModuleManager::FindPluginsForProvider(const PluginID & providerID,
       // If it couldn't be created, just give up and return an empty list
       if (!CreateProviderInstance(providerID, path))
       {
-         return wxArrayString();
+         return {};
       }
    }
 
-   return mDynModules[providerID]->FindPlugins(PluginManager::Get());
+   return mDynModules[providerID]->FindPluginPaths(PluginManager::Get());
 }
 
-bool ModuleManager::RegisterPlugin(const PluginID & providerID, const wxString & path)
+bool ModuleManager::RegisterEffectPlugin(const PluginID & providerID, const PluginPath & path, wxString &errMsg)
 {
+   errMsg.clear();
    if (mDynModules.find(providerID) == mDynModules.end())
    {
       return false;
    }
 
-   return mDynModules[providerID]->RegisterPlugin(PluginManager::Get(), path);
+   auto nFound = mDynModules[providerID]->DiscoverPluginsAtPath(path, errMsg, PluginManagerInterface::DefaultRegistrationCallback);
+
+   return nFound > 0;
 }
 
-IdentInterface *ModuleManager::CreateProviderInstance(const PluginID & providerID,
-                                                      const wxString & path)
+ModuleInterface *ModuleManager::CreateProviderInstance(const PluginID & providerID,
+                                                      const PluginPath & path)
 {
-   if (path.IsEmpty() && mDynModules.find(providerID) != mDynModules.end())
+   if (path.empty() && mDynModules.find(providerID) != mDynModules.end())
    {
-      return mDynModules[providerID];
+      return mDynModules[providerID].get();
    }
 
    return LoadModule(path);
 }
 
-IdentInterface *ModuleManager::CreateInstance(const PluginID & providerID,
-                                              const wxString & path)
+ComponentInterface *ModuleManager::CreateInstance(const PluginID & providerID,
+                                              const PluginPath & path)
 {
    if (mDynModules.find(providerID) == mDynModules.end())
    {
@@ -593,7 +573,7 @@ IdentInterface *ModuleManager::CreateInstance(const PluginID & providerID,
 }
 
 void ModuleManager::DeleteInstance(const PluginID & providerID,
-                                   IdentInterface *instance)
+                                   ComponentInterface *instance)
 {
    if (mDynModules.find(providerID) == mDynModules.end())
    {
@@ -604,10 +584,10 @@ void ModuleManager::DeleteInstance(const PluginID & providerID,
 }
 
 bool ModuleManager::IsProviderValid(const PluginID & WXUNUSED(providerID),
-                                    const wxString & path)
+                                    const PluginPath & path)
 {
    // Builtin modules do not have a path
-   if (path.IsEmpty())
+   if (path.empty())
    {
       return true;  
    }
@@ -622,13 +602,14 @@ bool ModuleManager::IsProviderValid(const PluginID & WXUNUSED(providerID),
 }
 
 bool ModuleManager::IsPluginValid(const PluginID & providerID,
-                                  const wxString & path)
+                                  const PluginPath & path,
+                                  bool bFast)
 {
    if (mDynModules.find(providerID) == mDynModules.end())
    {
       return false;
    }
 
-   return mDynModules[providerID]->IsPluginValid(path);
+   return mDynModules[providerID]->IsPluginValid(path, bFast);
 }
 
