@@ -6,12 +6,20 @@
 
   Dominic Mazzoni
 
-*******************************************************************/
+*******************************************************************//**
+
+\class ScreenFrame
+\brief ScreenFrame provides an alternative Gui for ScreenshotCommand.
+It adds a timer that allows a delay before taking a screenshot,
+provides lots of one-click buttons, options to resize the screen.
+It forwards the actual work of doing the commands to the ScreenshotCommand.
+
+***********************************************************************/
 
 #include "Screenshot.h"
 #include "commands/ScreenshotCommand.h"
 #include "commands/CommandTargets.h"
-#include "commands/CommandDirectory.h"
+#include "commands/CommandContext.h"
 #include <wx/defs.h>
 #include <wx/event.h>
 #include <wx/frame.h>
@@ -23,31 +31,36 @@
 #include <wx/image.h>
 #include <wx/intl.h>
 #include <wx/panel.h>
+#include <wx/sizer.h>
 #include <wx/statusbr.h>
 #include <wx/textctrl.h>
 #include <wx/timer.h>
 #include <wx/tglbtn.h>
 #include <wx/window.h>
 
-#include "AudacityApp.h"
 #include "Project.h"
+#include "ProjectStatus.h"
+#include "ProjectWindow.h"
 #include "Prefs.h"
 #include "toolbars/ToolManager.h"
+#include "tracks/ui/TrackView.h"
+#include "widgets/HelpSystem.h"
 
-#include "Track.h"
+#include "ViewInfo.h"
+#include "WaveTrack.h"
 
-class CommandType;
+class OldStyleCommandType;
 
 ////////////////////////////////////////////////////////////////////////////////
 
-class ScreenFrame:public wxFrame
+class ScreenFrame final : public wxFrame
 {
  public:
    // constructors and destructors
    ScreenFrame(wxWindow *parent, wxWindowID id);
    virtual ~ScreenFrame();
 
-   virtual bool ProcessEvent(wxEvent & event);
+   bool ProcessEvent(wxEvent & event) override;
 
  private:
    void Populate();
@@ -56,6 +69,9 @@ class ScreenFrame:public wxFrame
    void OnCloseWindow(wxCloseEvent & event);
    void OnUIUpdate(wxUpdateUIEvent & event);
    void OnDirChoose(wxCommandEvent & event);
+   void OnGetURL(wxCommandEvent & event);
+   void OnClose(wxCommandEvent & event );
+
 
    void SizeMainWindow(int w, int h);
    void OnMainWindowSmall(wxCommandEvent & event);
@@ -63,20 +79,8 @@ class ScreenFrame:public wxFrame
    void OnToggleBackgroundBlue(wxCommandEvent & event);
    void OnToggleBackgroundWhite(wxCommandEvent & event);
 
-   void DoCapture(wxString captureMode);
-   void OnCaptureWindowContents(wxCommandEvent & event);
-   void OnCaptureFullWindow(wxCommandEvent & event);
-   void OnCaptureWindowPlus(wxCommandEvent & event);
-   void OnCaptureFullScreen(wxCommandEvent & event);
-   void OnCaptureToolbars(wxCommandEvent & event);
-   void OnCaptureSelectionBar(wxCommandEvent & event);
-   void OnCaptureTools(wxCommandEvent & event);
-   void OnCaptureTransport(wxCommandEvent & event);
-   void OnCaptureMixer(wxCommandEvent & event);
-   void OnCaptureMeter(wxCommandEvent & event);
-   void OnCaptureEdit(wxCommandEvent & event);
-   void OnCaptureDevice(wxCommandEvent & event);
-   void OnCaptureTranscription(wxCommandEvent & event);
+   void DoCapture(int captureMode);
+   void OnCaptureSomething(wxCommandEvent & event);
 
    void TimeZoom(double seconds);
    void OnOneSec(wxCommandEvent & event);
@@ -90,13 +94,7 @@ class ScreenFrame:public wxFrame
    void OnMedTracks(wxCommandEvent & event);
    void OnTallTracks(wxCommandEvent & event);
 
-   void OnCaptureTrackPanel(wxCommandEvent & event);
-   void OnCaptureRuler(wxCommandEvent & event);
-   void OnCaptureTracks(wxCommandEvent & event);
-   void OnCaptureFirstTrack(wxCommandEvent & event);
-   void OnCaptureSecondTrack(wxCommandEvent & event);
-
-   ScreenshotCommand *CreateCommand();
+   std::unique_ptr<ScreenshotCommand> CreateCommand();
 
    wxCheckBox *mDelayCheckBox;
    wxTextCtrl *mDirectoryTextBox;
@@ -104,20 +102,30 @@ class ScreenFrame:public wxFrame
    wxToggleButton *mWhite;
    wxStatusBar *mStatus;
 
-   ScreenshotCommand *mCommand;
-   CommandExecutionContext mContext;
+   std::unique_ptr<ScreenshotCommand> mCommand;
+   const CommandContext mContext;
 
    DECLARE_EVENT_TABLE()
 };
 
-static ScreenFrame *mFrame = NULL;
+// Static pointer to the unique ScreenFrame window.
+// Formerly it was parentless, therefore this was a Destroy_ptr<ScreenFrame>
+// But now the window is owned, so just use a bare pointer, and null it when
+// the unique window is destroyed.
+using ScreenFramePtr = ScreenFrame*;
+ScreenFramePtr mFrame;
 
 ////////////////////////////////////////////////////////////////////////////////
 
 void OpenScreenshotTools()
 {
    if (!mFrame) {
-      mFrame = new ScreenFrame(NULL, -1);
+      auto parent = wxTheApp->GetTopWindow();
+      if (!parent) {
+         wxASSERT(false);
+         return;
+      }
+      mFrame = ScreenFramePtr{ safenew ScreenFrame(parent, -1) };
    }
    mFrame->Show();
    mFrame->Raise();
@@ -125,35 +133,32 @@ void OpenScreenshotTools()
 
 void CloseScreenshotTools()
 {
-   if (mFrame) {
-      mFrame->Destroy();
-      mFrame = NULL;
-   }
+   mFrame = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-class ScreenFrameTimer:public wxTimer
+class ScreenFrameTimer final : public wxTimer
 {
  public:
    ScreenFrameTimer(ScreenFrame *frame,
                     wxEvent & event)
    {
       screenFrame = frame;
-      evt = event.Clone();
+      evt.reset(event.Clone());
    }
 
-   virtual void Notify()
+   void Notify() override
    {
+      // Process timer notification just once, then destroy self
       evt->SetEventObject(NULL);
       screenFrame->ProcessEvent(*evt);
-      delete evt;
       delete this;
    }
 
  private:
    ScreenFrame *screenFrame;
-   wxEvent *evt;
+   std::unique_ptr<wxEvent> evt;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -178,38 +183,51 @@ enum
 
    IdDelayCheckBox,
 
+   IdCaptureFirst,
+   // No point delaying the capture of sets of things.
+   IdCaptureEffects= IdCaptureFirst,
+   IdCaptureScriptables,
+   IdCapturePreferences,
    IdCaptureToolbars,
+
+   // Put all events that need delay between AllDelayed and LastDelayed.
+   IdAllDelayedEvents,
+   IdCaptureWindowContents=IdAllDelayedEvents,
+   IdCaptureFullWindow,
+   IdCaptureWindowPlus,
+   IdCaptureFullScreen,
+  
    IdCaptureSelectionBar,
+   IdCaptureSpectralSelection,
    IdCaptureTools,
    IdCaptureTransport,
    IdCaptureMixer,
    IdCaptureMeter,
+   IdCapturePlayMeter,
+   IdCaptureRecordMeter,
    IdCaptureEdit,
    IdCaptureDevice,
    IdCaptureTranscription,
+   IdCaptureScrub,
 
    IdCaptureTrackPanel,
    IdCaptureRuler,
    IdCaptureTracks,
    IdCaptureFirstTrack,
    IdCaptureSecondTrack,
+   IdCaptureLast = IdCaptureSecondTrack,
+
+   IdLastDelayedEvent,
 
    IdToggleBackgroundBlue,
    IdToggleBackgroundWhite,
 
-   // Put all events that might need delay below:
-   IdAllDelayedEvents,
-
-   IdCaptureWindowContents,
-   IdCaptureFullWindow,
-   IdCaptureWindowPlus,
-   IdCaptureFullScreen,
-
-   IdLastDelayedEvent,
 };
 
 BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
    EVT_CLOSE(ScreenFrame::OnCloseWindow)
+   EVT_BUTTON(wxID_HELP, ScreenFrame::OnGetURL)
+   EVT_BUTTON(wxID_CANCEL, ScreenFrame::OnClose)
 
    EVT_UPDATE_UI(IdCaptureFullScreen,   ScreenFrame::OnUIUpdate)
 
@@ -217,20 +235,7 @@ BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
    EVT_BUTTON(IdMainWindowLarge,        ScreenFrame::OnMainWindowLarge)
    EVT_TOGGLEBUTTON(IdToggleBackgroundBlue,   ScreenFrame::OnToggleBackgroundBlue)
    EVT_TOGGLEBUTTON(IdToggleBackgroundWhite,  ScreenFrame::OnToggleBackgroundWhite)
-   EVT_BUTTON(IdCaptureWindowContents,  ScreenFrame::OnCaptureWindowContents)
-   EVT_BUTTON(IdCaptureFullWindow,      ScreenFrame::OnCaptureFullWindow)
-   EVT_BUTTON(IdCaptureWindowPlus,      ScreenFrame::OnCaptureWindowPlus)
-   EVT_BUTTON(IdCaptureFullScreen,      ScreenFrame::OnCaptureFullScreen)
-
-   EVT_BUTTON(IdCaptureToolbars,        ScreenFrame::OnCaptureToolbars)
-   EVT_BUTTON(IdCaptureSelectionBar,    ScreenFrame::OnCaptureSelectionBar)
-   EVT_BUTTON(IdCaptureTools,           ScreenFrame::OnCaptureTools)
-   EVT_BUTTON(IdCaptureTransport,       ScreenFrame::OnCaptureTransport)
-   EVT_BUTTON(IdCaptureMixer,           ScreenFrame::OnCaptureMixer)
-   EVT_BUTTON(IdCaptureMeter,           ScreenFrame::OnCaptureMeter)
-   EVT_BUTTON(IdCaptureEdit,            ScreenFrame::OnCaptureEdit)
-   EVT_BUTTON(IdCaptureDevice,          ScreenFrame::OnCaptureDevice)
-   EVT_BUTTON(IdCaptureTranscription,   ScreenFrame::OnCaptureTranscription)
+   EVT_COMMAND_RANGE(IdCaptureFirst, IdCaptureLast, wxEVT_COMMAND_BUTTON_CLICKED, ScreenFrame::OnCaptureSomething)
 
    EVT_BUTTON(IdOneSec,                 ScreenFrame::OnOneSec)
    EVT_BUTTON(IdTenSec,                 ScreenFrame::OnTenSec)
@@ -242,43 +247,43 @@ BEGIN_EVENT_TABLE(ScreenFrame, wxFrame)
    EVT_BUTTON(IdMedTracks,              ScreenFrame::OnMedTracks)
    EVT_BUTTON(IdTallTracks,             ScreenFrame::OnTallTracks)
 
-   EVT_BUTTON(IdCaptureTrackPanel,      ScreenFrame::OnCaptureTrackPanel)
-   EVT_BUTTON(IdCaptureRuler,           ScreenFrame::OnCaptureRuler)
-   EVT_BUTTON(IdCaptureTracks,          ScreenFrame::OnCaptureTracks)
-   EVT_BUTTON(IdCaptureFirstTrack,      ScreenFrame::OnCaptureFirstTrack)
-   EVT_BUTTON(IdCaptureSecondTrack,     ScreenFrame::OnCaptureSecondTrack)
-
    EVT_BUTTON(IdDirChoose,              ScreenFrame::OnDirChoose)
 END_EVENT_TABLE();
 
 // Must not be called before CreateStatusBar!
-ScreenshotCommand *ScreenFrame::CreateCommand()
+std::unique_ptr<ScreenshotCommand> ScreenFrame::CreateCommand()
 {
    wxASSERT(mStatus != NULL);
-   CommandOutputTarget *output =
-      new CommandOutputTarget(new NullProgressTarget(),
-                              new StatusBarTarget(*mStatus),
-                              new MessageBoxTarget());
-   CommandType *type = CommandDirectory::Get()->LookUp(wxT("Screenshot"));
-   wxASSERT_MSG(type != NULL, wxT("Screenshot command doesn't exist!"));
-   return new ScreenshotCommand(*type, output, this);
+   auto output =
+      std::make_unique<CommandOutputTargets>(std::make_unique<NullProgressTarget>(),
+                              std::make_shared<StatusBarTarget>(*mStatus),
+                              std::make_shared<MessageBoxTarget>());
+   return std::make_unique<ScreenshotCommand>();//*type, std::move(output), this);
 }
 
 ScreenFrame::ScreenFrame(wxWindow * parent, wxWindowID id)
 :  wxFrame(parent, id, _("Screen Capture Frame"),
            wxDefaultPosition, wxDefaultSize,
+
 #if !defined(__WXMSW__)
-           wxFRAME_TOOL_WINDOW|
+
+   #if !defined(__WXMAC__) // bug1358
+           wxFRAME_TOOL_WINDOW |
+   #endif
+
 #else
+
            wxSTAY_ON_TOP|
+
 #endif
+
            wxSYSTEM_MENU|wxCAPTION|wxCLOSE_BOX),
-   mContext(&wxGetApp(), GetActiveProject())
+   mContext( *GetActiveProject() )
 {
    mDelayCheckBox = NULL;
    mDirectoryTextBox = NULL;
 
-   mStatus = CreateStatusBar();
+   mStatus = CreateStatusBar(3);
    mCommand = CreateCommand();
 
    Populate();
@@ -287,12 +292,19 @@ ScreenFrame::ScreenFrame(wxWindow * parent, wxWindowID id)
    // Note that the audio could be playing.
    // The monitoring will switch off temporarily
    // because we've switched monitor mid play.
-   mContext.GetProject()->mToolManager->Reset();
+   // Bug 383 - Resetting the toolbars is not wanted.
+   // Any that are invisible will be amde visible as/when needed.
+   //ToolManager::Get( mContext.project ).Reset();
+   Center();
 }
 
 ScreenFrame::~ScreenFrame()
 {
-   delete mCommand;
+   if (this == mFrame)
+      mFrame = nullptr;
+   else
+      // There should only be one!
+      wxASSERT(false);
 }
 
 void ScreenFrame::Populate()
@@ -304,6 +316,7 @@ void ScreenFrame::Populate()
 void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
 {
    wxPanel *p = S.StartPanel();
+   RTL_WORKAROUND(p);
    {
       S.SetBorder(3);
 
@@ -333,13 +346,13 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
             S.Id(IdMainWindowLarge).AddButton(_("Resize Large"));
             /* i18n-hint: Bkgnd is short for background and appears on a small button
              * It is OK to just translate this item as if it said 'Blue' */
-            mBlue = new wxToggleButton(p,
+            mBlue = safenew wxToggleButton(S.GetParent(),
                                        IdToggleBackgroundBlue,
                                        _("Blue Bkgnd"));
             S.AddWindow(mBlue);
             /* i18n-hint: Bkgnd is short for background and appears on a small button
              * It is OK to just translate this item as if it said 'White' */
-            mWhite = new wxToggleButton(p,
+            mWhite = safenew wxToggleButton(S.GetParent(),
                                         IdToggleBackgroundWhite,
                                         _("White Bkgnd"));
             S.AddWindow(mWhite);
@@ -364,7 +377,7 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          {
             mDelayCheckBox = S.Id(IdDelayCheckBox).AddCheckBox
                (_("Wait 5 seconds and capture frontmost window/dialog"),
-                _("false"));
+                false);
          }
          S.EndHorizontalLay();
       }
@@ -375,7 +388,16 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          S.StartHorizontalLay();
          {
             S.Id(IdCaptureToolbars).AddButton(_("All Toolbars"));
+            S.Id(IdCaptureEffects).AddButton(_("All Effects"));
+            S.Id(IdCaptureScriptables).AddButton(_("All Scriptables"));
+            S.Id(IdCapturePreferences).AddButton(_("All Preferences"));
+         }
+         S.EndHorizontalLay();
+
+         S.StartHorizontalLay();
+         {
             S.Id(IdCaptureSelectionBar).AddButton(_("SelectionBar"));
+            S.Id(IdCaptureSpectralSelection).AddButton(_("Spectral Selection"));
             S.Id(IdCaptureTools).AddButton(_("Tools"));
             S.Id(IdCaptureTransport).AddButton(_("Transport"));
          }
@@ -385,9 +407,17 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          {
             S.Id(IdCaptureMixer).AddButton(_("Mixer"));
             S.Id(IdCaptureMeter).AddButton(_("Meter"));
+            S.Id(IdCapturePlayMeter).AddButton(_("Play Meter"));
+            S.Id(IdCaptureRecordMeter).AddButton(_("Record Meter"));
+         }
+         S.EndHorizontalLay();
+
+         S.StartHorizontalLay();
+         {
             S.Id(IdCaptureEdit).AddButton(_("Edit"));
             S.Id(IdCaptureDevice).AddButton(_("Device"));
-            S.Id(IdCaptureTranscription).AddButton(_("Transcription"));
+            S.Id(IdCaptureTranscription).AddButton(_("Play-at-Speed"));
+            S.Id(IdCaptureScrub).AddButton(_("Scrub"));
          }
          S.EndHorizontalLay();
 
@@ -424,6 +454,7 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
          S.EndHorizontalLay();
       }
       S.EndStatic();
+      S.AddStandardButtons(eCloseButton |eHelpButton);
    }
    S.EndPanel();
 
@@ -449,22 +480,27 @@ void ScreenFrame::PopulateOrExchange(ShuttleGui & S)
       CentreOnParent();
    }
 
-   SetIcon(mContext.GetProject()->GetIcon());
+   SetIcon( GetProjectFrame( mContext.project ).GetIcon() );
 }
 
 bool ScreenFrame::ProcessEvent(wxEvent & e)
 {
    int id = e.GetId();
 
+   // If split into two parts to make for easier breakpoint
+   // when testing timer.
    if (mDelayCheckBox &&
        mDelayCheckBox->GetValue() &&
        e.IsCommandEvent() &&
-       e.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED &&
-       id >= IdAllDelayedEvents && id <= IdLastDelayedEvent &&
+       e.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED)
+   {
+      if( id >= IdAllDelayedEvents && id <= IdLastDelayedEvent &&
        e.GetEventObject() != NULL) {
-      ScreenFrameTimer *timer = new ScreenFrameTimer(this, e);
-      timer->Start(5000, true);
-      return true;
+         // safenew because it's a one-shot that deletes itself
+         ScreenFrameTimer *timer = safenew ScreenFrameTimer(this, e);
+         timer->Start(5000, true);
+         return true;
+      }
    }
 
    if (e.IsCommandEvent() && e.GetEventObject() == NULL) {
@@ -475,10 +511,19 @@ bool ScreenFrame::ProcessEvent(wxEvent & e)
 
 void ScreenFrame::OnCloseWindow(wxCloseEvent &  WXUNUSED(event))
 {
-   mFrame = NULL;
    Destroy();
 }
 
+void ScreenFrame::OnClose(wxCommandEvent &  WXUNUSED(event))
+{
+   Destroy();
+}
+
+void ScreenFrame::OnGetURL(wxCommandEvent & WXUNUSED(event))
+{
+   HelpSystem::ShowHelp(this, wxT("Screenshot"));
+}
+
 void ScreenFrame::OnUIUpdate(wxUpdateUIEvent &  WXUNUSED(event))
 {
 #ifdef __WXMAC__
@@ -510,42 +555,41 @@ void ScreenFrame::OnDirChoose(wxCommandEvent & WXUNUSED(event))
 {
    wxString current = mDirectoryTextBox->GetValue();
 
-   wxDirDialog dlog(this,
+   wxDirDialogWrapper dlog(this,
                     _("Choose a location to save screenshot images"),
                     current);
 
    dlog.ShowModal();
-   if (dlog.GetPath() != wxT("")) {
+   if (!dlog.GetPath().empty()) {
       wxFileName tmpDirPath;
       tmpDirPath.AssignDir(dlog.GetPath());
       wxString path = tmpDirPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR);
       mDirectoryTextBox->SetValue(path);
       gPrefs->Write(wxT("/ScreenshotPath"), path);
       gPrefs->Flush();
+      mCommand->mPath = path;
    }
 }
 
 void ScreenFrame::OnToggleBackgroundBlue(wxCommandEvent & WXUNUSED(event))
 {
    mWhite->SetValue(false);
-   mCommand->SetParameter(wxT("Background"),
-         mBlue->GetValue() ? wxT("Blue") : wxT("None"));
 }
 
 void ScreenFrame::OnToggleBackgroundWhite(wxCommandEvent & WXUNUSED(event))
 {
    mBlue->SetValue(false);
-   mCommand->SetParameter(wxT("Background"),
-         mWhite->GetValue() ? wxT("White") : wxT("None"));
 }
 
 void ScreenFrame::SizeMainWindow(int w, int h)
 {
    int top = 20;
 
-   mContext.GetProject()->Maximize(false);
-   mContext.GetProject()->SetSize(16, 16 + top, w, h);
-   mContext.GetProject()->mToolManager->Reset();
+   auto &window = GetProjectFrame( mContext.project );
+   window.Maximize(false);
+   window.SetSize(16, 16 + top, w, h);
+   //Bug383 - Toolbar Resets not wanted.
+   //ToolManager::Get( mContext.project ).Reset();
 }
 
 void ScreenFrame::OnMainWindowSmall(wxCommandEvent & WXUNUSED(event))
@@ -558,113 +602,100 @@ void ScreenFrame::OnMainWindowLarge(wxCommandEvent & WXUNUSED(event))
    SizeMainWindow(900, 600);
 }
 
-void ScreenFrame::DoCapture(wxString captureMode)
+void ScreenFrame::DoCapture(int captureMode)
 {
    Hide();
-   mCommand->SetParameter(wxT("FilePath"), mDirectoryTextBox->GetValue());
-
-   mCommand->SetParameter(wxT("CaptureMode"), captureMode);
+   //mCommand->SetParameter(wxT("FilePath"), mDirectoryTextBox->GetValue());
+   //mCommand->SetParameter(wxT("CaptureMode"), captureMode);
+   mCommand->mBack = mWhite->GetValue()
+      ? ScreenshotCommand::kWhite
+      : mBlue->GetValue()
+         ? ScreenshotCommand::kBlue : ScreenshotCommand::kNone;
+   mCommand->mPath = mDirectoryTextBox->GetValue();
+   mCommand->mWhat = captureMode;
    if (!mCommand->Apply(mContext))
-      mStatus->SetStatusText(wxT("Capture failed!"), mainStatusBarField);
+      mStatus->SetStatusText(_("Capture failed!"), mainStatusBarField);
    Show();
 }
 
-void ScreenFrame::OnCaptureWindowContents(wxCommandEvent & WXUNUSED(event))
+void ScreenFrame::OnCaptureSomething(wxCommandEvent &  event)
 {
-   DoCapture(wxT("window"));
-}
+   int i = event.GetId() - IdCaptureFirst;
 
-void ScreenFrame::OnCaptureFullWindow(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("fullwindow"));
-}
+   /*
+   IdCaptureEffects= IdCaptureFirst,
+   IdCaptureScriptables,
+   IdCapturePreferences,
+   IdCaptureToolbars,
 
-void ScreenFrame::OnCaptureWindowPlus(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("windowplus"));
-}
+   // Put all events that need delay between AllDelayed and LastDelayed.
+   IdAllDelayedEvents,
+   IdCaptureWindowContents=IdAllDelayedEvents,
+   IdCaptureFullWindow,
+   IdCaptureWindowPlus,
+   IdCaptureFullScreen,
+  
+   IdCaptureSelectionBar,
+   IdCaptureSpectralSelection,
+   IdCaptureTools,
+   IdCaptureTransport,
+   IdCaptureMixer,
+   IdCaptureMeter,
+   IdCapturePlayMeter,
+   IdCaptureRecordMeter,
+   IdCaptureEdit,
+   IdCaptureDevice,
+   IdCaptureTranscription,
+   IdCaptureScrub,
 
-void ScreenFrame::OnCaptureFullScreen(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("fullscreen"));
-}
+   IdCaptureTrackPanel,
+   IdCaptureRuler,
+   IdCaptureTracks,
+   IdCaptureFirstTrack,
+   IdCaptureSecondTrack,
+   IdCaptureLast = IdCaptureSecondTrack,
+    */
 
-void ScreenFrame::OnCaptureToolbars(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("toolbars"));
-}
+   static const int codes[] = {
+      ScreenshotCommand::keffects,
+      ScreenshotCommand::kscriptables,
+      ScreenshotCommand::kpreferences,
+      ScreenshotCommand::ktoolbars,
 
-void ScreenFrame::OnCaptureSelectionBar(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("selectionbar"));
-}
+      ScreenshotCommand::kwindow,
+      ScreenshotCommand::kfullwindow,
+      ScreenshotCommand::kwindowplus,
+      ScreenshotCommand::kfullscreen,
+      ScreenshotCommand::kselectionbar,
+      ScreenshotCommand::kspectralselection,
+      ScreenshotCommand::ktools,
+      ScreenshotCommand::ktransport,
+      ScreenshotCommand::kmixer,
+      ScreenshotCommand::kmeter,
+      ScreenshotCommand::kplaymeter,
+      ScreenshotCommand::krecordmeter,
+      ScreenshotCommand::kedit,
+      ScreenshotCommand::kdevice,
+      ScreenshotCommand::ktranscription,
+      ScreenshotCommand::kscrub,
+      ScreenshotCommand::ktrackpanel,
+      ScreenshotCommand::kruler,
+      ScreenshotCommand::ktracks,
+      ScreenshotCommand::kfirsttrack,
+      ScreenshotCommand::ksecondtrack,
+   };
 
-void ScreenFrame::OnCaptureTools(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("tools"));
-}
-
-void ScreenFrame::OnCaptureTransport(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("transport"));
-}
-
-void ScreenFrame::OnCaptureMixer(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("mixer"));
-}
-
-void ScreenFrame::OnCaptureMeter(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("meter"));
-}
-
-void ScreenFrame::OnCaptureEdit(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("edit"));
-}
-
-void ScreenFrame::OnCaptureDevice(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("device"));
-}
-
-void ScreenFrame::OnCaptureTranscription(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("transcription"));
-}
-
-void ScreenFrame::OnCaptureTrackPanel(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("trackpanel"));
-}
-
-void ScreenFrame::OnCaptureRuler(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("ruler"));
-}
-
-void ScreenFrame::OnCaptureTracks(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("tracks"));
-}
-
-void ScreenFrame::OnCaptureFirstTrack(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("firsttrack"));
-}
-
-void ScreenFrame::OnCaptureSecondTrack(wxCommandEvent & WXUNUSED(event))
-{
-   DoCapture(wxT("secondtrack"));
+   DoCapture(codes[i]);
 }
 
 void ScreenFrame::TimeZoom(double seconds)
 {
+   auto &viewInfo = ViewInfo::Get( mContext.project );
+   auto &window = ProjectWindow::Get( mContext.project );
    int width, height;
-   mContext.GetProject()->GetClientSize(&width, &height);
-   mContext.GetProject()->mViewInfo.SetZoom((0.75 * width) / seconds);
-   mContext.GetProject()->RedrawProject();
+   window.GetClientSize(&width, &height);
+   viewInfo.SetZoom((0.75 * width) / seconds);
+   window.RedrawProject();
 }
 
 void ScreenFrame::OnOneSec(wxCommandEvent & WXUNUSED(event))
@@ -694,29 +725,32 @@ void ScreenFrame::OnOneHour(wxCommandEvent & WXUNUSED(event))
 
 void ScreenFrame::SizeTracks(int h)
 {
-   TrackListIterator iter(mContext.GetProject()->GetTracks());
-   for (Track * t = iter.First(); t; t = iter.Next()) {
-      if (t->GetKind() == Track::Wave) {
-         if (t->GetLink()) {
-            t->SetHeight(h);
-         }
-         else {
-            t->SetHeight(h*2);
-         }
-      }
+   // h is the height for a channel
+   // Set the height of a mono track twice as high
+
+   // TODO: more-than-two-channels
+   // If there should be more-than-stereo tracks, this makes
+   // each channel as high as for a stereo channel
+
+   auto &tracks = TrackList::Get( mContext.project );
+   for (auto t : tracks.Leaders<WaveTrack>()) {
+      auto channels = TrackList::Channels(t);
+      auto nChannels = channels.size();
+      auto height = nChannels == 1 ? 2 * h : h;
+      for (auto channel : channels)
+         TrackView::Get( *channel ).SetHeight(height);
    }
-   mContext.GetProject()->RedrawProject();
+   ProjectWindow::Get( mContext.project ).RedrawProject();
 }
 
 void ScreenFrame::OnShortTracks(wxCommandEvent & WXUNUSED(event))
 {
-   TrackListIterator iter(mContext.GetProject()->GetTracks());
-   for (Track * t = iter.First(); t; t = iter.Next()) {
-      if (t->GetKind() == Track::Wave) {
-         t->SetHeight(t->GetMinimizedHeight());
-      }
+   for (auto t : TrackList::Get( mContext.project ).Any<WaveTrack>()) {
+      auto &view = TrackView::Get( *t );
+      view.SetHeight( view.GetMinimizedHeight() );
    }
-   mContext.GetProject()->RedrawProject();
+
+   ProjectWindow::Get( mContext.project ).RedrawProject();
 }
 
 void ScreenFrame::OnMedTracks(wxCommandEvent & WXUNUSED(event))
