@@ -16,11 +16,11 @@
 #ifndef __AUDACITY_SOUND_ACTIVATED_RECORD__
 #define __AUDACITY_SOUND_ACTIVATED_RECORD__
 
-#include <wx/dialog.h>
+#include "widgets/wxPanelWrapper.h" // to inherit
 
 class ShuttleGui;
 
-class SoundActivatedRecord : public wxDialog
+class SoundActivatedRecord final : public wxDialogWrapper
 {
 public:
    SoundActivatedRecord(wxWindow* parent);
@@ -31,7 +31,7 @@ private:
 
    void PopulateOrExchange(ShuttleGui& S);
 
-   DECLARE_EVENT_TABLE();
+   DECLARE_EVENT_TABLE()
 };
 
 #endif
