@@ -11,7 +11,6 @@
 #ifndef __AUDACITY_SPLASH_DLG__
 #define __AUDACITY_SPLASH_DLG__
 
-#include "MemoryX.h"
 #include "widgets/wxPanelWrapper.h" // to inherit
 
 class wxBitmap;
@@ -22,6 +21,9 @@ class HtmlWindow;
 class SplashDialog final : public wxDialogWrapper {
    DECLARE_DYNAMIC_CLASS(SplashDialog)
 public:
+
+   static void DoHelpWelcome( AudacityProject &project );
+
    SplashDialog(wxWindow * parent);
    virtual ~ SplashDialog();
    void OnOK(wxCommandEvent & event);
