@@ -30,33 +30,39 @@
 
 *//*******************************************************************/
 
+#include "Audacity.h"
+#include "Tags.h"
+
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
+#include <wx/setup.h> // for wxUSE_* macros
+
 #ifndef WX_PRECOMP
 // Include your minimal set of headers here, or wx.h
 #include <wx/window.h>
 #endif
 
-#include "Tags.h"
-
-#include "Audacity.h"
-#include "FileDialog.h"
 #include "FileNames.h"
-#include "Internat.h"
 #include "Prefs.h"
+#include "Project.h"
+#include "ProjectFileIORegistry.h"
 #include "ShuttleGui.h"
+#include "TranslatableStringArray.h"
 #include "widgets/Grid.h"
+#include "widgets/AudacityMessageBox.h"
+#include "widgets/HelpSystem.h"
 #include "xml/XMLFileReader.h"
 
 #include <wx/button.h>
 #include <wx/choice.h>
+#include <wx/filedlg.h>
 #include <wx/filename.h>
 #include <wx/intl.h>
 #include <wx/listctrl.h>
-#include <wx/msgdlg.h>
 #include <wx/notebook.h>
 #include <wx/radiobut.h>
+#include <wx/scrolbar.h>
 #include <wx/sizer.h>
 #include <wx/stattext.h>
 #include <wx/string.h>
@@ -220,6 +226,32 @@ static const wxChar *DefaultGenres[] =
    wxT("Synthpop")
 };
 
+static ProjectFileIORegistry::Entry registerFactory{
+   wxT( "tags" ),
+   []( AudacityProject &project ){ return &Tags::Get( project ); }
+};
+
+static const AudacityProject::AttachedObjects::RegisteredFactory key{
+  [](AudacityProject &){ return std::make_shared< Tags >(); }
+};
+
+Tags &Tags::Get( AudacityProject &project )
+{
+   return project.AttachedObjects::Get< Tags >( key );
+}
+
+const Tags &Tags::Get( const AudacityProject &project )
+{
+   return Get( const_cast< AudacityProject & >( project ) );
+}
+
+Tags &Tags::Set( AudacityProject &project, const std::shared_ptr< Tags > &tags )
+{
+   auto &result = *tags;
+   project.AttachedObjects::Assign( key, tags );
+   return result;
+}
+
 Tags::Tags()
 {
    mEditTitle = true;
@@ -233,6 +265,11 @@ Tags::~Tags()
 {
 }
 
+std::shared_ptr<Tags> Tags::Duplicate() const
+{
+   return std::make_shared<Tags>(*this);
+}
+
 Tags & Tags::operator=(const Tags & src)
 {
    mEditTitle = src.mEditTitle;
@@ -297,6 +334,41 @@ void Tags::Clear()
    mMap.clear();
 }
 
+namespace {
+   bool EqualMaps(const TagMap &map1, const TagMap &map2)
+   {
+      // Maps are unordered, hash maps; can't just iterate in tandem and
+      // compare.
+      if (map1.size() != map2.size())
+         return false;
+
+      for (const auto &pair : map2) {
+         auto iter = map1.find(pair.first);
+         if (iter == map1.end() || iter->second != pair.second)
+            return false;
+      }
+
+      return true;
+   }
+}
+
+bool operator== (const Tags &lhs, const Tags &rhs)
+{
+   if (!EqualMaps(lhs.mXref, rhs.mXref))
+      return false;
+
+   if (!EqualMaps(lhs.mMap, rhs.mMap))
+      return false;
+
+   return
+      lhs.mGenres == rhs.mGenres
+      &&
+      lhs.mEditTitle == rhs.mEditTitle
+      &&
+      lhs.mEditTrackNumber == rhs.mEditTrackNumber
+   ;
+}
+
 void Tags::AllowEditTitle(bool editTitle)
 {
    mEditTitle = editTitle;
@@ -309,14 +381,14 @@ void Tags::AllowEditTrackNumber(bool editTrackNumber)
 
 int Tags::GetNumUserGenres()
 {
-   return mGenres.GetCount();
+   return mGenres.size();
 }
 
 void Tags::LoadDefaultGenres()
 {
-   mGenres.Clear();
+   mGenres.clear();
    for (size_t i = 0; i < WXSIZEOF(DefaultGenres); i++) {
-      mGenres.Add(DefaultGenres[i]);
+      mGenres.push_back(DefaultGenres[i]);
    }
 }
 
@@ -330,11 +402,11 @@ void Tags::LoadGenres()
       return;
    }
 
-   mGenres.Clear();
+   mGenres.clear();
 
    int cnt = tf.GetLineCount();
    for (int i = 0; i < cnt; i++) {
-      mGenres.Add(tf.GetLine(i));
+      mGenres.push_back(tf.GetLine(i));
    }
 }
 
@@ -371,59 +443,50 @@ int Tags::GetGenre(const wxString & name)
    return 255;
 }
 
-bool Tags::HasTag(const wxString & name)
+bool Tags::HasTag(const wxString & name) const
 {
    wxString key = name;
    key.UpperCase();
 
-   TagMap::iterator iter = mXref.find(key);
+   auto iter = mXref.find(key);
    return (iter != mXref.end());
 }
 
-wxString Tags::GetTag(const wxString & name)
+wxString Tags::GetTag(const wxString & name) const
 {
    wxString key = name;
    key.UpperCase();
 
-   TagMap::iterator iter = mXref.find(key);
+   auto iter = mXref.find(key);
 
    if (iter == mXref.end()) {
       return wxEmptyString;
    }
 
-   return mMap[iter->second];
-}
-
-bool Tags::GetFirst(wxString & name, wxString & value)
-{
-   mIter = mMap.begin();
-   if (mIter == mMap.end()) {
-      return false;
+   auto iter2 = mMap.find(iter->second);
+   if (iter2 == mMap.end()) {
+      wxASSERT(false);
+      return wxEmptyString;
    }
-
-   name = mIter->first;
-   value = mIter->second;
-
-   return true;
+   else
+      return iter2->second;
 }
 
-bool Tags::GetNext(wxString & name, wxString & value)
+Tags::Iterators Tags::GetRange() const
 {
-   ++mIter;
-   if (mIter == mMap.end()) {
-      return false;
-   }
-
-   name = mIter->first;
-   value = mIter->second;
-
-   return true;
+   return { mMap.begin(), mMap.end() };
 }
 
-void Tags::SetTag(const wxString & name, const wxString & value)
+void Tags::SetTag(const wxString & name, const wxString & value, const bool bSpecialTag)
 {
    // We don't like empty names
-   if (name.IsEmpty()) {
+   if (name.empty()) {
+      return;
+   }
+
+   // Tag name must be ascii
+   if (!name.IsAscii()) {
+      wxLogError("Tag rejected (Non-ascii character in name)");
       return;
    }
 
@@ -434,28 +497,39 @@ void Tags::SetTag(const wxString & name, const wxString & value)
    // Look it up
    TagMap::iterator iter = mXref.find(key);
 
-   // Didn't find the tag
-   if (iter == mXref.end()) {
-      // Intention was to delete so no need to add it
-      if (value.IsEmpty()) {
-         return;
+   // The special tags, if empty, should not exist.
+   // However it is allowable for a custom tag to be empty.
+   // See Bug 440 and Bug 1382 
+   if (value.empty() && bSpecialTag) {
+      // Erase the tag
+      if (iter == mXref.end())
+         // nothing to do
+         ;
+      else {
+         mMap.erase(iter->second);
+         mXref.erase(iter);
       }
-
-      // Add a new tag
-      mXref[key] = name;
-      mMap[name] = value;
-      return;
    }
+   else 
+   {
+      if (iter == mXref.end()) {
+         // Didn't find the tag
 
-   // Intention was to delete
-   if (value.IsEmpty()) {
-      mMap.erase(iter->second);
-      mXref.erase(iter);
-      return;
+         // Add a NEW tag
+         mXref[key] = name;
+         mMap[name] = value;
+      }
+      else if (iter->second != name) {
+         // Watch out for case differences!
+         mMap[name] = value;
+         mMap.erase(iter->second);
+         iter->second = name;
+      }
+      else {
+         // Update the value
+         mMap[iter->second] = value;
+      }
    }
-
-   // Update the value
-   mMap[iter->second] = value;
 }
 
 void Tags::SetTag(const wxString & name, const int & value)
@@ -474,7 +548,7 @@ bool Tags::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
 
       while (*attrs) {
          wxString attr = *attrs++;
-         if (attr.IsEmpty())
+         if (attr.empty())
             break;
          wxString value = *attrs++;
 
@@ -492,7 +566,7 @@ bool Tags::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
       }
 
       if (n == wxT("id3v2")) {
-         // LLL:  This is obsolute, but it must be handled and ignored.
+         // LLL:  This is obsolete, but it must be handled and ignored.
       }
       else {
          SetTag(n, v);
@@ -517,12 +591,14 @@ XMLTagHandler *Tags::HandleXMLChild(const wxChar *tag)
    return NULL;
 }
 
-void Tags::WriteXML(XMLWriter &xmlFile)
+void Tags::WriteXML(XMLWriter &xmlFile) const
+// may throw
 {
    xmlFile.StartTag(wxT("tags"));
 
-   wxString n, v;
-   for (bool cont = GetFirst(n, v); cont; cont = GetNext(n, v)) {
+   for (const auto &pair : GetRange()) {
+      const auto &n = pair.first;
+      const auto &v = pair.second;
       xmlFile.StartTag(wxT("tag"));
       xmlFile.WriteAttr(wxT("name"), n);
       xmlFile.WriteAttr(wxT("value"), v);
@@ -532,7 +608,7 @@ void Tags::WriteXML(XMLWriter &xmlFile)
    xmlFile.EndTag(wxT("tags"));
 }
 
-bool Tags::ShowEditDialog(wxWindow *parent, wxString title, bool force)
+bool Tags::ShowEditDialog(wxWindow *parent, const wxString &title, bool force)
 {
    if (force) {
       TagsEditor dlg(parent, title, this, mEditTitle, mEditTrackNumber);
@@ -546,31 +622,33 @@ bool Tags::ShowEditDialog(wxWindow *parent, wxString title, bool force)
 // ComboEditor - Wrapper to prevent unwanted background erasure
 //
 
-class ComboEditor:public wxGridCellChoiceEditor
+class ComboEditor final : public wxGridCellChoiceEditor
 {
 public:
    ComboEditor(const wxArrayString& choices, bool allowOthers = false)
    :  wxGridCellChoiceEditor(choices, allowOthers)
+   ,  m_choices{ choices }
+   ,  m_allowOthers{ allowOthers }
    {
    }
 
-   virtual void PaintBackground(const wxRect& WXUNUSED(rectCell), wxGridCellAttr * WXUNUSED(attr))
+   void PaintBackground(wxDC&, const wxRect& WXUNUSED(rectCell), const wxGridCellAttr & WXUNUSED(attr)) override
    {
       // Ignore it (a must on the Mac as the erasure causes problems.)
    }
 
-   void SetParameters(const wxString& params)
+   void SetParameters(const wxString& params) override
    {
       wxGridCellChoiceEditor::SetParameters(params);
 
-      // Refresh the wxComboBox with new values
+      // Refresh the wxComboBox with NEW values
       if (Combo()) {
          Combo()->Clear();
          Combo()->Append(m_choices);
       }
    }
 
-   void SetSize(const wxRect& rectOrig)
+   void SetSize(const wxRect& rectOrig) override
    {
       wxRect rect(rectOrig);
       wxRect r = Combo()->GetRect();
@@ -581,32 +659,111 @@ public:
 
       wxGridCellChoiceEditor::SetSize(rect);
    }
+
+   // Fix for Bug 1389
+   // July 2016: ANSWER-ME: Does this need reporting upstream to wxWidgets?
+   virtual void StartingKey(wxKeyEvent& event) override
+   {
+       // Lifted from wxGridCellTextEditor and adapted to combo.
+
+       // [Below is comment from wxWidgets code]
+       // Since this is now happening in the EVT_CHAR event EmulateKeyPress is no
+       // longer an appropriate way to get the character into the text control.
+       // Do it ourselves instead.  We know that if we get this far that we have
+       // a valid character, so not a whole lot of testing needs to be done.
+
+       //The only difference to wxGridCellTextEditor.
+       //wxTextCtrl* tc = (wxTextCtrl *)m_control;
+       wxComboBox * tc = Combo();
+       int ch;
+
+       bool isPrintable;
+
+   #if wxUSE_UNICODE
+       ch = event.GetUnicodeKey();
+       if ( ch != WXK_NONE )
+           isPrintable = true;
+       else
+   #endif // wxUSE_UNICODE
+       {
+           ch = event.GetKeyCode();
+           isPrintable = ch >= WXK_SPACE && ch < WXK_START;
+       }
+
+       switch (ch)
+       {
+           case WXK_DELETE:
+               // Delete the initial character when starting to edit with DELETE.
+               tc->Remove(0, 1);
+               break;
+
+           case WXK_BACK:
+               // Delete the last character when starting to edit with BACKSPACE.
+               {
+                   const long pos = tc->GetLastPosition();
+                   tc->Remove(pos - 1, pos);
+               }
+               break;
+
+           default:
+               if ( isPrintable )
+                   tc->WriteText(static_cast<wxChar>(ch));
+               break;
+       }
+   }
+
+   // Clone is required by wxwidgets; implemented via copy constructor
+   wxGridCellEditor *Clone() const override
+   {
+      return safenew ComboEditor{ m_choices, m_allowOthers };
+   }
+
+private:
+   wxArrayString   m_choices;
+   bool            m_allowOthers;
 };
 
 //
 // Editor
 //
 
-#define LABEL_ARTIST    _("Artist Name")
-#define LABEL_TITLE     _("Track Title")
-#define LABEL_ALBUM     _("Album Title")
-#define LABEL_TRACK     _("Track Number")
-#define LABEL_YEAR      _("Year")
-#define LABEL_GENRE     _("Genre")
-#define LABEL_COMMENTS  _("Comments")
+#define LABEL_ARTIST    XO("Artist Name")
+#define LABEL_TITLE     XO("Track Title")
+#define LABEL_ALBUM     XO("Album Title")
+#define LABEL_TRACK     XO("Track Number")
+#define LABEL_YEAR      XO("Year")
+#define LABEL_GENRE     XO("Genre")
+#define LABEL_COMMENTS  XO("Comments")
 
-static wxString names[] =
+static wxArrayString names()
 {
-   LABEL_ARTIST,
-   LABEL_TITLE,
-   LABEL_ALBUM,
-   LABEL_TRACK,
-   LABEL_YEAR,
-   LABEL_GENRE,
-   LABEL_COMMENTS
-};
+   static wxString theNames[] =
+   {
+      LABEL_ARTIST,
+      LABEL_TITLE,
+      LABEL_ALBUM,
+      LABEL_TRACK,
+      LABEL_YEAR,
+      LABEL_GENRE,
+      LABEL_COMMENTS
+   };
 
-static struct
+   class NamesArray final : public TranslatableStringArray
+   {
+      void Populate() override
+      {
+         for (auto &name : theNames)
+            mContents.push_back( wxGetTranslation( name ) );
+      }
+   };
+
+   static NamesArray theArray;
+
+   // Yes, return array by value
+   return theArray.Get();
+}
+
+static const struct
 {
    wxString label;
    wxString name;
@@ -632,10 +789,11 @@ enum {
    SaveID,
    SaveDefaultsID,
    AddID,
-   RemoveID
+   RemoveID,
+   DontShowID
 };
 
-BEGIN_EVENT_TABLE(TagsEditor, wxDialog)
+BEGIN_EVENT_TABLE(TagsEditor, wxDialogWrapper)
    EVT_GRID_CELL_CHANGED(TagsEditor::OnChange)
    EVT_BUTTON(EditID, TagsEditor::OnEdit)
    EVT_BUTTON(ResetID, TagsEditor::OnReset)
@@ -645,16 +803,19 @@ BEGIN_EVENT_TABLE(TagsEditor, wxDialog)
    EVT_BUTTON(SaveDefaultsID, TagsEditor::OnSaveDefaults)
    EVT_BUTTON(AddID, TagsEditor::OnAdd)
    EVT_BUTTON(RemoveID, TagsEditor::OnRemove)
+   EVT_BUTTON(wxID_HELP, TagsEditor::OnHelp)
    EVT_BUTTON(wxID_CANCEL, TagsEditor::OnCancel)
    EVT_BUTTON(wxID_OK, TagsEditor::OnOk)
+   EVT_CHECKBOX( DontShowID, TagsEditor::OnDontShow )
+   EVT_KEY_DOWN(TagsEditor::OnKeyDown)
 END_EVENT_TABLE()
 
 TagsEditor::TagsEditor(wxWindow * parent,
-                       wxString title,
+                       const wxString &title,
                        Tags * tags,
                        bool editTitle,
                        bool editTrack)
-:  wxDialog(parent, wxID_ANY, title, wxDefaultPosition, wxDefaultSize,
+:  wxDialogWrapper(parent, wxID_ANY, title, wxDefaultPosition, wxDefaultSize,
             wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER),
    mTags(tags),
    mEditTitle(editTitle),
@@ -662,30 +823,6 @@ TagsEditor::TagsEditor(wxWindow * parent,
 {
    SetName(GetTitle());
 
-   names[0] = LABEL_ARTIST;
-   names[1] = LABEL_TITLE;
-   names[2] = LABEL_ALBUM;
-   names[3] = LABEL_TRACK;
-   names[4] = LABEL_YEAR;
-   names[5] = LABEL_GENRE;
-   names[6] = LABEL_COMMENTS;
-
-   labelmap[0].label = LABEL_ARTIST;
-   labelmap[1].label = LABEL_TITLE;
-   labelmap[2].label = LABEL_ALBUM;
-   labelmap[3].label = LABEL_TRACK;
-   labelmap[4].label = LABEL_YEAR;
-   labelmap[5].label = LABEL_GENRE;
-   labelmap[6].label = LABEL_COMMENTS;
-
-   labelmap[0].name = TAG_ARTIST;
-   labelmap[1].name = TAG_TITLE;
-   labelmap[2].name = TAG_ALBUM;
-   labelmap[3].name = TAG_TRACK;
-   labelmap[4].name = TAG_YEAR;
-   labelmap[5].name = TAG_GENRE;
-   labelmap[6].name = TAG_COMMENTS;
-
    mGrid = NULL;
 
    // Make a local copy of the passed in tags
@@ -728,6 +865,8 @@ TagsEditor::TagsEditor(wxWindow * parent,
    r.width -= 10;
    r.width -= r.x;
    mGrid->SetColSize(1, r.width);
+   //Bug 2038
+   mGrid->SetFocus();
 
    // Load the genres
    PopulateGenres();
@@ -735,39 +874,44 @@ TagsEditor::TagsEditor(wxWindow * parent,
 
 TagsEditor::~TagsEditor()
 {
-   delete mGrid;
+   // This DELETE is not needed because wxWidgets owns the grid.
+// DELETE mGrid;
+
 // TODO:  Need to figure out if these should be deleted.  Looks like the wxGrid
 //        code takes ownership and uses reference counting, but there's been
 //        cases where they show up as memory leaks.
-//   delete mStringRenderer;
-//   delete mComboEditor;
+//  PRL: Fixed the leaks, see commit c87eb0804bc5f40659b133cab6e2ade061959645
+//   DELETE mStringRenderer;
+//   DELETE mComboEditor;
 }
 
 void TagsEditor::PopulateOrExchange(ShuttleGui & S)
 {
+   bool bShow;
+   gPrefs->Read(wxT("/AudioFiles/ShowId3Dialog"), &bShow, true );
+
    S.StartVerticalLay();
    {
-      S.StartHorizontalLay(wxALIGN_LEFT, false);
+      S.StartHorizontalLay(wxALIGN_LEFT, 0);
       {
          S.AddUnits(_("Use arrow keys (or ENTER key after editing) to navigate fields."));
       }
       S.EndHorizontalLay();
 
       if (mGrid == NULL) {
-         mGrid = new Grid(S.GetParent(),
+         mGrid = safenew Grid(S.GetParent(),
                           wxID_ANY,
                           wxDefaultPosition,
                           wxDefaultSize,
                           wxSUNKEN_BORDER);
 
-         mStringRenderer = new wxGridCellStringRenderer;
-         mComboEditor = new ComboEditor(wxArrayString(), true);
-
-         mGrid->RegisterDataType(wxT("Combo"), mStringRenderer, mComboEditor);
+         mGrid->RegisterDataType(wxT("Combo"),
+            (mStringRenderer = safenew wxGridCellStringRenderer),
+            (mComboEditor = safenew ComboEditor(wxArrayString(), true)));
 
          mGrid->SetColLabelSize(mGrid->GetDefaultRowSize());
 
-         wxArrayString cs(WXSIZEOF(names), names);
+         auto  cs = names();
 
          // Build the initial (empty) grid
          mGrid->CreateGrid(0, 2);
@@ -781,19 +925,19 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
          mGrid->SetColSize(0, tc.GetSize().x);
          mGrid->SetColMinimalWidth(0, tc.GetSize().x);
       }
-      S.Prop(true);
+      S.Prop(1);
       S.AddWindow(mGrid, wxEXPAND | wxALL);
 
       S.StartMultiColumn(4, wxALIGN_CENTER);
       {
          S.Id(AddID).AddButton(_("&Add"));
          S.Id(RemoveID).AddButton(_("&Remove"));
-         S.AddTitle(wxT(" "));
+         S.AddTitle( {} );
          S.Id(ClearID).AddButton(_("Cl&ear"));
       }
       S.EndMultiColumn();
 
-      S.StartHorizontalLay(wxALIGN_CENTRE, false);
+      S.StartHorizontalLay(wxALIGN_CENTRE, 0);
       {
          S.StartStatic(_("Genres"));
          {
@@ -811,7 +955,7 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
             {
                S.Id(LoadID).AddButton(_("&Load..."));
                S.Id(SaveID).AddButton(_("&Save..."));
-               S.AddTitle(wxT(" "));
+               S.AddTitle( {} );
                S.Id(SaveDefaultsID).AddButton(_("Set De&fault"));
             }
             S.EndMultiColumn();
@@ -819,10 +963,27 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
          S.EndStatic();
       }
       S.EndHorizontalLay();
+      S.StartHorizontalLay(wxALIGN_LEFT, 0);
+      {
+         S.Id( DontShowID ).AddCheckBox( _("Don't show this when exporting audio"), !bShow );
+      }
+      S.EndHorizontalLay();
    }
    S.EndVerticalLay();
 
-   S.AddStandardButtons(eOkButton | eCancelButton);
+   S.AddStandardButtons(eOkButton | eCancelButton | eHelpButton);
+}
+
+void TagsEditor::OnDontShow( wxCommandEvent & Evt )
+{
+   bool bShow = !Evt.IsChecked();
+   gPrefs->Write(wxT("/AudioFiles/ShowId3Dialog"), bShow );
+   gPrefs->Flush();
+}
+
+void TagsEditor::OnHelp(wxCommandEvent& WXUNUSED(event))
+{
+   HelpSystem::ShowHelp(this, wxT("Metadata_Editor"), true);
 }
 
 bool TagsEditor::TransferDataFromWindow()
@@ -836,36 +997,44 @@ bool TagsEditor::TransferDataFromWindow()
 
    mLocal.Clear();
    for (i = 0; i < cnt; i++) {
+      // Get tag name from the grid
+
       wxString n = mGrid->GetCellValue(i, 0);
       wxString v = mGrid->GetCellValue(i, 1);
 
-      if (n.IsEmpty()) {
+      if (n.empty()) {
          continue;
       }
 
-      if (n.CmpNoCase(LABEL_ARTIST) == 0) {
+      bool bSpecialTag = true;
+
+      // Map special tag names back to internal keys
+      if (n.CmpNoCase(wxGetTranslation(LABEL_ARTIST)) == 0) {
          n = TAG_ARTIST;
       }
-      else if (n.CmpNoCase(LABEL_TITLE) == 0) {
+      else if (n.CmpNoCase(wxGetTranslation(LABEL_TITLE)) == 0) {
          n = TAG_TITLE;
       }
-      else if (n.CmpNoCase(LABEL_ALBUM) == 0) {
+      else if (n.CmpNoCase(wxGetTranslation(LABEL_ALBUM)) == 0) {
          n = TAG_ALBUM;
       }
-      else if (n.CmpNoCase(LABEL_TRACK) == 0) {
+      else if (n.CmpNoCase(wxGetTranslation(LABEL_TRACK)) == 0) {
          n = TAG_TRACK;
       }
-      else if (n.CmpNoCase(LABEL_YEAR) == 0) {
+      else if (n.CmpNoCase(wxGetTranslation(LABEL_YEAR)) == 0) {
          n = TAG_YEAR;
       }
-      else if (n.CmpNoCase(LABEL_GENRE) == 0) {
+      else if (n.CmpNoCase(wxGetTranslation(LABEL_GENRE)) == 0) {
          n = TAG_GENRE;
       }
-      else if (n.CmpNoCase(LABEL_COMMENTS) == 0) {
+      else if (n.CmpNoCase(wxGetTranslation(LABEL_COMMENTS)) == 0) {
          n = TAG_COMMENTS;
       }
+      else {
+         bSpecialTag = false;
+      }
 
-      mLocal.SetTag(n, v);
+      mLocal.SetTag(n, v, bSpecialTag);
    }
 
    return true;
@@ -874,8 +1043,7 @@ bool TagsEditor::TransferDataFromWindow()
 bool TagsEditor::TransferDataToWindow()
 {
    size_t i;
-   wxString n;
-   wxString v;
+   TagMap popTagMap;
 
    // Disable redrawing until we're done
    mGrid->BeginBatch();
@@ -890,27 +1058,34 @@ bool TagsEditor::TransferDataToWindow()
       mGrid->AppendRows();
 
       mGrid->SetReadOnly(i, 0);
-      mGrid->SetCellValue(i, 0, labelmap[i].label);
+      // The special tag name that's displayed and translated may not match
+      // the key string used for internal lookup.
+      mGrid->SetCellValue(i, 0, wxGetTranslation( labelmap[i].label ) );
       mGrid->SetCellValue(i, 1, mLocal.GetTag(labelmap[i].name));
 
-      if (!mEditTitle && mGrid->GetCellValue(i, 0).CmpNoCase(LABEL_TITLE) == 0) {
+      if (!mEditTitle &&
+          mGrid->GetCellValue(i, 0).CmpNoCase(wxGetTranslation(LABEL_TITLE)) == 0) {
          mGrid->SetReadOnly(i, 1);
       }
 
-      if (!mEditTrack && mGrid->GetCellValue(i, 0).CmpNoCase(LABEL_TRACK) == 0) {
+      if (!mEditTrack &&
+          mGrid->GetCellValue(i, 0).CmpNoCase(wxGetTranslation(LABEL_TRACK)) == 0) {
          mGrid->SetReadOnly(i, 1);
       }
 
-      mLocal.SetTag(labelmap[i].name, wxEmptyString);
+      popTagMap[ labelmap[i].name ] = mGrid->GetCellValue(i, 1);
    }
 
    // Populate the rest
-   for (bool cont = mLocal.GetFirst(n, v); cont; cont = mLocal.GetNext(n, v)) {
-      mGrid->AppendRows();
-
-      mGrid->SetCellValue(i, 0, n);
-      mGrid->SetCellValue(i, 1, v);
-      i++;
+   for (const auto &pair : mLocal.GetRange()) {
+      const auto &n = pair.first;
+      const auto &v = pair.second;
+      if (popTagMap.find(n) == popTagMap.end()) {
+         mGrid->AppendRows();
+         mGrid->SetCellValue(i, 0, n);
+         mGrid->SetCellValue(i, 1, v);
+         i++;
+      }
    }
 
    // Add an extra one to help with initial sizing and to show it can be done
@@ -921,6 +1096,8 @@ bool TagsEditor::TransferDataToWindow()
 
    // Set the editors
    SetEditors();
+   Layout();
+   Fit();
 
    return true;
 }
@@ -940,12 +1117,19 @@ void TagsEditor::OnChange(wxGridEvent & event)
       return;
    }
 
-   wxString n = mGrid->GetCellValue(event.GetRow(), 0);
-   for (size_t i = 0; i < STATICCNT; i++) {
-      if (n.CmpNoCase(labelmap[i].label) == 0) {
+   // Do not permit duplication of any of the tags.
+   // Tags differing only in case are nondistinct.
+   auto row = event.GetRow();
+   const wxString key0 = mGrid->GetCellValue(row, 0).Upper();
+   auto nn = mGrid->GetNumberRows();
+   for (decltype(nn) ii = 0; ii < nn; ++ii) {
+      if (ii == row)
+         continue;
+      auto key = mGrid->GetCellValue(ii, 0).Upper();
+      if (key0.CmpNoCase(key) == 0) {
          ischanging = true;
          wxBell();
-         mGrid->SetGridCursor(i, 0);
+         mGrid->SetGridCursor(ii, 0);
          event.Veto();
          ischanging = false;
          break;
@@ -962,7 +1146,7 @@ void TagsEditor::OnEdit(wxCommandEvent & WXUNUSED(event))
       mGrid->HideCellEditControl();
    }
 
-   wxDialog dlg(this, wxID_ANY, _("Edit Genres"),
+   wxDialogWrapper dlg(this, wxID_ANY, _("Edit Genres"),
                 wxDefaultPosition, wxDefaultSize,
                 wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
    dlg.SetName(dlg.GetTitle());
@@ -978,11 +1162,12 @@ void TagsEditor::OnEdit(wxCommandEvent & WXUNUSED(event))
 
    S.AddStandardButtons();
 
-   wxSortedArrayString g;
+   wxArrayString g;
    int cnt = mLocal.GetNumUserGenres();
    for (int i = 0; i < cnt; i++) {
-      g.Add(mLocal.GetUserGenre(i));
+      g.push_back(mLocal.GetUserGenre(i));
    }
+   std::sort( g.begin(), g.end() );
 
    for (int i = 0; i < cnt; i++) {
       tc->AppendText(g[i] + wxT("\n"));
@@ -996,7 +1181,7 @@ void TagsEditor::OnEdit(wxCommandEvent & WXUNUSED(event))
    wxFileName fn(FileNames::DataDir(), wxT("genres.txt"));
    wxFile f(fn.GetFullPath(), wxFile::write);
    if (!f.IsOpened() || !f.Write(tc->GetValue())) {
-      wxMessageBox(_("Unable to save genre file."), _("Reset Genres"));
+      AudacityMessageBox(_("Unable to save genre file."), _("Reset Genres"));
       return;
    }
 
@@ -1007,7 +1192,7 @@ void TagsEditor::OnEdit(wxCommandEvent & WXUNUSED(event))
 
 void TagsEditor::OnReset(wxCommandEvent & WXUNUSED(event))
 {
-   int id = wxMessageBox(_("Are you sure you want to reset the genre list to defaults?"),
+   int id = AudacityMessageBox(_("Are you sure you want to reset the genre list to defaults?"),
                          _("Reset Genres"),
                          wxYES_NO);
 
@@ -1023,7 +1208,7 @@ void TagsEditor::OnReset(wxCommandEvent & WXUNUSED(event))
                (!tf.Exists() && tf.Create());
 
    if (!open) {
-      wxMessageBox(_("Unable to open genre file."), _("Reset Genres"));
+      AudacityMessageBox(_("Unable to open genre file."), _("Reset Genres"));
       mLocal.LoadGenres();
       return;
    }
@@ -1035,7 +1220,7 @@ void TagsEditor::OnReset(wxCommandEvent & WXUNUSED(event))
    }
 
    if (!tf.Write()) {
-      wxMessageBox(_("Unable to save genre file."), _("Reset Genres"));
+      AudacityMessageBox(_("Unable to save genre file."), _("Reset Genres"));
       mLocal.LoadGenres();
       return;
    }
@@ -1057,7 +1242,8 @@ void TagsEditor::OnLoad(wxCommandEvent & WXUNUSED(event))
    wxString fn;
 
    // Ask the user for the real name
-   fn = FileSelector(_("Load Metadata As:"),
+   fn = FileNames::SelectFile(FileNames::Operation::_None,
+                     _("Load Metadata As:"),
                      FileNames::DataDir(),
                      wxT("Tags.xml"),
                      wxT("xml"),
@@ -1066,7 +1252,7 @@ void TagsEditor::OnLoad(wxCommandEvent & WXUNUSED(event))
                      this);
 
    // User canceled...
-   if (fn.IsEmpty()) {
+   if (fn.empty()) {
       return;
    }
 
@@ -1081,7 +1267,7 @@ void TagsEditor::OnLoad(wxCommandEvent & WXUNUSED(event))
    XMLFileReader reader;
    if (!reader.Parse(&mLocal, fn)) {
       // Inform user of load failure
-      wxMessageBox(reader.GetErrorStr(),
+      AudacityMessageBox(reader.GetErrorStr(),
                    _("Error Loading Metadata"),
                    wxOK | wxCENTRE,
                    this);
@@ -1111,7 +1297,8 @@ void TagsEditor::OnSave(wxCommandEvent & WXUNUSED(event))
    TransferDataFromWindow();
 
    // Ask the user for the real name
-   fn = FileSelector(_("Save Metadata As:"),
+   fn = FileNames::SelectFile(FileNames::Operation::_None,
+                     _("Save Metadata As:"),
                      FileNames::DataDir(),
                      wxT("Tags.xml"),
                      wxT("xml"),
@@ -1120,16 +1307,13 @@ void TagsEditor::OnSave(wxCommandEvent & WXUNUSED(event))
                      this);
 
    // User canceled...
-   if (fn.IsEmpty()) {
+   if (fn.empty()) {
       return;
    }
 
-   // Create/Open the file
-   XMLFileWriter writer;
-
-   try
-   {
-      writer.Open(fn, wxT("wb"));
+   GuardedCall( [&] {
+      // Create/Open the file
+      XMLFileWriter writer{ fn, _("Error Saving Tags File") };
 
       // Remember title and track in case they're read only
       wxString title = mLocal.GetTag(TAG_TITLE);
@@ -1145,31 +1329,23 @@ void TagsEditor::OnSave(wxCommandEvent & WXUNUSED(event))
          mLocal.SetTag(TAG_TRACK, wxEmptyString);
       }
 
+      auto cleanup = finally( [&] {
+         // Restore title
+         if (!mEditTitle) {
+            mLocal.SetTag(TAG_TITLE, title);
+         }
+
+         // Restore track
+         if (!mEditTrack) {
+            mLocal.SetTag(TAG_TRACK, track);
+         }
+      } );
+
       // Write the metadata
       mLocal.WriteXML(writer);
 
-      // Restore title
-      if (!mEditTitle) {
-         mLocal.SetTag(TAG_TITLE, title);
-      }
-
-      // Restore track
-      if (!mEditTrack) {
-         mLocal.SetTag(TAG_TRACK, track);
-      }
-
-      // Close the file
-      writer.Close();
-   }
-   catch (XMLFileWriterException* pException)
-   {
-      wxMessageBox(wxString::Format(
-         _("Couldn't write to file \"%s\": %s"),
-         fn.c_str(), pException->GetMessage().c_str()),
-         _("Error Saving Tags File"), wxICON_ERROR, this);
-
-      delete pException;
-   }
+      writer.Commit();
+   } );
 }
 
 void TagsEditor::OnSaveDefaults(wxCommandEvent & WXUNUSED(event))
@@ -1195,8 +1371,9 @@ void TagsEditor::OnSaveDefaults(wxCommandEvent & WXUNUSED(event))
    gPrefs->DeleteGroup(wxT("/Tags"));
 
    // Write out each tag
-   wxString n, v;
-   for (bool cont = mLocal.GetFirst(n, v); cont; cont = mLocal.GetNext(n, v)) {
+   for (const auto &pair : mLocal.GetRange()) {
+      const auto &n = pair.first;
+      const auto &v = pair.second;
       gPrefs->Write(wxT("/Tags/") + n, v);
    }
    gPrefs->Flush();
@@ -1221,10 +1398,13 @@ void TagsEditor::OnRemove(wxCommandEvent & WXUNUSED(event))
 {
    size_t row = mGrid->GetGridCursorRow();
 
-   if (!mEditTitle && mGrid->GetCellValue(row, 0).CmpNoCase(LABEL_TITLE) == 0) {
+   if (!mEditTitle &&
+       mGrid->GetCellValue(row, 0).CmpNoCase(wxGetTranslation(LABEL_TITLE)) == 0) {
       return;
    }
-   else if (!mEditTrack && mGrid->GetCellValue(row, 0).CmpNoCase(LABEL_TRACK) == 0) {
+   else if (!mEditTrack &&
+            mGrid->GetCellValue(row, 0)
+               .CmpNoCase(wxGetTranslation(LABEL_TRACK)) == 0) {
       return;
    }
    else if (row < STATICCNT) {
@@ -1240,7 +1420,6 @@ void TagsEditor::OnOk(wxCommandEvent & WXUNUSED(event))
    if (mGrid->IsCellEditControlShown()) {
       mGrid->SaveEditControlValue();
       mGrid->HideCellEditControl();
-      return;
    }
 
    if (!Validate() || !TransferDataFromWindow()) {
@@ -1260,25 +1439,47 @@ void TagsEditor::OnOk(wxCommandEvent & WXUNUSED(event))
 }
 
 void TagsEditor::OnCancel(wxCommandEvent & WXUNUSED(event))
+{
+   DoCancel(false);
+}
+
+void TagsEditor::DoCancel(bool escKey)
 {
    if (mGrid->IsCellEditControlShown()) {
-      mGrid->GetCellEditor(mGrid->GetGridCursorRow(),
-                           mGrid->GetGridCursorCol())
-                           ->Reset();
+      auto editor = mGrid->GetCellEditor(mGrid->GetGridCursorRow(),
+         mGrid->GetGridCursorCol());
+      editor->Reset();
+      // To avoid memory leak, don't forget DecRef()!
+      editor->DecRef();
       mGrid->HideCellEditControl();
+#if defined(__WXMSW__)
       return;
+#endif
    }
 
+   auto focus = wxWindow::FindFocus();
+   if (escKey && focus == mGrid)
+      return;
+
    EndModal(wxID_CANCEL);
 }
 
+void TagsEditor::OnKeyDown(wxKeyEvent &event)
+{
+   if (event.GetKeyCode() == WXK_ESCAPE)
+      DoCancel(true);
+   else
+      event.Skip();
+}
+
 void TagsEditor::SetEditors()
 {
    int cnt = mGrid->GetNumberRows();
 
    for (int i = 0; i < cnt; i++) {
       wxString label = mGrid->GetCellValue(i, 0);
-      if (label.CmpNoCase(LABEL_GENRE) == 0) {
+      if (label.CmpNoCase(wxGetTranslation(LABEL_GENRE)) == 0) {
+         // This use of GetDefaultEditorForType does not require DecRef.
          mGrid->SetCellEditor(i, 1, mGrid->GetDefaultEditorForType(wxT("Combo")));
       }
       else {
@@ -1292,17 +1493,22 @@ void TagsEditor::PopulateGenres()
    int cnt = mLocal.GetNumUserGenres();
    int i;
    wxString parm;
-   wxSortedArrayString g;
+   wxArrayString g;
 
    for (i = 0; i < cnt; i++) {
-      g.Add(mLocal.GetUserGenre(i));
+      g.push_back(mLocal.GetUserGenre(i));
    }
+   std::sort( g.begin(), g.end() );
 
    for (i = 0; i < cnt; i++) {
       parm = parm + (i == 0 ? wxT("") : wxT(",")) + g[i];
    }
 
-   mGrid->GetDefaultEditorForType(wxT("Combo"))->SetParameters(parm);
+   // Here was a memory leak!  wxWidgets docs for wxGrid::GetDefaultEditorForType() say:
+   // "The caller must call DecRef() on the returned pointer."
+   auto editor = mGrid->GetDefaultEditorForType(wxT("Combo"));
+   editor->SetParameters(parm);
+   editor->DecRef();
 }
 
 bool TagsEditor::IsWindowRectValid(const wxRect *windowRect) const
