@@ -11,31 +11,30 @@
 #ifndef __AUDACITY_LEGACYALIASBLOCKFILE__
 #define __AUDACITY_LEGACYALIASBLOCKFILE__
 
-#include "../BlockFile.h"
 #include "PCMAliasBlockFile.h"
 
 /// An AliasBlockFile that references uncompressed data in an existing file
-class LegacyAliasBlockFile : public PCMAliasBlockFile
+class LegacyAliasBlockFile final : public PCMAliasBlockFile
 {
  public:
 
    // Constructor / Destructor
 
    /// Constructs a LegacyAliasBlockFile, writing the summary to disk
-   LegacyAliasBlockFile(wxFileName fileName,
-                        wxFileName aliasedFileName,
+   LegacyAliasBlockFile(wxFileNameWrapper &&fileName,
+                        wxFileNameWrapper &&aliasedFileName,
                         sampleCount aliasStart,
-                        sampleCount aliasLen,
+                        size_t aliasLen,
                         int aliasChannel,
-                        sampleCount summaryLen,
+                        size_t summaryLen,
                         bool noRMS);
    virtual ~LegacyAliasBlockFile();
 
-   virtual void SaveXML(XMLWriter &xmlFile);
-   virtual BlockFile *Copy(wxFileName fileName);
-   virtual void Recover();
+   void SaveXML(XMLWriter &xmlFile) override;
+   BlockFilePtr Copy(wxFileNameWrapper &&fileName) override;
+   void Recover() override;
 
-   static BlockFile *BuildFromXML(wxString projDir, const wxChar **attrs);
+   static BlockFilePtr BuildFromXML(const FilePath &projDir, const wxChar **attrs);
 };
 
 #endif
