@@ -16,7 +16,10 @@
 #ifndef __COMMANDBUILDER__
 #define __COMMANDBUILDER__
 
-class Command;
+#include "../MemoryX.h"
+
+class OldStyleCommand;
+using OldStyleCommandPointer = std::shared_ptr<OldStyleCommand>;
 class wxString;
 
 // CommandBuilder has the task of validating and interpreting a command string.
@@ -26,21 +29,20 @@ class CommandBuilder
 {
    private:
       bool mValid;
-      Command *mCommand;
+      OldStyleCommandPointer mCommand;
       wxString mError;
 
-      void Failure(const wxString &msg = wxEmptyString);
-      void Success(Command *cmd);
-      void BuildCommand(const wxString &cmdName, wxString cmdParams);
-      void BuildCommand(wxString cmdString);
+      void Failure(const wxString &msg = {});
+      void Success(const OldStyleCommandPointer &cmd);
+      void BuildCommand(const wxString &cmdName, const wxString &cmdParams);
+      void BuildCommand(const wxString &cmdString);
    public:
       CommandBuilder(const wxString &cmdString);
       CommandBuilder(const wxString &cmdName,
                      const wxString &cmdParams);
       ~CommandBuilder();
       bool WasValid();
-      Command *GetCommand();
-      void Cleanup();
+      OldStyleCommandPointer GetCommand();
       const wxString &GetErrorMessage();
 };
 #endif /* End of include guard: __COMMANDBUILDER__ */
