@@ -16,18 +16,19 @@
 #ifndef __COMMANDHANDLER__
 #define __COMMANDHANDLER__
 
-#include "../AudacityApp.h"
+#include "../MemoryX.h"
+class AudacityApp;
 class AudacityProject;
 class AppCommandEvent;
-class CommandExecutionContext;
+class CommandContext;
 
 class CommandHandler
 {
    private:
-      CommandExecutionContext *mCurrentContext;
+      std::unique_ptr<const CommandContext> mCurrentContext;
 
    public:
-      CommandHandler(AudacityApp &app);
+      CommandHandler();
       ~CommandHandler();
 
       // This should only be used during initialization
