@@ -11,6 +11,9 @@
 \class EffectAutoDuck
 \brief Implements the Auto Ducking effect
 
+\class AutoDuckRegion
+\brief a struct that holds a start and end time.
+
 *******************************************************************/
 
 #include "../Audacity.h"
@@ -21,37 +24,35 @@
 
 #include <wx/dcclient.h>
 #include <wx/dcmemory.h>
-#include <wx/dynarray.h>
 #include <wx/intl.h>
-#include <wx/msgdlg.h>
 
 #include "../AColor.h"
 #include "../AllThemeResources.h"
-#include "../Internat.h"
 #include "../Prefs.h"
+#include "../Shuttle.h"
 #include "../ShuttleGui.h"
-#include "../Theme.h"
 #include "../widgets/valnum.h"
 
 #include "../WaveTrack.h"
+#include "../widgets/AudacityMessageBox.h"
 
 // Define keys, defaults, minimums, and maximums for the effect parameters
 //
 //     Name                Type     Key                     Def      Min      Max      Scale
-Param( DuckAmountDb,       double,  XO("DuckAmountDb"),     -12.0,   -24.0,   0.0,     1  );
-Param( InnerFadeDownLen,   double,  XO("InnerFadeDownLen"), 0.0,     0.0,     3.0,     1  );
-Param( InnerFadeUpLen,     double,  XO("InnerFadeUpLen"),   0.0,     0.0,     3.0,     1  );
-Param( OuterFadeDownLen,   double,  XO("OuterFadeDownLen"), 0.5,     0.0,     3.0,     1  );
-Param( OuterFadeUpLen,     double,  XO("OuterFadeUpLen"),   0.5,     0.0,     3.0,     1  );
-Param( ThresholdDb,        double,  XO("ThresholdDb"),      -30.0,   -100.0,  0.0,     1  );
-Param( MaximumPause,       double,  XO("MaximumPause"),     1.0,     0.0,     DBL_MAX, 1  );
+Param( DuckAmountDb,       double,  wxT("DuckAmountDb"),     -12.0,   -24.0,   0.0,     1  );
+Param( InnerFadeDownLen,   double,  wxT("InnerFadeDownLen"), 0.0,     0.0,     3.0,     1  );
+Param( InnerFadeUpLen,     double,  wxT("InnerFadeUpLen"),   0.0,     0.0,     3.0,     1  );
+Param( OuterFadeDownLen,   double,  wxT("OuterFadeDownLen"), 0.5,     0.0,     3.0,     1  );
+Param( OuterFadeUpLen,     double,  wxT("OuterFadeUpLen"),   0.5,     0.0,     3.0,     1  );
+Param( ThresholdDb,        double,  wxT("ThresholdDb"),      -30.0,   -100.0,  0.0,     1  );
+Param( MaximumPause,       double,  wxT("MaximumPause"),     1.0,     0.0,     DBL_MAX, 1  );
 
 /*
  * Common constants
  */
 
-static const int kBufSize = 131072;     // number of samples to process at once
-static const int kRMSWindowSize = 100;  // samples in circular RMS window buffer
+static const size_t kBufSize = 131072u;     // number of samples to process at once
+static const size_t kRMSWindowSize = 100u;  // samples in circular RMS window buffer
 
 /*
  * A auto duck region and an array of auto duck regions
@@ -69,11 +70,6 @@ struct AutoDuckRegion
    double t1;
 };
 
-#include <wx/arrimpl.cpp>
-
-WX_DECLARE_OBJARRAY(AutoDuckRegion, AutoDuckRegionArray);
-WX_DEFINE_OBJARRAY(AutoDuckRegionArray);
-
 /*
  * Effect implementation
  */
@@ -103,19 +99,24 @@ EffectAutoDuck::~EffectAutoDuck()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectAutoDuck::GetSymbol()
+ComponentInterfaceSymbol EffectAutoDuck::GetSymbol()
 {
    return AUTODUCK_PLUGIN_SYMBOL;
 }
 
 wxString EffectAutoDuck::GetDescription()
 {
-   return XO("Reduces (ducks) the volume of one or more tracks whenever the volume of a specified \"control\" track reaches a particular level");
+   return _("Reduces (ducks) the volume of one or more tracks whenever the volume of a specified \"control\" track reaches a particular level");
 }
 
-// EffectIdentInterface implementation
+wxString EffectAutoDuck::ManualPage()
+{
+   return wxT("Auto_Duck");
+}
+
+// EffectDefinitionInterface implementation
 
 EffectType EffectAutoDuck::GetType()
 {
@@ -123,8 +124,18 @@ EffectType EffectAutoDuck::GetType()
 }
 
 // EffectClientInterface implementation
+bool EffectAutoDuck::DefineParams( ShuttleParams & S ){
+   S.SHUTTLE_PARAM(  mDuckAmountDb, DuckAmountDb);
+   S.SHUTTLE_PARAM(  mInnerFadeDownLen, InnerFadeDownLen);
+   S.SHUTTLE_PARAM(  mInnerFadeUpLen, InnerFadeUpLen);
+   S.SHUTTLE_PARAM(  mOuterFadeDownLen, OuterFadeDownLen);
+   S.SHUTTLE_PARAM(  mOuterFadeUpLen, OuterFadeUpLen);
+   S.SHUTTLE_PARAM(  mThresholdDb, ThresholdDb);
+   S.SHUTTLE_PARAM(  mMaximumPause, MaximumPause);
+   return true;
+}
 
-bool EffectAutoDuck::GetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectAutoDuck::GetAutomationParameters(CommandParameters & parms)
 {
    parms.Write(KEY_DuckAmountDb, mDuckAmountDb);
    parms.Write(KEY_InnerFadeDownLen, mInnerFadeDownLen);
@@ -137,7 +148,7 @@ bool EffectAutoDuck::GetAutomationParameters(EffectAutomationParameters & parms)
    return true;
 }
 
-bool EffectAutoDuck::SetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectAutoDuck::SetAutomationParameters(CommandParameters & parms)
 {
    ReadAndVerifyDouble(DuckAmountDb);
    ReadAndVerifyDouble(InnerFadeDownLen);
@@ -197,48 +208,44 @@ bool EffectAutoDuck::Init()
 {
    mControlTrack = NULL;
 
-   TrackListIterator iter(mTracks);
-   Track *t = iter.First();
-
    bool lastWasSelectedWaveTrack = false;
-   WaveTrack *controlTrackCandidate = NULL;
+   const WaveTrack *controlTrackCandidate = NULL;
 
-   while(t)
+   for (auto t : inputTracks()->Any())
    {
-      if (lastWasSelectedWaveTrack && !t->GetSelected() &&
-          t->GetKind() == Track::Wave)
-      {
+      if (lastWasSelectedWaveTrack && !t->GetSelected()) {
          // This could be the control track, so remember it
-         controlTrackCandidate = (WaveTrack*)t;
+         controlTrackCandidate = track_cast<const WaveTrack *>(t);
       }
 
       lastWasSelectedWaveTrack = false;
 
-      if (t->GetSelected())
-      {
-         if (t->GetKind() == Track::Wave)
-         {
-            lastWasSelectedWaveTrack = true;
-         } else
-         {
-            wxMessageBox(
-               _("You selected a track which does not contain audio. AutoDuck can only process audio tracks."),
-               /* i18n-hint: Auto duck is the name of an effect that 'ducks' (reduces the volume)
-                * of the audio automatically when there is sound on another track.  Not as
-                * in 'Donald-Duck'!*/
-               _("Auto Duck"), wxICON_ERROR, mUIParent);
+      if (t->GetSelected()) {
+         bool ok = t->TypeSwitch<bool>(
+            [&](const WaveTrack *) {
+               lastWasSelectedWaveTrack = true;
+               return true;
+            },
+            [&](const Track *) {
+               Effect::MessageBox(
+                  _("You selected a track which does not contain audio. AutoDuck can only process audio tracks."),
+                  /* i18n-hint: Auto duck is the name of an effect that 'ducks' (reduces the volume)
+                   * of the audio automatically when there is sound on another track.  Not as
+                   * in 'Donald-Duck'!*/
+                  wxICON_ERROR);
+               return false;
+            }
+         );
+         if (!ok)
             return false;
-         }
       }
-
-      t = iter.Next();
    }
 
    if (!controlTrackCandidate)
    {
-      wxMessageBox(
+      Effect::MessageBox(
          _("Auto Duck needs a control track which must be placed below the selected track(s)."),
-         _("Auto Duck"), wxICON_ERROR, mUIParent);
+         wxICON_ERROR);
       return false;
    }
 
@@ -254,16 +261,14 @@ void EffectAutoDuck::End()
 
 bool EffectAutoDuck::Process()
 {
-   sampleCount i;
-
    if (GetNumWaveTracks() == 0 || !mControlTrack)
       return false;
 
    bool cancel = false;
 
-   sampleCount start =
+   auto start =
       mControlTrack->TimeToLongSamples(mT0 + mOuterFadeDownLen);
-   sampleCount end =
+   auto end =
       mControlTrack->TimeToLongSamples(mT1 - mOuterFadeUpLen);
 
    if (end <= start)
@@ -277,7 +282,7 @@ bool EffectAutoDuck::Process()
    if (maxPause < mOuterFadeDownLen + mOuterFadeUpLen)
       maxPause = mOuterFadeDownLen + mOuterFadeUpLen;
 
-   sampleCount minSamplesPause =
+   auto minSamplesPause =
       mControlTrack->TimeToLongSamples(maxPause);
 
    double threshold = DB_TO_LINEAR(mThresholdDb);
@@ -287,118 +292,111 @@ bool EffectAutoDuck::Process()
 
    int rmsPos = 0;
    float rmsSum = 0;
-   float *rmsWindow = new float[kRMSWindowSize];
-   for (i = 0; i < kRMSWindowSize; i++)
-      rmsWindow[i] = 0;
-
-   float *buf = new float[kBufSize];
-
-   bool inDuckRegion = false;
-
-   // initialize the following two variables to prevent compiler warning
-   double duckRegionStart = 0;
-   sampleCount curSamplesPause = 0;
-
    // to make the progress bar appear more natural, we first look for all
    // duck regions and apply them all at once afterwards
-   AutoDuckRegionArray regions;
-   sampleCount pos = start;
-
-   while (pos < end)
+   std::vector<AutoDuckRegion> regions;
+   bool inDuckRegion = false;
    {
-      sampleCount len = end - pos;
-      if (len > kBufSize)
-         len = kBufSize;
+      Floats rmsWindow{ kRMSWindowSize, true };
 
-      mControlTrack->Get((samplePtr)buf, floatSample, pos, (sampleCount)len);
+      Floats buf{ kBufSize };
 
-      for (i = pos; i < pos + len; i++)
+      // initialize the following two variables to prevent compiler warning
+      double duckRegionStart = 0;
+      sampleCount curSamplesPause = 0;
+
+      auto pos = start;
+
+      while (pos < end)
       {
-         rmsSum -= rmsWindow[rmsPos];
-         rmsWindow[rmsPos] = buf[i - pos] * buf[i - pos];
-         rmsSum += rmsWindow[rmsPos];
-         rmsPos = (rmsPos + 1) % kRMSWindowSize;
+         const auto len = limitSampleBufferSize( kBufSize, end - pos );
+         
+         mControlTrack->Get((samplePtr)buf.get(), floatSample, pos, len);
 
-         bool thresholdExceeded = rmsSum > threshold;
-
-         if (thresholdExceeded)
+         for (auto i = pos; i < pos + len; i++)
          {
-            // everytime the threshold is exceeded, reset our count for
-            // the number of pause samples
-            curSamplesPause = 0;
+            rmsSum -= rmsWindow[rmsPos];
+            // i - pos is bounded by len:
+            auto index = ( i - pos ).as_size_t();
+            rmsWindow[rmsPos] = buf[ index ] * buf[ index ];
+            rmsSum += rmsWindow[rmsPos];
+            rmsPos = (rmsPos + 1) % kRMSWindowSize;
 
-            if (!inDuckRegion)
+            bool thresholdExceeded = rmsSum > threshold;
+
+            if (thresholdExceeded)
             {
-               // the threshold has been exceeded for the first time, so
-               // let the duck region begin here
-               inDuckRegion = true;
-               duckRegionStart = mControlTrack->LongSamplesToTime(i);
+               // everytime the threshold is exceeded, reset our count for
+               // the number of pause samples
+               curSamplesPause = 0;
+
+               if (!inDuckRegion)
+               {
+                  // the threshold has been exceeded for the first time, so
+                  // let the duck region begin here
+                  inDuckRegion = true;
+                  duckRegionStart = mControlTrack->LongSamplesToTime(i);
+               }
             }
-         }
 
-         if (!thresholdExceeded && inDuckRegion)
-         {
-            // the threshold has not been exceeded and we are in a duck
-            // region, but only fade in if the maximum pause has been
-            // exceeded
-            curSamplesPause += 1;
-
-            if (curSamplesPause >= minSamplesPause)
+            if (!thresholdExceeded && inDuckRegion)
             {
-               // do the actual duck fade and reset all values
-               double duckRegionEnd =
-                  mControlTrack->LongSamplesToTime(i - curSamplesPause);
+               // the threshold has not been exceeded and we are in a duck
+               // region, but only fade in if the maximum pause has been
+               // exceeded
+               curSamplesPause += 1;
 
-               regions.Add(AutoDuckRegion(
-                              duckRegionStart - mOuterFadeDownLen,
-                              duckRegionEnd + mOuterFadeUpLen));
+               if (curSamplesPause >= minSamplesPause)
+               {
+                  // do the actual duck fade and reset all values
+                  double duckRegionEnd =
+                     mControlTrack->LongSamplesToTime(i - curSamplesPause);
 
-               inDuckRegion = false;
-            }
-         }
-      }
-
-      pos += len;
-
-      if (TotalProgress( ((double)(pos-start)) / (end-start) /
-                         (GetNumWaveTracks() + 1) ))
-      {
-         cancel = true;
-         break;
-      }
-   }
-
-   // apply last duck fade, if any
-   if (inDuckRegion)
-   {
-      double duckRegionEnd =
-         mControlTrack->LongSamplesToTime(end - curSamplesPause);
-      regions.Add(AutoDuckRegion(
+                  regions.push_back(AutoDuckRegion(
                      duckRegionStart - mOuterFadeDownLen,
                      duckRegionEnd + mOuterFadeUpLen));
-   }
 
-   delete[] buf;
-   delete[] rmsWindow;
+                  inDuckRegion = false;
+               }
+            }
+         }
+
+         pos += len;
+
+         if (TotalProgress(
+            (pos - start).as_double() /
+            (end - start).as_double() /
+            (GetNumWaveTracks() + 1)
+         ))
+         {
+            cancel = true;
+            break;
+         }
+      }
+
+      // apply last duck fade, if any
+      if (inDuckRegion)
+      {
+         double duckRegionEnd =
+            mControlTrack->LongSamplesToTime(end - curSamplesPause);
+         regions.push_back(AutoDuckRegion(
+            duckRegionStart - mOuterFadeDownLen,
+            duckRegionEnd + mOuterFadeUpLen));
+      }
+   }
 
    if (!cancel)
    {
       CopyInputTracks(); // Set up mOutputTracks.
-      SelectedTrackListOfKindIterator iter(Track::Wave, mOutputTracks);
-      Track *iterTrack = iter.First();
 
-      int trackNumber = 0;
+      int trackNum = 0;
 
-      while (iterTrack)
+      for( auto iterTrack : mOutputTracks->Selected< WaveTrack >() )
       {
-         wxASSERT(iterTrack->GetKind() == Track::Wave);
-
-         WaveTrack* t = (WaveTrack*)iterTrack;
-
-         for (i = 0; i < (int)regions.GetCount(); i++)
+         for (size_t i = 0; i < regions.size(); i++)
          {
             const AutoDuckRegion& region = regions[i];
-            if (ApplyDuckFade(trackNumber, t, region.t0, region.t1))
+            if (ApplyDuckFade(trackNum, iterTrack, region.t0, region.t1))
             {
                cancel = true;
                break;
@@ -408,8 +406,7 @@ bool EffectAutoDuck::Process()
          if (cancel)
             break;
 
-         iterTrack = iter.Next();
-         trackNumber++;
+         trackNum++;
       }
    }
 
@@ -424,44 +421,44 @@ void EffectAutoDuck::PopulateOrExchange(ShuttleGui & S)
    {
       S.AddSpace(0, 5);
 
-      mPanel = new EffectAutoDuckPanel(S.GetParent(), this);
+      mPanel = safenew EffectAutoDuckPanel(S.GetParent(), wxID_ANY, this);
       S.AddWindow(mPanel);
 
       S.AddSpace(0, 5);
 
       S.StartMultiColumn(6, wxCENTER);
       {
-         FloatingPointValidator<double> vldDuckAmountDb(1, &mDuckAmountDb, NUM_VAL_NO_TRAILING_ZEROES);
+         FloatingPointValidator<double> vldDuckAmountDb(1, &mDuckAmountDb, NumValidatorStyle::NO_TRAILING_ZEROES);
          vldDuckAmountDb.SetRange(MIN_DuckAmountDb, MAX_DuckAmountDb);
          mDuckAmountDbBox = S.AddTextBox(_("Duck amount:"), wxT(""), 10);
          mDuckAmountDbBox->SetValidator(vldDuckAmountDb);
          S.AddUnits(_("dB"));
 
-         FloatingPointValidator<double> vldMaximumPause(2, &mMaximumPause, NUM_VAL_NO_TRAILING_ZEROES);
+         FloatingPointValidator<double> vldMaximumPause(2, &mMaximumPause, NumValidatorStyle::NO_TRAILING_ZEROES);
          vldMaximumPause.SetRange(MIN_MaximumPause, MAX_MaximumPause);
          mMaximumPauseBox = S.AddTextBox(_("Maximum pause:"), wxT(""), 10);
          mMaximumPauseBox->SetValidator(vldMaximumPause);
          S.AddUnits(_("seconds"));
 
-         FloatingPointValidator<double> vldOuterFadeDownLen(2, &mOuterFadeDownLen, NUM_VAL_NO_TRAILING_ZEROES);
+         FloatingPointValidator<double> vldOuterFadeDownLen(2, &mOuterFadeDownLen, NumValidatorStyle::NO_TRAILING_ZEROES);
          vldOuterFadeDownLen.SetRange(MIN_OuterFadeDownLen, MAX_OuterFadeDownLen);
          mOuterFadeDownLenBox = S.AddTextBox(_("Outer fade down length:"), wxT(""), 10);
          mOuterFadeDownLenBox->SetValidator(vldOuterFadeDownLen);
          S.AddUnits(_("seconds"));
 
-         FloatingPointValidator<double> vldOuterFadeUpLen(2, &mOuterFadeUpLen, NUM_VAL_NO_TRAILING_ZEROES);
+         FloatingPointValidator<double> vldOuterFadeUpLen(2, &mOuterFadeUpLen, NumValidatorStyle::NO_TRAILING_ZEROES);
          vldOuterFadeUpLen.SetRange(MIN_OuterFadeUpLen, MAX_OuterFadeUpLen);
          mOuterFadeUpLenBox = S.AddTextBox(_("Outer fade up length:"), wxT(""), 10);
          mOuterFadeUpLenBox->SetValidator(vldOuterFadeUpLen);
          S.AddUnits(_("seconds"));
 
-         FloatingPointValidator<double> vldInnerFadeDownLen(2, &mInnerFadeDownLen, NUM_VAL_NO_TRAILING_ZEROES);
+         FloatingPointValidator<double> vldInnerFadeDownLen(2, &mInnerFadeDownLen, NumValidatorStyle::NO_TRAILING_ZEROES);
          vldInnerFadeDownLen.SetRange(MIN_InnerFadeDownLen, MAX_InnerFadeDownLen);
          mInnerFadeDownLenBox = S.AddTextBox(_("Inner fade down length:"), wxT(""), 10);
          mInnerFadeDownLenBox->SetValidator(vldInnerFadeDownLen);
          S.AddUnits(_("seconds"));
 
-         FloatingPointValidator<double> vldInnerFadeUpLen(2, &mInnerFadeUpLen, NUM_VAL_NO_TRAILING_ZEROES);
+         FloatingPointValidator<double> vldInnerFadeUpLen(2, &mInnerFadeUpLen, NumValidatorStyle::NO_TRAILING_ZEROES);
          vldInnerFadeUpLen.SetRange(MIN_InnerFadeUpLen, MAX_InnerFadeUpLen);
          mInnerFadeUpLenBox = S.AddTextBox(_("Inner fade up length:"), wxT(""), 10);
          mInnerFadeUpLenBox->SetValidator(vldInnerFadeUpLen);
@@ -471,7 +468,7 @@ void EffectAutoDuck::PopulateOrExchange(ShuttleGui & S)
 
       S.StartMultiColumn(3, wxCENTER);
       {
-         FloatingPointValidator<double> vldThresholdDb(2, &mThresholdDb, NUM_VAL_NO_TRAILING_ZEROES);
+         FloatingPointValidator<double> vldThresholdDb(2, &mThresholdDb, NumValidatorStyle::NO_TRAILING_ZEROES);
          vldThresholdDb.SetRange(MIN_ThresholdDb, MAX_ThresholdDb);
          mThresholdDbBox = S.AddTextBox(_("Threshold:"), wxT(""), 10);
          mThresholdDbBox->SetValidator(vldThresholdDb);
@@ -510,42 +507,40 @@ bool EffectAutoDuck::TransferDataFromWindow()
 // EffectAutoDuck implementation
 
 // this currently does an exponential fade
-bool EffectAutoDuck::ApplyDuckFade(int trackNumber, WaveTrack* t,
+bool EffectAutoDuck::ApplyDuckFade(int trackNum, WaveTrack* t,
                                    double t0, double t1)
 {
    bool cancel = false;
 
-   sampleCount start = t->TimeToLongSamples(t0);
-   sampleCount end = t->TimeToLongSamples(t1);
+   auto start = t->TimeToLongSamples(t0);
+   auto end = t->TimeToLongSamples(t1);
 
-   float *buf = new float[kBufSize];
-   sampleCount pos = start;
+   Floats buf{ kBufSize };
+   auto pos = start;
 
-   int fadeDownSamples = t->TimeToLongSamples(
+   auto fadeDownSamples = t->TimeToLongSamples(
       mOuterFadeDownLen + mInnerFadeDownLen);
    if (fadeDownSamples < 1)
       fadeDownSamples = 1;
 
-   int fadeUpSamples = t->TimeToLongSamples(
+   auto fadeUpSamples = t->TimeToLongSamples(
       mOuterFadeUpLen + mInnerFadeUpLen);
    if (fadeUpSamples < 1)
       fadeUpSamples = 1;
 
-   float fadeDownStep = mDuckAmountDb / fadeDownSamples;
-   float fadeUpStep = mDuckAmountDb / fadeUpSamples;
+   float fadeDownStep = mDuckAmountDb / fadeDownSamples.as_double();
+   float fadeUpStep = mDuckAmountDb / fadeUpSamples.as_double();
 
    while (pos < end)
    {
-      sampleCount len = end - pos;
-      if (len > kBufSize)
-         len = kBufSize;
+      const auto len = limitSampleBufferSize( kBufSize, end - pos );
 
-      t->Get((samplePtr)buf, floatSample, pos, len);
+      t->Get((samplePtr)buf.get(), floatSample, pos, len);
 
-      for (sampleCount i = pos; i < pos + len; i++)
+      for (auto i = pos; i < pos + len; i++)
       {
-         float gainDown = fadeDownStep * (i - start);
-         float gainUp = fadeUpStep * (end - i);;
+         float gainDown = fadeDownStep * (i - start).as_float();
+         float gainUp = fadeUpStep * (end - i).as_float();
 
          float gain;
          if (gainDown > gainUp)
@@ -555,16 +550,17 @@ bool EffectAutoDuck::ApplyDuckFade(int trackNumber, WaveTrack* t,
          if (gain < mDuckAmountDb)
             gain = mDuckAmountDb;
 
-         buf[i - pos] *= DB_TO_LINEAR(gain);
+         // i - pos is bounded by len:
+         buf[ ( i - pos ).as_size_t() ] *= DB_TO_LINEAR(gain);
       }
 
-      t->Set((samplePtr)buf, floatSample, pos, len);
+      t->Set((samplePtr)buf.get(), floatSample, pos, len);
 
       pos += len;
 
       float curTime = t->LongSamplesToTime(pos);
       float fractionFinished = (curTime - mT0) / (mT1 - mT0);
-      if (TotalProgress( (trackNumber + 1 + fractionFinished) /
+      if (TotalProgress( (trackNum + 1 + fractionFinished) /
                          (GetNumWaveTracks() + 1) ))
       {
          cancel = true;
@@ -572,7 +568,6 @@ bool EffectAutoDuck::ApplyDuckFade(int trackNumber, WaveTrack* t,
       }
    }
 
-   delete[] buf;
    return cancel;
 }
 
@@ -607,7 +602,7 @@ static int GetDistance(const wxPoint& first, const wxPoint& second)
       return distanceY;
 }
 
-BEGIN_EVENT_TABLE(EffectAutoDuckPanel, wxPanel)
+BEGIN_EVENT_TABLE(EffectAutoDuckPanel, wxPanelWrapper)
    EVT_PAINT(EffectAutoDuckPanel::OnPaint)
    EVT_MOUSE_CAPTURE_CHANGED(EffectAutoDuckPanel::OnMouseCaptureChanged)
    EVT_MOUSE_CAPTURE_LOST(EffectAutoDuckPanel::OnMouseCaptureLost)
@@ -616,8 +611,9 @@ BEGIN_EVENT_TABLE(EffectAutoDuckPanel, wxPanel)
    EVT_MOTION(EffectAutoDuckPanel::OnMotion)
 END_EVENT_TABLE()
 
-EffectAutoDuckPanel::EffectAutoDuckPanel(wxWindow *parent, EffectAutoDuck *effect)
-:  wxPanel(parent, wxID_ANY, wxDefaultPosition, wxSize(600, 300))
+EffectAutoDuckPanel::EffectAutoDuckPanel(
+   wxWindow *parent, wxWindowID winid, EffectAutoDuck *effect)
+:  wxPanelWrapper(parent, winid, wxDefaultPosition, wxSize(600, 300))
 {
    mParent = parent;
    mEffect = effect;
@@ -629,8 +625,8 @@ EffectAutoDuckPanel::EffectAutoDuckPanel(wxWindow *parent, EffectAutoDuck *effec
 
 EffectAutoDuckPanel::~EffectAutoDuckPanel()
 {
-   if (mBackgroundBitmap)
-      delete mBackgroundBitmap;
+   if(HasCapture())
+      ReleaseMouse();
 }
 
 void EffectAutoDuckPanel::ResetControlPoints()
@@ -650,9 +646,7 @@ void EffectAutoDuckPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
    if (!mBackgroundBitmap || mBackgroundBitmap->GetWidth() != clientWidth ||
        mBackgroundBitmap->GetHeight() != clientHeight)
    {
-      if (mBackgroundBitmap)
-         delete mBackgroundBitmap;
-      mBackgroundBitmap = new wxBitmap(clientWidth, clientHeight);
+      mBackgroundBitmap = std::make_unique<wxBitmap>(clientWidth, clientHeight,24);
    }
 
    wxMemoryDC dc;
@@ -685,7 +679,7 @@ void EffectAutoDuckPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
        duckAmountDb < MIN_DuckAmountDb         || duckAmountDb > MAX_DuckAmountDb)
    {
       // values are out of range, no preview available
-      wxString message = wxString::Format(_("Preview not available"));
+      wxString message = _("Preview not available");
       int textWidth = 0, textHeight = 0;
       dc.GetTextExtent(message, &textWidth, &textHeight);
       dc.DrawText(message, (clientWidth - textWidth) / 2,
@@ -696,7 +690,7 @@ void EffectAutoDuckPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
    {
       // draw preview
       dc.SetBrush(*wxTRANSPARENT_BRUSH);
-      dc.SetPen(wxPen(theTheme.Colour(clrGraphLines), 3, wxSOLID));
+      dc.SetPen(wxPen(theTheme.Colour(clrGraphLines), 3, wxPENSTYLE_SOLID));
 
       wxPoint points[6];
 
@@ -720,9 +714,9 @@ void EffectAutoDuckPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
       points[5].x = clientWidth - 10;
       points[5].y = DUCK_AMOUNT_START;
 
-      dc.DrawLines(6, points);
+      AColor::Lines(dc, 6, points);
 
-      dc.SetPen(wxPen(*wxBLACK, 1, wxDOT));
+      dc.SetPen(wxPen(*wxBLACK, 1, wxPENSTYLE_DOT));
 
       AColor::Line(dc, FADE_DOWN_START, 10, FADE_DOWN_START, clientHeight - 10);
       AColor::Line(dc, FADE_UP_START, 10, FADE_UP_START, clientHeight - 10);
@@ -860,7 +854,8 @@ void EffectAutoDuckPanel::OnLeftDown(wxMouseEvent & evt)
       for (int i = 0; i < AUTO_DUCK_PANEL_NUM_CONTROL_POINTS; i++)
          mMoveStartControlPoints[i] = mControlPoints[i];
 
-      CaptureMouse();
+      if( !HasCapture() )
+         CaptureMouse();
    }
 }
 
