@@ -18,22 +18,44 @@ the pitch without changing the tempo.
 #include "../Audacity.h" // for USE_SOUNDTOUCH
 
 #if USE_SOUNDTOUCH
-
 #include "ChangePitch.h"
 
+#if USE_SBSMS
+#include <wx/valgen.h>
+#endif
+
 #include <float.h>
 #include <math.h>
 
+#include <wx/checkbox.h>
+#include <wx/choice.h>
 #include <wx/intl.h>
+#include <wx/slider.h>
+#include <wx/spinctrl.h>
 #include <wx/valtext.h>
 
 #include "../PitchName.h"
+#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../Spectrum.h"
 #include "../WaveTrack.h"
 #include "../widgets/valnum.h"
 #include "TimeWarper.h"
 
+// Soundtouch defines these as well, which are also in generated configmac.h
+// and configunix.h, so get rid of them before including,
+// to avoid compiler warnings, and be sure to do this
+// after all other #includes, to avoid any mischief that might result
+// from doing the un-definitions before seeing any wx headers.
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE
+#undef VERSION
+#include "SoundTouch.h"
+
 enum {
    ID_PercentChange = 10000,
    ID_FromPitch,
@@ -50,7 +72,8 @@ enum {
 // Define keys, defaults, minimums, and maximums for the effect parameters
 //
 //     Name          Type     Key               Def   Min      Max      Scale
-Param( Percentage,   double,  XO("Percentage"), 0.0,  -99.0,   3000.0,  1  );
+Param( Percentage,   double,  wxT("Percentage"), 0.0,  -99.0,   3000.0,  1  );
+Param( UseSBSMS,     bool,    wxT("SBSMS"),     false, false,   true,    1  );
 
 // We warp the slider to go up to 400%, but user can enter up to 3000%
 static const double kSliderMax = 100.0;          // warped above zero to actually go up to 400%
@@ -80,6 +103,12 @@ EffectChangePitch::EffectChangePitch()
    m_dStartFrequency = 0.0; // 0.0 => uninitialized
    m_bLoopDetect = false;
 
+#if USE_SBSMS
+   mUseSBSMS = DEF_UseSBSMS;
+#else
+   mUseSBSMS = false;
+#endif
+
    // NULL out these control members because there are some cases where the
    // event table handlers get called during this method, and those handlers that
    // can cause trouble check for NULL.
@@ -103,19 +132,24 @@ EffectChangePitch::~EffectChangePitch()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectChangePitch::GetSymbol()
+ComponentInterfaceSymbol EffectChangePitch::GetSymbol()
 {
    return CHANGEPITCH_PLUGIN_SYMBOL;
 }
 
 wxString EffectChangePitch::GetDescription()
 {
-   return XO("Change the pitch of a track without changing its tempo");
+   return _("Changes the pitch of a track without changing its tempo");
 }
 
-// EffectIdentInterface implementation
+wxString EffectChangePitch::ManualPage()
+{
+   return wxT("Change_Pitch");
+}
+
+// EffectDefinitionInterface implementation
 
 EffectType EffectChangePitch::GetType()
 {
@@ -123,21 +157,35 @@ EffectType EffectChangePitch::GetType()
 }
 
 // EffectClientInterface implementation
+bool EffectChangePitch::DefineParams( ShuttleParams & S ){
+   S.SHUTTLE_PARAM( m_dPercentChange, Percentage );
+   S.SHUTTLE_PARAM( mUseSBSMS, UseSBSMS );
+   return true;
+}
 
-bool EffectChangePitch::GetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectChangePitch::GetAutomationParameters(CommandParameters & parms)
 {
    parms.Write(KEY_Percentage, m_dPercentChange);
+   parms.Write(KEY_UseSBSMS, mUseSBSMS);
 
    return true;
 }
 
-bool EffectChangePitch::SetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectChangePitch::SetAutomationParameters(CommandParameters & parms)
 {
    // Vaughan, 2013-06: Long lost to history, I don't see why m_dPercentChange was chosen to be shuttled.
    // Only m_dSemitonesChange is used in Process().
    ReadAndVerifyDouble(Percentage);
 
    m_dPercentChange = Percentage;
+   Calc_SemitonesChange_fromPercentChange();
+
+#if USE_SBSMS
+   ReadAndVerifyBool(UseSBSMS);
+   mUseSBSMS = UseSBSMS;
+#else
+   mUseSBSMS = false;
+#endif
 
    return true;
 }
@@ -153,26 +201,47 @@ bool EffectChangePitch::LoadFactoryDefaults()
 
 bool EffectChangePitch::Init()
 {
-   mSoundTouch = NULL;
+   mSoundTouch.reset();
    return true;
 }
 
 bool EffectChangePitch::Process()
 {
-   mSoundTouch = new SoundTouch();
-   SetTimeWarper(new IdentityTimeWarper());
-   mSoundTouch->setPitchSemiTones((float)(m_dSemitonesChange));
-#ifdef USE_MIDI
-   // Note: m_dSemitonesChange is private to ChangePitch because it only
-   // needs to pass it along to mSoundTouch (above). I added mSemitones
-   // to SoundTouchEffect (the super class) to convey this value
-   // to process Note tracks. This approach minimizes changes to existing
-   // code, but it would be cleaner to change all m_dSemitonesChange to
-   // mSemitones, make mSemitones exist with or without USE_MIDI, and
-   // eliminate the next line:
-   mSemitones = m_dSemitonesChange;
+#if USE_SBSMS
+   if (mUseSBSMS)
+   {
+      double pitchRatio = 1.0 + m_dPercentChange / 100.0;
+      EffectSBSMS proxy;
+      proxy.mProxyEffectName = XO("High Quality Pitch Change");
+      proxy.setParameters(1.0, pitchRatio);
+
+      return Delegate(proxy, mUIParent, false);
+   }
+   else
 #endif
-   return EffectSoundTouch::Process();
+   {
+      // Macros save m_dPercentChange and not m_dSemitonesChange, so we must
+      // ensure that m_dSemitonesChange is set.
+      Calc_SemitonesChange_fromPercentChange();
+
+      mSoundTouch = std::make_unique<soundtouch::SoundTouch>();
+      IdentityTimeWarper warper;
+      mSoundTouch->setPitchSemiTones((float)(m_dSemitonesChange));
+#ifdef USE_MIDI
+      // Pitch shifting note tracks is currently only supported by SoundTouchEffect
+      // and non-real-time-preview effects require an audio track selection.
+      //
+      // Note: m_dSemitonesChange is private to ChangePitch because it only
+      // needs to pass it along to mSoundTouch (above). I added mSemitones
+      // to SoundTouchEffect (the super class) to convey this value
+      // to process Note tracks. This approach minimizes changes to existing
+      // code, but it would be cleaner to change all m_dSemitonesChange to
+      // mSemitones, make mSemitones exist with or without USE_MIDI, and
+      // eliminate the next line:
+      mSemitones = m_dSemitonesChange;
+#endif
+      return EffectSoundTouch::ProcessWithTimeWarper(warper);
+   }
 }
 
 bool EffectChangePitch::CheckWhetherSkipEffect()
@@ -184,19 +253,9 @@ void EffectChangePitch::PopulateOrExchange(ShuttleGui & S)
 {
    DeduceFrequencies(); // Set frequency-related control values based on sample.
 
-   wxArrayString pitch;
-   pitch.Add(wxT("C"));
-   pitch.Add(wxT("C#/Db"));
-   pitch.Add(wxT("D"));
-   pitch.Add(wxT("D#/Eb"));
-   pitch.Add(wxT("E"));
-   pitch.Add(wxT("F"));
-   pitch.Add(wxT("F#/Gb"));
-   pitch.Add(wxT("G"));
-   pitch.Add(wxT("G#/Ab"));
-   pitch.Add(wxT("A"));
-   pitch.Add(wxT("A#/Bb"));
-   pitch.Add(wxT("B"));
+   wxArrayStringEx pitch;
+   for (int ii = 0; ii < 12; ++ii)
+      pitch.push_back( PitchName( ii, PitchNameChoice::Both ) );
 
    S.SetBorder(5);
 
@@ -207,7 +266,7 @@ void EffectChangePitch::PopulateOrExchange(ShuttleGui & S)
          S.AddTitle(_("Change Pitch without Changing Tempo"));
          S.AddTitle(
             wxString::Format(_("Estimated Start Pitch: %s%d (%.3f Hz)"),
-                              pitch[m_nFromPitch].c_str(), m_nFromOctave, m_FromFrequency));
+                              pitch[m_nFromPitch], m_nFromOctave, m_FromFrequency));
       }
       S.EndVerticalLay();
 
@@ -216,20 +275,20 @@ void EffectChangePitch::PopulateOrExchange(ShuttleGui & S)
       {
          S.StartMultiColumn(6, wxALIGN_CENTER); // 6 controls, because each AddChoice adds a wxStaticText and a wxChoice.
          {
-            m_pChoice_FromPitch = S.Id(ID_FromPitch).AddChoice(_("from"), wxT(""), &pitch);
+            m_pChoice_FromPitch = S.Id(ID_FromPitch).AddChoice(_("from"), pitch);
             m_pChoice_FromPitch->SetName(_("from"));
             m_pChoice_FromPitch->SetSizeHints(80, -1);
 
-            m_pSpin_FromOctave = S.Id(ID_FromOctave).AddSpinCtrl(wxT(""), m_nFromOctave, INT_MAX, INT_MIN);
+            m_pSpin_FromOctave = S.Id(ID_FromOctave).AddSpinCtrl( {}, m_nFromOctave, INT_MAX, INT_MIN);
             m_pSpin_FromOctave->SetName(_("from Octave"));
             m_pSpin_FromOctave->SetSizeHints(50, -1);
 
-            m_pChoice_ToPitch = S.Id(ID_ToPitch).AddChoice(_("to"), wxT(""), &pitch);
+            m_pChoice_ToPitch = S.Id(ID_ToPitch).AddChoice(_("to"), pitch);
             m_pChoice_ToPitch->SetName(_("to"));
             m_pChoice_ToPitch->SetSizeHints(80, -1);
 
             m_pSpin_ToOctave =
-               S.Id(ID_ToOctave).AddSpinCtrl(wxT(""), m_nToOctave, INT_MAX, INT_MIN);
+               S.Id(ID_ToOctave).AddSpinCtrl( {}, m_nToOctave, INT_MAX, INT_MIN);
             m_pSpin_ToOctave->SetName(_("to Octave"));
             m_pSpin_ToOctave->SetSizeHints(50, -1);
          }
@@ -237,7 +296,7 @@ void EffectChangePitch::PopulateOrExchange(ShuttleGui & S)
 
          S.StartHorizontalLay(wxALIGN_CENTER);
          {
-            FloatingPointValidator<double> vldSemitones(2, &m_dSemitonesChange, NUM_VAL_TWO_TRAILING_ZEROES);
+            FloatingPointValidator<double> vldSemitones(2, &m_dSemitonesChange, NumValidatorStyle::TWO_TRAILING_ZEROES);
             m_pTextCtrl_SemitonesChange =
                S.Id(ID_SemitonesChange).AddTextBox(_("Semitones (half-steps):"), wxT(""), 12);
             m_pTextCtrl_SemitonesChange->SetName(_("Semitones (half-steps)"));
@@ -251,13 +310,13 @@ void EffectChangePitch::PopulateOrExchange(ShuttleGui & S)
       {
          S.StartMultiColumn(5, wxALIGN_CENTER); // 5, because AddTextBox adds a wxStaticText and a wxTextCtrl.
          {
-            FloatingPointValidator<double> vldFromFrequency(3, &m_FromFrequency, NUM_VAL_THREE_TRAILING_ZEROES);
+            FloatingPointValidator<double> vldFromFrequency(3, &m_FromFrequency, NumValidatorStyle::THREE_TRAILING_ZEROES);
             vldFromFrequency.SetMin(0.0);
             m_pTextCtrl_FromFrequency = S.Id(ID_FromFrequency).AddTextBox(_("from"), wxT(""), 12);
             m_pTextCtrl_FromFrequency->SetName(_("from (Hz)"));
             m_pTextCtrl_FromFrequency->SetValidator(vldFromFrequency);
 
-            FloatingPointValidator<double> vldToFrequency(3, &m_ToFrequency, NUM_VAL_THREE_TRAILING_ZEROES);
+            FloatingPointValidator<double> vldToFrequency(3, &m_ToFrequency, NumValidatorStyle::THREE_TRAILING_ZEROES);
             vldToFrequency.SetMin(0.0);
             m_pTextCtrl_ToFrequency = S.Id(ID_ToFrequency).AddTextBox(_("to"), wxT(""), 12);
             m_pTextCtrl_ToFrequency->SetName(_("to (Hz)"));
@@ -269,7 +328,7 @@ void EffectChangePitch::PopulateOrExchange(ShuttleGui & S)
 
          S.StartHorizontalLay(wxALIGN_CENTER);
          {
-            FloatingPointValidator<double> vldPercentage(3, &m_dPercentChange, NUM_VAL_THREE_TRAILING_ZEROES);
+            FloatingPointValidator<double> vldPercentage(3, &m_dPercentChange, NumValidatorStyle::THREE_TRAILING_ZEROES);
             vldPercentage.SetRange(MIN_Percentage, MAX_Percentage);
             m_pTextCtrl_PercentChange = S.Id(ID_PercentChange).AddTextBox(_("Percent Change:"), wxT(""), 12);
             m_pTextCtrl_PercentChange->SetValidator(vldPercentage);
@@ -280,15 +339,25 @@ void EffectChangePitch::PopulateOrExchange(ShuttleGui & S)
          {
             S.SetStyle(wxSL_HORIZONTAL);
             m_pSlider_PercentChange = S.Id(ID_PercentChange)
-               .AddSlider(wxT(""), 0, (int)kSliderMax, (int)MIN_Percentage);
+               .AddSlider( {}, 0, (int)kSliderMax, (int)MIN_Percentage);
             m_pSlider_PercentChange->SetName(_("Percent Change"));
          }
          S.EndHorizontalLay();
       }
       S.EndStatic();
+
+#if USE_SBSMS
+      S.StartMultiColumn(2);
+      {
+         mUseSBSMSCheckBox = S.AddCheckBox(_("Use high quality stretching (slow)"),
+                                             mUseSBSMS);
+         mUseSBSMSCheckBox->SetValidator(wxGenericValidator(&mUseSBSMS));
+      }
+      S.EndMultiColumn();
+#endif
+
    }
    S.EndVerticalLay();
-
    return;
 }
 
@@ -346,10 +415,17 @@ bool EffectChangePitch::TransferDataFromWindow()
 // the selection. Then set some other params accordingly.
 void EffectChangePitch::DeduceFrequencies()
 {
+    auto FirstTrack = [&]()->const WaveTrack *{
+      if( !inputTracks() )
+         return nullptr;
+      return *( inputTracks()->Selected< const WaveTrack >() ).first;
+   };
+
+   m_dStartFrequency = 261.265;// Middle C.
+
    // As a neat trick, attempt to get the frequency of the note at the
    // beginning of the selection.
-   SelectedTrackListOfKindIterator iter(Track::Wave, mTracks);
-   WaveTrack *track = (WaveTrack *) iter.First();
+   auto track = FirstTrack();
    if (track) {
       double rate = track->GetRate();
 
@@ -357,52 +433,38 @@ void EffectChangePitch::DeduceFrequencies()
       // Aim for around 2048 samples at 44.1 kHz (good down to about 100 Hz).
       // To detect single notes, analysis period should be about 0.2 seconds.
       // windowSize must be a power of 2.
-      int windowSize = wxRound(pow(2.0, floor((log(rate / 20.0)/log(2.0)) + 0.5)));
-      // windowSize < 256 too inaccurate
-      windowSize = (windowSize > 256)? windowSize : 256;
+      const size_t windowSize =
+         // windowSize < 256 too inaccurate
+         std::max(256, wxRound(pow(2.0, floor((log(rate / 20.0)/log(2.0)) + 0.5))));
 
       // we want about 0.2 seconds to catch the first note.
       // number of windows rounded to nearest integer >= 1.
-      int numWindows = wxRound((double)(rate / (5.0f * windowSize)));
-      numWindows = (numWindows > 0)? numWindows : 1;
+      const unsigned numWindows =
+         std::max(1, wxRound((double)(rate / (5.0f * windowSize))));
 
       double trackStart = track->GetStartTime();
       double t0 = mT0 < trackStart? trackStart: mT0;
-      sampleCount start = track->TimeToLongSamples(t0);
+      auto start = track->TimeToLongSamples(t0);
 
-      int analyzeSize = windowSize * numWindows;
-      float * buffer;
-      buffer = new float[analyzeSize];
+      auto analyzeSize = windowSize * numWindows;
+      Floats buffer{ analyzeSize };
 
-      float * freq;
-      freq = new float[windowSize/2];
+      Floats freq{ windowSize / 2 };
+      Floats freqa{ windowSize / 2, true };
 
-      float * freqa;
-      freqa = new float[windowSize/2];
-
-      int i, j, argmax;
-      int lag;
-
-      for(j=0; j<windowSize/2; j++)
-         freqa[j] = 0;
-
-      track->Get((samplePtr) buffer, floatSample, start, analyzeSize);
-      for(i=0; i<numWindows; i++) {
-         ComputeSpectrum(buffer+i*windowSize, windowSize,
-                         windowSize, rate, freq, true);
-         for(j=0; j<windowSize/2; j++)
+      track->Get((samplePtr) buffer.get(), floatSample, start, analyzeSize);
+      for(unsigned i = 0; i < numWindows; i++) {
+         ComputeSpectrum(buffer.get() + i * windowSize, windowSize,
+                         windowSize, rate, freq.get(), true);
+         for(size_t j = 0; j < windowSize / 2; j++)
             freqa[j] += freq[j];
       }
-      argmax=0;
-      for(j=1; j<windowSize/2; j++)
+      size_t argmax = 0;
+      for(size_t j = 1; j < windowSize / 2; j++)
          if (freqa[j] > freqa[argmax])
             argmax = j;
 
-      delete [] freq;
-      delete [] freqa;
-      delete [] buffer;
-
-      lag = (windowSize/2 - 1) - argmax;
+      auto lag = (windowSize / 2 - 1) - argmax;
       m_dStartFrequency = rate / lag;
    }
 
@@ -414,7 +476,7 @@ void EffectChangePitch::DeduceFrequencies()
    m_nToOctave = PitchOctave(dToMIDInote);
 
    m_FromFrequency = m_dStartFrequency;
-   Calc_PercentChange();
+   // Calc_PercentChange();  // This will reset m_dPercentChange
    Calc_ToFrequency();
 }
 
@@ -459,7 +521,6 @@ void EffectChangePitch::Calc_PercentChange()
 
 
 // handlers
-
 void EffectChangePitch::OnChoice_FromPitch(wxCommandEvent & WXUNUSED(evt))
 {
    if (m_bLoopDetect)
