@@ -15,11 +15,10 @@
 *//*******************************************************************/
 
 #include "../Audacity.h"
+#include "Invert.h"
 
 #include <wx/intl.h>
 
-#include "Invert.h"
-
 EffectInvert::EffectInvert()
 {
 }
@@ -28,19 +27,19 @@ EffectInvert::~EffectInvert()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectInvert::GetSymbol()
+ComponentInterfaceSymbol EffectInvert::GetSymbol()
 {
    return INVERT_PLUGIN_SYMBOL;
 }
 
 wxString EffectInvert::GetDescription()
 {
-   return XO("Flips the audio samples upside-down, reversing their polarity");
+   return _("Flips the audio samples upside-down, reversing their polarity");
 }
 
-// EffectIdentInterface implementation
+// EffectDefinitionInterface implementation
 
 EffectType EffectInvert::GetType()
 {
@@ -54,22 +53,22 @@ bool EffectInvert::IsInteractive()
 
 // EffectClientInterface implementation
 
-int EffectInvert::GetAudioInCount()
+unsigned EffectInvert::GetAudioInCount()
 {
    return 1;
 }
 
-int EffectInvert::GetAudioOutCount()
+unsigned EffectInvert::GetAudioOutCount()
 {
    return 1;
 }
 
-sampleCount EffectInvert::ProcessBlock(float **inBlock, float **outBlock, sampleCount blockLen)
+size_t EffectInvert::ProcessBlock(float **inBlock, float **outBlock, size_t blockLen)
 {
    float *ibuf = inBlock[0];
    float *obuf = outBlock[0];
 
-   for (sampleCount i = 0; i < blockLen; i++)
+   for (decltype(blockLen) i = 0; i < blockLen; i++)
    {
       obuf[i] = -ibuf[i];
    }
