@@ -13,31 +13,29 @@
 #ifndef __AUDACITY_EFFECT_REVERSE__
 #define __AUDACITY_EFFECT_REVERSE__
 
-#include <wx/string.h>
-
 #include "Effect.h"
 
-#define REVERSE_PLUGIN_SYMBOL XO("Reverse")
+#define REVERSE_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Reverse") }
 
-class EffectReverse : public Effect
+class EffectReverse final : public Effect
 {
 public:
    EffectReverse();
    virtual ~EffectReverse();
 
-   // IdentInterface implementation
+   // ComponentInterface implementation
 
-   virtual wxString GetSymbol();
-   virtual wxString GetDescription();
+   ComponentInterfaceSymbol GetSymbol() override;
+   wxString GetDescription() override;
 
-   // EffectIdentInterface implementation
+   // EffectDefinitionInterface implementation
 
-   virtual EffectType GetType();
-   virtual bool IsInteractive();
+   EffectType GetType() override;
+   bool IsInteractive() override;
 
    // Effect implementation
 
-   virtual bool Process();
+   bool Process() override;
 
 private:
    // EffectReverse implementation
