@@ -38,11 +38,16 @@ a graph for EffectScienFilter.
 #include <math.h>
 #include <float.h>
 
+#include <wx/setup.h> // for wxUSE_* macros
+
 #include <wx/brush.h>
+#include <wx/choice.h>
+#include <wx/dcclient.h>
 #include <wx/dcmemory.h>
 #include <wx/intl.h>
-#include <wx/msgdlg.h>
 #include <wx/settings.h>
+#include <wx/slider.h>
+#include <wx/stattext.h>
 #include <wx/utils.h>
 #include <wx/valgen.h>
 
@@ -51,12 +56,14 @@ a graph for EffectScienFilter.
 #include "../PlatformCompatibility.h"
 #include "../Prefs.h"
 #include "../Project.h"
+#include "../ProjectSettings.h"
+#include "../Shuttle.h"
 #include "../ShuttleGui.h"
-#include "../Theme.h"
 #include "../WaveTrack.h"
 #include "../widgets/valnum.h"
-
-#include "Equalization.h" // For SliderAx
+#include "../widgets/AudacityMessageBox.h"
+#include "../widgets/Ruler.h"
+#include "../widgets/WindowAccessible.h"
 
 #if !defined(M_PI)
 #define PI = 3.1415926535897932384626433832795
@@ -83,41 +90,42 @@ enum kTypes
    kButterworth,
    kChebyshevTypeI,
    kChebyshevTypeII,
-   kNumTypes
+   nTypes
 };
 
-static const wxChar *kTypeStrings[] =
+static const EnumValueSymbol kTypeStrings[nTypes] =
 {
    /*i18n-hint: Butterworth is the name of the person after whom the filter type is named.*/
-   XO("Butterworth"),
+   { XO("Butterworth") },
    /*i18n-hint: Chebyshev is the name of the person after whom the filter type is named.*/
-   XO("Chebyshev Type I"),
+   { XO("Chebyshev Type I") },
    /*i18n-hint: Chebyshev is the name of the person after whom the filter type is named.*/
-   XO("Chebyshev Type II")
+   { XO("Chebyshev Type II") }
 };
 
 enum kSubTypes
 {
    kLowPass,
    kHighPass,
-   kNumSubTypes
+   nSubTypes
 };
 
-static const wxChar *kSubTypeStrings[] =
+static const EnumValueSymbol kSubTypeStrings[nSubTypes] =
 {
-   XO("Lowpass"),
-   XO("Highpass")
+   // These are acceptable dual purpose internal/visible names
+   { XO("Lowpass") },
+   { XO("Highpass") }
 };
 
 // Define keys, defaults, minimums, and maximums for the effect parameters
 //
 //     Name       Type     Key                     Def            Min   Max               Scale
-Param( Type,      int,     XO("FilterType"),       kButterworth,  0,    kNumTypes - 1,    1  );
-Param( Subtype,   int,     XO("FilterSubtype"),    kLowPass,      0,    kNumSubTypes - 1, 1  );
-Param( Order,     int,     XO("Order"),            1,             1,    10,               1  );
-Param( Cutoff,    float,   XO("Cutoff"),           1000.0,        1.0,  FLT_MAX,          1  );
-Param( Passband,  float,   XO("PassbandRipple"),   1.0,           0.0,  100.0,            1  );
-Param( Stopband,  float,   XO("StopbandRipple"),   30.0,          0.0,  100.0,            1  );
+Param( Type,      int,     wxT("FilterType"),       kButterworth,  0,    nTypes - 1,    1  );
+Param( Subtype,   int,     wxT("FilterSubtype"),    kLowPass,      0,    nSubTypes - 1, 1  );
+Param( Order,     int,     wxT("Order"),            1,             1,    10,               1  );
+Param( Cutoff,    float,   wxT("Cutoff"),           1000.0,        1.0,  FLT_MAX,          1  );
+Param( Passband,  float,   wxT("PassbandRipple"),   1.0,           0.0,  100.0,            1  );
+Param( Stopband,  float,   wxT("StopbandRipple"),   30.0,          0.0,  100.0,            1  );
 
 static const double s_fChebyCoeffs[MAX_Order][MAX_Order + 1] =
 {
@@ -153,6 +161,7 @@ BEGIN_EVENT_TABLE(EffectScienFilter, wxEvtHandler)
 END_EVENT_TABLE()
 
 EffectScienFilter::EffectScienFilter()
+: mpBiquad{ size_t( MAX_Order / 2 ), true }
 {
    mOrder = DEF_Order;
    mFilterType = DEF_Type;
@@ -170,33 +179,31 @@ EffectScienFilter::EffectScienFilter()
 
    mLoFreq = 20;              // Lowest frequency to display in response graph
    mNyquist = 44100.0 / 2.0;  // only used during initialization, updated when effect is used
-
-   mpBiquad = new BiquadStruct[MAX_Order / 2];
-   memset(mpBiquad, 0, sizeof(BiquadStruct) * MAX_Order / 2);
-   for (int i = 0; i < MAX_Order / 2; i++)
-   {
-      mpBiquad[i].fNumerCoeffs[0] = 1.0;	// straight-through
-   }
 }
 
 EffectScienFilter::~EffectScienFilter()
 {
-   delete [] mpBiquad;
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectScienFilter::GetSymbol()
+ComponentInterfaceSymbol EffectScienFilter::GetSymbol()
 {
    return CLASSICFILTERS_PLUGIN_SYMBOL;
 }
 
 wxString EffectScienFilter::GetDescription()
 {
-   return XO("Performs IIR filtering that emulates analog filters");
+   return _("Performs IIR filtering that emulates analog filters");
 }
 
-// EffectIdentInterface implementation
+wxString EffectScienFilter::ManualPage()
+{
+   return wxT("Classic_Filters");
+}
+
+
+// EffectDefinitionInterface implementation
 
 EffectType EffectScienFilter::GetType()
 {
@@ -205,12 +212,12 @@ EffectType EffectScienFilter::GetType()
 
 // EffectClientInterface implementation
 
-int EffectScienFilter::GetAudioInCount()
+unsigned EffectScienFilter::GetAudioInCount()
 {
    return 1;
 }
 
-int EffectScienFilter::GetAudioOutCount()
+unsigned EffectScienFilter::GetAudioOutCount()
 {
    return 1;
 }
@@ -218,34 +225,38 @@ int EffectScienFilter::GetAudioOutCount()
 bool EffectScienFilter::ProcessInitialize(sampleCount WXUNUSED(totalLen), ChannelNames WXUNUSED(chanMap))
 {
    for (int iPair = 0; iPair < (mOrder + 1) / 2; iPair++)
-   {
-      mpBiquad[iPair].fPrevIn = 0;
-      mpBiquad[iPair].fPrevPrevIn = 0;
-      mpBiquad[iPair].fPrevOut = 0;
-      mpBiquad[iPair].fPrevPrevOut = 0;
-   }
+      mpBiquad[iPair].Reset();
 
    return true;
 }
 
-sampleCount EffectScienFilter::ProcessBlock(float **inBlock, float **outBlock, sampleCount blockLen)
+size_t EffectScienFilter::ProcessBlock(float **inBlock, float **outBlock, size_t blockLen)
 {
    float *ibuf = inBlock[0];
    for (int iPair = 0; iPair < (mOrder + 1) / 2; iPair++)
    {
       mpBiquad[iPair].pfIn = ibuf;
       mpBiquad[iPair].pfOut = outBlock[0];
-      Biquad_Process(&mpBiquad[iPair], blockLen);
+      mpBiquad[iPair].Process(blockLen);
       ibuf = outBlock[0];
    }
 
    return blockLen;
 }
+bool EffectScienFilter::DefineParams( ShuttleParams & S ){
+   S.SHUTTLE_ENUM_PARAM( mFilterType, Type, kTypeStrings, nTypes );
+   S.SHUTTLE_ENUM_PARAM( mFilterSubtype, Subtype, kSubTypeStrings, nSubTypes );
+   S.SHUTTLE_PARAM( mOrder, Order );
+   S.SHUTTLE_PARAM( mCutoff, Cutoff );
+   S.SHUTTLE_PARAM( mRipple, Passband );
+   S.SHUTTLE_PARAM( mStopbandRipple, Stopband );
+   return true;
+}
 
-bool EffectScienFilter::GetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectScienFilter::GetAutomationParameters(CommandParameters & parms)
 {
-   parms.Write(KEY_Type, kTypeStrings[mFilterType]);
-   parms.Write(KEY_Subtype, kSubTypeStrings[mFilterSubtype]);
+   parms.Write(KEY_Type, kTypeStrings[mFilterType].Internal());
+   parms.Write(KEY_Subtype, kSubTypeStrings[mFilterSubtype].Internal());
    parms.Write(KEY_Order, mOrder);
    parms.WriteFloat(KEY_Cutoff, mCutoff);
    parms.WriteFloat(KEY_Passband, mRipple);
@@ -254,10 +265,10 @@ bool EffectScienFilter::GetAutomationParameters(EffectAutomationParameters & par
    return true;
 }
 
-bool EffectScienFilter::SetAutomationParameters(EffectAutomationParameters & parms)
+bool EffectScienFilter::SetAutomationParameters(CommandParameters & parms)
 {
-   ReadAndVerifyEnum(Type, wxArrayString(kNumTypes, kTypeStrings));
-   ReadAndVerifyEnum(Subtype, wxArrayString(kNumSubTypes, kSubTypeStrings));
+   ReadAndVerifyEnum(Type, kTypeStrings, nTypes);
+   ReadAndVerifyEnum(Subtype, kSubTypeStrings, nSubTypes);
    ReadAndVerifyInt(Order);
    ReadAndVerifyFloat(Cutoff);
    ReadAndVerifyFloat(Passband);
@@ -332,30 +343,32 @@ bool EffectScienFilter::Init()
    int selcount = 0;
    double rate = 0.0;
 
-   TrackListOfKindIterator iter(Track::Wave, mTracks);
-   WaveTrack *t = (WaveTrack *) iter.First();
+   auto trackRange = inputTracks()->Selected< const WaveTrack >();
 
-   mNyquist = (t ? t->GetRate() : GetActiveProject()->GetRate()) / 2.0;
-
-   while (t)
    {
-      if (t->GetSelected() && t->GetKind() == Track::Wave)
+      auto t = *trackRange.begin();
+      mNyquist =
+         (t
+            ? t->GetRate()
+            : ProjectSettings::Get( *GetActiveProject() ).GetRate())
+         / 2.0;
+   }
+
+   for (auto t : trackRange)
+   {
+      if (selcount == 0)
       {
-         if (selcount == 0)
-         {
-            rate = t->GetRate();
-         }
-         else
-         {
-            if (t->GetRate() != rate)
-            {
-               wxMessageBox(_("To apply a filter, all selected tracks must have the same sample rate."));
-               return false;
-            }
-         }
-         selcount++;
+         rate = t->GetRate();
       }
-      t = (WaveTrack *) iter.Next();
+      else
+      {
+         if (t->GetRate() != rate)
+         {
+            Effect::MessageBox(_("To apply a filter, all selected tracks must have the same sample rate."));
+            return false;
+         }
+      }
+      selcount++;
    }
 
    return true;
@@ -363,7 +376,7 @@ bool EffectScienFilter::Init()
 
 void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
 {
-   wxWindow *parent = S.GetParent();
+   wxWindow *const parent = S.GetParent();
 
    S.AddSpace(5);
    S.SetSizerProportion(1);
@@ -378,16 +391,15 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
 
       S.StartVerticalLay();
       {
-         mdBRuler = new RulerPanel(parent, wxID_ANY);
-         mdBRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
-         mdBRuler->ruler.SetOrientation(wxVERTICAL);
-         mdBRuler->ruler.SetRange(30.0, -120.0);
-         mdBRuler->ruler.SetFormat(Ruler::LinearDBFormat);
-         mdBRuler->ruler.SetUnits(_("dB"));
-         mdBRuler->ruler.SetLabelEdges(true);
-         int w;
-         mdBRuler->ruler.GetMaxSize(&w, NULL);
-         mdBRuler->SetSize(wxSize(w, 150));  // height needed for wxGTK
+         mdBRuler = safenew RulerPanel(
+            parent, wxID_ANY, wxVERTICAL,
+            wxSize{ 100, 100 }, // Ruler can't handle small sizes
+            RulerPanel::Range{ 30.0, -120.0 },
+            Ruler::LinearDBFormat,
+            _("dB"),
+            RulerPanel::Options{}
+               .LabelEdges(true)
+         );
 
          S.SetBorder(1);
          S.AddSpace(1, 1);
@@ -397,8 +409,10 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
       }
       S.EndVerticalLay();
 
-      mPanel = new EffectScienFilterPanel(this, parent);
-      mPanel->SetFreqRange(mLoFreq, mNyquist);
+      mPanel = safenew EffectScienFilterPanel(
+         parent, wxID_ANY,
+         this, mLoFreq, mNyquist
+      );
 
       S.SetBorder(5);
       S.Prop(1);
@@ -409,18 +423,18 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
       {
          S.AddVariableText(_("+ dB"), false, wxCENTER);
          S.SetStyle(wxSL_VERTICAL | wxSL_INVERSE);
-         mdBMaxSlider = S.Id(ID_dBMax).AddSlider(wxT(""), 10, 20, 0);
+         mdBMaxSlider = S.Id(ID_dBMax).AddSlider( {}, 10, 20, 0);
 #if wxUSE_ACCESSIBILITY
          mdBMaxSlider->SetName(_("Max dB"));
-         mdBMaxSlider->SetAccessible(new SliderAx(mdBMaxSlider, wxString(wxT("%d ")) + _("dB")));
+         mdBMaxSlider->SetAccessible(safenew SliderAx(mdBMaxSlider, _("%d dB")));
 #endif
 
          S.SetStyle(wxSL_VERTICAL | wxSL_INVERSE);
-         mdBMinSlider = S.Id(ID_dBMin).AddSlider(wxT(""), -10, -10, -120);
+         mdBMinSlider = S.Id(ID_dBMin).AddSlider( {}, -10, -10, -120);
          S.AddVariableText(_("- dB"), false, wxCENTER);
 #if wxUSE_ACCESSIBILITY
          mdBMinSlider->SetName(_("Min dB"));
-         mdBMinSlider->SetAccessible(new SliderAx(mdBMinSlider, wxString(wxT("%d ")) + _("dB")));
+         mdBMinSlider->SetAccessible(safenew SliderAx(mdBMinSlider, _("%d dB")));
 #endif
       }
       S.EndVerticalLay();
@@ -431,18 +445,17 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
 
       S.AddSpace(1, 1);
 
-      mfreqRuler  = new RulerPanel(parent, wxID_ANY);
-      mfreqRuler->ruler.SetBounds(0, 0, 100, 100); // Ruler can't handle small sizes
-      mfreqRuler->ruler.SetOrientation(wxHORIZONTAL);
-      mfreqRuler->ruler.SetLog(true);
-      mfreqRuler->ruler.SetRange(mLoFreq, mNyquist);
-      mfreqRuler->ruler.SetFormat(Ruler::IntFormat);
-      mfreqRuler->ruler.SetUnits(wxT(""));
-      mfreqRuler->ruler.SetFlip(true);
-      mfreqRuler->ruler.SetLabelEdges(true);
-      int h;
-      mfreqRuler->ruler.GetMaxSize(NULL, &h);
-      mfreqRuler->SetMinSize(wxSize(-1, h));
+      mfreqRuler  = safenew RulerPanel(
+         parent, wxID_ANY, wxHORIZONTAL,
+         wxSize{ 100, 100 }, // Ruler can't handle small sizes
+         RulerPanel::Range{ mLoFreq, mNyquist },
+         Ruler::IntFormat,
+         wxT(""),
+         RulerPanel::Options{}
+            .Log(true)
+            .Flip(true)
+            .LabelEdges(true)
+      );
 
       S.Prop(1);
       S.AddWindow(mfreqRuler, wxEXPAND | wxALIGN_LEFT | wxRIGHT);
@@ -457,25 +470,21 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
       S.SetSizerProportion(0);
       S.StartMultiColumn(8, wxALIGN_CENTER);
       {
-         wxASSERT(kNumTypes == WXSIZEOF(kTypeStrings));
+         wxASSERT(nTypes == WXSIZEOF(kTypeStrings));
 
-         wxArrayString typeChoices;
-         for (int i = 0; i < kNumTypes; i++)
-         {
-            typeChoices.Add(wxGetTranslation(kTypeStrings[i]));
-         }
-
-         mFilterTypeCtl = S.Id(ID_Type).AddChoice(_("&Filter Type:"), wxT(""), &typeChoices);
+         auto typeChoices = LocalizedStrings(kTypeStrings, nTypes);
+         mFilterTypeCtl = S.Id(ID_Type)
+            .AddChoice(_("&Filter Type:"), typeChoices);
          mFilterTypeCtl->SetValidator(wxGenericValidator(&mFilterType));
          S.SetSizeHints(-1, -1);
 
-         wxArrayString orders;
+         wxArrayStringEx orders;
          for (int i = 1; i <= 10; i++)
          {
-            orders.Add(wxString::Format(wxT("%d"), i));
+            orders.push_back(wxString::Format(wxT("%d"), i));
          }
          /*i18n-hint: 'Order' means the complexity of the filter, and is a number between 1 and 10.*/
-         mFilterOrderCtl = S.Id(ID_Order).AddChoice(_("O&rder:"), wxT(""), &orders);
+         mFilterOrderCtl = S.Id(ID_Order).AddChoice(_("O&rder:"), orders);
          mFilterOrderCtl->SetValidator(wxGenericValidator(&mOrderIndex));
          S.SetSizeHints(-1, -1);
          S.AddSpace(1, 1);
@@ -484,20 +493,16 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
          vldRipple.SetRange(MIN_Passband, MAX_Passband);
          
          mRippleCtlP = S.AddVariableText(_("&Passband Ripple:"), false, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
-         mRippleCtl = S.Id(ID_Ripple).AddTextBox(wxT(""), wxT(""), 10);
+         mRippleCtl = S.Id(ID_Ripple).AddTextBox( {}, wxT(""), 10);
          mRippleCtl->SetName(_("Passband Ripple (dB)"));
          mRippleCtl->SetValidator(vldRipple);
          mRippleCtlU = S.AddVariableText(_("dB"), false, wxALL | wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
 
-         wxASSERT(kNumSubTypes == WXSIZEOF(kSubTypeStrings));
+         wxASSERT(nSubTypes == WXSIZEOF(kSubTypeStrings));
 
-         wxArrayString subTypeChoices;
-         for (int i = 0; i < kNumSubTypes; i++)
-         {
-            subTypeChoices.Add(wxGetTranslation(kSubTypeStrings[i]));
-         }
-
-         mFilterSubTypeCtl = S.Id(ID_SubType).AddChoice(_("&Subtype:"), wxT(""), &subTypeChoices);
+         auto subTypeChoices = LocalizedStrings(kSubTypeStrings, nSubTypes);
+         mFilterSubTypeCtl = S.Id(ID_SubType)
+            .AddChoice(_("&Subtype:"), subTypeChoices);
          mFilterSubTypeCtl->SetValidator(wxGenericValidator(&mFilterSubtype));
          S.SetSizeHints(-1, -1);
       
@@ -513,7 +518,7 @@ void EffectScienFilter::PopulateOrExchange(ShuttleGui & S)
          vldStopbandRipple.SetRange(MIN_Stopband, MAX_Stopband);
          
          mStopbandRippleCtlP = S.AddVariableText(_("Minimum S&topband Attenuation:"), false, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
-         mStopbandRippleCtl = S.Id(ID_StopbandRipple).AddTextBox(wxT(""), wxT(""), 10);
+         mStopbandRippleCtl = S.Id(ID_StopbandRipple).AddTextBox( {}, wxT(""), 10);
          mStopbandRippleCtl->SetName(_("Minimum S&topband Attenuation (dB)"));
          mStopbandRippleCtl->SetValidator(vldStopbandRipple);
          mStopbandRippleCtlU = S.AddVariableText(_("dB"), false, wxALL | wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
@@ -580,7 +585,7 @@ bool EffectScienFilter::TransferGraphLimitsFromWindow()
    if (dB != mdBMin) {
       rr = true;
       mdBMin = dB;
-      tip.Printf(wxString(wxT("%d ")) + _("dB"), (int)mdBMin);
+      tip.Printf(_("%d dB"), (int)mdBMin);
       mdBMinSlider->SetToolTip(tip);
    }
 
@@ -588,7 +593,7 @@ bool EffectScienFilter::TransferGraphLimitsFromWindow()
    if (dB != mdBMax) {
       rr = true;
       mdBMax = dB;
-      tip.Printf(wxString(wxT("%d ")) + _("dB"),(int)mdBMax);
+      tip.Printf(_("%d dB"),(int)mdBMax);
       mdBMaxSlider->SetToolTip(tip);
    }
 
@@ -672,8 +677,8 @@ bool EffectScienFilter::CalcFilter()
             fDCPoleDistSqr = fZPoleX + 1;    // dist from Nyquist
          for (int iPair = 1; iPair <= mOrder/2; iPair++)
          {
-            float fSPoleX = fC * cos (PI - iPair * PI / mOrder);
-            float fSPoleY = fC * sin (PI - iPair * PI / mOrder);
+            fSPoleX = fC * cos (PI - iPair * PI / mOrder);
+            fSPoleY = fC * sin (PI - iPair * PI / mOrder);
             BilinTransform (fSPoleX, fSPoleY, &fZPoleX, &fZPoleY);
             mpBiquad[iPair].fNumerCoeffs [0] = 1;
             if (mFilterSubtype == kLowPass)		// LOWPASS
@@ -1008,13 +1013,15 @@ void EffectScienFilter::EnableDisableRippleCtl(int FilterType)
 // EffectScienFilterPanel
 //----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(EffectScienFilterPanel, wxPanel)
+BEGIN_EVENT_TABLE(EffectScienFilterPanel, wxPanelWrapper)
     EVT_PAINT(EffectScienFilterPanel::OnPaint)
     EVT_SIZE(EffectScienFilterPanel::OnSize)
 END_EVENT_TABLE()
 
-EffectScienFilterPanel::EffectScienFilterPanel(EffectScienFilter *effect, wxWindow *parent)
-:  wxPanel(parent, wxID_ANY, wxDefaultPosition, wxSize(400, 200))
+EffectScienFilterPanel::EffectScienFilterPanel(
+   wxWindow *parent, wxWindowID winid,
+   EffectScienFilter *effect, double lo, double hi)
+:  wxPanelWrapper(parent, winid, wxDefaultPosition, wxSize(400, 200))
 {
    mEffect = effect;
    mParent = parent;
@@ -1026,14 +1033,12 @@ EffectScienFilterPanel::EffectScienFilterPanel(EffectScienFilter *effect, wxWind
    mHiFreq = 0.0;
    mDbMin = 0.0;
    mDbMax = 0.0;
+
+   SetFreqRange(lo, hi);
 }
 
 EffectScienFilterPanel::~EffectScienFilterPanel()
 {
-   if (mBitmap)
-   {
-      delete mBitmap;
-   }
 }
 
 void EffectScienFilterPanel::SetFreqRange(double lo, double hi)
@@ -1073,14 +1078,9 @@ void EffectScienFilterPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
 
    if (!mBitmap || mWidth != width || mHeight != height)
    {
-      if (mBitmap)
-      {
-         delete mBitmap;
-      }
-
       mWidth = width;
       mHeight = height;
-      mBitmap = new wxBitmap(mWidth, mHeight);
+      mBitmap = std::make_unique<wxBitmap>(mWidth, mHeight,24);
    }
 
    wxBrush bkgndBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
@@ -1114,7 +1114,7 @@ void EffectScienFilterPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
    mEnvRect.Deflate(2, 2);
 
    // Pure blue x-axis line
-   memDC.SetPen(wxPen(theTheme.Colour(clrGraphLines), 1, wxSOLID));
+   memDC.SetPen(wxPen(theTheme.Colour(clrGraphLines), 1, wxPENSTYLE_SOLID));
    int center = (int) (mEnvRect.height * mDbMax / (mDbMax - mDbMin) + 0.5);
    AColor::Line(memDC,
                 mEnvRect.GetLeft(), mEnvRect.y + center,
@@ -1122,7 +1122,7 @@ void EffectScienFilterPanel::OnPaint(wxPaintEvent & WXUNUSED(evt))
 
    //Now draw the actual response that you will get.
    //mFilterFunc has a linear scale, window has a log one so we have to fiddle about
-   memDC.SetPen(wxPen(theTheme.Colour(clrResponseLines), 3, wxSOLID));
+   memDC.SetPen(wxPen(theTheme.Colour(clrResponseLines), 3, wxPENSTYLE_SOLID));
    double scale = (double) mEnvRect.height / (mDbMax - mDbMin);    // pixels per dB
    double yF;                                                     // gain at this freq
 
