@@ -15,13 +15,11 @@
 #include <Carbon/Carbon.h>
 #endif
 
-#include <wx/osx/private.h>
+#include <wx/osx/private.h> // to inherit wxWidgetCocoaImpl
 
-#include <wx/control.h>
+#include "VSTControl.h"
 
-#include "aeffectx.h"
-
-class VSTControlImpl : public wxWidgetCocoaImpl
+class VSTControlImpl final : public wxWidgetCocoaImpl
 {
 public :
    VSTControlImpl(wxWindowMac *peer, NSView *view);
@@ -35,6 +33,7 @@ public:
    ~VSTControl();
 
    bool Create(wxWindow *parent, VSTEffectLink *link);
+   void Close();
 
 private:
    void CreateCocoa();
