@@ -20,26 +20,30 @@ Licensed under the GNU General Public License v2 or later
 
 *//*******************************************************************/
 
+#include "../Audacity.h"    // needed before FFmpeg.h // for USE_* macros
+
+#include "../Experimental.h"
+
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
-#include "../Audacity.h"    // needed before FFmpeg.h
 #include "../FFmpeg.h"      // which brings in avcodec.h, avformat.h
+#include "../WaveClip.h"
+#include "../blockfile/ODDecodeBlockFile.h"
 #include "../ondemand/ODManager.h"
 #ifndef WX_PRECOMP
 // Include your minimal set of headers here, or wx.h
 #include <wx/window.h>
 #endif
 
-#include "../Experimental.h"
+#include "../widgets/ProgressDialog.h"
 
 
 #define DESC _("FFmpeg-compatible files")
 
 //TODO: remove non-audio extensions
 #if defined(USE_FFMPEG)
-static const wxChar *exts[] =
-{
+static const auto exts = {
    wxT("4xm"),
    wxT("MTV"),
    wxT("roq"),
@@ -150,9 +154,7 @@ static const wxChar *exts[] =
 
 // all the includes live here by default
 #include "Import.h"
-#include "ImportFFmpeg.h"
 #include "../Tags.h"
-#include "../Internat.h"
 #include "../WaveTrack.h"
 #include "ImportPlugin.h"
 
@@ -161,36 +163,37 @@ static const wxChar *exts[] =
 #include "../ondemand/ODDecodeFFmpegTask.h"
 #endif
 
-extern FFmpegLibs *FFmpegLibsInst;
+extern FFmpegLibs *FFmpegLibsInst();
 
 class FFmpegImportFileHandle;
 
 /// A representative of FFmpeg loader in
 /// the Audacity import plugin list
-class FFmpegImportPlugin : public ImportPlugin
+class FFmpegImportPlugin final : public ImportPlugin
 {
 public:
    FFmpegImportPlugin():
-      ImportPlugin(wxArrayString(WXSIZEOF(exts),exts))
-      {
-
-      }
+      ImportPlugin( FileExtensions( exts.begin(), exts.end() ) )
+   {
+   }
 
    ~FFmpegImportPlugin() { }
 
-   wxString GetPluginStringID() { return wxT("libav"); }
-   wxString GetPluginFormatDescription();
+   wxString GetPluginStringID() override { return wxT("libav"); }
+   wxString GetPluginFormatDescription() override;
 
    ///! Probes the file and opens it if appropriate
-   ImportFileHandle *Open(wxString Filename);
+   std::unique_ptr<ImportFileHandle> Open(const FilePath &Filename) override;
+   
+   unsigned SequenceNumber() const override;
 };
 
 ///! Does acual import, returned by FFmpegImportPlugin::Open
-class FFmpegImportFileHandle : public ImportFileHandle
+class FFmpegImportFileHandle final : public ImportFileHandle
 {
 
 public:
-   FFmpegImportFileHandle(const wxString & name);
+   FFmpegImportFileHandle(const FilePath & name);
    ~FFmpegImportFileHandle();
 
    ///! Format initialization
@@ -201,13 +204,13 @@ public:
    bool InitCodecs();
 
 
-   wxString GetFileDescription();
-   int GetFileUncompressedBytes();
+   wxString GetFileDescription() override;
+   ByteCount GetFileUncompressedBytes() override;
 
    ///! Imports audio
    ///\return import status (see Import.cpp)
-   int Import(TrackFactory *trackFactory, Track ***outTracks,
-      int *outNumTracks, Tags *tags);
+   ProgressResult Import(TrackFactory *trackFactory, TrackHolders &outTracks,
+      Tags *tags) override;
 
    ///! Reads next audio frame
    ///\return pointer to the stream context structure to which the frame belongs to or NULL on error, or 1 if stream is not to be imported.
@@ -221,8 +224,7 @@ public:
 
    ///! Writes decoded data into WaveTracks. Called by DecodeFrame
    ///\param sc - stream context
-   ///\return 0 on success, 1 on error or interruption
-   int WriteData(streamContext *sc);
+   ProgressResult WriteData(streamContext *sc);
 
    ///! Writes extracted metadata to tags object
    ///\param avf - file context
@@ -238,14 +240,14 @@ public:
 
    ///! Called by Import.cpp
    ///\return number of readable streams in the file
-   wxInt32 GetStreamCount()
+   wxInt32 GetStreamCount() override
    {
       return mNumStreams;
    }
 
    ///! Called by Import.cpp
    ///\return array of strings - descriptions of the streams
-   wxArrayString *GetStreamInfo()
+   const wxArrayString &GetStreamInfo() override
    {
       return mStreamInfo;
    }
@@ -253,26 +255,30 @@ public:
    ///! Called by Import.cpp
    ///\param StreamID - index of the stream in mStreamInfo and mScs arrays
    ///\param Use - true if this stream should be imported, false otherwise
-   void SetStreamUsage(wxInt32 StreamID, bool Use)
+   void SetStreamUsage(wxInt32 StreamID, bool Use) override
    {
       if (StreamID < mNumStreams)
-         mScs[StreamID]->m_use = Use;
+         mScs->get()[StreamID]->m_use = Use;
    }
 
 private:
 
+   std::shared_ptr<FFmpegContext> mContext; // An object that does proper IO shutdown in its destructor; may be shared with decoder task.
    AVFormatContext      *mFormatContext; //!< Format description, also contains metadata and some useful info
    int                   mNumStreams;    //!< mNumstreams is less or equal to mFormatContext->nb_streams
-   streamContext       **mScs;           //!< Array of pointers to stream contexts. Length is mNumStreams.
-   wxArrayString        *mStreamInfo;    //!< Array of stream descriptions. Length is mNumStreams
+   ScsPtr                mScs;           //!< Points to array of pointers to stream contexts, which may be shared with a decoder task.
+   wxArrayString         mStreamInfo;    //!< Array of stream descriptions. Length is mNumStreams
 
    wxInt64               mProgressPos;   //!< Current timestamp, file position or whatever is used as first argument for Update()
    wxInt64               mProgressLen;   //!< Duration, total length or whatever is used as second argument for Update()
 
    bool                  mCancelled;     //!< True if importing was canceled by user
    bool                  mStopped;       //!< True if importing was stopped by user
-   wxString              mName;
-   WaveTrack           ***mChannels;     //!< 2-dimentional array of WaveTrack's. First dimention - streams, second - channels of a stream. Length is mNumStreams
+   FilePath              mName;
+   TrackHolders mChannels;               //!< 2-dimensional array of WaveTrack's.
+                                         //!< First dimension - streams,
+                                         //!< second - channels of a stream.
+                                         //!< Length is mNumStreams
 #ifdef EXPERIMENTAL_OD_FFMPEG
    bool                  mUsingOD;
 #endif
@@ -280,21 +286,14 @@ private:
 };
 
 
-void GetFFmpegImportPlugin(ImportPluginList *importPluginList,
-                           UnusableImportPluginList *WXUNUSED(unusableImportPluginList))
-{
-   importPluginList->Append(new FFmpegImportPlugin);
-}
-
-
 wxString FFmpegImportPlugin::GetPluginFormatDescription()
 {
    return DESC;
 }
 
-ImportFileHandle *FFmpegImportPlugin::Open(wxString filename)
+std::unique_ptr<ImportFileHandle> FFmpegImportPlugin::Open(const FilePath &filename)
 {
-   FFmpegImportFileHandle *handle = new FFmpegImportFileHandle(filename);
+   auto handle = std::make_unique<FFmpegImportFileHandle>(filename);
 
    //Check if we're loading explicitly supported format
    wxString extension = filename.AfterLast(wxT('.'));
@@ -304,81 +303,94 @@ ImportFileHandle *FFmpegImportPlugin::Open(wxString filename)
       //officially supported by this plugin.
       //If we don't have FFmpeg configured - tell the user about it.
       //Since this will be happening often, use disableable "FFmpeg not found" dialog
-      //insdead of usual wxMessageBox()
+      //insdead of usual AudacityMessageBox()
       bool newsession = false;
       gPrefs->Read(wxT("/NewImportingSession"), &newsession);
-      if (!FFmpegLibsInst->ValidLibsLoaded())
+      if (!FFmpegLibsInst()->ValidLibsLoaded())
       {
          int dontShowDlg;
-         FFmpegNotFoundDialog *dlg;
          gPrefs->Read(wxT("/FFmpeg/NotFoundDontShow"),&dontShowDlg,0);
          if (dontShowDlg == 0 && newsession)
          {
             gPrefs->Write(wxT("/NewImportingSession"), false);
             gPrefs->Flush();
-            dlg = new FFmpegNotFoundDialog(NULL);
-            dlg->ShowModal();
-            delete dlg;
+            FFmpegNotFoundDialog{ nullptr }.ShowModal();
          }
       }
    }
-   if (!FFmpegLibsInst->ValidLibsLoaded())
+   if (!FFmpegLibsInst()->ValidLibsLoaded())
    {
-      delete handle;
-      return NULL;
+      return nullptr;
    }
 
    // Open the file for import
    bool success = handle->Init();
    if (!success) {
-      delete handle;
-      return NULL;
+      return nullptr;
    }
 
-   return handle;
+   // This std::move is needed to "upcast" the pointer type
+   return std::move(handle);
 }
 
+unsigned FFmpegImportPlugin::SequenceNumber() const
+{
+   return 60;
+}
 
-FFmpegImportFileHandle::FFmpegImportFileHandle(const wxString & name)
+static Importer::RegisteredImportPlugin registered{
+   std::make_unique< FFmpegImportPlugin >()
+};
+
+
+FFmpegImportFileHandle::FFmpegImportFileHandle(const FilePath & name)
 :  ImportFileHandle(name)
 {
    PickFFmpegLibs();
 
-   mStreamInfo = new wxArrayString();
    mFormatContext = NULL;
    mNumStreams = 0;
-   mScs = NULL;
    mCancelled = false;
    mStopped = false;
    mName = name;
-   mChannels = NULL;
    mProgressPos = 0;
    mProgressLen = 1;
 }
 
 bool FFmpegImportFileHandle::Init()
 {
-   //FFmpegLibsInst->LoadLibs(NULL,false); //Loaded at startup or from Prefs now
+   //FFmpegLibsInst()->LoadLibs(NULL,false); //Loaded at startup or from Prefs now
 
-   if (!FFmpegLibsInst->ValidLibsLoaded()) return false;
+   if (!FFmpegLibsInst()->ValidLibsLoaded())
+      return false;
 
    av_log_set_callback(av_log_wx_callback);
 
-   int err = ufile_fopen_input(&mFormatContext, mName);
+   int err;
+   std::unique_ptr<FFmpegContext> tempContext;
+   err = ufile_fopen_input(tempContext, mName);
    if (err < 0)
    {
-      wxLogError(wxT("FFmpeg : av_open_input_file() failed for file %s"),mName.c_str());
+      wxLogError(wxT("FFmpeg : av_open_input_file() failed for file %s"), mName);
       return false;
    }
+   wxASSERT(tempContext.get());
+   mFormatContext = tempContext->ic_ptr;
 
    err = avformat_find_stream_info(mFormatContext, NULL);
    if (err < 0)
    {
-      wxLogError(wxT("FFmpeg: avformat_find_stream_info() failed for file %s"),mName.c_str());
+      wxLogError(wxT("FFmpeg: avformat_find_stream_info() failed for file %s"),mName);
       return false;
    }
 
-   InitCodecs();
+   if (!InitCodecs())
+      return false;
+
+   // Only now do we postpone destroying the FFmpegContext.
+   // Move from unique to shared pointer
+   mContext.reset(tempContext.release());
+
    return true;
 }
 
@@ -386,40 +398,40 @@ bool FFmpegImportFileHandle::InitCodecs()
 {
    // Allocate the array of pointers to hold stream contexts pointers
    // Some of the allocated space may be unused (corresponds to video, subtitle, or undecodeable audio streams)
-   mScs = (streamContext**)malloc(sizeof(streamContext**)*mFormatContext->nb_streams);
+   mScs = std::make_shared<Scs>(size_t{mFormatContext->nb_streams});
    // Fill the stream contexts
    for (unsigned int i = 0; i < mFormatContext->nb_streams; i++)
    {
       if (mFormatContext->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO)
       {
          //Create a context
-         streamContext *sc = new streamContext;
-         memset(sc,0,sizeof(*sc));
+         auto sc = std::make_unique<streamContext>();
 
          sc->m_stream = mFormatContext->streams[i];
          sc->m_codecCtx = sc->m_stream->codec;
 
-         AVCodec *codec = avcodec_find_decoder(sc->m_codecCtx->codec_id);
+         const AVCodecID id   = sc->m_codecCtx->codec_id;
+         const char*     name = avcodec_get_name(id);
+         const AVCodec *codec = avcodec_find_decoder(id);
+
          if (codec == NULL)
          {
-            wxLogError(wxT("FFmpeg : avcodec_find_decoder() failed. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
+            wxLogError(wxT("FFmpeg : avcodec_find_decoder() failed. Index[%02d], Codec[%02x - %s]"),i,id,name);
             //FFmpeg can't decode this stream, skip it
-            delete sc;
             continue;
          }
+
          if (codec->type != sc->m_codecCtx->codec_type)
          {
-            wxLogError(wxT("FFmpeg : Codec type mismatch, skipping. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
+            wxLogError(wxT("FFmpeg : Codec type mismatch, skipping. Index[%02d], Codec[%02x - %s]"),i,id,name);
             //Non-audio codec reported as audio? Nevertheless, we don't need THIS.
-            delete sc;
             continue;
          }
 
          if (avcodec_open2(sc->m_codecCtx, codec, NULL) < 0)
          {
-            wxLogError(wxT("FFmpeg : avcodec_open() failed. Index[%02d], Codec[%02x - %s]"),i,sc->m_codecCtx->codec_id,sc->m_codecCtx->codec_name);
+            wxLogError(wxT("FFmpeg : avcodec_open() failed. Index[%02d], Codec[%02x - %s]"),i,id,name);
             //Can't open decoder - skip this stream
-            delete sc;
             continue;
          }
 
@@ -430,9 +442,9 @@ bool FFmpegImportFileHandle::InitCodecs()
             duration = sc->m_stream->duration * sc->m_stream->time_base.num / sc->m_stream->time_base.den;
          else
             duration = mFormatContext->duration / AV_TIME_BASE;
-         wxString bitrate = wxT("");
+         wxString bitrate;
          if (sc->m_codecCtx->bit_rate > 0)
-            bitrate.Printf(wxT("%d"),sc->m_codecCtx->bit_rate);
+            bitrate.Printf(wxT("%d"),(int)sc->m_codecCtx->bit_rate);
          else
             bitrate.Printf(wxT("?"));
 
@@ -442,9 +454,10 @@ bool FFmpegImportFileHandle::InitCodecs()
          {
             lang.FromUTF8(tag->value);
          }
-         strinfo.Printf(_("Index[%02x] Codec[%s], Language[%s], Bitrate[%s], Channels[%d], Duration[%d]"),sc->m_stream->id,codec->name,lang.c_str(),bitrate.c_str(),sc->m_stream->codec->channels, duration);
-         mStreamInfo->Add(strinfo);
-         mScs[mNumStreams++] = sc;
+         strinfo.Printf(_("Index[%02x] Codec[%s], Language[%s], Bitrate[%s], Channels[%d], Duration[%d]"),
+                        sc->m_stream->id,codec->name,lang,bitrate,(int)sc->m_stream->codec->channels,(int)duration);
+         mStreamInfo.push_back(strinfo);
+         mScs->get()[mNumStreams++] = std::move(sc);
       }
       //for video and unknown streams do nothing
    }
@@ -458,105 +471,95 @@ wxString FFmpegImportFileHandle::GetFileDescription()
 }
 
 
-int FFmpegImportFileHandle::GetFileUncompressedBytes()
+auto FFmpegImportFileHandle::GetFileUncompressedBytes() -> ByteCount
 {
    // TODO: Get Uncompressed byte count.
    return 0;
 }
 
-int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
-              Track ***outTracks,
-              int *outNumTracks,
+ProgressResult FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
+              TrackHolders &outTracks,
               Tags *tags)
 {
+   outTracks.clear();
 
    CreateProgress();
 
    // Remove stream contexts which are not marked for importing and adjust mScs and mNumStreams accordingly
+   const auto scs = mScs->get();
    for (int i = 0; i < mNumStreams;)
    {
-      if (!mScs[i]->m_use)
+      if (!scs[i]->m_use)
       {
-         delete mScs[i];
          for (int j = i; j < mNumStreams - 1; j++)
          {
-            mScs[j] = mScs[j+1];
+            scs[j] = std::move(scs[j+1]);
          }
          mNumStreams--;
       }
       else i++;
    }
 
-   mChannels = new WaveTrack **[mNumStreams];
+   mChannels.resize(mNumStreams);
 
-   for (int s = 0; s < mNumStreams; s++)
+   int s = -1;
+   for (auto &stream : mChannels)
    {
-      switch (mScs[s]->m_stream->codec->sample_fmt)
+      ++s;
+
+      auto sc = scs[s].get();
+      switch (sc->m_stream->codec->sample_fmt)
       {
          case AV_SAMPLE_FMT_U8:
          case AV_SAMPLE_FMT_S16:
          case AV_SAMPLE_FMT_U8P:
          case AV_SAMPLE_FMT_S16P:
-            mScs[s]->m_osamplesize = sizeof(int16_t);
-            mScs[s]->m_osamplefmt = int16Sample;
+            sc->m_osamplesize = sizeof(int16_t);
+            sc->m_osamplefmt = int16Sample;
          break;
          default:
-            mScs[s]->m_osamplesize = sizeof(float);
-            mScs[s]->m_osamplefmt = floatSample;
+            sc->m_osamplesize = sizeof(float);
+            sc->m_osamplefmt = floatSample;
          break;
       }
 
-      // There is a possibility that number of channels will change over time, but we do not have WaveTracks for new channels. Remember the number of channels and stick to it.
-      mScs[s]->m_initialchannels = mScs[s]->m_stream->codec->channels;
-      mChannels[s] = new WaveTrack *[mScs[s]->m_stream->codec->channels];
-      int c;
-      for (c = 0; c < mScs[s]->m_stream->codec->channels; c++)
-      {
-         mChannels[s][c] = trackFactory->NewWaveTrack(mScs[s]->m_osamplefmt, mScs[s]->m_stream->codec->sample_rate);
-
-         if (mScs[s]->m_stream->codec->channels == 2)
-         {
-            switch (c)
-            {
-            case 0:
-               mChannels[s][c]->SetChannel(Track::LeftChannel);
-               mChannels[s][c]->SetLinked(true);
-               break;
-            case 1:
-               mChannels[s][c]->SetChannel(Track::RightChannel);
-               break;
-            }
-         }
-         else
-         {
-            mChannels[s][c]->SetChannel(Track::MonoChannel);
-         }
-      }
+      // There is a possibility that number of channels will change over time, but we do not have WaveTracks for NEW channels. Remember the number of channels and stick to it.
+      sc->m_initialchannels = sc->m_stream->codec->channels;
+      stream.resize(sc->m_stream->codec->channels);
+      for (auto &channel : stream)
+         channel = trackFactory->NewWaveTrack(sc->m_osamplefmt, sc->m_stream->codec->sample_rate);
    }
 
    // Handles the start_time by creating silence. This may or may not be correct.
    // There is a possibility that we should ignore first N milliseconds of audio instead. I do not know.
    /// TODO: Nag FFmpeg devs about start_time until they finally say WHAT is this and HOW to handle it.
-   for (int s = 0; s < mNumStreams; s++)
+   s = -1;
+   for (auto &stream : mChannels)
    {
+      ++s;
+
       int64_t stream_delay = 0;
-      if (mScs[s]->m_stream->start_time != int64_t(AV_NOPTS_VALUE) && mScs[s]->m_stream->start_time > 0)
+      auto sc = scs[s].get();
+      if (sc->m_stream->start_time != int64_t(AV_NOPTS_VALUE) && sc->m_stream->start_time > 0)
       {
-         stream_delay = mScs[s]->m_stream->start_time;
-         wxLogDebug(wxT("Stream %d start_time = %lld, that would be %f milliseconds."), s, (long long) mScs[s]->m_stream->start_time, double(mScs[s]->m_stream->start_time)/AV_TIME_BASE*1000);
+         stream_delay = sc->m_stream->start_time;
+         wxLogDebug(wxT("Stream %d start_time = %lld, that would be %f milliseconds."), s, (long long) sc->m_stream->start_time, double(sc->m_stream->start_time)/AV_TIME_BASE*1000);
       }
-      if (stream_delay != 0)
+      if (stream_delay > 0)
       {
-         for (int c = 0; c < mScs[s]->m_stream->codec->channels; c++)
+         int c = -1;
+         for (auto &channel : stream)
          {
-            WaveTrack *t = mChannels[s][c];
+            ++c;
+
+            WaveTrack *t = channel.get();
             t->InsertSilence(0,double(stream_delay)/AV_TIME_BASE);
          }
       }
    }
    // This is the heart of the importing process
    // The result of Import() to be returend. It will be something other than zero if user canceled or some error appears.
-   int res = eProgressSuccess;
+   auto res = ProgressResult::Success;
 
 #ifdef EXPERIMENTAL_OD_FFMPEG
    mUsingOD = false;
@@ -564,66 +567,77 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
    //at this point we know the file is good and that we have to load the number of channels in mScs[s]->m_stream->codec->channels;
    //so for OD loading we create the tracks and releasee the modal lock after starting the ODTask.
    if (mUsingOD) {
-      std::vector<ODDecodeFFmpegTask*> tasks;
+      std::vector<std::unique_ptr<ODDecodeFFmpegTask>> tasks;
       //append blockfiles to each stream and add an individual ODDecodeTask for each one.
-      for (int s = 0; s < mNumStreams; s++) {
-         ODDecodeFFmpegTask* odTask=new ODDecodeFFmpegTask(mScs,mNumStreams,mChannels,mFormatContext, s);
+      s = -1;
+      for (const auto &stream : mChannels) {
+         ++s;
+         auto odTask =
+            std::make_unique<ODDecodeFFmpegTask>(mScs, ODDecodeFFmpegTask::FromList(mChannels), mContext, s);
          odTask->CreateFileDecoder(mFilename);
 
          //each stream has different duration.  We need to know it if seeking is to be allowed.
          sampleCount sampleDuration = 0;
-         if (mScs[s]->m_stream->duration > 0)
-            sampleDuration = ((sampleCount)mScs[s]->m_stream->duration * mScs[s]->m_stream->time_base.num) *mScs[s]->m_stream->codec->sample_rate / mScs[s]->m_stream->time_base.den;
+         auto sc = scs[s].get();
+         if (sc->m_stream->duration > 0)
+            sampleDuration = ((sampleCount)sc->m_stream->duration * sc->m_stream->time_base.num) * sc->m_stream->codec->sample_rate / sc->m_stream->time_base.den;
          else
-            sampleDuration = ((sampleCount)mFormatContext->duration *mScs[s]->m_stream->codec->sample_rate) / AV_TIME_BASE;
+            sampleDuration = ((sampleCount)mFormatContext->duration *sc->m_stream->codec->sample_rate) / AV_TIME_BASE;
 
-         //      printf(" OD duration samples %qi, sr %d, secs %d\n",sampleDuration, (int)mScs[s]->m_stream->codec->sample_rate,(int)sampleDuration/mScs[s]->m_stream->codec->sample_rate);
+         //      wxPrintf(" OD duration samples %qi, sr %d, secs %d\n",sampleDuration, (int)sc->m_stream->codec->sample_rate, (int)sampleDuration/sc->m_stream->codec->sample_rate);
 
          //for each wavetrack within the stream add coded blockfiles
-         for (int c = 0; c < mScs[s]->m_stream->codec->channels; c++) {
-            WaveTrack *t = mChannels[s][c];
+         for (int c = 0; c < sc->m_stream->codec->channels; c++) {
+            auto t = stream[c];
             odTask->AddWaveTrack(t);
 
-            sampleCount maxBlockSize = t->GetMaxBlockSize();
+            auto maxBlockSize = t->GetMaxBlockSize();
             //use the maximum blockfile size to divide the sections (about 11secs per blockfile at 44.1khz)
-            for (sampleCount i = 0; i < sampleDuration; i += maxBlockSize) {
-               sampleCount blockLen = maxBlockSize;
-               if (i + blockLen > sampleDuration)
-                  blockLen = sampleDuration - i;
 
-               t->AppendCoded(mFilename, i, blockLen, c,ODTask::eODFFMPEG);
+            for (decltype(sampleDuration) i = 0; i < sampleDuration; i += maxBlockSize) {
+               const auto blockLen =
+                  limitSampleBufferSize( maxBlockSize, sampleDuration - i );
+
+               t->RightmostOrNewClip()->AppendBlockFile(
+                  [&]( wxFileNameWrapper filePath, size_t len ) {
+                     return make_blockfile<ODDecodeBlockFile>(
+                        std::move(filePath), wxFileNameWrapper{ mFilename },
+                        i, len, c, ODTask::eODFFMPEG);
+                  },
+                  blockLen
+               );
 
                // This only works well for single streams since we assume
                // each stream is of the same duration and channels
-               res = mProgress->Update(i+sampleDuration*c+ sampleDuration*mScs[s]->m_stream->codec->channels*s,
-                                       sampleDuration*mScs[s]->m_stream->codec->channels*mNumStreams);
-               if (res != eProgressSuccess)
+               res = mProgress->Update(
+                  (i+sampleDuration * c +
+                      sampleDuration*sc->m_stream->codec->channels * s
+                  ).as_long_long(),
+                  (sampleDuration *
+                      sc->m_stream->codec->channels * mNumStreams
+                  ).as_long_long()
+               );
+               if (res != ProgressResult::Success)
                   break;
             }
          }
-         tasks.push_back(odTask);
-      }
-      //Now we add the tasks and let them run, or delete them if the user cancelled
-      for(int i=0; i < (int)tasks.size(); i++) {
-         if(res==eProgressSuccess)
-            ODManager::Instance()->AddNewTask(tasks[i]);
-         else
-            {
-               delete tasks[i];
-            }
+         tasks.push_back(std::move(odTask));
       }
+      //Now we add the tasks and let them run, or DELETE them if the user cancelled
+      if (res == ProgressResult::Success)
+         for (int i = 0; i < (int)tasks.size(); i++)
+            ODManager::Instance()->AddNewTask(std::move(tasks[i]));
    } else {
 #endif
-   streamContext *sc = NULL;
 
    // Read next frame.
-   while ((sc = ReadNextFrame()) != NULL && (res == eProgressSuccess))
+   for (streamContext *sc; (sc = ReadNextFrame()) != NULL && (res == ProgressResult::Success);)
    {
       // ReadNextFrame returns 1 if stream is not to be imported
       if (sc != (streamContext*)1)
       {
          // Decode frame until it is not possible to decode any further
-         while (sc->m_pktRemainingSiz > 0 && (res == eProgressSuccess || res == eProgressStopped))
+         while (sc->m_pktRemainingSiz > 0 && (res == ProgressResult::Success || res == ProgressResult::Stopped))
          {
             if (DecodeFrame(sc,false) < 0)
                break;
@@ -634,28 +648,22 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
          }
 
          // Cleanup after frame decoding
-         if (sc->m_pktValid)
-         {
-            av_free_packet(&sc->m_pkt);
-            sc->m_pktValid = 0;
-         }
+         sc->m_pkt.reset();
       }
    }
 
    // Flush the decoders.
-   if ((mNumStreams != 0) && (res == eProgressSuccess || res == eProgressStopped))
+   if ((mNumStreams != 0) && (res == ProgressResult::Success || res == ProgressResult::Stopped))
    {
       for (int i = 0; i < mNumStreams; i++)
       {
-         if (DecodeFrame(mScs[i], true) == 0)
+         auto sc = scs[i].get();
+         sc->m_pkt.create();
+         if (DecodeFrame(sc, true) == 0)
          {
-            WriteData(mScs[i]);
+            WriteData(sc);
 
-            if (mScs[i]->m_pktValid)
-            {
-               av_free_packet(&mScs[i]->m_pkt);
-               mScs[i]->m_pktValid = 0;
-            }
+            sc->m_pkt.reset();
          }
       }
    }
@@ -664,39 +672,16 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
 #endif   //EXPERIMENTAL_OD_FFMPEG
 
    // Something bad happened - destroy everything!
-   if (res == eProgressCancelled || res == eProgressFailed)
-   {
-      for (int s = 0; s < mNumStreams; s++)
-      {
-         delete[] mChannels[s];
-      }
-      delete[] mChannels;
-
+   if (res == ProgressResult::Cancelled || res == ProgressResult::Failed)
       return res;
-   }
    //else if (res == 2), we just stop the decoding as if the file has ended
 
-   *outNumTracks = 0;
-   for (int s = 0; s < mNumStreams; s++)
-   {
-      *outNumTracks += mScs[s]->m_initialchannels;
-   }
-
-   // Create new tracks
-   *outTracks = new Track *[*outNumTracks];
-
    // Copy audio from mChannels to newly created tracks (destroying mChannels elements in process)
-   int trackindex = 0;
-   for (int s = 0; s < mNumStreams; s++)
-   {
-      for(int c = 0; c < mScs[s]->m_initialchannels; c++)
-      {
-         mChannels[s][c]->Flush();
-         (*outTracks)[trackindex++] = mChannels[s][c];
-      }
-      delete[] mChannels[s];
-   }
-   delete[] mChannels;
+   for (auto &stream : mChannels)
+      for(auto &channel : stream)
+         channel->Flush();
+
+   outTracks.swap(mChannels);
 
    // Save metadata
    WriteMetadata(tags);
@@ -706,7 +691,11 @@ int FFmpegImportFileHandle::Import(TrackFactory *trackFactory,
 
 streamContext *FFmpegImportFileHandle::ReadNextFrame()
 {
-   return import_ffmpeg_read_next_frame(mFormatContext, mScs, mNumStreams);
+   // Get pointer to array of contiguous unique_ptrs
+   auto scs = mScs->get();
+   // This reinterpret_cast to array of plain pointers is innocent
+   return import_ffmpeg_read_next_frame
+      (mFormatContext, reinterpret_cast<streamContext**>(scs), mNumStreams);
 }
 
 int FFmpegImportFileHandle::DecodeFrame(streamContext *sc, bool flushing)
@@ -714,13 +703,15 @@ int FFmpegImportFileHandle::DecodeFrame(streamContext *sc, bool flushing)
    return import_ffmpeg_decode_frame(sc, flushing);
 }
 
-int FFmpegImportFileHandle::WriteData(streamContext *sc)
+ProgressResult FFmpegImportFileHandle::WriteData(streamContext *sc)
 {
    // Find the stream index in mScs array
    int streamid = -1;
-   for (int i = 0; i < mNumStreams; i++)
+   auto iter = mChannels.begin();
+   auto scs = mScs->get();
+   for (int i = 0; i < mNumStreams; ++iter, ++i)
    {
-      if (mScs[i] == sc)
+      if (scs[i].get() == sc)
       {
          streamid = i;
          break;
@@ -729,25 +720,22 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
    // Stream is not found. This should not really happen
    if (streamid == -1)
    {
-      return 1;
+      return ProgressResult::Success;
    }
 
    // Allocate the buffer to store audio.
-   int insamples = sc->m_decodedAudioSamplesValidSiz / sc->m_samplesize;
-   int nChannels = sc->m_stream->codec->channels < sc->m_initialchannels ? sc->m_stream->codec->channels : sc->m_initialchannels;
-   uint8_t **tmp = (uint8_t **) malloc(sizeof(uint8_t *) * nChannels);
-   for (int chn = 0; chn < nChannels; chn++)
-   {
-      tmp[chn] = (uint8_t *) malloc(sc->m_osamplesize * (insamples / nChannels));
-   }
+   auto insamples = sc->m_decodedAudioSamplesValidSiz / sc->m_samplesize;
+   size_t nChannels = std::min(sc->m_stream->codec->channels, sc->m_initialchannels);
+
+   ArraysOf<uint8_t> tmp{ nChannels, sc->m_osamplesize * (insamples / nChannels) };
 
    // Separate the channels and convert input sample format to 16-bit
-   uint8_t *in = sc->m_decodedAudioSamples;
+   uint8_t *in = sc->m_decodedAudioSamples.get();
    int index = 0;
-   int pos = 0;
+   unsigned int pos = 0;
    while (pos < insamples)
    {
-      for (int chn = 0; chn < sc->m_stream->codec->channels; chn++)
+      for (size_t chn = 0; (int)chn < sc->m_stream->codec->channels; chn++)
       {
          if (chn < nChannels)
          {
@@ -755,37 +743,32 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
             {
                case AV_SAMPLE_FMT_U8:
                case AV_SAMPLE_FMT_U8P:
-                  ((int16_t *)tmp[chn])[index] = (int16_t) (*(uint8_t *)in - 0x80) << 8;
+                  ((int16_t *)tmp[chn].get())[index] = (int16_t) (*(uint8_t *)in - 0x80) << 8;
                break;
 
                case AV_SAMPLE_FMT_S16:
                case AV_SAMPLE_FMT_S16P:
-                  ((int16_t *)tmp[chn])[index] = (int16_t) *(int16_t *)in;
+                  ((int16_t *)tmp[chn].get())[index] = (int16_t) *(int16_t *)in;
                break;
 
                case AV_SAMPLE_FMT_S32:
                case AV_SAMPLE_FMT_S32P:
-                  ((float *)tmp[chn])[index] = (float) *(int32_t *)in * (1.0 / (1 << 31));
+                  ((float *)tmp[chn].get())[index] = (float) *(int32_t *)in * (1.0 / (1u << 31));
                break;
 
                case AV_SAMPLE_FMT_FLT:
                case AV_SAMPLE_FMT_FLTP:
-                  ((float *)tmp[chn])[index] = (float) *(float *)in;
+                  ((float *)tmp[chn].get())[index] = (float) *(float *)in;
                break;
 
                case AV_SAMPLE_FMT_DBL:
                case AV_SAMPLE_FMT_DBLP:
-                  ((float *)tmp[chn])[index] = (float) *(double *)in;
+                  ((float *)tmp[chn].get())[index] = (float) *(double *)in;
                break;
 
                default:
                   wxLogError(wxT("Stream %d has unrecognized sample format %d."), streamid, sc->m_samplefmt);
-                  for (int chn=0; chn < nChannels; chn++)
-                  {
-                     free(tmp[chn]);
-                  }
-                  free(tmp);
-                  return 1;
+                  return ProgressResult::Success;
                break;
             }
          }
@@ -796,21 +779,19 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
    }
 
    // Write audio into WaveTracks
-   for (int chn=0; chn < nChannels; chn++)
+   auto iter2 = iter->begin();
+   for (size_t chn=0; chn < nChannels; ++iter2, ++chn)
    {
-      mChannels[streamid][chn]->Append((samplePtr)tmp[chn],sc->m_osamplefmt,index);
-      free(tmp[chn]);
+      iter2->get()->Append((samplePtr)tmp[chn].get(), sc->m_osamplefmt, index);
    }
 
-   free(tmp);
-
    // Try to update the progress indicator (and see if user wants to cancel)
-   int updateResult = eProgressSuccess;
+   auto updateResult = ProgressResult::Success;
    int64_t filesize = avio_size(mFormatContext->pb);
    // PTS (presentation time) is the proper way of getting current position
-   if (sc->m_pkt.pts != int64_t(AV_NOPTS_VALUE) && mFormatContext->duration != int64_t(AV_NOPTS_VALUE))
+   if (sc->m_pkt->pts != int64_t(AV_NOPTS_VALUE) && mFormatContext->duration != int64_t(AV_NOPTS_VALUE))
    {
-      mProgressPos = sc->m_pkt.pts * sc->m_stream->time_base.num / sc->m_stream->time_base.den;
+      mProgressPos = sc->m_pkt->pts * sc->m_stream->time_base.num / sc->m_stream->time_base.den;
       mProgressLen = (mFormatContext->duration > 0 ? mFormatContext->duration / AV_TIME_BASE: 1);
    }
    // When PTS is not set, use number of frames and number of current frame
@@ -820,9 +801,9 @@ int FFmpegImportFileHandle::WriteData(streamContext *sc)
       mProgressLen = sc->m_stream->nb_frames;
    }
    // When number of frames is unknown, use position in file
-   else if (filesize > 0 && sc->m_pkt.pos > 0 && sc->m_pkt.pos <= filesize)
+   else if (filesize > 0 && sc->m_pkt->pos > 0 && sc->m_pkt->pos <= filesize)
    {
-      mProgressPos = sc->m_pkt.pos;
+      mProgressPos = sc->m_pkt->pos;
       mProgressLen = filesize;
    }
    updateResult = mProgress->Update(mProgressPos, mProgressLen != 0 ? mProgressLen : 1);
@@ -858,31 +839,11 @@ void FFmpegImportFileHandle::GetMetadata(Tags *tags, const wxChar *tag, const ch
 
 FFmpegImportFileHandle::~FFmpegImportFileHandle()
 {
-#ifdef EXPERIMENTAL_OD_FFMPEG
-   //ODDecodeFFmpegTask takes ownership and deltes it there.
-   if(!mUsingOD)
-   {
-#endif
-   if (FFmpegLibsInst->ValidLibsLoaded())
-   {
-      if (mFormatContext) avformat_close_input(&mFormatContext);
+   if (FFmpegLibsInst()->ValidLibsLoaded())
       av_log_set_callback(av_log_default_callback);
-   }
 
-   for (int i = 0; i < mNumStreams; i++)
-   {
-      if (mScs[i]->m_decodedAudioSamples != NULL)
-         av_free(mScs[i]->m_decodedAudioSamples);
-
-      delete mScs[i];
-   }
-   free(mScs);
-#ifdef EXPERIMENTAL_OD_FFMPEG
-   }//mUsingOD
-#endif
-
-
-   delete mStreamInfo;
+   // Do this before unloading the libraries
+   mContext.reset();
 
    DropFFmpegLibs();
 }
