@@ -11,19 +11,22 @@
 #ifndef __AUDACITY_SPECPOWERMETER_H_
 #define __AUDACITY_SPECPOWERMETER_H_
 
-class SpecPowerMeter
+#include <cstddef>
+#include "../SampleFormat.h"
+
+class SpecPowerCalculation
 {
-   int mSigLen;
+   const size_t mSigLen;
    
-   float* mSigI;
-   float* mSigFR;
-   float* mSigFI;
+   Floats mSigI;
+   Floats mSigFR;
+   Floats mSigFI;
 
    float CalcBinPower(float* sig_f_r, float* sig_f_i, int loBin, int hiBin);
    int Freq2Bin(float fc);
 public:
-   SpecPowerMeter(int sigLen);
-   ~SpecPowerMeter();
+   SpecPowerCalculation(size_t sigLen);
+   ~SpecPowerCalculation();
    
    float CalcPower(float* sig, float fc, float bw);
 };
