@@ -1,8 +1,12 @@
 #include "../Audacity.h"
 
-#include "../Menus.h"
+#include "../CommonCommandFlags.h"
 #include "../Prefs.h"
 #include "../Project.h"
+#include "../ProjectHistory.h"
+#include "../ProjectWindow.h"
+#include "../Track.h"
+#include "../SelectionState.h"
 #include "../TrackPanel.h"
 #include "../TrackPanelAx.h"
 #include "../commands/CommandContext.h"
@@ -25,16 +29,16 @@ void NextOrPrevFrame(AudacityProject &project, bool forward)
    auto temp2 = ASlider::TemporarilyAllowFocus();
    auto temp3 = MeterPanel::TemporarilyAllowFocus();
 
-   auto toolManager = project.GetToolManager();
-   auto botDock = toolManager->GetBotDock();
+   auto &toolManager = ToolManager::Get( project );
+   auto botDock = toolManager.GetBotDock();
 
 
    // Define the set of windows we rotate among.
    static const unsigned rotationSize = 3u;
 
    wxWindow *const begin [rotationSize] = {
-      project.GetTopPanel(),
-      project.GetTrackPanel(),
+      ProjectWindow::Get( project ).GetTopPanel(),
+      &TrackPanel::Get( project ),
       botDock,
    };
 
@@ -65,7 +69,7 @@ void NextOrPrevFrame(AudacityProject &project, bool forward)
       wxWindow *toFocus = begin[idx2];
       bool bIsAnEmptyDock=false;
       if( idx2 != 1 )
-         bIsAnEmptyDock = ((idx2==0) ? toolManager->GetTopDock() : botDock)->
+         bIsAnEmptyDock = ((idx2==0) ? toolManager.GetTopDock() : botDock)->
          GetChildren().GetCount() < 1;
 
       // Skip docks that are empty (Bug 1564).
@@ -82,17 +86,18 @@ void NextOrPrevFrame(AudacityProject &project, bool forward)
 void DoPrevTrack(
    AudacityProject &project, bool shift, bool circularTrackNavigation )
 {
-   auto trackPanel = project.GetTrackPanel();
-   auto tracks = project.GetTracks();
-   auto &selectionState = project.GetSelectionState();
+   auto &projectHistory = ProjectHistory::Get( project );
+   auto &trackFocus = TrackFocus::Get( project );
+   auto &tracks = TrackList::Get( project );
+   auto &selectionState = SelectionState::Get( project );
 
-   Track* t = trackPanel->GetFocusedTrack();
+   auto t = trackFocus.Get();
    if( t == NULL )   // if there isn't one, focus on last
    {
-      t = *tracks->Any().rbegin();
-      trackPanel->SetFocusedTrack( t );
-      trackPanel->EnsureVisible( t );
-      project.ModifyState(false);
+      t = *tracks.Any().rbegin();
+      trackFocus.Set( t );
+      if (t)
+         t->EnsureVisible( true );
       return;
    }
 
@@ -101,17 +106,17 @@ void DoPrevTrack(
    bool pSelected = false;
    if( shift )
    {
-      p = * -- tracks->FindLeader( t ); // Get previous track
+      p = * -- tracks.FindLeader( t ); // Get previous track
       if( p == NULL )   // On first track
       {
          // JKC: wxBell() is probably for accessibility, so a blind
          // user knows they were at the top track.
          wxBell();
          if( circularTrackNavigation )
-            p = *tracks->Any().rbegin();
+            p = *tracks.Any().rbegin();
          else
          {
-            trackPanel->EnsureVisible( t );
+            t->EnsureVisible();
             return;
          }
       }
@@ -122,65 +127,64 @@ void DoPrevTrack(
       {
          selectionState.SelectTrack
             ( *t, false, false );
-         trackPanel->SetFocusedTrack( p );   // move focus to next track up
-         trackPanel->EnsureVisible( p );
-         project.ModifyState(false);
+         trackFocus.Set( p );   // move focus to next track up
+         if (p)
+            p->EnsureVisible( true );
          return;
       }
       if( tSelected && !pSelected )
       {
          selectionState.SelectTrack
             ( *p, true, false );
-         trackPanel->SetFocusedTrack( p );   // move focus to next track up
-         trackPanel->EnsureVisible( p );
-         project.ModifyState(false);
+         trackFocus.Set( p );   // move focus to next track up
+         if (p)
+            p->EnsureVisible( true );
          return;
       }
       if( !tSelected && pSelected )
       {
          selectionState.SelectTrack
             ( *p, false, false );
-         trackPanel->SetFocusedTrack( p );   // move focus to next track up
-         trackPanel->EnsureVisible( p );
-         project.ModifyState(false);
+         trackFocus.Set( p );   // move focus to next track up
+         if (p)
+            p->EnsureVisible( true );
          return;
       }
       if( !tSelected && !pSelected )
       {
          selectionState.SelectTrack
             ( *t, true, false );
-         trackPanel->SetFocusedTrack( p );   // move focus to next track up
-         trackPanel->EnsureVisible( p );
-          project.ModifyState(false);
+         trackFocus.Set( p );   // move focus to next track up
+         if (p)
+            p->EnsureVisible( true );
          return;
       }
    }
    else
    {
-      p = * -- tracks->FindLeader( t ); // Get previous track
+      p = * -- tracks.FindLeader( t ); // Get previous track
       if( p == NULL )   // On first track so stay there?
       {
          wxBell();
          if( circularTrackNavigation )
          {
-            auto range = tracks->Leaders();
+            auto range = tracks.Leaders();
             p = * range.rbegin(); // null if range is empty
-            trackPanel->SetFocusedTrack( p );   // Wrap to the last track
-            trackPanel->EnsureVisible( p );
-             project.ModifyState(false);
+            trackFocus.Set( p );   // Wrap to the last track
+            if (p)
+               p->EnsureVisible( true );
             return;
          }
          else
          {
-            trackPanel->EnsureVisible( t );
+            t->EnsureVisible();
             return;
          }
       }
       else
       {
-         trackPanel->SetFocusedTrack( p );   // move focus to next track up
-         trackPanel->EnsureVisible( p );
-         project.ModifyState(false);
+         trackFocus.Set( p );   // move focus to next track up
+         p->EnsureVisible( true );
          return;
       }
    }
@@ -192,31 +196,32 @@ void DoPrevTrack(
 void DoNextTrack(
    AudacityProject &project, bool shift, bool circularTrackNavigation )
 {
-   auto trackPanel = project.GetTrackPanel();
-   auto tracks = project.GetTracks();
-   auto &selectionState = project.GetSelectionState();
+   auto &projectHistory = ProjectHistory::Get( project );
+   auto &trackFocus = TrackFocus::Get( project );
+   auto &tracks = TrackList::Get( project );
+   auto &selectionState = SelectionState::Get( project );
 
-   auto t = trackPanel->GetFocusedTrack();   // Get currently focused track
+   auto t = trackFocus.Get();   // Get currently focused track
    if( t == NULL )   // if there isn't one, focus on first
    {
-      t = *tracks->Any().begin();
-      trackPanel->SetFocusedTrack( t );
-      trackPanel->EnsureVisible( t );
-      project.ModifyState(false);
+      t = *tracks.Any().begin();
+      trackFocus.Set( t );
+      if (t)
+         t->EnsureVisible( true );
       return;
    }
 
    if( shift )
    {
-      auto n = * ++ tracks->FindLeader( t ); // Get next track
+      auto n = * ++ tracks.FindLeader( t ); // Get next track
       if( n == NULL )   // On last track so stay there
       {
          wxBell();
          if( circularTrackNavigation )
-            n = *tracks->Any().begin();
+            n = *tracks.Any().begin();
          else
          {
-            trackPanel->EnsureVisible( t );
+            t->EnsureVisible();
             return;
          }
       }
@@ -226,64 +231,63 @@ void DoNextTrack(
       {
          selectionState.SelectTrack
             ( *t, false, false );
-         trackPanel->SetFocusedTrack( n );   // move focus to next track down
-         trackPanel->EnsureVisible( n );
-         project.ModifyState(false);
+         trackFocus.Set( n );   // move focus to next track down
+         if (n)
+            n->EnsureVisible( true );
          return;
       }
       if( tSelected && !nSelected )
       {
          selectionState.SelectTrack
             ( *n, true, false );
-         trackPanel->SetFocusedTrack( n );   // move focus to next track down
-         trackPanel->EnsureVisible( n );
-         project.ModifyState(false);
+         trackFocus.Set( n );   // move focus to next track down
+         if (n)
+            n->EnsureVisible( true );
          return;
       }
       if( !tSelected && nSelected )
       {
          selectionState.SelectTrack
             ( *n, false, false );
-         trackPanel->SetFocusedTrack( n );   // move focus to next track down
-         trackPanel->EnsureVisible( n );
-         project.ModifyState(false);
+         trackFocus.Set( n );   // move focus to next track down
+         if (n)
+            n->EnsureVisible( true );
          return;
       }
       if( !tSelected && !nSelected )
       {
          selectionState.SelectTrack
             ( *t, true, false );
-         trackPanel->SetFocusedTrack( n );   // move focus to next track down
-         trackPanel->EnsureVisible( n );
-         project.ModifyState(false);
+         trackFocus.Set( n );   // move focus to next track down
+         if (n)
+            n->EnsureVisible( true );
          return;
       }
    }
    else
    {
-      auto n = * ++ tracks->FindLeader( t ); // Get next track
+      auto n = * ++ tracks.FindLeader( t ); // Get next track
       if( n == NULL )   // On last track so stay there
       {
          wxBell();
          if( circularTrackNavigation )
          {
-            n = *tracks->Any().begin();
-            trackPanel->SetFocusedTrack( n );   // Wrap to the first track
-            trackPanel->EnsureVisible( n );
-            project.ModifyState(false);
+            n = *tracks.Any().begin();
+            trackFocus.Set( n );   // Wrap to the first track
+            if (n)
+               n->EnsureVisible( true );
             return;
          }
          else
          {
-            trackPanel->EnsureVisible( t );
+            t->EnsureVisible();
             return;
          }
       }
       else
       {
-         trackPanel->SetFocusedTrack( n );   // move focus to next track down
-         trackPanel->EnsureVisible( n );
-         project.ModifyState(false);
+         trackFocus.Set( n );   // move focus to next track down
+         n->EnsureVisible( true );
          return;
       }
    }
@@ -301,21 +305,23 @@ namespace NavigationActions {
 
 struct Handler
    : CommandHandlerObject // MUST be the first base class!
+   , ClientData::Base
    , PrefsListener
 {
 
 void OnPrevWindow(const CommandContext &context)
 {
    auto &project = context.project;
-   auto isEnabled = project.IsEnabled();
+   auto &window = GetProjectFrame( project );
+   auto isEnabled = window.IsEnabled();
 
    wxWindow *w = wxGetTopLevelParent(wxWindow::FindFocus());
-   const auto & list = project.GetChildren();
+   const auto & list = window.GetChildren();
    auto iter = list.rbegin(), end = list.rend();
 
    // If the project window has the current focus, start the search with the
    // last child
-   if (w == &project)
+   if (w == &window)
    {
    }
    // Otherwise start the search with the current window's previous sibling
@@ -342,7 +348,7 @@ void OnPrevWindow(const CommandContext &context)
    // Ran out of siblings, so make the current project active
    if ((iter == end) && isEnabled)
    {
-      w = &project;
+      w = &window;
    }
 
    // And make sure it's on top (only for floating windows...project window will
@@ -365,15 +371,16 @@ void OnPrevWindow(const CommandContext &context)
 void OnNextWindow(const CommandContext &context)
 {
    auto &project = context.project;
-   auto isEnabled = project.IsEnabled();
+   auto &window = GetProjectFrame( project );
+   auto isEnabled = window.IsEnabled();
 
    wxWindow *w = wxGetTopLevelParent(wxWindow::FindFocus());
-   const auto & list = project.GetChildren();
+   const auto & list = window.GetChildren();
    auto iter = list.begin(), end = list.end();
 
    // If the project window has the current focus, start the search with the
    // first child
-   if (w == &project)
+   if (w == &window)
    {
    }
    // Otherwise start the search with the current window's next sibling
@@ -404,7 +411,7 @@ void OnNextWindow(const CommandContext &context)
    // Ran out of siblings, so make the current project active
    if ((iter == end) && isEnabled)
    {
-      w = &project;
+      w = &window;
    }
 
    // And make sure it's on top (only for floating windows...project window will
@@ -454,39 +461,35 @@ void OnCursorDown(const CommandContext &context)
 void OnFirstTrack(const CommandContext &context)
 {
    auto &project = context.project;
-   auto trackPanel = project.GetTrackPanel();
-   auto tracks = project.GetTracks();
+   auto &trackFocus = TrackFocus::Get( project );
+   auto &tracks = TrackList::Get( project );
 
-   Track *t = trackPanel->GetFocusedTrack();
+   auto t = trackFocus.Get();
    if (!t)
       return;
 
-   auto f = *tracks->Any().begin();
+   auto f = *tracks.Any().begin();
    if (t != f)
-   {
-      trackPanel->SetFocusedTrack(f);
-      project.ModifyState(false);
-   }
-   trackPanel->EnsureVisible(f);
+      trackFocus.Set(f);
+   if (f)
+      f->EnsureVisible( t != f );
 }
 
 void OnLastTrack(const CommandContext &context)
 {
    auto &project = context.project;
-   auto trackPanel = project.GetTrackPanel();
-   auto tracks = project.GetTracks();
+   auto &trackFocus = TrackFocus::Get( project );
+   auto &tracks = TrackList::Get( project );
 
-   Track *t = trackPanel->GetFocusedTrack();
+   Track *t = trackFocus.Get();
    if (!t)
       return;
 
-   auto l = *tracks->Any().rbegin();
+   auto l = *tracks.Any().rbegin();
    if (t != l)
-   {
-      trackPanel->SetFocusedTrack(l);
-      project.ModifyState(false);
-   }
-   trackPanel->EnsureVisible(l);
+      trackFocus.Set(l);
+   if (l)
+      l->EnsureVisible( t != l );
 }
 
 void OnShiftUp(const CommandContext &context)
@@ -504,21 +507,20 @@ void OnShiftDown(const CommandContext &context)
 void OnToggle(const CommandContext &context)
 {
    auto &project = context.project;
-   auto trackPanel = project.GetTrackPanel();
-   auto &selectionState = project.GetSelectionState();
+   auto &trackFocus = TrackFocus::Get( project );
+   auto &selectionState = SelectionState::Get( project );
 
    Track *t;
 
-   t = trackPanel->GetFocusedTrack();   // Get currently focused track
+   t = trackFocus.Get();   // Get currently focused track
    if (!t)
       return;
 
    selectionState.SelectTrack
       ( *t, !t->GetSelected(), true );
-   trackPanel->EnsureVisible( t );
-   project.ModifyState(false);
+   t->EnsureVisible( true );
 
-   trackPanel->GetAx().Updated();
+   trackFocus.UpdateAccessibility();
 
    return;
 }
@@ -532,6 +534,8 @@ Handler()
 {
    UpdatePrefs();
 }
+Handler( const Handler & ) PROHIBITED;
+Handler &operator=( const Handler & ) PROHIBITED;
 
 }; // struct Handler
 
@@ -539,12 +543,12 @@ Handler()
 
 // Handler is stateful.  Needs a factory registered with
 // AudacityProject.
-static const AudacityProject::RegisteredAttachedObjectFactory factory{ []{
-   return std::make_unique< NavigationActions::Handler >();
-} };
+static const AudacityProject::AttachedObjects::RegisteredFactory key{
+   [](AudacityProject&) {
+      return std::make_unique< NavigationActions::Handler >(); } };
+
 static CommandHandlerObject &findCommandHandler(AudacityProject &project) {
-   return static_cast<NavigationActions::Handler&>(
-      project.GetAttachedObject(factory));
+   return project.AttachedObjects::Get< NavigationActions::Handler >( key );
 };
 
 // Menu definitions
@@ -571,7 +575,7 @@ MenuTable::BaseItemPtr ExtraGlobalCommands( AudacityProject & )
 MenuTable::BaseItemPtr ExtraFocusMenu( AudacityProject & )
 {
    using namespace MenuTable;
-   constexpr auto FocusedTracksFlags = TracksExistFlag | TrackPanelHasFocus;
+   static const auto FocusedTracksFlags = TracksExistFlag | TrackPanelHasFocus;
 
    return Menu( _("F&ocus"),
       Command( wxT("PrevFrame"),
