@@ -9,8 +9,12 @@
 
 ******************************************************************/
 
-#include "../Audacity.h"   // needed before FFmpeg.h
+#include "../Audacity.h"   // needed before FFmpeg.h // for USE_* macros
+#include "ODDecodeFFmpegTask.h"
 
+#include "../Experimental.h"
+
+#include <wx/crt.h>
 #include <wx/wxprec.h>
 // For compilers that support precompilation, includes "wx/wx.h".
 #ifndef WX_PRECOMP
@@ -18,17 +22,16 @@
 #include <wx/window.h>
 #endif
 
-#include "../Experimental.h"
-
 #ifdef USE_FFMPEG
 #ifdef EXPERIMENTAL_OD_FFMPEG
 
+#include <algorithm>
+#include <functional>
+
 #include "../FFmpeg.h"      // which brings in avcodec.h, avformat.h
-#include "../import/ImportFFmpeg.h"
 
 
-extern FFmpegLibs *FFmpegLibsInst;
-#include "ODDecodeFFmpegTask.h"
+extern FFmpegLibs *FFmpegLibsInst();
 
 
 #define ODFFMPEG_SEEKING_TEST_UNKNOWN 0
@@ -38,27 +41,29 @@ extern FFmpegLibs *FFmpegLibsInst;
 #define kMaxSamplesInCache 4410000
 
 //struct for caching the decoded samples to be used over multiple blockfiles
-typedef struct _FFmpegDecodeCache
+struct FFMpegDecodeCache
 {
-   uint8_t* samplePtr;//interleaved samples
+   ArrayOf<uint8_t> samplePtr;//interleaved samples
    sampleCount start;
-   sampleCount len;
-   int         numChannels;
+   size_t         len;
+   unsigned    numChannels;
    AVSampleFormat samplefmt; // input (from libav) sample format
 
-} FFMpegDecodeCache;
+};
 
 
 //------ ODFFmpegDecoder declaration and defs - here because we strip dependencies from .h files
 
 ///class to decode a particular file (one per file).  Saves info such as filename and length (after the header is read.)
-class ODFFmpegDecoder:public ODFileDecoder
+class ODFFmpegDecoder final : public ODFileDecoder
 {
-
-
 public:
    ///This should handle unicode converted to UTF-8 on mac/linux, but OD TODO:check on windows
-   ODFFmpegDecoder(const wxString & fileName, streamContext** scs, int numStreams,WaveTrack*** channels, AVFormatContext* formatContext, int streamIndex);
+   ODFFmpegDecoder(const wxString & fileName,
+      const ScsPtr &scs,
+      ODDecodeFFmpegTask::Streams &&channels,
+      const std::shared_ptr<FFmpegContext> &formatContext,
+      int streamIndex);
    virtual ~ODFFmpegDecoder();
 
    ///Decodes the samples for this blockfile from the real file into a float buffer.
@@ -68,19 +73,19 @@ public:
    ///this->ReadData(sampleData, floatSample, 0, mLen);
    ///This class should call ReadHeader() first, so it knows the length, and can prepare
    ///the file object if it needs to.
-   virtual int Decode(samplePtr & data, sampleFormat & format, sampleCount start, sampleCount len, unsigned int channel);
+   int Decode(SampleBuffer & data, sampleFormat & format, sampleCount start, size_t len, unsigned int channel) override;
 
    ///This is a must implement abstract virtual in the superclass.
    ///However it doesn't do anything because ImportFFMpeg does all that for us.
-   virtual bool ReadHeader() {return true;}
+   bool ReadHeader() override {return true;}
 
-   bool SeekingAllowed() ;
+   bool SeekingAllowed() override;
 
 private:
-   void InsertCache(FFMpegDecodeCache* cache);
+   void InsertCache(std::unique_ptr<FFMpegDecodeCache> &&cache);
 
    //puts the actual audio samples into the blockfile's data array
-   int FillDataFromCache(samplePtr & data, sampleFormat outFormat, sampleCount & start, sampleCount& len, unsigned int channel);
+   int FillDataFromCache(samplePtr & data, sampleFormat outFormat, sampleCount & start, size_t& len, unsigned int channel);
          ///REFACTORABLE CODE FROM IMPORT FFMPEG
       ///! Reads next audio frame
    ///\return pointer to the stream context structure to which the frame belongs to or NULL on error, or 1 if stream is not to be imported.
@@ -92,46 +97,50 @@ private:
    ///\return 0 on success, -1 if it can't decode any further
    int DecodeFrame(streamContext *sc, bool flushing);
 
-   int                  mNumStreams;
-   streamContext       **mScs;           //!< Array of pointers to stream contexts. Length is mNumStreams.
-   WaveTrack***        mChannels;
-   AVFormatContext      *mFormatContext; //!< Format description, also contains metadata and some useful info
-   std::vector<FFMpegDecodeCache*> mDecodeCache;
+   ScsPtr mScs;           //!< Pointer to array of pointers to stream contexts.
+   ODDecodeFFmpegTask::Streams mChannels;
+   std::shared_ptr<FFmpegContext> mContext; //!< Format description, also contains metadata and some useful info
+   std::vector<std::unique_ptr<FFMpegDecodeCache>> mDecodeCache;
    int                  mNumSamplesInCache;
    sampleCount                  mCurrentPos;     //the index of the next sample to be decoded
-   sampleCount                  mCurrentLen;     //length of the last packet decoded
+   size_t                       mCurrentLen;     //length of the last packet decoded
 
    bool                 mSeekingAllowedStatus;
    int                  mStreamIndex;
 };
 
+auto ODDecodeFFmpegTask::FromList( const TrackHolders &channels ) -> Streams
+{
+   // Convert array of array of unique_ptr to array of array of bare pointers
+   return transform_container<Streams>( channels,
+      [](const NewChannelGroup &holders) {
+         return transform_container<Channels>( holders,
+            std::mem_fn(&NewChannelGroup::value_type::get) );
+      }
+   );
+}
 
 //------ ODDecodeFFmpegTask definitions
-ODDecodeFFmpegTask::ODDecodeFFmpegTask(void* scs,int numStreams, WaveTrack*** channels, void* formatContext, int streamIndex)
+ODDecodeFFmpegTask::ODDecodeFFmpegTask(const ScsPtr &scs, Streams &&channels, const std::shared_ptr<FFmpegContext> &context, int streamIndex)
+   : mChannels(std::move(channels))
 {
    mScs=scs;
-   mNumStreams=numStreams;
-   mChannels=channels;
-   mFormatContext = formatContext;
+   mContext = context;
    mStreamIndex = streamIndex;
-   //TODO we probably need to create a new WaveTrack*** pointer and copy.
-   //same for streamContext, but we should also use a ref counting system - this should be added to streamContext
- //  mScs = (streamContext**)malloc(sizeof(streamContext**)*mFormatContext->nb_streams);
 }
 ODDecodeFFmpegTask::~ODDecodeFFmpegTask()
 {
 }
 
 
-ODTask* ODDecodeFFmpegTask::Clone()
+std::unique_ptr<ODTask> ODDecodeFFmpegTask::Clone() const
 {
-   //we need to create copies of mScs.  It would be better to use a reference counter system.
-
-   ODDecodeFFmpegTask* clone = new ODDecodeFFmpegTask((void*)mScs,mNumStreams,mChannels,mFormatContext,mStreamIndex);
+   auto clone = std::make_unique<ODDecodeFFmpegTask>(mScs, Streams{ mChannels }, mContext, mStreamIndex);
    clone->mDemandSample=GetDemandSample();
 
    //the decoders and blockfiles should not be copied.  They are created as the task runs.
-   return clone;
+   // This std::move is needed to "upcast" the pointer type
+   return std::move(clone);
 }
 
 ///Creates an ODFileDecoder that decodes a file of filetype the subclass handles.
@@ -140,10 +149,12 @@ ODTask* ODDecodeFFmpegTask::Clone()
 ODFileDecoder* ODDecodeFFmpegTask::CreateFileDecoder(const wxString & fileName)
 {
    // Open the file for import
-   ODFFmpegDecoder *decoder = new ODFFmpegDecoder(fileName, (streamContext**) mScs,mNumStreams,mChannels,(AVFormatContext*)mFormatContext, mStreamIndex);
+   auto decoder =
+      std::make_unique<ODFFmpegDecoder>(fileName, mScs, ODDecodeFFmpegTask::Streams{ mChannels },
+      mContext, mStreamIndex);
 
-   mDecoders.push_back(decoder);
-   return decoder;
+   mDecoders.push_back(std::move(decoder));
+   return mDecoders.back().get();
 
 }
 
@@ -185,17 +196,19 @@ bool ODFFmpegDecoder::SeekingAllowed()
    if(!audioStreamExists)
       goto test_failed;
    //TODO: now try a seek and see if dts/pts (decode/presentation timestamp) is updated as we expected it to be.
-   //This should be done using a new AVFormatContext clone so that we don't ruin the file pointer if we fail.
+   //This should be done using a NEW AVFormatContext clone so that we don't ruin the file pointer if we fail.
 //   url_fseek(mFormatContext->pb,0,SEEK_SET);
 
 
+   std::unique_ptr<FFmpegContext> tempMpegContext;
    AVFormatContext* tempContext;
    int err;
-   err = ufile_fopen_input(&tempContext, mFName);
+   err = ufile_fopen_input(tempMpegContext, mFName);
    if (err < 0)
    {
       goto test_failed;
    }
+   tempContext = tempMpegContext->ic_ptr;
 
    err = avformat_find_stream_info(tempContext, NULL);
    if (err < 0)
@@ -205,12 +218,9 @@ bool ODFFmpegDecoder::SeekingAllowed()
 
    if(av_seek_frame(tempContext, st->index, 0, 0) >= 0) {
       mSeekingAllowedStatus = ODFFMPEG_SEEKING_TEST_SUCCESS;
-      if (tempContext) avformat_close_input(&tempContext);
       return SeekingAllowed();
    }
 
-   if (tempContext) avformat_close_input(&tempContext);
-
 test_failed:
    mSeekingAllowedStatus = ODFFMPEG_SEEKING_TEST_FAILED;
    return SeekingAllowed();
@@ -219,12 +229,15 @@ test_failed:
 
 
 //------ ODDecodeFFmpegFileDecoder
-ODFFmpegDecoder::ODFFmpegDecoder(const wxString & fileName, streamContext** scs,int numStreams,WaveTrack*** channels, AVFormatContext* formatContext, int streamIndex)
+ODFFmpegDecoder::ODFFmpegDecoder(const wxString & fileName,
+   const ScsPtr &scs,
+   ODDecodeFFmpegTask::Streams &&channels,
+   const std::shared_ptr<FFmpegContext> &context,
+int streamIndex)
 :ODFileDecoder(fileName),
 //mSamplesDone(0),
-mNumStreams(numStreams),
 mScs(scs),
-mFormatContext(formatContext),
+mContext(context),
 mNumSamplesInCache(0),
 mCurrentLen(0),
 mSeekingAllowedStatus(ODFFMPEG_SEEKING_TEST_UNKNOWN),
@@ -233,61 +246,31 @@ mStreamIndex(streamIndex)
    PickFFmpegLibs();
 
    //do a shallow copy of the 2d array.
-   mChannels = new WaveTrack **[mNumStreams];
+   mChannels = std::move(channels);
 
-   for (int s = 0; s < mNumStreams; s++)
-   {
-      mChannels[s] = new WaveTrack *[mScs[s]->m_stream->codec->channels];
-      int c;
-      for (c = 0; c < mScs[s]->m_stream->codec->channels; c++)
-      {
-         mChannels[s][c] = channels[s][c];
-      }
-   }
    // get the current stream start time.
    int64_t stream_delay = 0;
-   if (mScs[streamIndex]->m_stream->start_time != int64_t(AV_NOPTS_VALUE) &&
-       mScs[streamIndex]->m_stream->start_time > 0) {
-      stream_delay = mScs[streamIndex]->m_stream->start_time;
+   const auto sc = mScs->get()[streamIndex].get();
+   if (sc->m_stream->start_time != int64_t(AV_NOPTS_VALUE) &&
+       sc->m_stream->start_time > 0) {
+      stream_delay = sc->m_stream->start_time;
    }
-   mCurrentPos = double(stream_delay) / AV_TIME_BASE;
+   mCurrentPos = sampleCount{ double(stream_delay) / AV_TIME_BASE };
 
    //TODO: add a ref counter to scs?  This will be necessary if we want to allow copy and paste of not-yet decoded
    //ODDecodeBlockFiles that point to FFmpeg files.
 }
 
-//we have taken ownership, so delete the ffmpeg stuff allocated in ImportFFmpeg that was given to us.
+//we have taken ownership, so DELETE the ffmpeg stuff allocated in ImportFFmpeg that was given to us.
 ODFFmpegDecoder::~ODFFmpegDecoder()
 {
-   if (FFmpegLibsInst->ValidLibsLoaded())
-   {
-      if (mFormatContext) avformat_close_input(&mFormatContext);
-      av_log_set_callback(av_log_default_callback);
-   }
+   // Do this before unloading libraries
+   mContext.reset();
 
-   for (int i = 0; i < mNumStreams; i++)
-   {
-      if (mScs[i]->m_decodedAudioSamples != NULL)
-         av_free(mScs[i]->m_decodedAudioSamples);
-
-      delete mScs[i];
-   }
-   free(mScs);
-
-   //delete our caches.
+   //DELETE our caches.
    while(mDecodeCache.size())
-   {
-      free(mDecodeCache[0]->samplePtr);
-      delete mDecodeCache[0];
       mDecodeCache.erase(mDecodeCache.begin());
-   }
 
-   //free the channel pointer arrays
-   for (int s = 0; s < mNumStreams; s++)
-   {
-      delete[] mChannels[s];
-   }
-   delete[] mChannels;
    DropFFmpegLibs();
 }
 
@@ -296,31 +279,35 @@ ODFFmpegDecoder::~ODFFmpegDecoder()
 #define kDecodeSampleAllowance 400000
 //number of jump backwards seeks
 #define kMaxSeekRewindAttempts 8
-int ODFFmpegDecoder::Decode(samplePtr & data, sampleFormat & format, sampleCount start, sampleCount len, unsigned int channel)
+int ODFFmpegDecoder::Decode(SampleBuffer & data, sampleFormat & format, sampleCount start, size_t len, unsigned int channel)
 {
-   format = mScs[mStreamIndex]->m_osamplefmt;
+   auto mFormatContext = mContext->ic_ptr;
 
-   data = NewSamples(len, format);
-   samplePtr bufStart = data;
+   auto scs = mScs->get();
+   auto sci = scs[mStreamIndex].get();
+   format = sci->m_osamplefmt;
+
+   data.Allocate(len, format);
+   samplePtr bufStart = data.ptr();
    streamContext* sc = NULL;
 
-   // printf("start %llu len %llu\n", start, len);
+   // wxPrintf("start %llu len %lu\n", start, len);
    //TODO update this to work with seek - this only works linearly now.
    if(mCurrentPos > start && mCurrentPos  <= start+len + kDecodeSampleAllowance)
    {
-      //this next call takes data, start and len as reference variables and updates them to reflect the new area that is needed.
+      //this next call takes data, start and len as reference variables and updates them to reflect the NEW area that is needed.
       FillDataFromCache(bufStart, format, start,len,channel);
    }
 
    bool seeking = false;
    //look at the decoding timestamp and see if the next sample that will be decoded is not the next sample we need.
    if(len && (mCurrentPos > start + len  || mCurrentPos + kDecodeSampleAllowance < start ) && SeekingAllowed()) {
-      sc = mScs[mStreamIndex];
+      sc = sci;
       AVStream* st = sc->m_stream;
       int stindex = -1;
       uint64_t targetts;
 
-      //printf("attempting seek to %llu\n", start);
+      //wxPrintf("attempting seek to %llu\n", start);
       //we have to find the index for this stream.
       for (unsigned int i = 0; i < mFormatContext->nb_streams; i++) {
          if (mFormatContext->streams[i] == sc->m_stream )
@@ -333,29 +320,31 @@ int ODFFmpegDecoder::Decode(samplePtr & data, sampleFormat & format, sampleCount
          mCurrentPos = start+len +1;
          while(numAttempts++ < kMaxSeekRewindAttempts && mCurrentPos > start) {
             //we want to move slightly before the start of the block file, but not too far ahead
-            targetts = (start-kDecodeSampleAllowance*numAttempts/kMaxSeekRewindAttempts)  * ((double)st->time_base.den/(st->time_base.num * st->codec->sample_rate ));
-            if(targetts<0)
-               targetts=0;
+            targetts = std::max( 0.0,
+               (start - kDecodeSampleAllowance * numAttempts / kMaxSeekRewindAttempts)
+                  .as_long_long() *
+               ((double)st->time_base.den/(st->time_base.num * st->codec->sample_rate ))
+            );
 
-            //printf("attempting seek to %llu, attempts %d\n", targetts, numAttempts);
+            //wxPrintf("attempting seek to %llu, attempts %d\n", targetts, numAttempts);
             if(av_seek_frame(mFormatContext,stindex,targetts,0) >= 0){
                //find out the dts we've seekd to.
-               sampleCount actualDecodeStart = 0.5 + st->codec->sample_rate * st->cur_dts  * ((double)st->time_base.num/st->time_base.den);      //this is mostly safe because den is usually 1 or low number but check for high values.
+               sampleCount actualDecodeStart { 0.5 + st->codec->sample_rate * st->cur_dts  * ((double)st->time_base.num/st->time_base.den) };      //this is mostly safe because den is usually 1 or low number but check for high values.
 
                mCurrentPos = actualDecodeStart;
                seeking = true;
 
                //if the seek was past our desired position, rewind a bit.
-               //printf("seek ok to %llu samps, float: %f\n",actualDecodeStart,actualDecodeStartDouble);
+               //wxPrintf("seek ok to %llu samps, float: %f\n",actualDecodeStart,actualDecodeStartDouble);
             } else {
-               printf("seek failed");
+               wxPrintf("seek failed");
                break;
             }
          }
          if(mCurrentPos>start){
             mSeekingAllowedStatus = (bool)ODFFMPEG_SEEKING_TEST_FAILED;
             //               url_fseek(mFormatContext->pb,sc->m_pkt.pos,SEEK_SET);
-            printf("seek fail, reverting to previous pos\n");
+            wxPrintf("seek fail, reverting to previous pos\n");
             return -1;
          }
       }
@@ -369,14 +358,14 @@ int ODFFmpegDecoder::Decode(samplePtr & data, sampleFormat & format, sampleCount
       if (sc != (streamContext*)1)
       {
          //find out the dts we've seekd to.  can't use the stream->cur_dts because it is faulty.  also note that until we do the first seek, pkt.dts can be false and will change for the same samples after the initial seek.
-         sampleCount actualDecodeStart = mCurrentPos;
+         auto actualDecodeStart = mCurrentPos;
 
          // we need adjacent samples, so don't use dts most of the time which will leave gaps between frames
          // for some formats
          // The only other case for inserting silence is for initial offset and ImportFFmpeg.cpp does this for us
          if (seeking) {
-            actualDecodeStart = 0.52 + (sc->m_stream->codec->sample_rate * sc->m_pkt.dts
-                                        * ((double)sc->m_stream->time_base.num / sc->m_stream->time_base.den));
+            actualDecodeStart = sampleCount{ 0.52 + (sc->m_stream->codec->sample_rate * sc->m_pkt->dts
+                                        * ((double)sc->m_stream->time_base.num / sc->m_stream->time_base.den)) };
             //this is mostly safe because den is usually 1 or low number but check for high values.
 
             //hack to get rounding to work to neareset frame size since dts isn't exact
@@ -387,32 +376,36 @@ int ODFFmpegDecoder::Decode(samplePtr & data, sampleFormat & format, sampleCount
             seeking = false;
          }
          if(actualDecodeStart != mCurrentPos)
-            printf("ts not matching - now:%llu , last:%llu, lastlen:%llu, start %llu, len %llu\n",actualDecodeStart, mCurrentPos, mCurrentLen, start, len);
+            wxPrintf("ts not matching - now:%llu , last:%llu, lastlen:%lu, start %llu, len %lu\n",actualDecodeStart.as_long_long(), mCurrentPos.as_long_long(), mCurrentLen, start.as_long_long(), len);
             //if we've skipped over some samples, fill the gap with silence.  This could happen often in the beginning of the file.
          if(actualDecodeStart>start && firstpass) {
             // find the number of samples for the leading silence
-            int amt = actualDecodeStart - start;
-            FFMpegDecodeCache* cache = new FFMpegDecodeCache;
 
-            //printf("skipping/zeroing %i samples. - now:%llu (%f), last:%llu, lastlen:%llu, start %llu, len %llu\n",amt,actualDecodeStart, actualDecodeStartdouble, mCurrentPos, mCurrentLen, start, len);
+            // UNSAFE_SAMPLE_COUNT_TRUNCATION
+            // -- but used only experimentally as of this writing
+            // Is there a proof size_t will not overflow size_t?
+            // Result is surely nonnegative.
+            auto amt = (actualDecodeStart - start).as_size_t();
+            auto cache = std::make_unique<FFMpegDecodeCache>();
+
+            //wxPrintf("skipping/zeroing %i samples. - now:%llu (%f), last:%llu, lastlen:%lu, start %llu, len %lu\n",amt,actualDecodeStart, actualDecodeStartdouble, mCurrentPos, mCurrentLen, start, len);
 
             //put it in the cache so the other channels can use it.
-            cache->numChannels = sc->m_stream->codec->channels;
+            // wxASSERT(sc->m_stream->codec->channels > 0);
+            cache->numChannels = std::max<unsigned>(0, sc->m_stream->codec->channels);
             cache->len = amt;
             cache->start=start;
             // 8 bit and 16 bit audio output from ffmpeg means
             // 16 bit int out.
             // 32 bit int, float, double mean float out.
             if (format == int16Sample)
-               cache->samplefmt = SAMPLE_FMT_S16;
+               cache->samplefmt = AV_SAMPLE_FMT_S16;
             else
-               cache->samplefmt = SAMPLE_FMT_FLT;
+               cache->samplefmt = AV_SAMPLE_FMT_FLT;
 
-            cache->samplePtr = (uint8_t*) malloc(amt * cache->numChannels * SAMPLE_SIZE(format));
+            cache->samplePtr.reinit(amt * cache->numChannels * SAMPLE_SIZE(format), true);
 
-            memset(cache->samplePtr, 0, amt * cache->numChannels * SAMPLE_SIZE(format));
-
-            InsertCache(cache);
+            InsertCache(std::move(cache));
          }
          firstpass=false;
          mCurrentPos = actualDecodeStart;
@@ -424,34 +417,28 @@ int ODFFmpegDecoder::Decode(samplePtr & data, sampleFormat & format, sampleCount
                   break;
 
          // Cleanup after frame decoding
-         if (sc->m_pktValid)
-         {
-            av_free_packet(&sc->m_pkt);
-            sc->m_pktValid = 0;
-         }
+         sc->m_pkt.reset();
       }
    }
 
    // Flush the decoders if we're done.
    if((!sc || sc == (streamContext*) 1)&& len>0)
    {
-      for (int i = 0; i < mNumStreams; i++)
+      for (int i = 0; i < mChannels.size(); i++)
       {
-         if (DecodeFrame(mScs[i], true) == 0)
+         sc = scs[i].get();
+         sc->m_pkt.create();
+         if (DecodeFrame(sc, true) == 0)
          {
-            if (mScs[i]->m_pktValid)
-            {
-               av_free_packet(&mScs[i]->m_pkt);
-               mScs[i]->m_pktValid = 0;
-            }
+            sc->m_pkt.reset();
          }
       }
    }
 
-   //this next call takes data, start and len as reference variables and updates them to reflect the new area that is needed.
+   //this next call takes data, start and len as reference variables and updates them to reflect the NEW area that is needed.
    FillDataFromCache(bufStart, format, start, len, channel);
 
-   // CHECK: not sure if we need this.  In any case it has to be updated for the new float case (not just int16)
+   // CHECK: not sure if we need this.  In any case it has to be updated for the NEW float case (not just int16)
    //if for some reason we couldn't get the samples, fill them with silence
    /*
    int16_t* outBuf = (int16_t*) bufStart;
@@ -466,8 +453,8 @@ int ODFFmpegDecoder::Decode(samplePtr & data, sampleFormat & format, sampleCount
 // the minimum amount of cache entries necessary to warrant a binary search.
 #define kODFFmpegSearchThreshold 10
 ///returns the number of samples filled in from start.
-//also updates data and len to reflect new unfilled area - start is unmodified.
-int ODFFmpegDecoder::FillDataFromCache(samplePtr & data, sampleFormat outFormat, sampleCount &start, sampleCount& len, unsigned int channel)
+//also updates data and len to reflect NEW unfilled area - start is unmodified.
+int ODFFmpegDecoder::FillDataFromCache(samplePtr & data, sampleFormat outFormat, sampleCount &start, size_t& len, unsigned int channel)
 {
    if(mDecodeCache.size() <= 0)
       return 0;
@@ -513,50 +500,57 @@ int ODFFmpegDecoder::FillDataFromCache(samplePtr & data, sampleFormat outFormat,
          if(start<mDecodeCache[i]->start && start+len  > mDecodeCache[i]->start+mDecodeCache[i]->len)
             continue;
 
-         int samplesHit;
-         int hitStartInCache;
-         int hitStartInRequest;
-         int nChannels = mDecodeCache[i]->numChannels;
-         samplesHit = FFMIN(start+len,mDecodeCache[i]->start+mDecodeCache[i]->len)
-                        - FFMAX(mDecodeCache[i]->start,start);
+         auto nChannels = mDecodeCache[i]->numChannels;
+         auto samplesHit = (
+            // Proof that the result is never negative: consider four cases
+            // of FFMIN and FFMAX choices, and use the if-condition enclosing.
+            // The result is not more than len.
+            FFMIN(start+len,mDecodeCache[i]->start+mDecodeCache[i]->len)
+               - FFMAX(mDecodeCache[i]->start, start)
+         ).as_size_t();
          //find the start of the hit relative to the cache buffer start.
-         hitStartInCache   = FFMAX(0,start-mDecodeCache[i]->start);
+         // UNSAFE_SAMPLE_COUNT_TRUNCATION
+         // -- but used only experimentally as of this writing
+         // Is there a proof size_t will not overflow?
+         const auto hitStartInCache =
+            // result is less than mDecodeCache[i]->len:
+            FFMAX(sampleCount{0},start-mDecodeCache[i]->start).as_size_t();
          //we also need to find out which end was hit - if it is the tail only we need to update from a later index.
-         hitStartInRequest = start <mDecodeCache[i]->start?len - samplesHit: 0;
-         sampleCount outIndex,inIndex;
-         for(int j=0;j<samplesHit;j++)
+         const auto hitStartInRequest = start < mDecodeCache[i]->start
+            ? len - samplesHit : 0;
+         for(decltype(samplesHit) j = 0; j < samplesHit; j++)
          {
-            outIndex = hitStartInRequest + j;
-            inIndex = (hitStartInCache + j) * nChannels + channel;
+            const auto outIndex = hitStartInRequest + j;
+            const auto inIndex = (hitStartInCache + j) * nChannels + channel;
             switch (mDecodeCache[i]->samplefmt)
             {
-               case SAMPLE_FMT_U8:
-                  //printf("u8 in %llu out %llu cachelen %llu outLen %llu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
-                  ((int16_t *)outBuf)[outIndex] = (int16_t) (((uint8_t*)mDecodeCache[i]->samplePtr)[inIndex] - 0x80) << 8;
+               case AV_SAMPLE_FMT_U8:
+                  //wxPrintf("u8 in %lu out %lu cachelen %lu outLen %lu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
+                  ((int16_t *)outBuf)[outIndex] = (int16_t) (((uint8_t*)mDecodeCache[i]->samplePtr.get())[inIndex] - 0x80) << 8;
                break;
 
-               case SAMPLE_FMT_S16:
-                  //printf("u16 in %llu out %llu cachelen %llu outLen %llu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
-                  ((int16_t *)outBuf)[outIndex] = ((int16_t*)mDecodeCache[i]->samplePtr)[inIndex];
+               case AV_SAMPLE_FMT_S16:
+                  //wxPrintf("u16 in %lu out %lu cachelen %lu outLen % lu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
+                  ((int16_t *)outBuf)[outIndex] = ((int16_t*)mDecodeCache[i]->samplePtr.get())[inIndex];
                break;
 
-               case SAMPLE_FMT_S32:
-                  //printf("s32 in %llu out %llu cachelen %llu outLen %llu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
-                  ((float *)outBuf)[outIndex] = (float) ((int32_t*)mDecodeCache[i]->samplePtr)[inIndex] * (1.0 / (1 << 31));
+               case AV_SAMPLE_FMT_S32:
+                  //wxPrintf("s32 in %lu out %lu cachelen %lu outLen %lu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
+                  ((float *)outBuf)[outIndex] = (float) ((int32_t*)mDecodeCache[i]->samplePtr.get())[inIndex] * (1.0 / (1u << 31));
                break;
 
-               case SAMPLE_FMT_FLT:
-                  //printf("f in %llu out %llu cachelen %llu outLen %llu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
-                  ((float *)outBuf)[outIndex] = (float) ((float*)mDecodeCache[i]->samplePtr)[inIndex];
+               case AV_SAMPLE_FMT_FLT:
+                  //wxPrintf("f in %lu out %lu cachelen %lu outLen %lu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
+                  ((float *)outBuf)[outIndex] = (float) ((float*)mDecodeCache[i]->samplePtr.get())[inIndex];
                break;
 
-               case SAMPLE_FMT_DBL:
-                  //printf("dbl in %llu out %llu cachelen %llu outLen %llu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
-                  ((float *)outBuf)[outIndex] = (float) ((double*)mDecodeCache[i]->samplePtr)[inIndex];
+               case AV_SAMPLE_FMT_DBL:
+                  //wxPrintf("dbl in %lu out %lu cachelen %lu outLen %lu\n", inIndex, outIndex, mDecodeCache[i]->len, len);
+                  ((float *)outBuf)[outIndex] = (float) ((double*)mDecodeCache[i]->samplePtr.get())[inIndex];
                break;
 
                default:
-                  printf("ODDecodeFFMPEG TASK unrecognized sample format\n");
+                  wxPrintf("ODDecodeFFMPEG TASK unrecognized sample format\n");
                   return 1;
                break;
             }
@@ -588,7 +582,11 @@ int ODFFmpegDecoder::FillDataFromCache(samplePtr & data, sampleFormat outFormat,
 //get the right stream pointer.
 streamContext* ODFFmpegDecoder::ReadNextFrame()
 {
-   return import_ffmpeg_read_next_frame(mFormatContext, mScs, mNumStreams);
+   // Get pointer to array of contiguous unique_ptrs
+   auto scs = mScs->get();
+   // This reinterpret_cast to array of plain pointers is innocent
+   return import_ffmpeg_read_next_frame
+      (mContext->ic_ptr, reinterpret_cast<streamContext**>(scs), mChannels.size());
 }
 
 
@@ -600,37 +598,38 @@ int ODFFmpegDecoder::DecodeFrame(streamContext *sc, bool flushing)
       //stick it in the cache.
       //TODO- consider growing/unioning a few cache buffers like WaveCache does.
       //however we can't use wavecache as it isn't going to handle our stereo interleaved part, and isn't for samples
-      //However if other ODDecode tasks need this, we should do a new class for caching.
-      FFMpegDecodeCache* cache = new FFMpegDecodeCache;
+      //However if other ODDecode tasks need this, we should do a NEW class for caching.
+      auto cache = std::make_unique<FFMpegDecodeCache>();
       //len is number of samples per channel
-      cache->numChannels = sc->m_stream->codec->channels;
+      // wxASSERT(sc->m_stream->codec->channels > 0);
+      cache->numChannels = std::max<unsigned>(0, sc->m_stream->codec->channels);
 
       cache->len = (sc->m_decodedAudioSamplesValidSiz / sc->m_samplesize) / cache->numChannels;
       cache->start = mCurrentPos;
-      cache->samplePtr = (uint8_t*) malloc(sc->m_decodedAudioSamplesValidSiz);
+      cache->samplePtr.reinit(sc->m_decodedAudioSamplesValidSiz);
       cache->samplefmt = sc->m_samplefmt;
-      memcpy(cache->samplePtr, sc->m_decodedAudioSamples, sc->m_decodedAudioSamplesValidSiz);
+      memcpy(cache->samplePtr.get(), sc->m_decodedAudioSamples.get(), sc->m_decodedAudioSamplesValidSiz);
 
-      InsertCache(cache);
+      InsertCache(std::move(cache));
    }
    return ret;
 }
 
-void ODFFmpegDecoder::InsertCache(FFMpegDecodeCache* cache) {
+void ODFFmpegDecoder::InsertCache(std::unique_ptr<FFMpegDecodeCache> &&cache) {
    int searchStart = 0;
    int searchEnd = mDecodeCache.size(); //size() is also a valid insert index.
    int guess = 0;
    //first just guess that the cache is contiguous and we can just use math to figure it out like a dictionary
    //by guessing where our hit will be.
 
-//   printf("inserting cache start %llu, mCurrentPos %llu\n", cache->start, mCurrentPos);
+//   wxPrintf("inserting cache start %llu, mCurrentPos %llu\n", cache->start, mCurrentPos);
    while(searchStart<searchEnd)
    {
       guess = (searchStart+searchEnd)/2;//searchStart+ (searchEnd-searchStart)*  ((float)cache->start - mDecodeCache[searchStart]->start )/mDecodeCache[searchEnd]->start;
       //check greater than OR equals because we want to insert infront of old dupes.
       if(mDecodeCache[guess]->start>= cache->start) {
 //         if(mDecodeCache[guess]->start == cache->start) {
-//            printf("dupe! start cache %llu start new cache %llu, mCurrentPos %llu\n",mDecodeCache[guess]->start, cache->start, mCurrentPos);
+//            wxPrintf("dupe! start cache %llu start NEW cache %llu, mCurrentPos %llu\n",mDecodeCache[guess]->start, cache->start, mCurrentPos);
 //         }
          searchEnd = guess;
       }
@@ -639,7 +638,7 @@ void ODFFmpegDecoder::InsertCache(FFMpegDecodeCache* cache) {
    }
    mCurrentLen = cache->len;
    mCurrentPos=cache->start+cache->len;
-   mDecodeCache.insert(mDecodeCache.begin()+guess, cache);
+   mDecodeCache.insert(mDecodeCache.begin()+guess, std::move(cache));
    //      mDecodeCache.push_back(cache);
 
    mNumSamplesInCache+=cache->len;
@@ -651,8 +650,6 @@ void ODFFmpegDecoder::InsertCache(FFMpegDecodeCache* cache) {
       //drop which ever index is further from our newly added one.
       dropindex = (guess > (int)mDecodeCache.size()/2) ? 0 : (mDecodeCache.size()-1);
       mNumSamplesInCache-=mDecodeCache[dropindex]->len;
-      free(mDecodeCache[dropindex]->samplePtr);
-      delete mDecodeCache[dropindex];
       mDecodeCache.erase(mDecodeCache.begin()+dropindex);
    }
 }
