@@ -37,10 +37,6 @@ class EffectsPrefs final : public PrefsPanel
    void Populate();
 };
 
-/// A PrefsPanelFactory that creates one EffectsPrefs panel.
-class EffectsPrefsFactory final : public PrefsPanelFactory
-{
-public:
-   PrefsPanel *operator () (wxWindow *parent, wxWindowID winid) override;
-};
+/// A PrefsPanel::Factory that creates one EffectsPrefs panel.
+extern PrefsPanel::Factory EffectsPrefsFactory;
 #endif
