@@ -11,6 +11,8 @@
 
 #include "../Experimental.h"
 
+class wxChoice;
+class wxTextCtrl;
 class ShuttleGui;
 
 #ifdef EXPERIMENTAL_MIDI_OUT
@@ -20,30 +22,34 @@ class ShuttleGui;
 
 #include <wx/defs.h>
 
-#include <wx/choice.h>
-#include <wx/string.h>
-#include <wx/window.h>
-
 #include "PrefsPanel.h"
 
-class MidiIOPrefs:public PrefsPanel
+class wxArrayStringEx;
+
+#define MIDI_IO_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Midi IO") }
+
+class MidiIOPrefs final : public PrefsPanel
 {
  public:
-   MidiIOPrefs(wxWindow * parent);
+   MidiIOPrefs(wxWindow * parent, wxWindowID winid);
    virtual ~MidiIOPrefs();
-   virtual bool Apply();
-   virtual bool Validate();
+   ComponentInterfaceSymbol GetSymbol() override;
+   wxString GetDescription() override;
+
+   bool Commit() override;
+   bool Validate() override;
+   wxString HelpPageName() override;
+   void PopulateOrExchange(ShuttleGui & S) override;
 
  private:
    void Populate();
-   void PopulateOrExchange(ShuttleGui & S);
    void GetNamesAndLabels();
 
    void OnHost(wxCommandEvent & e);
 //   void OnDevice(wxCommandEvent & e);
 
-   wxArrayString mHostNames;
-   wxArrayString mHostLabels;
+   wxArrayStringEx mHostNames;
+   wxArrayStringEx mHostLabels;
 
    wxString mPlayDevice;
 #ifdef EXPERIMENTAL_MIDI_IN
@@ -59,14 +65,11 @@ class MidiIOPrefs:public PrefsPanel
 #endif
 //   wxChoice *mChannels;
 
-   DECLARE_EVENT_TABLE();
+   DECLARE_EVENT_TABLE()
 };
 
-class MidiIOPrefsFactory : public PrefsPanelFactory
-{
-public:
-   virtual PrefsPanel *Create(wxWindow *parent);
-};
+/// A PrefsPanel::Factory that creates one MidiIOPrefs panel.
+extern PrefsPanel::Factory MidiIOPrefsFactory;
 #endif
 
 #endif
