@@ -11,36 +11,36 @@
 
 #include <wx/defs.h>
 
-#include <wx/listctrl.h>
-#include <wx/string.h>
-#include <wx/window.h>
-
 #include "PrefsPanel.h"
 
+class wxListCtrl;
 class ShuttleGui;
 
-class MousePrefs :public PrefsPanel
+#define MOUSE_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Mouse") }
+
+class MousePrefs final : public PrefsPanel
 {
  public:
-   MousePrefs(wxWindow * parent);
+   MousePrefs(wxWindow * parent, wxWindowID winid);
    ~MousePrefs();
-   virtual bool Apply();
+   ComponentInterfaceSymbol GetSymbol() override;
+   wxString GetDescription() override;
+
+   bool Commit() override;
+   wxString HelpPageName() override;
+   void PopulateOrExchange(ShuttleGui & S) override;
 
  private:
    void Populate();
-   void PopulateOrExchange(ShuttleGui & S);
    void CreateList();
    void AddItem(wxString const & buttons,
                 wxString const & tool,
                 wxString const & action,
-                wxString const & comment = wxEmptyString);
+                wxString const & comment = {});
 
    wxListCtrl * mList;
 };
 
-class MousePrefsFactory : public PrefsPanelFactory
-{
-public:
-   virtual PrefsPanel *Create(wxWindow *parent);
-};
+/// A PrefsPanel::Factory that creates one MousePrefs panel.
+extern PrefsPanel::Factory MousePrefsFactory;
 #endif
