@@ -37,10 +37,6 @@ class ProjectsPrefs final : public PrefsPanel
    void Populate();
 };
 
-/// A PrefsPanelFactory that creates one ProjectPrefs panel.
-class ProjectsPrefsFactory final : public PrefsPanelFactory
-{
-public:
-   PrefsPanel *operator () (wxWindow *parent, wxWindowID winid) override;
-};
+/// A PrefsPanel::Factory that creates one ProjectPrefs panel.
+extern PrefsPanel::Factory ProjectsPrefsFactory;
 #endif
