@@ -31,13 +31,15 @@ Provides:
 #include "../Audacity.h"
 #include "ThemePrefs.h"
 
+#include <wx/app.h>
 #include <wx/wxprec.h>
 #include "../Prefs.h"
 #include "../Theme.h"
-#include "../Project.h"
 #include "../ShuttleGui.h"
 #include "../AColor.h"
 
+wxDEFINE_EVENT(EVT_THEME_CHANGE, wxCommandEvent);
+
 enum eThemePrefsIds {
    idLoadThemeCache=7000,
    idSaveThemeCache,
@@ -170,7 +172,7 @@ void ThemePrefs::PopulateOrExchange(ShuttleGui & S)
 void ThemePrefs::OnLoadThemeComponents(wxCommandEvent & WXUNUSED(event))
 {
    theTheme.LoadComponents();
-   theTheme.ApplyUpdatedImages();
+   ApplyUpdatedImages();
 }
 
 /// Save Theme to multiple png files.
@@ -183,7 +185,7 @@ void ThemePrefs::OnSaveThemeComponents(wxCommandEvent & WXUNUSED(event))
 void ThemePrefs::OnLoadThemeCache(wxCommandEvent & WXUNUSED(event))
 {
    theTheme.ReadImageCache();
-   theTheme.ApplyUpdatedImages();
+   ApplyUpdatedImages();
 }
 
 /// Save Theme to single png file.
@@ -197,7 +199,7 @@ void ThemePrefs::OnSaveThemeCache(wxCommandEvent & WXUNUSED(event))
 void ThemePrefs::OnReadThemeInternal(wxCommandEvent & WXUNUSED(event))
 {
    theTheme.ReadImageCache( theTheme.GetFallbackThemeType() );
-   theTheme.ApplyUpdatedImages();
+   ApplyUpdatedImages();
 }
 
 /// Save Theme as C source code.
@@ -207,6 +209,14 @@ void ThemePrefs::OnSaveThemeAsCode(wxCommandEvent & WXUNUSED(event))
    theTheme.WriteImageDefs();// bonus - give them the Defs too.
 }
 
+void ThemePrefs::ApplyUpdatedImages()
+{
+   AColor::ReInit();
+
+   wxCommandEvent e{ EVT_THEME_CHANGE };
+   wxTheApp->SafelyProcessEvent( e );
+}
+
 /// Update the preferences stored on disk.
 bool ThemePrefs::Commit()
 {
@@ -216,8 +226,9 @@ bool ThemePrefs::Commit()
    return true;
 }
 
-PrefsPanel *ThemePrefsFactory::operator () (wxWindow *parent, wxWindowID winid)
+PrefsPanel::Factory
+ThemePrefsFactory = [](wxWindow *parent, wxWindowID winid)
 {
    wxASSERT(parent); // to justify safenew
    return safenew ThemePrefs(parent, winid);
-}
+};
