@@ -15,22 +15,34 @@
 #define __AUDACITY_THEME_PREFS__
 
 #include <wx/defs.h>
-#include <wx/window.h>
+#include <wx/event.h> // to declare a custom event type
 
 #include "PrefsPanel.h"
 
 class ShuttleGui;
 
-class ThemePrefs :public PrefsPanel
+// An event sent to the application when the user changes choice of theme
+wxDECLARE_EXPORTED_EVENT(AUDACITY_DLL_API,
+                         EVT_THEME_CHANGE, wxCommandEvent);
+
+#define THEME_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Theme") }
+
+class ThemePrefs final : public PrefsPanel
 {
  public:
-   ThemePrefs(wxWindow * parent);
+   ThemePrefs(wxWindow * parent, wxWindowID winid);
    ~ThemePrefs(void);
-   virtual bool Apply();
+   ComponentInterfaceSymbol GetSymbol() override;
+   wxString GetDescription() override;
+
+   bool Commit() override;
+   wxString HelpPageName() override;
+
+   static void ApplyUpdatedImages();
 
  private:
    void Populate();
-   void PopulateOrExchange(ShuttleGui & S);
+   void PopulateOrExchange(ShuttleGui & S) override;
    void OnLoadThemeComponents(wxCommandEvent & e);
    void OnSaveThemeComponents(wxCommandEvent & e);
    void OnLoadThemeCache(wxCommandEvent & e);
@@ -38,12 +50,9 @@ class ThemePrefs :public PrefsPanel
    void OnReadThemeInternal(wxCommandEvent & e);
    void OnSaveThemeAsCode(wxCommandEvent & e);
 
-   DECLARE_EVENT_TABLE();
+   DECLARE_EVENT_TABLE()
 };
 
-class ThemePrefsFactory : public PrefsPanelFactory
-{
-public:
-   virtual PrefsPanel *Create(wxWindow *parent);
-};
+/// A PrefsPanel::Factory that creates one ThemePrefs panel.
+extern PrefsPanel::Factory ThemePrefsFactory;
 #endif
