@@ -20,18 +20,26 @@ class WaveTrack;
 class wxCheckBox;
 class wxChoice;
 
-class WaveformPrefs :public PrefsPanel
+class wxArrayStringEx;
+
+#define WAVEFORM_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Waveform") }
+
+class WaveformPrefs final : public PrefsPanel
 {
 public:
-   WaveformPrefs(wxWindow * parent, WaveTrack *wt);
+   WaveformPrefs(wxWindow * parent, wxWindowID winid, WaveTrack *wt);
    virtual ~WaveformPrefs();
-   virtual bool Apply();
-   virtual bool ShowsApplyButton();
-   virtual bool Validate();
+   ComponentInterfaceSymbol GetSymbol() override;
+   wxString GetDescription() override;
+   wxString HelpPageName() override;
+
+   bool Commit() override;
+   bool ShowsPreviewButton() override;
+   bool Validate() override;
+   void PopulateOrExchange(ShuttleGui & S) override;
 
 private:
    void Populate();
-   void PopulateOrExchange(ShuttleGui & S);
 
    void OnControl(wxCommandEvent&);
    void OnScale(wxCommandEvent&);
@@ -47,22 +55,16 @@ private:
    wxChoice *mScaleChoice;
    wxChoice *mRangeChoice;
 
-   wxArrayString mScaleChoices;
-   wxArrayString mRangeCodes;
-   wxArrayString mRangeChoices;
+   wxArrayStringEx mRangeCodes;
+   wxArrayStringEx mRangeChoices;
 
    WaveformSettings mTempSettings;
 
    bool mPopulating;
 };
 
-class WaveformPrefsFactory : public PrefsPanelFactory
-{
-public:
-   explicit WaveformPrefsFactory(WaveTrack *wt = 0);
-   virtual PrefsPanel *Create(wxWindow *parent);
-
-private:
-   WaveTrack *const mWt;
-};
+/// A PrefsPanel::Factory that creates one WaveformPrefs panel.
+/// This factory can be parametrized by a single track, to change settings
+/// non-globally
+extern PrefsPanel::Factory WaveformPrefsFactory(WaveTrack *wt);
 #endif
