@@ -16,6 +16,7 @@
 
 #include "../Audacity.h" // for USE_* macros
 #include "DeviceToolBar.h"
+#include "ToolManager.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
@@ -38,15 +39,15 @@
 
 #include "../AColor.h"
 #include "../AllThemeResources.h"
-#include "../AudioIO.h"
+#include "../AudioIOBase.h"
 #include "../ImageManipulation.h"
+#include "../KeyboardCapture.h"
 #include "../Prefs.h"
 #include "../Project.h"
 #include "../ShuttleGui.h"
-#include "../Theme.h"
 #include "../widgets/Grabber.h"
 #include "../DeviceManager.h"
-#include "../widgets/ErrorDialog.h"
+#include "../widgets/AudacityMessageBox.h"
 #include "../widgets/Grabber.h"
 
 #if wxUSE_ACCESSIBILITY
@@ -64,16 +65,35 @@ BEGIN_EVENT_TABLE(DeviceToolBar, ToolBar)
    EVT_COMMAND(wxID_ANY, EVT_CAPTURE_KEY, DeviceToolBar::OnCaptureKey)
 END_EVENT_TABLE()
 
-//Standard contructor
-DeviceToolBar::DeviceToolBar()
-: ToolBar(DeviceBarID, _("Device"), wxT("Device"), true)
+static int DeviceToolbarPrefsID()
 {
+   static int value = wxNewId();
+   return value;
+}
+
+//Standard contructor
+DeviceToolBar::DeviceToolBar( AudacityProject &project )
+: ToolBar( project, DeviceBarID, _("Device"), wxT("Device"), true )
+{
+   wxTheApp->Bind( EVT_RESCANNED_DEVICES,
+      &DeviceToolBar::OnRescannedDevices, this );
 }
 
 DeviceToolBar::~DeviceToolBar()
 {
 }
 
+DeviceToolBar &DeviceToolBar::Get( AudacityProject &project )
+{
+   auto &toolManager = ToolManager::Get( project );
+   return *static_cast<DeviceToolBar*>( toolManager.GetToolBar(DeviceBarID) );
+}
+
+const DeviceToolBar &DeviceToolBar::Get( const AudacityProject &project )
+{
+   return Get( const_cast<AudacityProject&>( project )) ;
+}
+
 void DeviceToolBar::Create(wxWindow *parent)
 {
    ToolBar::Create(parent);
@@ -189,16 +209,7 @@ void DeviceToolBar::RefillCombos()
 
 void DeviceToolBar::OnFocus(wxFocusEvent &event)
 {
-   if (event.GetEventType() == wxEVT_KILL_FOCUS) {
-      AudacityProject::ReleaseKeyboard(this);
-   }
-   else {
-      AudacityProject::CaptureKeyboard(this);
-   }
-
-   Refresh(false);
-
-   event.Skip();
+   KeyboardCapture::OnFocus( *this, event );
 }
 
 void DeviceToolBar::OnCaptureKey(wxCommandEvent &event)
@@ -322,9 +333,17 @@ void DeviceToolBar::UpdatePrefs()
    Refresh();
 }
 
+void DeviceToolBar::UpdateSelectedPrefs( int id )
+{
+   if (id == DeviceToolbarPrefsID())
+      UpdatePrefs();
+   ToolBar::UpdateSelectedPrefs( id );
+}
+
 
 void DeviceToolBar::EnableDisableButtons()
 {
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO) {
       // we allow changes when monitoring, but not when recording
       bool audioStreamActive = gAudioIO->IsStreamActive() && !gAudioIO->IsMonitoring();
@@ -332,12 +351,8 @@ void DeviceToolBar::EnableDisableButtons()
       // Here we should relinquish focus
       if (audioStreamActive) {
          wxWindow *focus = wxWindow::FindFocus();
-         if (focus == mHost || focus == mInput || focus == mOutput || focus == mInputChannels) {
-            AudacityProject *activeProject = GetActiveProject();
-            if (activeProject) {
-               activeProject->GetTrackPanel()->SetFocus();
-            }
-         }
+         if (focus == mHost || focus == mInput || focus == mOutput || focus == mInputChannels)
+            TrackPanel::Get( mProject ).SetFocus();
       }
 
       mHost->Enable(!audioStreamActive);
@@ -371,8 +386,8 @@ void DeviceToolBar::RegenerateTooltips()
 bool DeviceToolBar::Layout()
 {
    bool ret;
-   RepositionCombos();
    ret = ToolBar::Layout();
+   RepositionCombos();
    return ret;
 }
 
@@ -429,7 +444,7 @@ void DeviceToolBar::RepositionCombos()
    wxSize desiredInput, desiredOutput, desiredHost, desiredChannels;
    float hostRatio, outputRatio, inputRatio, channelsRatio;
    // if the toolbar is docked then the width we should use is the project width.
-   // as the toolbar's with can extend past this.
+   // as the toolbar's width can extend past this.
    GetClientSize(&w, &h);
 
    // FIXME: Note that there's some bug in here, in that even if the prefs show the toolbar
@@ -611,6 +626,13 @@ void DeviceToolBar::FillHostDevices()
    // The setting of the Device is left up to OnChoice
 }
 
+void DeviceToolBar::OnRescannedDevices( wxCommandEvent &event )
+{
+   event.Skip();
+   // Hosts may have disappeared or appeared so a complete repopulate is needed.
+   RefillCombos();
+}
+
 //return 1 if host changed, 0 otherwise.
 int DeviceToolBar::ChangeHost()
 {
@@ -759,6 +781,7 @@ void DeviceToolBar::OnChoice(wxCommandEvent &event)
       ChangeDevice(false);
    }
 
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO) {
       // We cannot have gotten here if gAudioIO->IsAudioTokenActive(),
       // per the setting of AudioIONotBusyFlag and AudioIOBusyFlag in
@@ -782,10 +805,8 @@ void DeviceToolBar::OnChoice(wxCommandEvent &event)
       gAudioIO->HandleDeviceChange();
    }
 
-   // Update all projects' DeviceToolBar.
-   for (size_t i = 0; i < gAudacityProjects.size(); i++) {
-      gAudacityProjects[i]->GetDeviceToolBar()->UpdatePrefs();
-   }
+   wxTheApp->AddPendingEvent(wxCommandEvent{
+      EVT_PREFS_UPDATE, DeviceToolbarPrefsID() });
 }
 
 void DeviceToolBar::ShowInputDialog()
@@ -846,3 +867,8 @@ void DeviceToolBar::ShowComboDialog(wxChoice *combo, const wxString &title)
    }
 #endif
 }
+
+static RegisteredToolbarFactory factory{ DeviceBarID,
+   []( AudacityProject &project ){
+      return ToolBar::Holder{ safenew DeviceToolBar{ project } }; }
+};
