@@ -11,7 +11,6 @@
 #ifndef __AUDACITY_DEVICE_TOOLBAR__
 #define __AUDACITY_DEVICE_TOOLBAR__
 
-#include "../MemoryX.h"
 #include <vector>
 #include "ToolBar.h"
 
@@ -20,16 +19,22 @@ class wxPoint;
 class wxChoice;
 struct DeviceSourceMap;
 
+class AudacityProject;
+
 class DeviceToolBar final : public ToolBar {
 
  public:
 
-   DeviceToolBar();
+   DeviceToolBar( AudacityProject &project );
    virtual ~DeviceToolBar();
 
+   static DeviceToolBar &Get( AudacityProject &project );
+   static const DeviceToolBar &Get( const AudacityProject &project );
+
    void Create(wxWindow * parent) override;
 
    void UpdatePrefs() override;
+   void UpdateSelectedPrefs( int ) override;
 
    void DeinitChildren();
    void Populate() override;
@@ -54,6 +59,8 @@ class DeviceToolBar final : public ToolBar {
    void RefillCombos();
 
  private:
+   void OnRescannedDevices( wxCommandEvent& );
+
    int  ChangeHost();
    void ChangeDevice(bool isInput);
    void FillHosts();
