@@ -13,30 +13,33 @@
 
 #include "ToolBar.h"
 
-class wxImage;
 class wxSize;
 class wxPoint;
-class wxChoice;
-class wxStaticBitmap;
 
 class ASlider;
+class AudacityProject;
 
-class MixerToolBar:public ToolBar {
+class MixerToolBar final : public ToolBar {
 
  public:
 
-   MixerToolBar();
+   MixerToolBar( AudacityProject &project );
    virtual ~MixerToolBar();
 
-   void Create(wxWindow * parent);
+   static MixerToolBar &Get( AudacityProject &project );
+   static const MixerToolBar &Get( const AudacityProject &project );
 
-   void UpdatePrefs();
+   void Create(wxWindow * parent) override;
+
+   void UpdatePrefs() override;
    void UpdateControls();
    void SetMixer(wxCommandEvent &event);
 
-   virtual void Populate();
-   virtual void Repaint(wxDC * WXUNUSED(dc)) {};
-   virtual void EnableDisableButtons() {};
+   void Populate() override;
+   void Repaint(wxDC * WXUNUSED(dc)) override {};
+   void EnableDisableButtons() override {};
+   int GetMinToolbarWidth() override { return 250; }
+   int GetInitialWidth()  override { return 327; }
 
    void OnFocus(wxFocusEvent &event);
    void OnCaptureKey(wxCommandEvent &event);
@@ -49,6 +52,8 @@ class MixerToolBar:public ToolBar {
    void AdjustOutputGain(int adj);
    void AdjustInputGain(int adj);
 
+   void RegenerateTooltips() override {};
+
  protected:
    float mInputSliderVolume;
    float mOutputSliderVolume;
@@ -58,16 +63,13 @@ class MixerToolBar:public ToolBar {
    void InitializeMixerToolBar();
    void SetToolTips();
 
-   wxBitmap *mPlayBitmap;
-   wxBitmap *mRecordBitmap;
-
    ASlider *mInputSlider;
    ASlider *mOutputSlider;
 
  public:
 
-   DECLARE_CLASS(MixerToolBar);
-   DECLARE_EVENT_TABLE();
+   DECLARE_CLASS(MixerToolBar)
+   DECLARE_EVENT_TABLE()
 };
 
 #endif
