@@ -24,12 +24,18 @@ in which buttons can be placed.
 
 *//**********************************************************************/
 
-#include "../Audacity.h"
+#include "../Audacity.h" // for USE_* macros
+#include "ToolBar.h"
+
+#include "../Experimental.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
+#include <wx/setup.h> // for wxUSE_* macros
+
 #ifndef WX_PRECOMP
+#include <wx/dcclient.h>
 #include <wx/defs.h>
 #include <wx/gdicmn.h>
 #include <wx/image.h>
@@ -40,16 +46,16 @@ in which buttons can be placed.
 #include <wx/window.h>
 #endif  /*  */
 
-#include "ToolBar.h"
-#include "../Experimental.h"
+#include "ToolDock.h"
 
 #include "../AllThemeResources.h"
 #include "../AColor.h"
 #include "../ImageManipulation.h"
 #include "../Project.h"
-#include "../Theme.h"
+#include "../commands/CommandManager.h"
 #include "../widgets/AButton.h"
 #include "../widgets/Grabber.h"
+#include "../Prefs.h"
 
 ////////////////////////////////////////////////////////////
 /// ToolBarResizer
@@ -60,14 +66,19 @@ in which buttons can be placed.
 //
 #define RWIDTH 4
 
-class ToolBarResizer : public wxWindow
+/// \brief a wxWindow that provides the resizer for a toolbar on the 
+/// right hand side.  Responsible for drawing the resizer appearance, 
+/// resizing mouse events and constraining the resizing.
+class ToolBarResizer final : public wxWindow
 {
 public:
    ToolBarResizer(ToolBar *mBar);
    virtual ~ToolBarResizer();
 
    // We don't need or want to accept focus.
-   bool AcceptsFocus() const;
+   // Note that AcceptsFocusFromKeyboard() is overriden rather than
+   // AcceptsFocus(), so that resize can be cancelled by ESC
+   bool AcceptsFocusFromKeyboard() const override {return false;}
 
 private:
    void OnErase(wxEraseEvent & event);
@@ -77,13 +88,17 @@ private:
    void OnEnter(wxMouseEvent & event);
    void OnLeave(wxMouseEvent & event);
    void OnMotion(wxMouseEvent & event);
+   void ResizeBar(const wxSize &size);
    void OnCaptureLost(wxMouseCaptureLostEvent & event);
+   void OnKeyDown(wxKeyEvent &event);
 
 private:
    ToolBar *mBar;
-   wxPoint mResizeStart;
+   wxPoint mResizeOffset;
+   wxSize mOrigSize;
+   wxWindowRef mOrigFocus{};
 
-   DECLARE_EVENT_TABLE();
+   DECLARE_EVENT_TABLE()
 };
 
 //
@@ -94,8 +109,11 @@ BEGIN_EVENT_TABLE( ToolBarResizer, wxWindow )
    EVT_PAINT( ToolBarResizer::OnPaint )
    EVT_LEFT_DOWN( ToolBarResizer::OnLeftDown )
    EVT_LEFT_UP( ToolBarResizer::OnLeftUp )
+   EVT_ENTER_WINDOW( ToolBarResizer::OnEnter )
+   EVT_LEAVE_WINDOW( ToolBarResizer::OnLeave )
    EVT_MOTION( ToolBarResizer::OnMotion )
    EVT_MOUSE_CAPTURE_LOST( ToolBarResizer::OnCaptureLost )
+   EVT_KEY_DOWN( ToolBarResizer::OnKeyDown )
 END_EVENT_TABLE();
 
 ToolBarResizer::ToolBarResizer(ToolBar *bar)
@@ -107,11 +125,8 @@ ToolBarResizer::ToolBarResizer(ToolBar *bar)
 
 ToolBarResizer::~ToolBarResizer()
 {
-}
-
-bool ToolBarResizer::AcceptsFocus() const
-{
-   return false;
+   if(HasCapture())
+      ReleaseMouse();
 }
 
 //
@@ -134,9 +149,9 @@ void ToolBarResizer::OnPaint( wxPaintEvent & event )
    // Under GTK, we specifically set the toolbar background to the background
    // colour in the system theme.
 #if defined( __WXGTK__ )
-   dc.SetBackground( wxBrush( wxSystemSettings::GetColour( wxSYS_COLOUR_BACKGROUND ) ) );
+//   dc.SetBackground( wxBrush( wxSystemSettings::GetColour( wxSYS_COLOUR_BACKGROUND ) ) );
 #endif
-
+   dc.SetBackground( wxBrush( theTheme.Colour( clrMedium  ) ) );
    dc.Clear();
 
    wxSize sz = GetSize();
@@ -155,10 +170,16 @@ void ToolBarResizer::OnLeftDown( wxMouseEvent & event )
    event.Skip();
 
    // Retrieve the mouse position
-   mResizeStart = ClientToScreen( event.GetPosition() );
+   // Bug 1896: This is at time of processing the event, rather than at time 
+   // of generation of event.  Works around event.GetPosition() giving
+   // incorrect values if position of resizer is changing.
+   mResizeOffset = wxGetMousePosition()-mBar->GetRect().GetBottomRight();
+
+   mOrigSize = mBar->GetSize();
 
    // We want all of the mouse events
-   CaptureMouse();
+   if( !HasCapture() )
+      CaptureMouse();
 }
 
 void ToolBarResizer::OnLeftUp( wxMouseEvent & event )
@@ -169,27 +190,48 @@ void ToolBarResizer::OnLeftUp( wxMouseEvent & event )
    if( HasCapture() )
    {
       ReleaseMouse();
+      if (mOrigFocus)
+         mOrigFocus->SetFocus();
+      mOrigFocus = nullptr;
    }
 }
 
+void ToolBarResizer::OnEnter( wxMouseEvent & /*event*/ )
+{
+   // Bug 1201:  On Mac, unsetting and re-setting the tooltip may be needed
+   // to make it pop up when we want it.
+   const auto text = GetToolTipText();
+   UnsetToolTip();
+   SetToolTip(text);
+   if (!mOrigFocus)
+      mOrigFocus = FindFocus();
+}
+
+void ToolBarResizer::OnLeave( wxMouseEvent & /*event*/ )
+{
+   if (!GetCapture())
+      mOrigFocus = nullptr;
+}
+
 void ToolBarResizer::OnMotion( wxMouseEvent & event )
 {
    // Go ahead and set the event to propagate
    event.Skip();
 
-   // Retrieve the mouse position
-   wxPoint raw_pos = event.GetPosition();
-   wxPoint pos = ClientToScreen( raw_pos );
-
-   if( event.Dragging() )
+   if( HasCapture() && event.Dragging() )
    {
+      // Retrieve the mouse position
+      // Bug 1896: This is at time of processing the event, rather than at time 
+      // of generation of event.  Works around event.GetPosition() giving
+      // incorrect values if position of resizer is changing.
+      wxPoint pos = wxGetMousePosition();
+
       wxRect r = mBar->GetRect();
       wxSize msz = mBar->GetMinSize();
       wxSize psz = mBar->GetParent()->GetClientSize();
 
-      // Adjust the size by the difference between the
-      // last mouse and current mouse positions.
-      r.width += ( pos.x - mResizeStart.x );
+      // Adjust the size based on updated mouse position.
+      r.width = ( pos.x - mResizeOffset.x ) - r.x;
 
       // Constrain
       if( r.width < msz.x )
@@ -206,29 +248,43 @@ void ToolBarResizer::OnMotion( wxMouseEvent & event )
          // the one that set them up.  :-)
          r.SetRight( psz.x - 3 );
       }
-      else
-      {
-         // Remember for next go round
-         mResizeStart = pos;
-      }
 
-      // Resize the bar
-      mBar->SetSize( r.GetSize() );
-
-      // Tell everyone we've changed sizes
-      mBar->Updated();
-
-      // Refresh our world
-      mBar->GetParent()->Refresh();
-      mBar->GetParent()->Update();
+      ResizeBar( r.GetSize() );
    }
 }
 
+void ToolBarResizer::ResizeBar(const wxSize &size)
+{
+   mBar->SetSize( size );
+
+   // Tell everyone we've changed sizes
+   mBar->Updated();
+
+   // Refresh our world
+   mBar->GetParent()->Refresh();
+   mBar->GetParent()->Update();
+}
+
 void ToolBarResizer::OnCaptureLost( wxMouseCaptureLostEvent & WXUNUSED(event) )
 {
    if( HasCapture() )
    {
       ReleaseMouse();
+      if (mOrigFocus)
+         mOrigFocus->SetFocus();
+      mOrigFocus = nullptr;
+   }
+}
+
+void ToolBarResizer::OnKeyDown(wxKeyEvent &event)
+{
+   event.Skip();
+   if (HasCapture() && WXK_ESCAPE == event.GetKeyCode()) {
+      ResizeBar( mOrigSize );
+      ReleaseMouse();
+      if (mOrigFocus)
+         mOrigFocus->SetFocus();
+      mOrigFocus = nullptr;
    }
 }
 
@@ -239,7 +295,7 @@ void ToolBarResizer::OnCaptureLost( wxMouseCaptureLostEvent & WXUNUSED(event) )
 //
 // Define class to RTTI
 //
-IMPLEMENT_CLASS( ToolBar, wxPanel );
+IMPLEMENT_CLASS( ToolBar, wxPanelWrapper );
 
 //
 // Custom event
@@ -249,19 +305,22 @@ DEFINE_EVENT_TYPE(EVT_TOOLBAR_UPDATED)
 //
 // Event table
 //
-BEGIN_EVENT_TABLE( ToolBar, wxPanel )
+BEGIN_EVENT_TABLE( ToolBar, wxPanelWrapper )
    EVT_PAINT( ToolBar::OnPaint )
    EVT_ERASE_BACKGROUND( ToolBar::OnErase )
+   EVT_MOUSE_EVENTS( ToolBar::OnMouseEvents )
 END_EVENT_TABLE()
 
 //
 // Constructor
 //
-ToolBar::ToolBar( int type,
+ToolBar::ToolBar( AudacityProject &project,
+                  int type,
                   const wxString &label,
                   const wxString &section,
                   bool resizable )
-: wxPanel()
+: wxPanelWrapper()
+, mProject{ project }
 {
    // Save parameters
    mType = type;
@@ -272,14 +331,11 @@ ToolBar::ToolBar( int type,
    // Initialize everything
    mParent = NULL;
    mHSizer = NULL;
-   mSpacer = NULL;
-   mDock = NULL;
    mVisible = false;
    mPositioned = false;
 
    mGrabber = NULL;
    mResizer = NULL;
-
    SetId(mType);
 }
 
@@ -296,7 +352,7 @@ ToolBar::~ToolBar()
 wxString ToolBar::GetTitle()
 {
    /* i18n-hint: %s will be replaced by the name of the kind of toolbar.*/
-   return wxString::Format( _("Audacity %s Toolbar"), GetLabel().c_str() );
+   return wxString::Format( _("Audacity %s Toolbar"), GetLabel() );
 }
 
 //
@@ -334,7 +390,7 @@ void ToolBar::SetLabel(const wxString & label)
 //
 // Returns whether the toolbar is resizable or not
 //
-bool ToolBar::IsResizable()
+bool ToolBar::IsResizable() const
 {
    return mResizable;
 }
@@ -342,15 +398,15 @@ bool ToolBar::IsResizable()
 //
 // Returns the dock state of the toolbar
 //
-bool ToolBar::IsDocked()
+bool ToolBar::IsDocked() const
 {
-   return mDock != NULL;
+   return const_cast<ToolBar*>(this)->GetDock() != nullptr;
 }
 
 //
 // Returns the visibility of the toolbar
 //
-bool ToolBar::IsVisible()
+bool ToolBar::IsVisible() const
 {
    return mVisible;
 }
@@ -396,13 +452,13 @@ void ToolBar::Create( wxWindow *parent )
    mParent = parent;
 
    // Create the window and label it
-   wxPanel::Create( mParent,
+   wxPanelWrapper::Create( mParent,
                     mType,
                     wxDefaultPosition,
                     wxDefaultSize,
                     wxNO_BORDER | wxTAB_TRAVERSAL,
                     GetTitle() );
-   wxPanel::SetLabel( GetLabel() );
+   wxPanelWrapper::SetLabel( GetLabel() );
 
    // Go do the rest of the creation
    ReCreateButtons();
@@ -411,8 +467,18 @@ void ToolBar::Create( wxWindow *parent )
    mVisible = true;
 }
 
+void ToolBar::SetToDefaultSize(){
+   wxSize sz;
+   sz.SetHeight( -1 );
+   sz.SetWidth( GetInitialWidth());
+   SetSize( sz );
+}
+
 void ToolBar::ReCreateButtons()
 {
+   wxSize sz3 = GetSize();
+   //wxLogDebug( "x:%i y:%i",sz3.x, sz3.y);
+
    // SetSizer(NULL) detaches mHSizer and deletes it.
    // Do not use Detach() here, as that attempts to detach mHSizer from itself!
    SetSizer( NULL );
@@ -421,35 +487,38 @@ void ToolBar::ReCreateButtons()
    DestroyChildren();
    mGrabber = NULL;
    mResizer = NULL;
+   SetLayoutDirection(wxLayout_LeftToRight);
 
-   // Create the main sizer
-   wxBoxSizer *ms = new wxBoxSizer( wxHORIZONTAL );
-
-   // Create the grabber and add it to the main sizer
-   mGrabber = new Grabber( this, mType );
-   ms->Add( mGrabber, 0, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP | wxRIGHT, 1 );
-
-   // Use a box sizer for laying out controls
-   mHSizer = new wxBoxSizer( wxHORIZONTAL );
-   ms->Add( mHSizer, 1, wxEXPAND );
-
-   // (Re)Establish dock state
-   SetDocked( GetDock(), false );
-
-   // Go add all the rest of the gadgets
-   Populate();
-
-   // Add some space for the resize border
-   if( IsResizable() )
    {
-      // Create the resizer and add it to the main sizer
-      mResizer = new ToolBarResizer( this );
-      ms->Add( mResizer, 0, wxEXPAND | wxALIGN_TOP | wxLEFT, 1 );
-      mResizer->SetToolTip( _("Click and drag to resize toolbar") );
-   }
+      // Create the main sizer
+      auto ms = std::make_unique<wxBoxSizer>(wxHORIZONTAL);
 
-   // Set the sizer
-   SetSizerAndFit( ms );
+      // Create the grabber and add it to the main sizer
+      mGrabber = safenew Grabber(this, mType);
+      ms->Add(mGrabber, 0, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP | wxRIGHT, 1);
+
+      // Use a box sizer for laying out controls
+      ms->Add((mHSizer = safenew wxBoxSizer(wxHORIZONTAL)), 1, wxEXPAND);
+
+      // Go add all the rest of the gadgets
+      Populate();
+
+      // Add some space for the resize border
+      if (IsResizable())
+      {
+         // Create the resizer and add it to the main sizer
+         mResizer = safenew ToolBarResizer(this);
+         ms->Add(mResizer, 0, wxEXPAND | wxALIGN_TOP | wxLEFT, 1);
+         mResizer->SetToolTip(_("Click and drag to resize toolbar"));
+      }
+      
+      // Set dock after possibly creating resizer.
+      // (Re)Establish dock state
+      SetDocked(GetDock(), false);
+
+      // Set the sizer
+      SetSizerAndFit(ms.release());
+   }
 
    // Recalculate the height to be a multiple of toolbarSingle
    const int tbs = toolbarSingle + toolbarGap;
@@ -459,14 +528,25 @@ void ToolBar::ReCreateButtons()
    // Set the true AND minimum sizes and do final layout
    if(IsResizable())
    {
-      sz.SetWidth(GetMinToolbarWidth());
       // JKC we're going to allow all resizable toolbars to be resized
-      // to 1 unit high!
+      // to 1 unit high, typically 27 pixels.
       wxSize sz2 = sz;
+      sz2.SetWidth(GetMinToolbarWidth());
       sz2.y = tbs -1;
       SetMinSize(sz2);
-      sz.SetWidth(GetInitialWidth());
-      SetSize(sz);
+      
+      // sz2 is now the minimum size.
+      // sz3 is the size we were.
+      // When recreating buttons, we want to preserve size.
+      // But not if that makes the size too small.
+
+      // Size at least as big as minimum.
+      if( sz3.y < sz2.y )
+         sz3.y = sz2.y;
+      if( sz3.x < sz2.x )
+         sz3.x = sz2.x;
+
+      SetSize(sz3);
    }
    else
    {
@@ -490,6 +570,9 @@ void ToolBar::UpdatePrefs()
    if ( mResizer )
    {
       mResizer->SetToolTip( _("Click and drag to resize toolbar") );
+      wxSizeEvent e;
+      GetParent()->GetEventHandler()->AddPendingEvent( e );
+      GetParent()->Refresh();
    }
 #endif
 
@@ -501,7 +584,7 @@ void ToolBar::UpdatePrefs()
 //
 ToolDock *ToolBar::GetDock()
 {
-   return mDock;
+   return dynamic_cast<ToolDock*>(GetParent());
 }
 
 //
@@ -510,7 +593,7 @@ ToolDock *ToolBar::GetDock()
 void ToolBar::SetDocked( ToolDock *dock, bool pushed )
 {
    // Remember it
-   mDock = dock;
+//   mDock = dock;
 
    // Change the tooltip of the grabber
 #if wxUSE_TOOLTIPS
@@ -522,9 +605,8 @@ void ToolBar::SetDocked( ToolDock *dock, bool pushed )
 
    if (mResizer)
    {
-      mResizer->Show(mDock != NULL);
+      mResizer->Show(dock != NULL);
       Layout();
-      Fit();
    }
 }
 
@@ -533,8 +615,16 @@ void ToolBar::SetDocked( ToolDock *dock, bool pushed )
 //
 void ToolBar::Updated()
 {
-   wxCommandEvent e( EVT_TOOLBAR_UPDATED, GetId() );
-   GetParent()->GetEventHandler()->AddPendingEvent( e );
+   if( IsDocked() )
+      GetDock()->Updated();
+   else
+      // Bug 2120.  Changing the choice also changes the size of the toolbar so
+      // we need to update the client size, even if undocked.
+      // If modifying/improving this, remember to test both changing the choice,
+      // and clciking on the choice but not actually changing it.
+      GetParent()->SetClientSize( GetSize() + wxSize( 2,2));
+   //wxCommandEvent e( EVT_TOOLBAR_UPDATED, GetId() );
+   //GetParent()->GetEventHandler()->AddPendingEvent( e );
 }
 
 //
@@ -634,48 +724,64 @@ void ToolBar::MakeMacRecoloredImage(teBmps eBmpOut, teBmps eBmpIn )
 
 void ToolBar::MakeRecoloredImage( teBmps eBmpOut, teBmps eBmpIn )
 {
-   wxImage * pPattern;
-   wxImage * pSrc = &theTheme.Image( eBmpIn );
-#if defined( __WXGTK__ )
-   wxColour newColour = wxSystemSettings::GetColour( wxSYS_COLOUR_BACKGROUND );
-#else
-   wxColour newColour = wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE );
-#endif
-   wxColour baseColour = wxColour( 204, 204, 204 );
-
-   pPattern = ChangeImageColour( pSrc, baseColour, newColour );
-
-   theTheme.ReplaceImage( eBmpOut, pPattern);
-   delete pPattern;
+   // Don't recolour the buttons...
+   MakeMacRecoloredImage( eBmpOut, eBmpIn );
 }
 
 void ToolBar:: MakeButtonBackgroundsLarge()
 {
-#ifdef USE_AQUA_THEME
-   MakeMacRecoloredImage( bmpRecoloredUpLarge,     bmpMacUpButton );
-   MakeMacRecoloredImage( bmpRecoloredDownLarge,   bmpMacDownButton );
-   MakeMacRecoloredImage( bmpRecoloredHiliteLarge, bmpMacHiliteButton );
-#else
-   MakeRecoloredImage( bmpRecoloredUpLarge,     bmpUpButtonLarge );
-   MakeRecoloredImage( bmpRecoloredDownLarge,   bmpDownButtonLarge );
-   MakeRecoloredImage( bmpRecoloredHiliteLarge, bmpHiliteButtonLarge );
+
+   bool bUseAqua = false;
+
+#ifdef EXPERIMENTAL_THEME_PREFS
+   gPrefs->Read( wxT("/GUI/ShowMac"), &bUseAqua, false);
 #endif
+
+#ifdef USE_AQUA_THEME
+   bUseAqua = !bUseAqua;
+#endif
+
+   if( bUseAqua ){
+      MakeMacRecoloredImage( bmpRecoloredUpLarge,       bmpMacUpButton );
+      MakeMacRecoloredImage( bmpRecoloredDownLarge,     bmpMacDownButton );
+      MakeMacRecoloredImage( bmpRecoloredUpHiliteLarge, bmpMacHiliteUpButton );
+      MakeMacRecoloredImage( bmpRecoloredHiliteLarge,   bmpMacHiliteButton );
+   } else {
+      MakeRecoloredImage( bmpRecoloredUpLarge,       bmpUpButtonLarge );
+      MakeRecoloredImage( bmpRecoloredDownLarge,     bmpDownButtonLarge );
+      MakeRecoloredImage( bmpRecoloredUpHiliteLarge, bmpHiliteUpButtonLarge );
+      MakeRecoloredImage( bmpRecoloredHiliteLarge,   bmpHiliteButtonLarge );
+   }
 }
 
 void ToolBar::MakeButtonBackgroundsSmall()
 {
-#ifdef USE_AQUA_THEME
-   MakeMacRecoloredImage( bmpRecoloredUpSmall,     bmpMacUpButtonSmall );
-   MakeMacRecoloredImage( bmpRecoloredDownSmall,   bmpMacDownButtonSmall );
-   MakeMacRecoloredImage( bmpRecoloredHiliteSmall, bmpMacHiliteButtonSmall );
-#else
-   MakeRecoloredImage( bmpRecoloredUpSmall,     bmpUpButtonSmall );
-   MakeRecoloredImage( bmpRecoloredDownSmall,   bmpDownButtonSmall );
-   MakeRecoloredImage( bmpRecoloredHiliteSmall, bmpHiliteButtonSmall );
+
+   bool bUseAqua = false;
+
+#ifdef EXPERIMENTAL_THEME_PREFS
+   gPrefs->Read( wxT("/GUI/ShowMac"), &bUseAqua, false);
 #endif
+
+#ifdef USE_AQUA_THEME
+   bUseAqua = !bUseAqua;
+#endif
+
+   if( bUseAqua ){
+      MakeMacRecoloredImage( bmpRecoloredUpSmall,       bmpMacUpButtonSmall );
+      MakeMacRecoloredImage( bmpRecoloredDownSmall,     bmpMacDownButtonSmall );
+      MakeMacRecoloredImage( bmpRecoloredUpHiliteSmall, bmpMacHiliteUpButtonSmall );
+      MakeMacRecoloredImage( bmpRecoloredHiliteSmall,   bmpMacHiliteButtonSmall );
+   } else {
+      MakeRecoloredImage(    bmpRecoloredUpSmall,       bmpUpButtonSmall );
+      MakeRecoloredImage(    bmpRecoloredDownSmall,     bmpDownButtonSmall );
+      MakeRecoloredImage(    bmpRecoloredUpHiliteSmall, bmpHiliteUpButtonSmall );
+      MakeRecoloredImage(    bmpRecoloredHiliteSmall,   bmpHiliteButtonSmall );
+   }
 }
 
 /// Makes a button and its four different state bitmaps
+/// @param parent            Parent window for the button.
 /// @param eUp               Background for when button is Up.
 /// @param eDown             Background for when button is Down.
 /// @param eHilite           Background for when button is Hilit.
@@ -686,9 +792,11 @@ void ToolBar::MakeButtonBackgroundsSmall()
 /// @param placement         Placement position
 /// @param processdownevents true iff button handles down events.
 /// @param size              Size of the background.
-AButton * ToolBar::MakeButton(teBmps eUp,
+AButton * ToolBar::MakeButton(wxWindow *parent,
+                              teBmps eUp,
                               teBmps eDown,
                               teBmps eHilite,
+                              teBmps eDownHi,
                               teBmps eStandardUp,
                               teBmps eStandardDown,
                               teBmps eDisabled,
@@ -697,17 +805,20 @@ AButton * ToolBar::MakeButton(teBmps eUp,
                               bool processdownevents,
                               wxSize size)
 {
-   int xoff = (size.GetWidth() - theTheme.Image(eStandardUp).GetWidth())/2;
-   int yoff = (size.GetHeight() - theTheme.Image(eStandardUp).GetHeight())/2;
+   // wxMax to cater for case of image being bigger than the button.
+   int xoff = wxMax( 0, (size.GetWidth() - theTheme.Image(eStandardUp).GetWidth())/2);
+   int yoff = wxMax( 0, (size.GetHeight() - theTheme.Image(eStandardUp).GetHeight())/2);
 
-   typedef std::auto_ptr<wxImage> wxImagePtr;
+   typedef std::unique_ptr<wxImage> wxImagePtr;
    wxImagePtr up2        (OverlayImage(eUp,     eStandardUp, xoff, yoff));
    wxImagePtr hilite2    (OverlayImage(eHilite, eStandardUp, xoff, yoff));
    wxImagePtr down2      (OverlayImage(eDown,   eStandardDown, xoff + 1, yoff + 1));
+   wxImagePtr downHi2    (OverlayImage(eDownHi, eStandardDown, xoff + 1, yoff + 1));
    wxImagePtr disable2   (OverlayImage(eUp,     eDisabled, xoff, yoff));
 
+   wxASSERT(parent); // to justify safenew
    AButton * button =
-      new AButton(this, id, placement, size, *up2, *hilite2, *down2,
+      safenew AButton(parent, id, placement, size, *up2, *hilite2, *down2, *downHi2,
             *disable2, processdownevents);
 
    return button;
@@ -718,21 +829,38 @@ void ToolBar::MakeAlternateImages(AButton &button, int idx,
                                   teBmps eUp,
                                   teBmps eDown,
                                   teBmps eHilite,
+                                  teBmps eDownHi,
                                   teBmps eStandardUp,
                                   teBmps eStandardDown,
                                   teBmps eDisabled,
                                   wxSize size)
 {
-   int xoff = (size.GetWidth() - theTheme.Image(eStandardUp).GetWidth())/2;
-   int yoff = (size.GetHeight() - theTheme.Image(eStandardUp).GetHeight())/2;
+   // wxMax to cater for case of image being bigger than the button.
+   int xoff = wxMax( 0, (size.GetWidth() - theTheme.Image(eStandardUp).GetWidth())/2);
+   int yoff = wxMax( 0, (size.GetHeight() - theTheme.Image(eStandardUp).GetHeight())/2);
 
-   typedef std::auto_ptr<wxImage> wxImagePtr;
+   typedef std::unique_ptr<wxImage> wxImagePtr;
    wxImagePtr up        (OverlayImage(eUp,     eStandardUp, xoff, yoff));
    wxImagePtr hilite    (OverlayImage(eHilite, eStandardUp, xoff, yoff));
    wxImagePtr down      (OverlayImage(eDown,   eStandardDown, xoff + 1, yoff + 1));
+   wxImagePtr downHi    (OverlayImage(eDownHi, eStandardDown, xoff + 1, yoff + 1));
    wxImagePtr disable   (OverlayImage(eUp,     eDisabled, xoff, yoff));
 
-   button.SetAlternateImages(idx, *up, *hilite, *down, *disable);
+   button.SetAlternateImages(idx, *up, *hilite, *down, *downHi, *disable);
+}
+
+void ToolBar::SetButtonToolTip
+(AudacityProject &theProject,
+ AButton &button, const TranslatedInternalString commands[], size_t nCommands)
+{
+   wxString result;
+   const auto project = &theProject;
+   const auto commandManager =
+      project ? &CommandManager::Get( *project ) : nullptr;
+   if (commandManager)
+      result =
+         commandManager->DescribeCommandsAndShortcuts(commands, nCommands);
+   button.SetToolTip(result);
 }
 
 //
@@ -763,62 +891,56 @@ void ToolBar::OnErase( wxEraseEvent & WXUNUSED(event) )
 //
 void ToolBar::OnPaint( wxPaintEvent & event )
 {
-   wxPaintDC dc( (wxWindow *) event.GetEventObject() );
-
+   (void)event;// compiler food.
+   //wxPaintDC dc( (wxWindow *) event.GetEventObject() );
+   wxPaintDC dc( this );
    // Start with a clean background
    //
    // Under GTK, we specifically set the toolbar background to the background
    // colour in the system theme.
 #if defined( __WXGTK__ )
-   dc.SetBackground( wxBrush( wxSystemSettings::GetColour( wxSYS_COLOUR_BACKGROUND ) ) );
+   //dc.SetBackground( wxBrush( wxSystemSettings::GetColour( wxSYS_COLOUR_BACKGROUND ) ) );
 #endif
 
+   // Themed background colour.
+   dc.SetBackground( wxBrush( theTheme.Colour( clrMedium  ) ) );
    dc.Clear();
 
-// EXPERIMENTAL_THEMING is set to not apply the gradient
-// on wxMAC builds.  on wxMAC we have the AQUA_THEME.
 #ifdef USE_AQUA_THEME
    Repaint( &dc );
-#else
-
-#ifdef EXPERIMENTAL_THEMING
-   wxImage * mpBackGradient =   &theTheme.Image( bmpRecoloredUpLarge  );
-
-   if( mpBackGradient != NULL )
-   {
-      wxSize imSz( mpBackGradient->GetWidth(), mpBackGradient->GetHeight() );
-      wxSize sz = GetSize();
-      int y;
-      for(y=0;y<sz.y;y++)
-      {
-         int yPix = ((float)y * imSz.y - 1.0f)/(sz.y-1);
-         wxColour col(
-            mpBackGradient->GetRed( 0, yPix),
-            mpBackGradient->GetGreen( 0, yPix),
-            mpBackGradient->GetBlue( 0, yPix));
-
-         // Set background colour so that controls placed on this
-         // toolbar such as radio buttons will draw reasonably.
-         // It's a little tacky setting the background colour
-         // here, but we can't do it in the constructor as the gradient
-         // may not be available yet.
-         // Better than this would be to set the colour when the image
-         // is loaded.
-         // We use the colour at the half way point as a suitable 'average'.
-         if( y==(sz.y/2) )
-         {
-            SetBackgroundColour( col );
-         }
-         wxPen Pen( col );
-         dc.SetPen(Pen );
-         AColor::Line(dc, 0, y, sz.x, y );
-      }
-   }
-#endif
 #endif
 }
 
+void ToolBar::OnMouseEvents(wxMouseEvent &event)
+{
+   // Do this hack so scrubber can detect mouse drags anywhere
+   event.ResumePropagation(wxEVENT_PROPAGATE_MAX);
+   event.Skip();
+}
+
 int ToolBar::GetResizeGrabberWidth()
 {
    return RWIDTH;
 }
+
+namespace {
+
+RegisteredToolbarFactory::Functions &GetFunctions()
+{
+   static RegisteredToolbarFactory::Functions factories( ToolBarCount );
+   return factories;
+}
+
+}
+
+RegisteredToolbarFactory::RegisteredToolbarFactory(
+   int id, const Function &function)
+{
+   wxASSERT( id >= 0 && id < ToolBarCount );
+   GetFunctions()[ id ] = function;
+}
+
+auto RegisteredToolbarFactory::GetFactories() -> const Functions&
+{
+   return GetFunctions();
+}
