@@ -16,6 +16,9 @@
 
 #include "../Audacity.h"
 #include "TranscriptionToolBar.h"
+#include "ToolManager.h"
+
+#include "../Experimental.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
@@ -30,17 +33,20 @@
 
 #include "../Envelope.h"
 
-#include "ControlToolBar.h"
-#include "../AudacityApp.h"
 #include "../AllThemeResources.h"
 #include "../AudioIO.h"
-#include "../Experimental.h"
 #include "../ImageManipulation.h"
+#include "../KeyboardCapture.h"
 #include "../Project.h"
-#include "../TimeTrack.h"
+#include "../ProjectAudioManager.h"
+#include "../ProjectSettings.h"
+#include "../Envelope.h"
+#include "../ViewInfo.h"
 #include "../WaveTrack.h"
 #include "../widgets/AButton.h"
 #include "../widgets/ASlider.h"
+#include "../tracks/ui/Scrubbing.h"
+#include "../Prefs.h"
 
 #ifdef EXPERIMENTAL_VOICE_DETECTION
 #include "../VoiceKey.h"
@@ -88,34 +94,35 @@ END_EVENT_TABLE()
    ;   //semicolon enforces  proper automatic indenting in emacs.
 
 ////Standard Constructor
-TranscriptionToolBar::TranscriptionToolBar()
-: ToolBar(TranscriptionBarID, _("Transcription"), wxT("Transcription"))
+TranscriptionToolBar::TranscriptionToolBar( AudacityProject &project )
+: ToolBar( project,
+   TranscriptionBarID, _("Play-at-Speed"), wxT("Transcription"), true )
 {
-   mPlaySpeed = 1.0 * 100.0;
-   mTimeTrack = NULL;
+   SetPlaySpeed( 1.0 * 100.0 );
 #ifdef EXPERIMENTAL_VOICE_DETECTION
-   mVk = new VoiceKey();
+   mVk = std::make_unique<VoiceKey>();
 #endif
 }
 
 TranscriptionToolBar::~TranscriptionToolBar()
 {
-#ifdef EXPERIMENTAL_VOICE_DETECTION
-   delete mVk;
-#endif
-   if (mTimeTrack) {
-      delete mTimeTrack;
-      mTimeTrack = NULL;
-   }
+}
+
+TranscriptionToolBar &TranscriptionToolBar::Get( AudacityProject &project )
+{
+   auto &toolManager = ToolManager::Get( project );
+   return *static_cast<TranscriptionToolBar*>( toolManager.GetToolBar(TranscriptionBarID) );
+}
+
+const TranscriptionToolBar &TranscriptionToolBar::Get( const AudacityProject &project )
+{
+   return Get( const_cast<AudacityProject&>( project )) ;
 }
 
 void TranscriptionToolBar::Create(wxWindow * parent)
 {
    ToolBar::Create(parent);
 
-   mBackgroundBrush.SetColour(wxColour(204, 204, 204));
-   mBackgroundPen.SetColour(wxColour(204, 204, 204));
-
    mBackgroundHeight = 0;
    mBackgroundWidth = 0;
 
@@ -139,21 +146,33 @@ void TranscriptionToolBar::Create(wxWindow * parent)
    //JKC: Set speed this way is better, as we don't
    //then stop Audio if it is playing, so we can be playing
    //audio and open a second project.
-   mPlaySpeed = (mPlaySpeedSlider->Get()) * 100;
+   SetPlaySpeed( (mPlaySpeedSlider->Get()) * 100 );
+
+   // Simulate a size event to set initial placement/size
+   wxSizeEvent event(GetSize(), GetId());
+   event.SetEventObject(this);
+   GetEventHandler()->ProcessEvent(event);
+}
+
+void TranscriptionToolBar::SetPlaySpeed( double value )
+{
+   mPlaySpeed = value;
+   ProjectSettings::Get( mProject ).SetPlaySpeed( GetPlaySpeed() );
 }
 
 /// This is a convenience function that allows for button creation in
 /// MakeButtons() with fewer arguments
 /// Very similar to code in ControlToolBar...
 AButton *TranscriptionToolBar::AddButton(
+   TranscriptionToolBar *pBar,
    teBmps eFore, teBmps eDisabled,
    int id,
    const wxChar *label)
 {
-   AButton *&r = mButtons[id];
+   AButton *&r = pBar->mButtons[id];
 
-   r = ToolBar::MakeButton(
-      bmpRecoloredUpSmall, bmpRecoloredDownSmall, bmpRecoloredHiliteSmall,
+   r = ToolBar::MakeButton(pBar,
+      bmpRecoloredUpSmall, bmpRecoloredDownSmall, bmpRecoloredUpHiliteSmall,bmpRecoloredHiliteSmall,
       eFore, eFore, eDisabled,
       wxWindowID(id),
       wxDefaultPosition,
@@ -164,7 +183,7 @@ AButton *TranscriptionToolBar::AddButton(
 // JKC: Unlike ControlToolBar, does not have a focus rect.  Shouldn't it?
 // r->SetFocusRect( r->GetRect().Deflate( 4, 4 ) );
 
-   Add( r, 0, wxALIGN_CENTER );
+   pBar->Add( r, 0, wxALIGN_CENTER );
 
    return r;
 }
@@ -174,18 +193,19 @@ void TranscriptionToolBar::MakeAlternateImages(
    int id, unsigned altIdx)
 {
    ToolBar::MakeAlternateImages(*mButtons[id], altIdx,
-      bmpRecoloredUpSmall, bmpRecoloredDownSmall, bmpRecoloredHiliteSmall,
+      bmpRecoloredUpSmall, bmpRecoloredDownSmall, bmpRecoloredUpHiliteSmall,bmpRecoloredHiliteSmall,
       eFore, eFore, eDisabled,
       theTheme.ImageSize( bmpRecoloredUpSmall ));
 }
 
 void TranscriptionToolBar::Populate()
 {
+   SetBackgroundColour( theTheme.Colour( clrMedium  ) );
 // Very similar to code in ControlToolBar...
 // Very similar to code in EditToolBar
    MakeButtonBackgroundsSmall();
 
-   AddButton(bmpPlay,     bmpPlayDisabled,   TTB_PlaySpeed,
+   AddButton(this, bmpPlay,     bmpPlayDisabled,   TTB_PlaySpeed,
       _("Play at selected speed"));
    MakeAlternateImages(bmpLoop, bmpLoopDisabled, TTB_PlaySpeed, 1);
    MakeAlternateImages(bmpCutPreview, bmpCutPreviewDisabled, TTB_PlaySpeed, 2);
@@ -193,50 +213,52 @@ void TranscriptionToolBar::Populate()
 
    //Add a slider that controls the speed of playback.
    const int SliderWidth=100;
-   mPlaySpeedSlider = new ASlider(this,
-                                  TTB_PlaySpeedSlider,
-                                  _("Playback Speed"),
-                                  wxDefaultPosition,
-                                  wxSize(SliderWidth,25),
-                                  SPEED_SLIDER);
+   mPlaySpeedSlider = safenew ASlider(this,
+      TTB_PlaySpeedSlider,
+      _("Playback Speed"),
+      wxDefaultPosition,
+      wxSize(SliderWidth,25),
+      ASlider::Options{}
+         .Style( SPEED_SLIDER )
+         //  6 steps using page up/down, and 60 using arrow keys
+         .Line( 0.16667f )
+         .Page( 1.6667f )
+   );
+   mPlaySpeedSlider->SetSizeHints(wxSize(100, 25), wxSize(2000, 25));
    mPlaySpeedSlider->Set(mPlaySpeed / 100.0);
    mPlaySpeedSlider->SetLabel(_("Playback Speed"));
-   //  6 steps using page up/down, and 60 using arrow keys
-   mPlaySpeedSlider->SetScroll(0.16667f, 1.6667f);
-   Add( mPlaySpeedSlider, 0, wxALIGN_CENTER );
-   mPlaySpeedSlider->Connect(wxEVT_SET_FOCUS,
-                 wxFocusEventHandler(TranscriptionToolBar::OnFocus),
-                 NULL,
+   Add( mPlaySpeedSlider, 1, wxALIGN_CENTER );
+   mPlaySpeedSlider->Bind(wxEVT_SET_FOCUS,
+                 &TranscriptionToolBar::OnFocus,
                  this);
-   mPlaySpeedSlider->Connect(wxEVT_KILL_FOCUS,
-                 wxFocusEventHandler(TranscriptionToolBar::OnFocus),
-                 NULL,
+   mPlaySpeedSlider->Bind(wxEVT_KILL_FOCUS,
+                 &TranscriptionToolBar::OnFocus,
                  this);
 
 #ifdef EXPERIMENTAL_VOICE_DETECTION
 // If we need these strings translated, then search and replace
 // TRANSLATBLE by _ and remove this #define.
 #define TRANSLATABLE( x ) wxT( x )
-   AddButton(bmpTnStartOn,     bmpTnStartOnDisabled,  TTB_StartOn,
+   AddButton(this, bmpTnStartOn,     bmpTnStartOnDisabled,  TTB_StartOn,
       TRANSLATABLE("Adjust left selection to next onset"));
-   AddButton(bmpTnEndOn,       bmpTnEndOnDisabled,   TTB_EndOn,
+   AddButton(this, bmpTnEndOn,       bmpTnEndOnDisabled,   TTB_EndOn,
       TRANSLATABLE("Adjust right selection to previous offset"));
-   AddButton(bmpTnStartOff,    bmpTnStartOffDisabled,  TTB_StartOff,
+   AddButton(this, bmpTnStartOff,    bmpTnStartOffDisabled,  TTB_StartOff,
       TRANSLATABLE("Adjust left selection to next offset"));
-   AddButton(bmpTnEndOff,      bmpTnEndOffDisabled,    TTB_EndOff,
+   AddButton(this, bmpTnEndOff,      bmpTnEndOffDisabled,    TTB_EndOff,
       TRANSLATABLE("Adjust right selection to previous onset"));
-   AddButton(bmpTnSelectSound, bmpTnSelectSoundDisabled, TTB_SelectSound,
+   AddButton(this, bmpTnSelectSound, bmpTnSelectSoundDisabled, TTB_SelectSound,
       TRANSLATABLE("Select region of sound around cursor"));
-   AddButton(bmpTnSelectSilence, bmpTnSelectSilenceDisabled, TTB_SelectSilence,
+   AddButton(this, bmpTnSelectSilence, bmpTnSelectSilenceDisabled, TTB_SelectSilence,
       TRANSLATABLE("Select region of silence around cursor"));
-   AddButton(bmpTnAutomateSelection,   bmpTnAutomateSelectionDisabled,  TTB_AutomateSelection,
+   AddButton(this, bmpTnAutomateSelection,   bmpTnAutomateSelectionDisabled,  TTB_AutomateSelection,
       TRANSLATABLE("Automatically make labels from words"));
-   AddButton(bmpTnMakeTag, bmpTnMakeTagDisabled,  TTB_MakeLabel,
+   AddButton(this, bmpTnMakeTag, bmpTnMakeTagDisabled,  TTB_MakeLabel,
       TRANSLATABLE("Add label at selection"));
-   AddButton(bmpTnCalibrate, bmpTnCalibrateDisabled, TTB_Calibrate,
+   AddButton(this, bmpTnCalibrate, bmpTnCalibrateDisabled, TTB_Calibrate,
       TRANSLATABLE("Calibrate voicekey"));
 
-   mSensitivitySlider = new ASlider(this,
+   mSensitivitySlider = safenew ASlider(this,
                                     TTB_SensitivitySlider,
                                     TRANSLATABLE("Adjust Sensitivity"),
                                     wxDefaultPosition,
@@ -255,7 +277,7 @@ void TranscriptionToolBar::Populate()
       TRANSLATABLE("Direction Changes (High Threshold)")
    };
 
-   mKeyTypeChoice = new wxChoice(this, TTB_KeyType,
+   mKeyTypeChoice = safenew wxChoice(this, TTB_KeyType,
                                  wxDefaultPosition,
                                  wxDefaultSize,
                                  5,
@@ -273,18 +295,23 @@ void TranscriptionToolBar::Populate()
 
 void TranscriptionToolBar::EnableDisableButtons()
 {
+   AudacityProject *p = &mProject;
+
+   auto gAudioIO = AudioIO::Get();
+   bool canStopAudioStream = (!gAudioIO->IsStreamActive() ||
+           gAudioIO->IsMonitoring() ||
+           gAudioIO->GetOwningProject() == p );
+   bool recording = gAudioIO->GetNumCaptureChannels() > 0;
+
+   // Only interested in audio type tracks
+   bool tracks = p && TrackList::Get( *p ).Any<AudioTrack>(); // PRL:  PlayableTrack ?
+   SetEnabled( canStopAudioStream && tracks && !recording );
+
 #ifdef EXPERIMENTAL_VOICE_DETECTION
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
    if (!p) return;
    // Is anything selected?
-   bool selection = false;
-   TrackListIterator iter(p->GetTracks());
-   for (Track *t = iter.First(); t; t = iter.Next())
-      if (t->GetSelected()) {
-         selection = true;
-         break;
-      }
-   selection &= (p->GetSel0() < p->GetSel1());
+   auto selection = p->GetSel0() < p->GetSel1() && p->GetTracks()->Selected();
 
    mButtons[TTB_Calibrate]->SetEnabled(selection);
 #endif
@@ -295,7 +322,7 @@ void TranscriptionToolBar::UpdatePrefs()
    RegenerateTooltips();
 
    // Set label to pull in language change
-   SetLabel(_("Transcription"));
+   SetLabel(_("Play-at-Speed"));
 
    // Give base class a chance
    ToolBar::UpdatePrefs();
@@ -303,7 +330,29 @@ void TranscriptionToolBar::UpdatePrefs()
 
 void TranscriptionToolBar::RegenerateTooltips()
 {
-   mButtons[TTB_PlaySpeed]->SetToolTip(_("Play-at-speed"));
+   // We could also mention the shift- and ctrl-modified versions in the
+   // tool tip... but it would get long
+
+   static const struct Entry {
+      int tool;
+      CommandID commandName;
+      wxString untranslatedLabel;
+      CommandID commandName2;
+      wxString untranslatedLabel2;
+   } table[] = {
+      { TTB_PlaySpeed,   wxT("PlayAtSpeed"),    XO("Play-at-Speed"),
+      wxT("PlayAtSpeedLooped"),    XO("Looped-Play-at-Speed")
+      },
+   };
+
+   for (const auto &entry : table) {
+      TranslatedInternalString commands[] = {
+         { entry.commandName,  wxGetTranslation(entry.untranslatedLabel)  },
+         { entry.commandName2, wxGetTranslation(entry.untranslatedLabel2) },
+      };
+      ToolBar::SetButtonToolTip( mProject,
+         *mButtons[entry.tool], commands, 2u );
+   }
 
 #ifdef EXPERIMENTAL_VOICE_DETECTION
    mButtons[TTB_StartOn]->SetToolTip(TRANSLATABLE("Left-to-On"));
@@ -323,16 +372,7 @@ void TranscriptionToolBar::RegenerateTooltips()
 
 void TranscriptionToolBar::OnFocus(wxFocusEvent &event)
 {
-   if (event.GetEventType() == wxEVT_KILL_FOCUS) {
-      AudacityProject::ReleaseKeyboard(this);
-   }
-   else {
-      AudacityProject::CaptureKeyboard(this);
-   }
-
-   Refresh(false);
-
-   event.Skip();
+   KeyboardCapture::OnFocus( *this, event );
 }
 
 void TranscriptionToolBar::OnCaptureKey(wxCommandEvent &event)
@@ -361,6 +401,7 @@ void TranscriptionToolBar::OnKeyEvent(wxKeyEvent & event)
    }
 
    if (event.GetKeyCode() == WXK_SPACE) {
+      auto gAudioIO = AudioIOBase::Get();
       if (gAudioIO->IsBusy()) {
          /*Do Stuff Here*/
       }
@@ -383,12 +424,13 @@ void TranscriptionToolBar::SetButton(bool down, AButton* button)
    }
 }
 
-void TranscriptionToolBar::GetSamples(WaveTrack *t, sampleCount *s0, sampleCount *slen)
+void TranscriptionToolBar::GetSamples(
+   const WaveTrack *t, sampleCount *s0, sampleCount *slen)
 {
    // GetSamples attempts to translate the start and end selection markers into sample indices
    // These selection numbers are doubles.
 
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
    if (!p) {
       return;
    }
@@ -396,11 +438,12 @@ void TranscriptionToolBar::GetSamples(WaveTrack *t, sampleCount *s0, sampleCount
    //First, get the current selection. It is part of the mViewInfo, which is
    //part of the project
 
-   double start = p->GetSel0();
-   double end = p->GetSel1();
+   const auto &selectedRegion = ViewInfo::Get( *p ).selectedRegion;
+   double start = selectedRegion.t0();
+   double end = selectedRegion.t1();
 
-   sampleCount ss0 = sampleCount( (start - t->GetOffset()) * t->GetRate() );
-   sampleCount ss1 = sampleCount( (end - t->GetOffset()) * t->GetRate() );
+   auto ss0 = sampleCount( (start - t->GetOffset()) * t->GetRate() );
+   auto ss1 = sampleCount( (end - t->GetOffset()) * t->GetRate() );
 
    if (start < t->GetOffset()) {
       ss0 = 0;
@@ -421,74 +464,108 @@ void TranscriptionToolBar::GetSamples(WaveTrack *t, sampleCount *s0, sampleCount
    *slen = ss1 - ss0;
 }
 
+// PRL: duplicating constants from TimeTrack.cpp
+//TODO-MB: are these sensible values?
+#define TIMETRACK_MIN 0.01
+#define TIMETRACK_MAX 10.0
+
 // Come here from button clicks, or commands
 void TranscriptionToolBar::PlayAtSpeed(bool looped, bool cutPreview)
 {
    // Can't do anything without an active project
-   AudacityProject * p = GetActiveProject();
+   AudacityProject *p = &mProject;
    if (!p) {
       return;
    }
 
-   // Create a TimeTrack if we haven't done so already
-   if (!mTimeTrack) {
-      mTimeTrack = new TimeTrack(p->GetDirManager());
-      if (!mTimeTrack) {
-         return;
+   auto &projectAudioManager = ProjectAudioManager::Get( mProject );
+
+   // Fixed speed play is the old method, that uses a time track.
+   // VariSpeed play reuses Scrubbing.
+   bool bFixedSpeedPlay = !gPrefs->ReadBool(wxT("/AudioIO/VariSpeedPlay"), true);
+   // Scrubbing doesn't support note tracks, but the fixed-speed method using time tracks does.
+   if ( TrackList::Get( *p ).Any< NoteTrack >() )
+      bFixedSpeedPlay = true;
+
+   // Scrubbing only supports straight through play.
+   // So if looped or cutPreview, we have to fall back to fixed speed.
+   bFixedSpeedPlay = bFixedSpeedPlay || looped || cutPreview;
+   if (bFixedSpeedPlay)
+   {
+      // Create a BoundedEnvelope if we haven't done so already
+      if (!mEnvelope) {
+         mEnvelope =
+            std::make_unique<BoundedEnvelope>(
+               true, TIMETRACK_MIN, TIMETRACK_MAX, 1.0);
+         // values as in the constructor for TimeTrack
+         mEnvelope->SetRangeLower( 0.9 );
+         mEnvelope->SetRangeUpper( 1.1 );
       }
+      // Set the speed range
+      //mTimeTrack->SetRangeUpper((double)mPlaySpeed / 100.0);
+      //mTimeTrack->SetRangeLower((double)mPlaySpeed / 100.0);
+      mEnvelope->Flatten((double)mPlaySpeed / 100.0);
    }
 
    // Pop up the button
    SetButton(false, mButtons[TTB_PlaySpeed]);
 
    // If IO is busy, abort immediately
-   if (gAudioIO->IsBusy()) {
-      p->GetControlToolBar()->StopPlaying();
-   }
-
-   // Set the speed range
-   //mTimeTrack->SetRangeUpper((double)mPlaySpeed / 100.0);
-   //mTimeTrack->SetRangeLower((double)mPlaySpeed / 100.0);
-   mTimeTrack->GetEnvelope()->Flatten((double)mPlaySpeed / 100.0);
+   auto gAudioIO = AudioIOBase::Get();
+   if (gAudioIO->IsBusy())
+      projectAudioManager.Stop();
 
    // Get the current play region
-   double playRegionStart, playRegionEnd;
-   p->GetPlayRegion(&playRegionStart, &playRegionEnd);
+   const auto &viewInfo = ViewInfo::Get( *p );
+   const auto &playRegion = viewInfo.playRegion;
 
    // Start playing
-   if (playRegionStart >= 0) {
-//      playRegionEnd = playRegionStart + (playRegionEnd-playRegionStart)* 100.0/mPlaySpeed;
-#ifdef EXPERIMENTAL_MIDI_OUT
-      gAudioIO->SetMidiPlaySpeed(mPlaySpeed);
-#endif
-      AudioIOStartStreamOptions options(p->GetDefaultPlayOptions());
+   if (playRegion.GetStart() < 0)
+      return;
+   if (bFixedSpeedPlay)
+   {
+      auto options = DefaultPlayOptions( *p );
       options.playLooped = looped;
-      options.timeTrack = mTimeTrack;
-      p->GetControlToolBar()->PlayPlayRegion
-         (SelectedRegion(playRegionStart, playRegionEnd),
-          options,
-          cutPreview);
+      // No need to set cutPreview options.
+      options.envelope = mEnvelope.get();
+      auto mode =
+         cutPreview ? PlayMode::cutPreviewPlay
+         : options.playLooped ? PlayMode::loopedPlay
+         : PlayMode::normalPlay;
+      projectAudioManager.PlayPlayRegion(
+         SelectedRegion(playRegion.GetStart(), playRegion.GetEnd()),
+            options,
+            mode);
+   }
+   else
+   {
+      auto &scrubber = Scrubber::Get( *p );
+      scrubber.StartSpeedPlay(GetPlaySpeed(),
+         playRegion.GetStart(), playRegion.GetEnd());
    }
 }
 
 // Come here from button clicks only
 void TranscriptionToolBar::OnPlaySpeed(wxCommandEvent & WXUNUSED(event))
 {
+   auto button = mButtons[TTB_PlaySpeed];
+
    // Let control have precedence over shift
    const bool cutPreview = mButtons[TTB_PlaySpeed]->WasControlDown();
    const bool looped = !cutPreview &&
-      mButtons[TTB_PlaySpeed]->WasShiftDown();
+      button->WasShiftDown();
    PlayAtSpeed(looped, cutPreview);
 }
 
 void TranscriptionToolBar::OnSpeedSlider(wxCommandEvent& WXUNUSED(event))
 {
-   mPlaySpeed = (mPlaySpeedSlider->Get()) * 100;
+   SetPlaySpeed( (mPlaySpeedSlider->Get()) * 100 );
    RegenerateTooltips();
 
    // If IO is busy, abort immediately
    // AWD: This is disabled to work around a hang on Linux when PulseAudio is
    // used.  If we figure that one out we can re-enable this code.
+   // auto gAudioIO = AudioIOBase::Get();
    //if (gAudioIO->IsBusy()) {
    //   OnPlaySpeed(event);
    //}
@@ -498,30 +575,30 @@ void TranscriptionToolBar::OnSpeedSlider(wxCommandEvent& WXUNUSED(event))
 void TranscriptionToolBar::OnStartOn(wxCommandEvent & WXUNUSED(event))
 {
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy()){
       SetButton(false,mButtons[TTB_StartOn]);
       return;
    }
 
    mVk->AdjustThreshold(GetSensitivity());
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
 
-   TrackList *tl = p->GetTracks();
-   TrackListOfKindIterator iter(Track::Wave, tl);
-
-   Track *t = iter.First();   //Make a track
+   auto t = *p->GetTracks()->Any< const WaveTrack >().begin();
    if(t ) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       //if(len == 0)
-      //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
+      //len = wt->GetSequence()->GetNumSamples()-start;
 
-      sampleCount newstart = mVk->OnForward(*(WaveTrack*)t,start,len);
-      double newpos = newstart / ((WaveTrack*)t)->GetRate();
+      auto newstart = mVk->OnForward(*wt, start, len);
+      double newpos = newstart.as_double() / wt->GetRate();
 
-      p->SetSel0(newpos);
+      auto &selectedRegion = p->GetViewInfo().selectedRegion;
+      selectedRegion.setT0( newpos );
       p->RedrawProject();
 
       SetButton(false, mButtons[TTB_StartOn]);
@@ -531,30 +608,30 @@ void TranscriptionToolBar::OnStartOn(wxCommandEvent & WXUNUSED(event))
 void TranscriptionToolBar::OnStartOff(wxCommandEvent & WXUNUSED(event))
 {
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy()){
       SetButton(false,mButtons[TTB_StartOff]);
       return;
    }
    mVk->AdjustThreshold(GetSensitivity());
-   AudacityProject *p = GetActiveProject();
-
-   TrackList *tl = p->GetTracks();
-   TrackListOfKindIterator iter(Track::Wave, tl);
+   AudacityProject *p = &mProject;
 
    SetButton(false, mButtons[TTB_StartOff]);
-   Track *t = iter.First();   //Make a track
+   auto t = *p->GetTracks()->Any< const WaveTrack >().begin();
    if(t) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       //if(len == 0)
-      //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
+      //len = wt->GetSequence()->GetNumSamples()-start;
 
-      sampleCount newstart = mVk->OffForward(*(WaveTrack*)t,start,len);
-      double newpos = newstart / ((WaveTrack*)t)->GetRate();
+      auto newstart = mVk->OffForward(*wt, start, len);
+      double newpos = newstart.as_double() / wt->GetRate();
 
-      p->SetSel0(newpos);
+      auto &selectedRegion = p->GetViewInfo().selectedRegion;
+      selectedRegion.setT0( newpos );
       p->RedrawProject();
 
       SetButton(false, mButtons[TTB_StartOn]);
@@ -565,20 +642,19 @@ void TranscriptionToolBar::OnEndOn(wxCommandEvent & WXUNUSED(event))
 {
 
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy()){
       SetButton(false,mButtons[TTB_EndOn]);
       return;
    }
 
    mVk->AdjustThreshold(GetSensitivity());
-   AudacityProject *p = GetActiveProject();
-   TrackList *tl = p->GetTracks();
-   TrackListOfKindIterator iter(Track::Wave, tl);
-
-   Track *t = iter.First();   //Make a track
+   AudacityProject *p = &mProject;
+   auto t = *p->GetTracks()->Any< const WaveTrack >().begin();
    if(t) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       if(len == 0)
@@ -586,8 +662,8 @@ void TranscriptionToolBar::OnEndOn(wxCommandEvent & WXUNUSED(event))
             len = start;
             start = 0;
          }
-      sampleCount newEnd = mVk->OnBackward(*(WaveTrack*)t,start+ len,len);
-      double newpos = newEnd / ((WaveTrack*)t)->GetRate();
+      auto newEnd = mVk->OnBackward(*wt, start + len, len);
+      double newpos = newEnd.as_double() / wt->GetRate();
 
       p->SetSel1(newpos);
       p->RedrawProject();
@@ -602,27 +678,27 @@ void TranscriptionToolBar::OnEndOff(wxCommandEvent & WXUNUSED(event))
 {
 
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy()){
       SetButton(false,mButtons[TTB_EndOff]);
       return;
    }
    mVk->AdjustThreshold(GetSensitivity());
-   AudacityProject *p = GetActiveProject();
-   TrackList *tl = p->GetTracks();
-   TrackListOfKindIterator iter(Track::Wave, tl);
+   AudacityProject *p = &mProject;
 
-   Track *t = iter.First();   //Make a track
+   auto t = *p->GetTracks()->Any< const WaveTrack >().begin();
    if(t) {
-      sampleCount start,len;
-      GetSamples((WaveTrack*)t, &start,&len);
+      auto wt = static_cast<const WaveTrack*>(t);
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
       //Adjust length to end if selection is null
       if(len == 0) {
          len = start;
          start = 0;
       }
-      sampleCount newEnd = mVk->OffBackward(*(WaveTrack*)t,start+ len,len);
-      double newpos = newEnd / ((WaveTrack*)t)->GetRate();
+      auto newEnd = mVk->OffBackward(*wt, start + len, len);
+      double newpos = newEnd.as_double() / wt->GetRate();
 
       p->SetSel1(newpos);
       p->RedrawProject();
@@ -637,6 +713,7 @@ void TranscriptionToolBar::OnSelectSound(wxCommandEvent & WXUNUSED(event))
 {
 
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy()){
       SetButton(false,mButtons[TTB_SelectSound]);
       return;
@@ -644,32 +721,30 @@ void TranscriptionToolBar::OnSelectSound(wxCommandEvent & WXUNUSED(event))
 
 
    mVk->AdjustThreshold(GetSensitivity());
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
 
 
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
+   if(auto wt = *tl->Any<const WaveTrack>().begin()) {
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
-   Track *t = iter.First();   //Make a track
-   if(t)
-      {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
+      //Adjust length to end if selection is null
+      //if(len == 0)
+      //len = wt->GetSequence()->GetNumSamples()-start;
 
-         //Adjust length to end if selection is null
-         //if(len == 0)
-         //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
+      double rate =  wt->GetRate();
+      auto newstart = mVk->OffBackward(*wt, start, start);
+      auto newend   =
+      mVk->OffForward(*wt, start + len, (int)(tl->GetEndTime() * rate));
 
-         double rate =  ((WaveTrack*)t)->GetRate();
-         sampleCount newstart = mVk->OffBackward(*(WaveTrack*)t,start,start);
-         sampleCount newend   = mVk->OffForward(*(WaveTrack*)t,start+len,(int)(tl->GetEndTime()*rate));
+      //reset the selection bounds.
+      auto &selectedRegion = p->GetViewInfo().selectedRegion;
+      selectedRegion.setTimes(
+         newstart.as_double() / rate, newend.as_double() /  rate );
+      p->RedrawProject();
 
-         //reset the selection bounds.
-         p->SetSel0(newstart / rate);
-         p->SetSel1(newend /  rate);
-         p->RedrawProject();
-
-      }
+   }
 
    SetButton(false,mButtons[TTB_SelectSound]);
 }
@@ -678,37 +753,35 @@ void TranscriptionToolBar::OnSelectSilence(wxCommandEvent & WXUNUSED(event))
 {
 
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy()){
       SetButton(false,mButtons[TTB_SelectSilence]);
       return;
    }
 
    mVk->AdjustThreshold(GetSensitivity());
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
 
 
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
+   if(auto wt = *tl->Any<const WaveTrack>().begin()) {
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
-   Track *t = iter.First();   //Make a track
-   if(t)
-      {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
+      //Adjust length to end if selection is null
+      //if(len == 0)
+      //len = wt->GetSequence()->GetNumSamples()-start;
+      double rate =  wt->GetRate();
+      auto newstart = mVk->OnBackward(*wt, start, start);
+      auto newend   =
+      mVk->OnForward(*wt, start + len, (int)(tl->GetEndTime() * rate));
 
-         //Adjust length to end if selection is null
-         //if(len == 0)
-         //len = (WaveTrack*)t->GetSequence()->GetNumSamples()-start;
-         double rate =  ((WaveTrack*)t)->GetRate();
-         sampleCount newstart = mVk->OnBackward(*(WaveTrack*)t,start,start);
-         sampleCount newend   = mVk->OnForward(*(WaveTrack*)t,start+len,(int)(tl->GetEndTime()*rate));
+      //reset the selection bounds.
+      p->SetSel0(newstart.as_double() /  rate);
+      p->SetSel1(newend.as_double() / rate);
+      p->RedrawProject();
 
-         //reset the selection bounds.
-         p->SetSel0(newstart /  rate);
-         p->SetSel1(newend / rate);
-         p->RedrawProject();
-
-      }
+   }
 
    SetButton(false,mButtons[TTB_SelectSilence]);
 
@@ -719,34 +792,31 @@ void TranscriptionToolBar::OnSelectSilence(wxCommandEvent & WXUNUSED(event))
 void TranscriptionToolBar::OnCalibrate(wxCommandEvent & WXUNUSED(event))
 {
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy()){
       SetButton(false,mButtons[TTB_Calibrate]);
       return;
    }
 
 
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
 
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
-   Track *t = iter.First();   //Get a track
+   if(auto wt = *tl->Any<const WaveTrack>().begin()) {
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
-   if(t)
-      {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
+      mVk->CalibrateNoise(*wt, start, len);
+      mVk->AdjustThreshold(3);
 
-         mVk->CalibrateNoise(*((WaveTrack*)t),start,len);
-         mVk->AdjustThreshold(3);
+      mButtons[TTB_StartOn]->Enable();
+      mButtons[TTB_StartOff]->Enable();
+      mButtons[TTB_EndOn]->Enable();
+      mButtons[TTB_EndOff]->Enable();
+      //mThresholdSensitivity->Set(3);
 
-         mButtons[TTB_StartOn]->Enable();
-         mButtons[TTB_StartOff]->Enable();
-         mButtons[TTB_EndOn]->Enable();
-         mButtons[TTB_EndOff]->Enable();
-         //mThresholdSensitivity->Set(3);
-
-         SetButton(false,mButtons[TTB_Calibrate]);
-      }
+      SetButton(false,mButtons[TTB_Calibrate]);
+   }
 
    mButtons[TTB_StartOn]->Enable();
    mButtons[TTB_StartOff]->Enable();
@@ -770,6 +840,7 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
 
 
    //If IO is busy, abort immediately
+   auto gAudioIO = AudioIOBase::Get();
    if (gAudioIO->IsBusy())
    {
       SetButton(false,mButtons[TTB_EndOff]);
@@ -779,83 +850,78 @@ void TranscriptionToolBar::OnAutomateSelection(wxCommandEvent & WXUNUSED(event))
    wxBusyCursor busy;
 
    mVk->AdjustThreshold(GetSensitivity());
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
    TrackList *tl = p->GetTracks();
-   TrackListIterator iter(tl);
+   if(auto wt = *tl->Any<const WaveTrack>().begin()) {
+      sampleCount start, len;
+      GetSamples(wt, &start, &len);
 
-   Track *t = iter.First();   //Make a track
-   if(t)
+      //Adjust length to end if selection is null
+      if(len == 0)
       {
-         sampleCount start,len;
-         GetSamples((WaveTrack*)t, &start,&len);
-
-         //Adjust length to end if selection is null
-         if(len == 0)
-            {
-               len = start;
-               start = 0;
-            }
-         int lastlen = 0;
-         sampleCount newStart, newEnd;
-         double newStartPos, newEndPos;
-
-
-         //This is the minumum word size in samples (.05 is 50 ms)
-         int minWordSize = (int)(((WaveTrack*)t)->GetRate() * .05);
-
-         //Continue until we have processed the entire
-         //region, or we are making no progress.
-         while(len > 0 && lastlen != len)
-            {
-
-               lastlen = len;
-
-               newStart = mVk->OnForward(*(WaveTrack*)t,start,len);
-
-               //JKC: If no start found then don't add any labels.
-               if( newStart==start)
-                  break;
-
-               //Adjust len by the new start position
-               len -= (newStart - start);
-
-               //Adjust len by the minimum word size
-               len -= minWordSize;
-
-
-
-               //OK, now we have found a new starting point.  A 'word' should be at least
-               //50 ms long, so jump ahead minWordSize
-
-               newEnd   = mVk->OffForward(*(WaveTrack*)t,newStart+minWordSize, len);
-
-               //If newEnd didn't move, we should give up, because
-               // there isn't another end before the end of the selection.
-               if(newEnd == (newStart + minWordSize))
-                  break;
-
-
-               //Adjust len by the new word end
-               len -= (newEnd - newStart);
-
-               //Calculate the start and end of the words, in seconds
-               newStartPos = newStart / ((WaveTrack*)t)->GetRate();
-               newEndPos = newEnd / ((WaveTrack*)t)->GetRate();
-
-
-               //Increment
-               start = newEnd;
-
-               p->DoAddLabel(SelectedRegion(newStartPos, newEndPos));
-               p->RedrawProject();
-            }
-         SetButton(false, mButtons[TTB_AutomateSelection]);
+         len = start;
+         start = 0;
       }
+      sampleCount lastlen = 0;
+      double newStartPos, newEndPos;
+
+      //This is the minumum word size in samples (.05 is 50 ms)
+      int minWordSize = (int)(wt->GetRate() * .05);
+
+      //Continue until we have processed the entire
+      //region, or we are making no progress.
+      while(len > 0 && lastlen != len)
+      {
+
+         lastlen = len;
+
+         auto newStart = mVk->OnForward(*wt, start, len);
+
+         //JKC: If no start found then don't add any labels.
+         if( newStart==start)
+            break;
+
+         //Adjust len by the NEW start position
+         len -= (newStart - start);
+
+         //Adjust len by the minimum word size
+         len -= minWordSize;
+
+
+
+         //OK, now we have found a NEW starting point.  A 'word' should be at least
+         //50 ms long, so jump ahead minWordSize
+
+         auto newEnd   =
+         mVk->OffForward(*wt, newStart + minWordSize, len);
+
+         //If newEnd didn't move, we should give up, because
+         // there isn't another end before the end of the selection.
+         if(newEnd == (newStart + minWordSize))
+            break;
+
+
+         //Adjust len by the NEW word end
+         len -= (newEnd - newStart);
+
+         //Calculate the start and end of the words, in seconds
+         newStartPos = newStart.as_double() / wt->GetRate();
+         newEndPos = newEnd.as_double() / wt->GetRate();
+
+
+         //Increment
+         start = newEnd;
+
+         p->DoAddLabel(SelectedRegion(newStartPos, newEndPos));
+         p->RedrawProject();
+      }
+      SetButton(false, mButtons[TTB_AutomateSelection]);
+   }
 }
 
 void TranscriptionToolBar::OnMakeLabel(wxCommandEvent & WXUNUSED(event))
 {
-   AudacityProject *p = GetActiveProject();
+   AudacityProject *p = &mProject;
    SetButton(false, mButtons[TTB_MakeLabel]);
    p->DoAddLabel(SelectedRegion(p->GetSel0(),  p->GetSel1()));
 }
@@ -936,3 +1002,7 @@ void TranscriptionToolBar::AdjustPlaySpeed(float adj)
    OnSpeedSlider(e);
 }
 
+static RegisteredToolbarFactory factory{ TranscriptionBarID,
+   []( AudacityProject &project ){
+      return ToolBar::Holder{ safenew TranscriptionToolBar{ project } }; }
+};
