@@ -63,7 +63,9 @@ ExpandingToolBar.
 
 *//*******************************************************************/
 
-#include "../Theme.h"
+#include "ExpandingToolBar.h"
+
+#include "../Experimental.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
@@ -75,14 +77,10 @@ ExpandingToolBar.
 #include <wx/wx.h>
 #include <wx/dcmemory.h>
 #include <wx/log.h>
-#include <wx/dragimag.h>
-#include <wx/arrimpl.cpp>
 #include <wx/dialog.h>
 
-#include "ExpandingToolBar.h"
 #include "AButton.h"
 #include "../AllThemeResources.h"
-#include "../Experimental.h"
 
 const int kToggleButtonHeight = 8;
 const int kTimerInterval = 50; // every 50 ms -> ~20 updates per second
@@ -93,27 +91,25 @@ enum {
    kTimerID
 };
 
-WX_DEFINE_OBJARRAY(wxArrayRect);
-
 class ToolBarArrangement
 {
 public:
-   ExpandingToolBarArray    childArray;
-   wxArrayRect              rectArray;
-   wxArrayInt               rowArray;
+   std::vector<ExpandingToolBar*> childArray;
+   std::vector<wxRect>      rectArray;
+   std::vector<int>         rowArray;
 };
 
 //
 // ExpandingToolBar
 //
 
-BEGIN_EVENT_TABLE(ExpandingToolBar, wxPanel)
+BEGIN_EVENT_TABLE(ExpandingToolBar, wxPanelWrapper)
    EVT_SIZE(ExpandingToolBar::OnSize)
    EVT_TIMER(kTimerID, ExpandingToolBar::OnTimer)
    EVT_BUTTON(kToggleButtonID, ExpandingToolBar::OnToggle)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ExpandingToolBar, wxPanel)
+IMPLEMENT_CLASS(ExpandingToolBar, wxPanelWrapper)
 
 //static
 int ExpandingToolBar::msNoAutoExpandStack = 0;
@@ -122,7 +118,7 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
                                    wxWindowID id,
                                    const wxPoint& pos,
                                    const wxSize& size):
-   wxPanel(parent, id, pos, size),
+   wxPanelWrapper(parent, id, pos, size),
    mIsAutoExpanded(false),
    mIsManualExpanded(false),
    mIsExpanded(false),
@@ -131,13 +127,12 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
    mFrameParent(NULL),
    mDialogParent(NULL),
    mAreaParent(NULL),
-   mSavedArrangement(NULL),
-   mDragImage(NULL),
+   mSavedArrangement{},
    mTopLevelParent(NULL)
 {
-   mMainPanel = new wxPanel(this, -1,
+   mMainPanel = safenew wxPanelWrapper(this, -1,
                             wxDefaultPosition, wxSize(1, 1));
-   mExtraPanel = new wxPanel(this, -1,
+   mExtraPanel = safenew wxPanelWrapper(this, -1,
                              wxDefaultPosition, wxSize(1, 1));
 
    mGrabber = NULL;
@@ -145,14 +140,14 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
    ToolBarArea *toolBarParent =
       dynamic_cast<ToolBarArea *>(GetParent());
    if (toolBarParent)
-      mGrabber = new ToolBarGrabber(this, -1, this);
+      mGrabber = safenew ToolBarGrabber(this, -1, this);
 
    /// \todo check whether this is a memory leak (and check similar code)
-   wxImage hbar = theTheme.Image(bmpToolBarToggle);
-   wxColour magicColor = wxColour(0, 255, 255);
-   ImageArray fourStates = ImageRoll::SplitV(hbar, magicColor);
-
-   mToggleButton = new AButton(this, kToggleButtonID,
+   //wxImage hbar = theTheme.Image(bmpToolBarToggle);
+   //wxColour magicColor = wxColour(0, 255, 255);
+   //ImageArray fourStates = ImageRoll::SplitV(hbar, magicColor);
+/*
+   mToggleButton = safenew AButton(this, kToggleButtonID,
                                wxDefaultPosition, wxDefaultSize,
                                ImageRoll(ImageRoll::HorizontalRoll,
                                          fourStates[0], magicColor),
@@ -164,7 +159,7 @@ ExpandingToolBar::ExpandingToolBar(wxWindow* parent,
                                          fourStates[3], magicColor),
                                true);
    mToggleButton->UseDisabledAsDownHiliteImage(true);
-
+*/
    SetAutoLayout(true);
    mTimer.SetOwner(this, kTimerID);
 }
@@ -276,49 +271,54 @@ void ExpandingToolBar::TryAutoCollapse()
 #endif
 }
 
-class ExpandingToolBarEvtHandler : public wxEvtHandler
+class ExpandingToolBarEvtHandler final : public wxEvtHandler
 {
  public:
    ExpandingToolBarEvtHandler(ExpandingToolBar *toolbar,
+                              wxWindow *window,
                               wxEvtHandler *inheritedEvtHandler)
    {
       mToolBar = toolbar;
+      mWindow = window;
       mInheritedEvtHandler = inheritedEvtHandler;
-   }
 
-   virtual bool ProcessEvent(wxEvent& evt)
+      window->PushEventHandler(this);
+}
+
+   bool ProcessEvent(wxEvent& evt) override
    {
-      if (mToolBar->IsCursorInWindow())
-         mToolBar->TryAutoExpand();
-      else
+//      if (mToolBar->IsCursorInWindow())
          mToolBar->TryAutoExpand();
+//      else
+//         mToolBar->TryAutoExpand();
 
       return mInheritedEvtHandler->ProcessEvent(evt);
    }
 
+   ~ExpandingToolBarEvtHandler()
+   {
+      mWindow->RemoveEventHandler(this);
+   }
+
 protected:
    ExpandingToolBar *mToolBar;
+   wxWindow *mWindow;
    wxEvtHandler *mInheritedEvtHandler;
 
-   DECLARE_NO_COPY_CLASS(ExpandingToolBarEvtHandler);
+   DECLARE_NO_COPY_CLASS(ExpandingToolBarEvtHandler)
 };
 
 void ExpandingToolBar::RecursivelyPushEventHandlers(wxWindow *win)
 {
    if (!mWindowHash[win]) {
-      ExpandingToolBarEvtHandler *evtHandler =
-         new ExpandingToolBarEvtHandler(this, win->GetEventHandler());
-      win->PushEventHandler(evtHandler);
+      mHandlers.push_back(std::make_unique<ExpandingToolBarEvtHandler>
+         (this, win, win->GetEventHandler()));
       mWindowHash[win] = 1;
    }
 
    wxWindowList children = win->GetChildren();
-
-   typedef wxWindowList::compatibility_iterator Node;
-   for(Node node = children.GetFirst(); node; node = node->GetNext()) {
-      wxWindow *child = node->GetData();
+   for(auto child : children)
       RecursivelyPushEventHandlers(child);
-   }
 }
 
 bool ExpandingToolBar::Layout()
@@ -538,11 +538,11 @@ void ExpandingToolBar::StartMoving()
    mDropTarget = kDummyRect;
 
    wxColour magicColor = wxColour(0, 255, 255);
-   wxImage tgtImage = theTheme.Image(bmpToolBarTarget);
-   ImageRoll tgtImageRoll = ImageRoll(ImageRoll::VerticalRoll,
-                                      tgtImage,
-                                      magicColor);
-   mTargetPanel = new ImageRollPanel(mAreaParent, -1, tgtImageRoll,
+//   wxImage tgtImage = theTheme.Image(bmpToolBarTarget);
+//   ImageRoll tgtImageRoll = ImageRoll(ImageRoll::VerticalRoll,
+//                                      tgtImage,
+//                                      magicColor);
+   mTargetPanel = safenew ImageRollPanel(mAreaParent, -1, //tgtImageRoll,
                                      wxDefaultPosition,
                                      wxDefaultSize,
                                      wxTRANSPARENT_WINDOW);
@@ -556,7 +556,7 @@ void ExpandingToolBar::StartMoving()
 
    mAreaParent->SetCapturedChild(this);
 
-   mDragImage = new wxDragImage(toolbarBitmap);
+   mDragImage = std::make_unique<wxDragImage>(toolbarBitmap);
    mDragImage->BeginDrag(hotSpot, mAreaParent, mTopLevelParent);
    mDragImage->Show();
    mDragImage->Move(ScreenToClient(wxGetMousePosition()));
@@ -572,7 +572,7 @@ void ExpandingToolBar::UpdateMoving()
    int best_dist_sq = 99999;
    int i;
 
-   for(i=0; i<(int)mDropTargets.GetCount(); i++) {
+   for(i = 0; i < (int)mDropTargets.size(); i++) {
       int x = (mDropTargets[i].x + (mDropTargets[i].width/2))-cursorPos.x;
       int y = (mDropTargets[i].y + (mDropTargets[i].height/2))-cursorPos.y;
       int dist_sq = (x*x) + (y*y);
@@ -622,7 +622,7 @@ void ExpandingToolBar::FinishMoving()
    if (!mAreaParent || !mSavedArrangement)
       return;
 
-   delete mTargetPanel;
+   // DELETE mTargetPanel; // I think this is not needed, but unreachable anyway -- PRL
 
    mAreaParent->SetCapturedChild(NULL);
 
@@ -632,13 +632,10 @@ void ExpandingToolBar::FinishMoving()
    msNoAutoExpandStack--;
 
    if (mDropTarget == kDummyRect) {
-      mAreaParent->RestoreArrangement(mSavedArrangement);
-      mSavedArrangement = NULL;
+      mAreaParent->RestoreArrangement(std::move(mSavedArrangement));
    }
    else {
-      delete mSavedArrangement;
-      mSavedArrangement = NULL;
-
+      mSavedArrangement.reset();
       mAreaParent->MoveChild(this, mDropTarget);
    }
 
@@ -655,22 +652,23 @@ void ExpandingToolBar::FinishMoving()
 // ToolBarGrabber
 //
 
-BEGIN_EVENT_TABLE(ToolBarGrabber, wxPanel)
+BEGIN_EVENT_TABLE(ToolBarGrabber, wxPanelWrapper)
    EVT_PAINT(ToolBarGrabber::OnPaint)
    EVT_SIZE(ToolBarGrabber::OnSize)
    EVT_MOUSE_EVENTS(ToolBarGrabber::OnMouse)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ToolBarGrabber, wxPanel)
+IMPLEMENT_CLASS(ToolBarGrabber, wxPanelWrapper)
 
 ToolBarGrabber::ToolBarGrabber(wxWindow *parent,
                                wxWindowID id,
                                ExpandingToolBar *ownerToolbar,
                                const wxPoint& pos,
                                const wxSize& size):
-   wxPanel(parent, id, pos, size),
+   wxPanelWrapper(parent, id, pos, size),
    mOwnerToolBar(ownerToolbar)
 {
+#if 0
    wxImage grabberImages = theTheme.Image(bmpToolBarGrabber);
    wxColour magicColor = wxColour(0, 255, 255);
    ImageArray images = ImageRoll::SplitH(grabberImages, magicColor);
@@ -684,7 +682,7 @@ ToolBarGrabber::ToolBarGrabber(wxWindow *parent,
 
    SetSizeHints(mImageRoll[0].GetMinSize(),
                 mImageRoll[1].GetMaxSize());
-
+#endif
    mState = 0;
 }
 
@@ -719,7 +717,7 @@ void ToolBarGrabber::OnPaint(wxPaintEvent & WXUNUSED(event))
 {
    wxPaintDC dc(this);
 
-   mImageRoll[mState].Draw(dc, GetClientRect());
+  // mImageRoll[mState].Draw(dc, GetClientRect());
 }
 
 void ToolBarGrabber::OnSize(wxSizeEvent & WXUNUSED(event))
@@ -731,16 +729,16 @@ void ToolBarGrabber::OnSize(wxSizeEvent & WXUNUSED(event))
 // ToolBarDialog
 //
 
-BEGIN_EVENT_TABLE(ToolBarDialog, wxDialog)
+BEGIN_EVENT_TABLE(ToolBarDialog, wxDialogWrapper)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ToolBarDialog, wxDialog)
+IMPLEMENT_CLASS(ToolBarDialog, wxDialogWrapper)
 
 ToolBarDialog::ToolBarDialog(wxWindow* parent,
                            wxWindowID id,
                            const wxString& name,
                            const wxPoint& pos):
-   wxDialog(parent, id, name, pos, wxSize(1, 1),
+   wxDialogWrapper(parent, id, name, pos, wxSize(1, 1),
 // Workaround for bug in __WXMSW__.  No close box on a wxDialog unless wxSYSTEM_MENU is used.
 #ifdef __WXMSW__
       wxSYSTEM_MENU |
@@ -835,18 +833,18 @@ void ToolBarFrame::Fit()
 // ToolBarArea
 //
 
-BEGIN_EVENT_TABLE(ToolBarArea, wxPanel)
+BEGIN_EVENT_TABLE(ToolBarArea, wxPanelWrapper)
    EVT_SIZE(ToolBarArea::OnSize)
    EVT_MOUSE_EVENTS(ToolBarArea::OnMouse)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(ToolBarArea, wxPanel)
+IMPLEMENT_CLASS(ToolBarArea, wxPanelWrapper)
 
 ToolBarArea::ToolBarArea(wxWindow* parent,
                          wxWindowID id,
                          const wxPoint& pos,
                          const wxSize& size):
-   wxPanel(parent, id, pos, size),
+   wxPanelWrapper(parent, id, pos, size),
    mInOnSize(false),
    mCapturedChild(NULL)
 {
@@ -865,7 +863,7 @@ void ToolBarArea::ContractRow(int rowIndex)
    int i;
    int x = 0;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++)
+   for(i = 0; i < (int)mChildArray.size(); i++)
       if (mRowArray[i] == rowIndex) {
          wxPoint childPos = mChildArray[i]->GetPosition();
          wxSize childMin = mChildArray[i]->GetMinSize();
@@ -891,7 +889,7 @@ bool ToolBarArea::ExpandRow(int rowIndex)
    int expandableCount = 0;
    int toolbarCount = 0;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++)
+   for(i = 0; i < (int)mChildArray.size(); i++)
       if (mRowArray[i] == rowIndex) {
          ExpandingToolBar *child = mChildArray[i];
          wxSize childMin = child->GetMinSize();
@@ -915,7 +913,7 @@ bool ToolBarArea::ExpandRow(int rowIndex)
 
    j = 0;
    x = 0;
-   for(i=0; i<(int)mChildArray.GetCount(); i++)
+   for(i = 0; i < (int)mChildArray.size(); i++)
       if (mRowArray[i] == rowIndex) {
          ExpandingToolBar *child = mChildArray[i];
          wxPoint childPos = child->GetPosition();
@@ -953,7 +951,7 @@ void ToolBarArea::LayoutOne(int childIndex)
       wxPoint p = mChildArray[childIndex]->GetPosition();
       wxSize s = mChildArray[childIndex]->GetSize();
 
-      printf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
+      wxPrintf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
              childIndex, mRowArray[childIndex],
              p.x, p.y, s.x, s.y);
       #endif
@@ -1005,7 +1003,7 @@ void ToolBarArea::LayoutOne(int childIndex)
    wxPoint p = mChildArray[childIndex]->GetPosition();
    wxSize s = mChildArray[childIndex]->GetSize();
 
-   printf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
+   wxPrintf("ToolBar %d moved to row %d at (%d, %d), size (%d x %d)\n",
           childIndex, mRowArray[childIndex],
           p.x, p.y, s.x, s.y);
    #endif
@@ -1018,10 +1016,10 @@ bool ToolBarArea::Layout()
 
    int i;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++)
+   for(i = 0; i < (int)mChildArray.size(); i++)
       mRowArray[i] = -1;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++)
+   for(i = 0; i < (int)mChildArray.size(); i++)
       LayoutOne(i);
 
    Refresh(true);
@@ -1037,13 +1035,13 @@ void ToolBarArea::AdjustLayout()
    int row = -1;
    int i, j;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++) {
+   for(i = 0; i < (int)mChildArray.size(); i++) {
       if (mRowArray[i] > row) {
          row = mRowArray[i];
          bool success = ExpandRow(row);
          if (!success) {
             // Re-layout all toolbars from this row on
-            for(j=i; j<(int)mChildArray.GetCount(); j++)
+            for(j = i; j < (int)mChildArray.size(); j++)
                LayoutOne(j);
             return;
          }
@@ -1068,7 +1066,7 @@ void ToolBarArea::Fit(bool horizontal, bool vertical)
    minSize.y = 0;
    maxSize.x = 9999;
    maxSize.y = 0;
-   for(i=0; i<(int)mChildArray.GetCount(); i++) {
+   for(i = 0; i < (int)mChildArray.size(); i++) {
       wxPoint childPos = mChildArray[i]->GetPosition();
       wxSize childSize = mChildArray[i]->GetSize();
 
@@ -1148,15 +1146,15 @@ void ToolBarArea::CollapseAll(bool now)
 {
    int i;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++)
+   for(i = 0; i < (int)mChildArray.size(); i++)
       mChildArray[i]->Collapse(now);
 }
 
 void ToolBarArea::AddChild(ExpandingToolBar *child)
 {
-   mChildArray.Add(child);
-   mRowArray.Add(-1); // unknown row
-   LayoutOne(mChildArray.GetCount()-1);
+   mChildArray.push_back(child);
+   mRowArray.push_back(-1); // unknown row
+   LayoutOne(mChildArray.size() - 1);
    Fit(false, true);
 }
 
@@ -1164,17 +1162,17 @@ void ToolBarArea::RemoveChild(ExpandingToolBar *child)
 {
    int i, j;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++) {
+   for(i = 0; i < (int)mChildArray.size(); i++) {
       if (mChildArray[i] == child) {
          child->Hide();
 
-         mChildArray.RemoveAt(i);
-         mRowArray.RemoveAt(i);
+         mChildArray.erase(mChildArray.begin() + i);
+         mRowArray.erase(mRowArray.begin() + i);
 
-         for(j=i; j<(int)mChildArray.GetCount(); j++)
+         for(j = i; j < (int)mChildArray.size(); j++)
             mRowArray[j] = -1;
 
-         for(j=i; j<(int)mChildArray.GetCount(); j++)
+         for(j = i; j < (int)mChildArray.size(); j++)
             LayoutOne(j);
 
          Fit(false, true);
@@ -1182,44 +1180,44 @@ void ToolBarArea::RemoveChild(ExpandingToolBar *child)
    }
 }
 
-ToolBarArrangement *ToolBarArea::SaveArrangement()
+std::unique_ptr<ToolBarArrangement> ToolBarArea::SaveArrangement()
 {
-   ToolBarArrangement *arrangement = new ToolBarArrangement();
+   auto arrangement = std::make_unique<ToolBarArrangement>();
    int i;
 
    arrangement->childArray = mChildArray;
    arrangement->rowArray = mRowArray;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++)
-      arrangement->rectArray.Add(mChildArray[i]->GetRect());
+   for(i = 0; i < (int)mChildArray.size(); i++)
+      arrangement->rectArray.push_back(mChildArray[i]->GetRect());
 
    return arrangement;
 }
 
-void ToolBarArea::RestoreArrangement(ToolBarArrangement *arrangement)
+void ToolBarArea::RestoreArrangement(std::unique_ptr<ToolBarArrangement>&& arrangement)
 {
    int i;
 
    mChildArray = arrangement->childArray;
    mRowArray = arrangement->rowArray;
 
-   for(i=0; i<(int)mChildArray.GetCount(); i++) {
+   for(i = 0; i < (int)mChildArray.size(); i++) {
       mChildArray[i]->SetSize(arrangement->rectArray[i]);
       mChildArray[i]->Show();
    }
 
    Fit(false, true);
 
-   delete arrangement;
+   arrangement.reset();
 }
 
-wxArrayRect ToolBarArea::GetDropTargets()
+std::vector<wxRect> ToolBarArea::GetDropTargets()
 {
-   mDropTargets.Clear();
-   mDropTargetIndices.Clear();
-   mDropTargetRows.Clear();
+   mDropTargets.clear();
+   mDropTargetIndices.clear();
+   mDropTargetRows.clear();
 
-   int numChildren = (int)mChildArray.GetCount();
+   int numChildren = (int)mChildArray.size();
    int i;
    int row = -1;
 
@@ -1233,16 +1231,16 @@ wxArrayRect ToolBarArea::GetDropTargets()
       if (childRow != row) {
          // Add a target before this child (at beginning of row only)
          row = childRow;
-         mDropTargetIndices.Add(i);
-         mDropTargetRows.Add(row);
-         mDropTargets.Add(wxRect(childRect.x, childRect.y,
+         mDropTargetIndices.push_back(i);
+         mDropTargetRows.push_back(row);
+         mDropTargets.push_back(wxRect(childRect.x, childRect.y,
                                  0, childRect.height));
       }
 
       // Add a target after this child (always)
-      mDropTargetIndices.Add(i+1);
-      mDropTargetRows.Add(row);
-      mDropTargets.Add(wxRect(childRect.x+childRect.width, childRect.y,
+      mDropTargetIndices.push_back(i+1);
+      mDropTargetRows.push_back(row);
+      mDropTargets.push_back(wxRect(childRect.x+childRect.width, childRect.y,
                               0, childRect.height));
    }
 
@@ -1253,22 +1251,22 @@ void ToolBarArea::MoveChild(ExpandingToolBar *toolBar, wxRect dropTarget)
 {
    int i, j;
 
-   for(i=0; i<(int)mDropTargets.GetCount(); i++) {
+   for(i = 0; i < (int)mDropTargets.size(); i++) {
       if (dropTarget == mDropTargets[i]) {
          int newIndex = mDropTargetIndices[i];
          int newRow = mDropTargetRows[i];
 
-         mChildArray.Insert(toolBar, newIndex);
-         mRowArray.Insert(newRow, newIndex);
+         mChildArray.insert(mChildArray.begin() + newIndex, toolBar);
+         mRowArray.insert(mRowArray.begin() + newIndex, newRow);
 
-         for(j=newIndex+1; j<(int)mChildArray.GetCount(); j++)
+         for(j = newIndex+1; j < (int)mChildArray.size(); j++)
             mRowArray[j] = -1;
 
          ContractRow(newRow);
 
          mChildArray[newIndex]->Show();
 
-         for(j=newIndex; j<(int)mChildArray.GetCount(); j++)
+         for(j = newIndex; j < (int)mChildArray.size(); j++)
             LayoutOne(j);
 
          Fit(false, true);
