@@ -8,6 +8,15 @@
   Leland Lucius
   Richard Ash
 
+  was merged with LinkingHtmlWindow.h
+
+  Vaughan Johnson
+  Dominic Mazzoni
+
+  utility fn and
+  descendant of HtmlWindow that opens links in the user's
+  default browser
+
 *//********************************************************************/
 
 #include "../Audacity.h" // for USE_* macros
@@ -16,12 +25,15 @@
 #include "../Experimental.h"
 
 #include <wx/setup.h> // for wxUSE_* macros
-#include <wx/bmpbuttn.h>
+#include <wx/button.h>
+#include <wx/frame.h>
 #include <wx/icon.h>
 #include <wx/dialog.h>
 #include <wx/intl.h>
+#include <wx/log.h>
 #include <wx/sizer.h>
 #include <wx/stattext.h>
+#include <wx/textctrl.h>
 #include <wx/utils.h>
 #include <wx/html/htmlwin.h>
 #include <wx/settings.h>
@@ -29,15 +41,11 @@
 #include <wx/regex.h>
 
 #include "../FileNames.h"
-#include "LinkingHtmlWindow.h"
-#include "../Theme.h"
 #include "../AllThemeResources.h"
 #include "../ShuttleGui.h"
 #include "../HelpText.h"
-#include "../Project.h"
 #include "../Prefs.h"
-
-#include "ErrorDialog.h"
+#include "../wxFileNameWrapper.h"
 
 #ifdef USE_ALPHA_MANUAL
 const wxString HelpSystem::HelpHostname = wxT("alphamanual.audacityteam.org");
@@ -227,17 +235,20 @@ void HelpSystem::ShowHelp(wxWindow *parent,
                     bool alwaysDefaultBrowser)
 {
    wxASSERT(parent); // to justify safenew
-   AudacityProject * pProj = GetActiveProject();
    wxString HelpMode = wxT("Local");
 
-   if( pProj )
+// DA: Default for DA is manual from internet.
+#ifdef EXPERIMENTAL_DA
+   gPrefs->Read(wxT("/GUI/Help"), &HelpMode, wxT("FromInternet") );
+#else
+   gPrefs->Read(wxT("/GUI/Help"), &HelpMode, wxT("Local") );
+#endif
+
    {
-      HelpMode = pProj->mHelpPref;
       // these next lines are for legacy cfg files (pre 2.0) where we had different modes
       if( (HelpMode == wxT("Standard")) || (HelpMode == wxT("InBrowser")) )
       {
          HelpMode = wxT("Local");
-         pProj->mHelpPref = HelpMode;
          gPrefs->Write(wxT("/GUI/Help"), HelpMode);
          gPrefs->Flush();
       }
@@ -272,7 +283,9 @@ void HelpSystem::ShowHelp(wxWindow *parent,
       // Use Built-in browser to suggest you use the remote url.
       wxString Text = HelpText( wxT("remotehelp") );
       Text.Replace( wxT("*URL*"), remoteURL );
-      ShowHtmlText( parent, _("Help on the Internet"), Text, false, bModal );
+      // Always make the 'help on the internet' dialog modal.
+      // Fixes Bug 1411.
+      ShowHtmlText( parent, _("Help on the Internet"), Text, false, true );
    }
    else if( HelpMode == wxT("Local") || alwaysDefaultBrowser)
    {
@@ -397,176 +410,163 @@ void HelpSystem::ShowHelp(wxWindow *parent,
       );
 }
 
-#include "../ShuttleGui.h"
-// These three are all for the OnReloadPreferences command.
-#include "../Project.h"
-#include "../commands/CommandContext.h"
-#include "../Menus.h"
+// For compilers that support precompilation, includes "wx/wx.h".
+#include <wx/wxprec.h>
 
-#define FixButtonID           7001
-#define HelpButtonID          7011
-#define FakeButtonID          7021
+#include <wx/mimetype.h>
+#include <wx/filename.h>
+#include <wx/uri.h>
 
-BEGIN_EVENT_TABLE(QuickFixDialog, wxDialogWrapper)
-   EVT_BUTTON(wxID_OK,                                            QuickFixDialog::OnOk)
-   EVT_BUTTON(wxID_CANCEL,                                        QuickFixDialog::OnCancel)
-   EVT_BUTTON(wxID_HELP,                                          QuickFixDialog::OnHelp)
-   EVT_COMMAND_RANGE(FixButtonID,  HelpButtonID-1, wxEVT_BUTTON,  QuickFixDialog::OnFix)
-   EVT_COMMAND_RANGE(HelpButtonID, FakeButtonID-1, wxEVT_BUTTON,  QuickFixDialog::OnHelp)
-END_EVENT_TABLE();
+BEGIN_EVENT_TABLE(BrowserDialog, wxDialogWrapper)
+   EVT_BUTTON(wxID_FORWARD,  BrowserDialog::OnForward)
+   EVT_BUTTON(wxID_BACKWARD, BrowserDialog::OnBackward)
+   EVT_BUTTON(wxID_CANCEL,   BrowserDialog::OnClose)
+   EVT_KEY_DOWN(BrowserDialog::OnKeyDown)
+END_EVENT_TABLE()
 
-QuickFixDialog::QuickFixDialog(wxWindow * pParent) :
-      wxDialogWrapper(pParent, wxID_ANY, _("Do you have these problems?"),
-            wxDefaultPosition, wxDefaultSize,
-            wxDEFAULT_DIALOG_STYLE )
+
+BrowserDialog::BrowserDialog(wxWindow *pParent, const wxString &title)
+   : wxDialogWrapper{ pParent, ID, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER /*| wxMAXIMIZE_BOX */  }
 {
-   const long SNAP_OFF = 0;
+   int width, height;
+   const int minWidth = 400;
+   const int minHeight = 250;
 
-   gPrefs->Read(wxT("/GUI/SyncLockTracks"), &mbSyncLocked, false);
-   mbInSnapTo = gPrefs->Read(wxT("/SnapTo"), SNAP_OFF) !=0;
-   gPrefs->Read(wxT("/AudioIO/SoundActivatedRecord"), &mbSoundActivated, false);
+   gPrefs->Read(wxT("/GUI/BrowserWidth"), &width, minWidth);
+   gPrefs->Read(wxT("/GUI/BrowserHeight"), &height, minHeight);
 
-   ShuttleGui S(this, eIsCreating);
-   PopulateOrExchange(S);
+   if (width < minWidth || width > wxSystemSettings::GetMetric(wxSYS_SCREEN_X))
+      width = minWidth;
+   if (height < minHeight || height > wxSystemSettings::GetMetric(wxSYS_SCREEN_Y))
+      height = minHeight;
 
-   Fit();
-   auto sz = GetSize();
-   SetMinSize( sz );
-   SetMaxSize( sz );
-
-   // The close button has the cancel id and acts exactly the same as cancel.
-   wxButton * pWin = (wxButton*)FindWindowById( wxID_CANCEL );
-   if( pWin )
-      pWin->SetFocus( );
-   Center();
+   SetMinSize(wxSize(minWidth, minHeight));
+   SetSize(wxDefaultPosition.x, wxDefaultPosition.y, width, height, wxSIZE_AUTO);
 }
 
-void QuickFixDialog::AddStuck( ShuttleGui & S, bool & bBool, wxString Pref, wxString Prompt, wxString Help )
+void BrowserDialog::OnForward(wxCommandEvent & WXUNUSED(event))
 {
-   mItem++;
-   if( !bBool)
-      return;
-   S.AddFixedText( Prompt );
-   S.Id(FixButtonID + mItem).AddButton( _("Fix") )->SetClientObject(
-      safenew wxStringClientData(Pref));
+   mpHtml->HistoryForward();
+   UpdateButtons();
+}
 
+void BrowserDialog::OnBackward(wxCommandEvent & WXUNUSED(event))
+{
+   mpHtml->HistoryBack();
+   UpdateButtons();
+}
+
+void BrowserDialog::OnClose(wxCommandEvent & WXUNUSED(event))
+{
+   if (IsModal() && !mDismissed)
    {
-     // Replace standard Help button with smaller icon button.
-      // bs->AddButton(safenew wxButton(parent, wxID_HELP));
-      auto b = safenew wxBitmapButton(S.GetParent(), HelpButtonID+mItem, theTheme.Bitmap( bmpHelpIcon ));
-      b->SetToolTip( _("Help") );
-      b->SetLabel(_("Help"));       // for screen readers
-      b->SetClientObject( safenew wxStringClientData( Help ));
-      S.AddWindow( b );
+      mDismissed = true;
+      EndModal(wxID_CANCEL);
    }
-}
+   auto parent = GetParent();
 
-void QuickFixDialog::PopulateOrExchange(ShuttleGui & S)
-{
-
-   S.StartVerticalLay(1);
-   S.StartStatic( _("Quick Fixes"));
-
-   // These aren't all possible modes one can be stuck in, but they are some of them.
-   bool bStuckInMode = mbSyncLocked || mbInSnapTo || mbSoundActivated;
-
-   if( !bStuckInMode ){
-      SetLabel(_("Nothing to do"));
-      S.AddFixedText(_("No quick, easily fixed problems were found"));
-   }
-   else {
-      S.StartMultiColumn(3, wxALIGN_CENTER);
-      {
-         mItem = -1;
-
-         // Use # in the URLs to ensure we go to the online version of help.
-         // Local help may well not be installed.
-         AddStuck( S, mbSyncLocked, "/GUI/SyncLockTracks", _("Clocks on the Tracks"), "Quick_Fix#sync_lock" );
-         AddStuck( S, mbInSnapTo, "/SnapTo", _("Can't select precisely"), "Quick_Fix#snap_to" );
-         AddStuck( S, mbSoundActivated, "/AudioIO/SoundActivatedRecord", _("Recording stops and starts"), "Quick_Fix#sound_activated_recording" );
-      }
-      S.EndMultiColumn();
-   }
-   S.EndStatic();
-
-   S.StartHorizontalLay(wxALIGN_CENTER_HORIZONTAL, 0);
-      S.AddStandardButtons(eCloseButton + (bStuckInMode ? 0 : eHelpButton));
-   S.EndHorizontalLay();
-
-   S.EndVerticalLay();
-
-   wxButton * pBtn = (wxButton*)FindWindowById( wxID_HELP );
-   if( pBtn )
-      pBtn->SetClientObject( safenew wxStringClientData( "Quick_Fix#" ));
-
-}
-
-void QuickFixDialog::OnOk(wxCommandEvent &event)
-{
-   (void)event;// Compiler food
-   EndModal(wxID_OK);
-}
-
-void QuickFixDialog::OnCancel(wxCommandEvent &event)
-{
-   (void)event;// Compiler food
-   EndModal(wxID_CANCEL);
-}
-
-wxString QuickFixDialog::StringFromEvent( wxCommandEvent &event )
-{
-   wxButton * pBtn = (wxButton*)event.GetEventObject();
-   if( !pBtn ){
-      wxFAIL_MSG( "Event Object not found");
-      return "";
-   }
-   wxStringClientData * pStrCd = (wxStringClientData*)(pBtn->GetClientObject());
-   if( !pStrCd ){
-      wxFAIL_MSG( "Client Data not found");
-      return "";
-   }
-   wxString Str = pStrCd->GetData();
-   if( Str.empty()){
-      wxFAIL_MSG( "String data empty");
-      return "";
-   }
-   return Str;
-}
-
-void QuickFixDialog::OnHelp(wxCommandEvent &event)
-{
-   HelpSystem::ShowHelp(this, StringFromEvent( event ), true);
-}
-
-void QuickFixDialog::OnFix(wxCommandEvent &event)
-{
-   wxString Str = StringFromEvent( event );
-   gPrefs->Write( Str, 0);
+   gPrefs->Write(wxT("/GUI/BrowserWidth"), GetSize().GetX());
+   gPrefs->Write(wxT("/GUI/BrowserHeight"), GetSize().GetY());
    gPrefs->Flush();
 
-   if ( auto pProject = GetActiveProject() ) {
-      // Sadly SnapTo has to be handled specially, as it is not part of the standard
-      // preference dialogs.
-      if( Str == "/SnapTo" )
+#ifdef __WXMAC__
+   auto grandparent = GetParent()->GetParent();
+#endif
+
+   parent->Destroy();
+
+#ifdef __WXMAC__
+   if(grandparent && grandparent->IsShown()) {
+      grandparent->Raise();
+   }
+#endif
+}
+
+void BrowserDialog::OnKeyDown(wxKeyEvent & event)
+{
+   bool bSkip = true;
+   if (event.GetKeyCode() == WXK_ESCAPE)
+   {
+      bSkip = false;
+      Close(false);
+   }
+   event.Skip(bSkip);
+}
+
+
+void BrowserDialog::UpdateButtons()
+{
+   wxWindow * pWnd;
+   if( (pWnd = FindWindowById( wxID_BACKWARD, this )) != NULL )
+   {
+      pWnd->Enable(mpHtml->HistoryCanBack());
+   }
+   if( (pWnd = FindWindowById( wxID_FORWARD, this )) != NULL )
+   {
+      pWnd->Enable(mpHtml->HistoryCanForward());
+   }
+}
+
+void OpenInDefaultBrowser(const wxHtmlLinkInfo& link)
+{
+   wxURI uri(link.GetHref());
+   wxLaunchDefaultBrowser(uri.BuildURI());
+}
+
+LinkingHtmlWindow::LinkingHtmlWindow(wxWindow *parent, wxWindowID id /*= -1*/,
+                                       const wxPoint& pos /*= wxDefaultPosition*/,
+                                       const wxSize& size /*= wxDefaultSize*/,
+                                       long style /*= wxHW_SCROLLBAR_AUTO*/) :
+   HtmlWindow(parent, id, pos, size, style)
+{
+}
+
+void LinkingHtmlWindow::OnLinkClicked(const wxHtmlLinkInfo& link)
+{
+   wxString href = link.GetHref();
+
+   if( href.StartsWith( wxT("innerlink:help:")))
+   {
+      HelpSystem::ShowHelp(this, href.Mid( 15 ), true );
+      return;
+   }
+   else if( href.StartsWith(wxT("innerlink:")) )
+   {
+      wxString FileName =
+         wxFileName( FileNames::HtmlHelpDir(), href.Mid( 10 ) + wxT(".htm") ).GetFullPath();
+      if( wxFileExists( FileName ) )
       {
-         pProject->SetSnapTo( 0 );
+         HelpSystem::ShowHelp(this, FileName, wxEmptyString, false);
+         return;
       }
       else
       {
-         // This is overkill (aka slow), as all preferences are reloaded and all 
-         // toolbars recreated.
-         // Overkill probably doesn't matter, as this command is infrequently used.
-         EditActions::DoReloadPreferences( *pProject );
+         SetPage( HelpText( href.Mid( 10 )));
+         wxGetTopLevelParent(this)->SetLabel( TitleText( href.Mid( 10 )));
       }
    }
-   
-   // Change the label after doing the fix, as the fix may take a second or two.
-   wxButton * pBtn = (wxButton*)event.GetEventObject();
-   if( pBtn )
-      pBtn->SetLabel( _("Fixed") );
-
-   // The close button has the cancel id and acts exactly the same as cancel.
-   wxButton * pWin = (wxButton*)FindWindowById( wxID_CANCEL );
-   if( pWin )
-      pWin->SetFocus( );
+   else if( href.StartsWith(wxT("mailto:")) || href.StartsWith(wxT("file:")) )
+   {
+      OpenInDefaultBrowser( link );
+      return;
+   }
+   else if( !href.StartsWith( wxT("http:"))  && !href.StartsWith( wxT("https:")) )
+   {
+      HtmlWindow::OnLinkClicked( link );
+   }
+   else
+   {
+      OpenInDefaultBrowser(link);
+      return;
+   }
+   wxFrame * pFrame = GetRelatedFrame();
+   if( !pFrame )
+      return;
+   wxWindow * pWnd = pFrame->FindWindow(BrowserDialog::ID);
+   if( !pWnd )
+      return;
+   BrowserDialog * pDlg = wxDynamicCast( pWnd , BrowserDialog );
+   if( !pDlg )
+      return;
+   pDlg->UpdateButtons();
 }
