@@ -27,6 +27,9 @@
 *//**********************************************************************/
 
 #include "../Audacity.h"
+#include "ProgressDialog.h"
+
+#include <algorithm>
 
 #include <wx/defs.h>
 #include <wx/app.h>
@@ -36,15 +39,17 @@
 #include <wx/dialog.h>
 #include <wx/event.h>
 #include <wx/evtloop.h>
+#include <wx/gauge.h>
 #include <wx/frame.h>
 #include <wx/intl.h>
+#include <wx/msgdlg.h>
 #include <wx/settings.h>
 #include <wx/sizer.h>
 #include <wx/sound.h>
 #include <wx/stopwatch.h>
 #include <wx/window.h>
+#include <wx/stattext.h>
 
-#include "ProgressDialog.h"
 #include "../Prefs.h"
 
 // This really should be a Preferences setting
@@ -979,7 +984,7 @@ static const unsigned char beep[] =
 /// Methods for ProgressDialog
 ////////////////////////////////////////////////////////////
 
-BEGIN_EVENT_TABLE(ProgressDialog, wxDialog)
+BEGIN_EVENT_TABLE(ProgressDialog, wxDialogWrapper)
    EVT_BUTTON(wxID_CANCEL, ProgressDialog::OnCancel)
    EVT_BUTTON(wxID_OK, ProgressDialog::OnStop)
    EVT_CLOSE(ProgressDialog::OnCloseWindow)
@@ -989,19 +994,26 @@ END_EVENT_TABLE()
 // Constructor
 //
 ProgressDialog::ProgressDialog()
-:  wxDialog()
+:  wxDialogWrapper()
 {
-   Init();
 }
 
 ProgressDialog::ProgressDialog(const wxString & title,
-                               const wxString & message,
-                               int flags)
-:  wxDialog()
+                               const wxString & message /* = {}*/,
+                               int flags /* = pdlgDefaultFlags */,
+                               const wxString & sRemainingLabelText /* = {} */)
+:  wxDialogWrapper()
 {
-   Init();
+   Create(title, message, flags, sRemainingLabelText);
+}
 
-   Create(title, message, flags);
+ProgressDialog::ProgressDialog(const wxString & title,
+                               const MessageTable &columns,
+                               int flags /* = pdlgDefaultFlags */,
+                               const wxString & sRemainingLabelText /* = {} */)
+:  wxDialogWrapper()
+{
+   Create(title, columns, flags, sRemainingLabelText);
 }
 
 //
@@ -1011,11 +1023,7 @@ ProgressDialog::~ProgressDialog()
 {
    // Delete the window disabler before hiding the dialog to allow
    // focus to return to the original window.
-   if (mDisable)
-   {
-      delete mDisable;
-      mDisable = NULL;
-   }
+   mDisable.reset();
 
    if (IsShown())
    {
@@ -1034,10 +1042,8 @@ ProgressDialog::~ProgressDialog()
 
    // Restore saved focus, but only if the window still exists.
    //
-   // It is possible that it was a deferred deletion and it was deleted since
-   // we captured the focused window.  So, we need to verify that the window
-   // still exists by searching all of the wxWidgets windows.  It's the only
-   // sure way.
+   // PRL:  I'm conservatively preserving the old existence test, but I think
+   // it's redundant now that we use wxWindowRef to avoid a dangling pointer
    if (mHadFocus && SearchForWindow(wxTopLevelWindows, mHadFocus)) {
       mHadFocus->SetFocus();
    }
@@ -1045,10 +1051,6 @@ ProgressDialog::~ProgressDialog()
 
 void ProgressDialog::Init()
 {
-   mLastValue = 0;
-   mDisable = NULL;
-   mIsTransparent = true;
-
    // There's a problem where the focus is not returned to the window that had
    // it before creating this object.  The reason is because the focus events
    // that are sent to the parent window after the wxWindowDisabler are created
@@ -1068,140 +1070,13 @@ void ProgressDialog::Init()
 #endif
 }
 
-bool ProgressDialog::Create(const wxString & title,
-                            const wxString & message,
-                            int flags)
+void ProgressDialog::Reinit()
 {
-   wxWindow *parent = GetParentForModalDialog(NULL, 0);
+   mLastValue = 0;
 
-   bool success = wxDialog::Create(parent,
-                                   wxID_ANY,
-                                   title,
-                                   wxDefaultPosition,
-                                   wxDefaultSize,
-                                   wxDEFAULT_DIALOG_STYLE |
-                                   wxFRAME_FLOAT_ON_PARENT);
-   if (!success)
-   {
-      return false;
-   }
-   SetName(GetTitle());
-
-   wxBoxSizer *v;
-   wxWindow *w;
-   wxSize ds;
-
-   SetExtraStyle(GetExtraStyle() | wxWS_EX_TRANSIENT);
-
-   v = new wxBoxSizer(wxVERTICAL);
-
-   mMessage = new wxStaticText(this,
-                               wxID_ANY,
-                               message,
-                               wxDefaultPosition,
-                               wxDefaultSize,
-                               wxALIGN_LEFT);
-   mMessage->SetName(message); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   v->Add(mMessage, 1, wxEXPAND | wxALL, 10);
-   ds.y += mMessage->GetSize().y + 20;
-
-   //
-   //
-   //
-   mGauge = new wxGauge(this,
-                        wxID_ANY,
-                        1000,
-                        wxDefaultPosition,
-                        wxDefaultSize,
-                        wxGA_HORIZONTAL);
-   v->Add(mGauge, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 10);
-   ds.y += mGauge->GetSize().y + 10;
-
-   //
-   //
-   //
-   wxFlexGridSizer *g = new wxFlexGridSizer(2, 2, 10, 10);
-
-   w = new wxStaticText(this,
-                        wxID_ANY,
-                        _("Elapsed Time:"),
-                        wxDefaultPosition,
-                        wxDefaultSize,
-                        wxALIGN_RIGHT);
-   w->SetName(w->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(w, 0, wxALIGN_RIGHT);
-
-   mElapsed = new wxStaticText(this,
-                               wxID_ANY,
-                               wxT("00:00:00"),
-                               wxDefaultPosition,
-                               wxDefaultSize,
-                               wxALIGN_LEFT);
-   mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(mElapsed, 0, wxALIGN_LEFT);
-   ds.y += mElapsed->GetSize().y + 10;
-
-   //
-   //
-   //
-   w = new wxStaticText(this,
-                        wxID_ANY,
-                        _("Remaining Time:"),
-                        wxDefaultPosition,
-                        wxDefaultSize,
-                        wxALIGN_RIGHT);
-   w->SetName(w->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(w, 0, wxALIGN_RIGHT);
-
-   mRemaining = new wxStaticText(this,
-                                 wxID_ANY,
-                                 wxT("00:00:00"),
-                                 wxDefaultPosition,
-                                 wxDefaultSize,
-                                 wxALIGN_LEFT);
-   mRemaining->SetName(mRemaining->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-   g->Add(mRemaining, 0, wxALIGN_LEFT);
-
-   v->Add(g, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
-   ds.y += mRemaining->GetSize().y + 10;
-
-   wxBoxSizer *h = new wxBoxSizer(wxHORIZONTAL);
-
-   if (!(flags & pdlgHideStopButton))
-   {
-      w = new wxButton(this, wxID_OK, _("Stop"));
-      h->Add(w, 0, wxRIGHT, 10);
-   }
-
-   if (!(flags & pdlgHideCancelButton))
-   {
-      w = new wxButton(this, wxID_CANCEL, _("Cancel"));
-      h->Add(w, 0, wxRIGHT, 10);
-   }
-
-   v->Add(h, 0, wxALIGN_RIGHT | wxRIGHT | wxBOTTOM, 10);
-
-   SetSizer(v);
-   Layout();
-
-   ds.x = wxMax(g->GetSize().x, h->GetSize().x) + 10;
-   ds.y += w->GetSize().y + 10;
-
-   wxClientDC dc(this);
-   dc.GetMultiLineTextExtent(message, &mLastW, &mLastH);
-
-#if defined(__WXMAC__)
-   mMessage->SetMinSize(wxSize(mLastW, mLastH));
-#endif
-
-   // The 300 really isn't needed, but it keeps it at a decent width.
-   ds.x = wxMax(wxMax(wxMax(ds.x, mLastW) + 20, wxMax(ds.y, mLastH)), 300);
-   SetClientSize(ds);
-
-   Centre(wxCENTER_FRAME | wxBOTH);
-
-   mStartTime = wxGetLocalTimeMillis().GetValue();
+   mStartTime = wxGetUTCTimeMillis().GetValue();
    mLastUpdate = mStartTime;
+   mYieldTimer = mStartTime;
    mCancel = false;
    mStop = false;
 
@@ -1213,7 +1088,210 @@ bool ProgressDialog::Create(const wxString & title,
    SetTransparent(0);
    mIsTransparent = true;
 
-   wxDialog::Show(true);
+   auto button = FindWindowById(wxID_CANCEL, this);
+   if (button)
+      button->Enable();
+   button = FindWindowById(wxID_OK, this);
+   if (button)
+      button->Enable();
+
+   wxDialogWrapper::Show(true);
+}
+
+// Add a NEW text column each time this is called.
+void ProgressDialog::AddMessageAsColumn(wxBoxSizer * pSizer,
+                                        const MessageColumn & column,
+                                        bool bFirstColumn) {
+
+   // Assuming that we don't want empty columns, bail out if there is no text.
+   if (column.empty())
+      return;
+
+   // Join strings
+   auto sText = column[0];
+   std::for_each( column.begin() + 1, column.end(),
+      [&](const wxString &text) { sText += wxT("\n") + text; });
+
+   // Create a statictext object and add to the sizer
+   wxStaticText* oText = safenew wxStaticText(this,
+                                              wxID_ANY,
+                                              sText,
+                                              wxDefaultPosition,
+                                              wxDefaultSize,
+                                              wxALIGN_LEFT);
+   oText->SetName(sText); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+
+   // If this is the first column then set the mMessage pointer so non-TimerRecord usages
+   // will still work correctly in SetMessage()
+   if (bFirstColumn) {
+      mMessage = oText;
+   }
+
+   pSizer->Add(oText, 1, wxEXPAND | wxALL, 5);
+}
+
+bool ProgressDialog::Create(const wxString & title,
+                            const wxString & message /* = {} */,
+                            int flags /* = pdlgDefaultFlags */,
+                            const wxString & sRemainingLabelText /* = {} */)
+{
+   MessageTable columns(1);
+   columns.back().push_back(message);
+   auto result = Create(title, columns, flags, sRemainingLabelText);
+
+   if (result) {
+      // Record some values used in case of change of message
+      // TODO: make the following work in case of message tables
+      wxClientDC dc(this);
+      dc.GetMultiLineTextExtent(message, &mLastW, &mLastH);
+   }
+
+   return result;
+}
+
+bool ProgressDialog::Create(const wxString & title,
+                            const MessageTable & columns,
+                            int flags /* = pdlgDefaultFlags */,
+                            const wxString & sRemainingLabelText /* = {} */)
+{
+   Init();
+
+   wxWindow *parent = GetParentForModalDialog(NULL, 0);
+
+   // Set this boolean to indicate if we are using the "Elapsed" labels
+   m_bShowElapsedTime = !(flags & pdlgHideElapsedTime);
+   // Set this boolean to indicate if we confirm the Cancel/Stop actions
+   m_bConfirmAction = (flags & pdlgConfirmStopCancel)!=0;
+
+   bool success = wxDialogWrapper::Create(parent,
+                                   wxID_ANY,
+                                   title,
+                                   wxDefaultPosition,
+                                   wxDefaultSize,
+                                   wxDEFAULT_DIALOG_STYLE |
+                                   wxFRAME_FLOAT_ON_PARENT);
+   if (!success)
+   {
+      return false;
+   }
+
+   SetName(GetTitle()); // This was added for NVDA screen reader and may now be redundant.
+   SetExtraStyle(GetExtraStyle() | wxWS_EX_TRANSIENT);   // Ancient code. Is it still required?
+
+   {
+      wxWindow *window;
+
+      // There may be more than one column, so create a BoxSizer container
+      auto uColSizer = std::make_unique<wxBoxSizer>(wxHORIZONTAL);
+      auto colSizer = uColSizer.get();
+
+      // TODO:  this setting-up of a grid of text in a sizer might be worth
+      // extracting as a utility for building other dialogs.
+      {
+         bool bFirstCol = true;
+         for (const auto &column : columns) {
+            AddMessageAsColumn(colSizer, column, bFirstCol);
+            bFirstCol = false;
+         }
+      }
+
+      // and put message column(s) into a main vertical sizer.
+      auto vertSizer = std::make_unique<wxBoxSizer>(wxVERTICAL);
+      vertSizer->Add(uColSizer.release(), 1, wxEXPAND | wxALL, 10);
+
+      mGauge = safenew wxGauge(this,
+                               wxID_ANY,
+                               1000,
+                               wxDefaultPosition,
+                               wxDefaultSize,
+                               wxGA_HORIZONTAL);
+      vertSizer->Add(mGauge, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 10);
+
+      {
+         auto uGridSizer = std::make_unique<wxFlexGridSizer>(2, 2, 10, 10);
+         // Only one row if we are not going to show the elapsed time
+         if (m_bShowElapsedTime == false) {
+            uGridSizer = std::make_unique<wxFlexGridSizer>(1, 2, 10, 10);
+         }
+         auto gridSizer = uGridSizer.get();
+
+         if (m_bShowElapsedTime) {
+            window = safenew wxStaticText(this,
+                                          wxID_ANY,
+                                          _("Elapsed Time:"),
+                                          wxDefaultPosition,
+                                          wxDefaultSize,
+                                          wxALIGN_RIGHT);
+
+            window->SetName(window->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+            gridSizer->Add(window, 0, wxALIGN_RIGHT);
+
+            mElapsed = safenew wxStaticText(this,
+                                            wxID_ANY,
+                                            wxT("00:00:00"),
+                                            wxDefaultPosition,
+                                            wxDefaultSize,
+                                            wxALIGN_LEFT);
+
+            mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+            gridSizer->Add(mElapsed, 0, wxALIGN_LEFT);
+         }
+
+         // Customised "Remaining" label text
+         wxString sRemainingText = sRemainingLabelText;
+         if (sRemainingText.empty()) {
+            sRemainingText = _("Remaining Time:");
+         }
+
+         window = safenew wxStaticText(this,
+                                       wxID_ANY,
+                                       sRemainingText,
+                                       wxDefaultPosition,
+                                       wxDefaultSize,
+                                       wxALIGN_RIGHT);
+         window->SetName(window->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+         gridSizer->Add(window, 0, wxALIGN_RIGHT);
+
+         mRemaining = safenew wxStaticText(this,
+                                           wxID_ANY,
+                                           wxT("00:00:00"),
+                                           wxDefaultPosition,
+                                           wxDefaultSize,
+                                           wxALIGN_LEFT);
+         mRemaining->SetName(mRemaining->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+         gridSizer->Add(mRemaining, 0, wxALIGN_LEFT);
+
+         vertSizer->Add(uGridSizer.release(), 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
+      }
+
+      {
+         auto uButtonBar = std::make_unique<wxBoxSizer>(wxHORIZONTAL);
+         auto buttonBarSizer = uButtonBar.get();
+
+         if (!(flags & pdlgHideStopButton)) {
+            window = safenew wxButton(this, wxID_OK, _("Stop"));
+            buttonBarSizer->Add(window, 0, wxRIGHT, 10);
+         }
+         if (!(flags & pdlgHideCancelButton)) {
+            window = safenew wxButton(this, wxID_CANCEL, _("Cancel"));
+            buttonBarSizer->Add(window, 0, wxRIGHT, 10);
+         }
+         vertSizer->Add(uButtonBar.release(), 0, wxALIGN_RIGHT | wxRIGHT | wxBOTTOM, 10);
+      }
+
+      SetSizerAndFit(vertSizer.release());
+   }
+   Layout();
+
+   // Center progress bar on Parent if it is nice and wide, otherwise Center on screen.
+   int parentWidth = -1, parentHeight=-1;
+   if( GetParent() )  GetParent()->GetSize( &parentWidth, &parentHeight );
+   if (parentWidth > 400) 
+      CenterOnParent();
+   else
+      CenterOnScreen();
+
+   Reinit();
 
    // Even though we won't necessarily show the dialog due to the 500ms
    // delay, we MUST disable other windows/menus anyway since we run the risk
@@ -1227,7 +1305,7 @@ bool ProgressDialog::Create(const wxString & title,
    // while waiting for Timer Record to start -- and then also
    // while it's recording, it has a ProgressDialog, so really,
    // no editing in any project until Timer Record finishes.
-   mDisable = new wxWindowDisabler(this);
+   mDisable = std::make_unique<wxWindowDisabler>(this);
 
    return true;
 }
@@ -1235,24 +1313,24 @@ bool ProgressDialog::Create(const wxString & title,
 //
 // Update the time and, optionally, the message
 //
-int ProgressDialog::Update(int value, const wxString & message)
+ProgressResult ProgressDialog::Update(int value, const wxString & message)
 {
    if (mCancel)
    {
       // for compatibility with old Update, that returned false on cancel
-      return eProgressCancelled;
+      return ProgressResult::Cancelled;
    }
    else if (mStop)
    {
-      return eProgressStopped;
+      return ProgressResult::Stopped;
    }
 
-   wxLongLong_t now = wxGetLocalTimeMillis().GetValue();
+   wxLongLong_t now = wxGetUTCTimeMillis().GetValue();
    wxLongLong_t elapsed = now - mStartTime;
 
    if (elapsed < 500)
    {
-      return eProgressSuccess;
+      return ProgressResult::Success;
    }
 
    if (mIsTransparent)
@@ -1286,12 +1364,14 @@ int ProgressDialog::Update(int value, const wxString & message)
    // Only update if a full second has passed or track progress is complete
    if ((now - mLastUpdate > 1000) || (value == 1000))
    {
-      wxTimeSpan tsElapsed(0, 0, 0, elapsed);
-      wxTimeSpan tsRemains(0, 0, 0, remains);
+      if (m_bShowElapsedTime) {
+         wxTimeSpan tsElapsed(0, 0, 0, elapsed);
+         mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")));
+         mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
+         mElapsed->Update();
+      }
 
-      mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")));
-      mElapsed->SetName(mElapsed->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
-      mElapsed->Update();
+      wxTimeSpan tsRemains(0, 0, 0, remains);
       mRemaining->SetLabel(tsRemains.Format(wxT("%H:%M:%S")));
       mRemaining->SetName(mRemaining->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
       mRemaining->Update();
@@ -1299,7 +1379,7 @@ int ProgressDialog::Update(int value, const wxString & message)
       mLastUpdate = now;
    }
 
-   wxDialog::Update();
+   wxDialogWrapper::Update();
 
    // Copied from wx 3.0.2 generic progress dialog
    //
@@ -1312,15 +1392,21 @@ int ProgressDialog::Update(int value, const wxString & message)
    //      (and probably other things).  I do not yet know why this happens and
    //      I'm not too keen on having timer events processed here, but you do
    //      what you have to do.
-   wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI | wxEVT_CATEGORY_USER_INPUT | wxEVT_CATEGORY_TIMER);
 
-   return eProgressSuccess;
+   // Nyquist effects call Update on every callback, but YieldFor is
+   // quite slow on Linux / Mac, so don't call too frequently. (bug 1575)
+   if ((now - mYieldTimer > 50) || (value == 1000)) {
+      wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI | wxEVT_CATEGORY_USER_INPUT | wxEVT_CATEGORY_TIMER);
+      mYieldTimer = now;
+   }
+
+   return ProgressResult::Success;
 }
 
 //
 // Update the time and, optionally, the message
 //
-int ProgressDialog::Update(double current, const wxString & message)
+ProgressResult ProgressDialog::Update(double current, const wxString & message)
 {
    return Update((int)(current * 1000), message);
 }
@@ -1328,7 +1414,7 @@ int ProgressDialog::Update(double current, const wxString & message)
 //
 // Update the time and, optionally, the message
 //
-int ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxString & message)
+ProgressResult ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1343,7 +1429,7 @@ int ProgressDialog::Update(wxULongLong_t current, wxULongLong_t total, const wxS
 //
 // Update the time and, optionally, the message
 //
-int ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString & message)
+ProgressResult ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString & message)
 {
    if (total.GetValue() != 0)
    {
@@ -1358,7 +1444,7 @@ int ProgressDialog::Update(wxLongLong current, wxLongLong total, const wxString
 //
 // Update the time and, optionally, the message
 //
-int ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxString & message)
+ProgressResult ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1373,7 +1459,7 @@ int ProgressDialog::Update(wxLongLong_t current, wxLongLong_t total, const wxStr
 //
 // Update the time and, optionally, the message
 //
-int ProgressDialog::Update(int current, int total, const wxString & message)
+ProgressResult ProgressDialog::Update(int current, int total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1388,7 +1474,7 @@ int ProgressDialog::Update(int current, int total, const wxString & message)
 //
 // Update the time and, optionally, the message
 //
-int ProgressDialog::Update(double current, double total, const wxString & message)
+ProgressResult ProgressDialog::Update(double current, double total, const wxString & message)
 {
    if (total != 0)
    {
@@ -1405,7 +1491,7 @@ int ProgressDialog::Update(double current, double total, const wxString & messag
 //
 void ProgressDialog::SetMessage(const wxString & message)
 {
-   if (!message.IsEmpty())
+   if (!message.empty())
    {
       mMessage->SetLabel(message);
 
@@ -1416,6 +1502,7 @@ void ProgressDialog::SetMessage(const wxString & message)
       bool sizeUpdated = false;
       wxSize ds = GetClientSize();
 
+      // TODO: make the following work in case of message tables
       if (w > mLastW)
       {
          ds.x += (w - mLastW);
@@ -1440,7 +1527,7 @@ void ProgressDialog::SetMessage(const wxString & message)
          // to the existing dimensions.
          ds.x = wxMax(wxMax(ds.x, mLastW), wxMax(ds.y, mLastH));
          SetClientSize(ds);
-         wxDialog::Update();
+         wxDialogWrapper::Update();
       }
    }
 }
@@ -1450,15 +1537,12 @@ void ProgressDialog::SetMessage(const wxString & message)
 //
 bool ProgressDialog::SearchForWindow(const wxWindowList & list, const wxWindow *searchfor) const
 {
-   wxWindowList::compatibility_iterator node = list.GetFirst();
-   while (node)
+   for (auto win : list)
    {
-      wxWindow *win = node->GetData();
       if (win == searchfor || SearchForWindow(win->GetChildren(), searchfor))
       {
          return true;
       }
-      node = node->GetNext();
    }
 
    return false;
@@ -1466,12 +1550,18 @@ bool ProgressDialog::SearchForWindow(const wxWindowList & list, const wxWindow *
 
 void ProgressDialog::OnCancel(wxCommandEvent & WXUNUSED(event))
 {
+   if (!ConfirmAction(_("Are you sure you wish to cancel?"), _("Confirm Cancel"), wxID_CANCEL)) {
+      return;
+   }
    FindWindowById(wxID_CANCEL, this)->Disable();
    mCancel = true;
 }
 
 void ProgressDialog::OnStop(wxCommandEvent & WXUNUSED(event))
 {
+   if (!ConfirmAction(_("Are you sure you wish to stop?"), _("Confirm Stop"), wxID_OK)) {
+      return;
+   }
    FindWindowById(wxID_OK, this)->Disable();
    mCancel = false;
    mStop = true;
@@ -1479,6 +1569,9 @@ void ProgressDialog::OnStop(wxCommandEvent & WXUNUSED(event))
 
 void ProgressDialog::OnCloseWindow(wxCloseEvent & WXUNUSED(event))
 {
+   if (!ConfirmAction(_("Are you sure you wish to close?"), _("Confirm Close"))) {
+      return;
+   }
    mCancel = true;
 }
 
@@ -1492,12 +1585,12 @@ void ProgressDialog::Beep() const
    gPrefs->Read(wxT("/GUI/BeepAfterDuration"), &after, 60);
    gPrefs->Read(wxT("/GUI/BeepFileName"), &name, wxEmptyString);
 
-   if (should && wxGetLocalTimeMillis().GetValue() - mStartTime > after * 1000)
+   if (should && wxGetUTCTimeMillis().GetValue() - mStartTime > after * 1000)
    {
       wxBusyCursor busy;
       wxSound s;
 
-      if (name.IsEmpty())
+      if (name.empty())
       {
          s.Create(sizeof(beep), beep);
       }
@@ -1513,33 +1606,61 @@ void ProgressDialog::Beep() const
    }
 }
 
+// Confirm action taken by user.
+// Returns TRUE if the user confirms Yes
+bool ProgressDialog::ConfirmAction(const wxString & sPrompt,
+                                   const wxString & sTitle,
+                                   int iButtonID /* = -1 */) {
+
+   // Check if confirmations are enabled?
+   // If not then return TRUE
+   if (m_bConfirmAction == false) {
+      return true;
+   }
+
+   AudacityMessageDialog dlgMessage(this,
+      sPrompt,
+      sTitle,
+      wxYES_NO | wxICON_QUESTION | wxNO_DEFAULT | wxSTAY_ON_TOP);
+   int iAction = dlgMessage.ShowModal();
+
+   bool bReturn = (iAction == wxID_YES);
+   if ((bReturn == false) && (iButtonID > -1)) {
+      // Set the focus back to the relevant button
+      FindWindowById(iButtonID, this)->SetFocus();
+   }
+
+   return bReturn;
+}
+
 TimerProgressDialog::TimerProgressDialog(const wxLongLong_t duration,
-                                          const wxString & title,
-                                          const wxString & message /*= wxEmptyString*/,
-                                          int flags /*= pdlgEmptyFlags*/)
-: ProgressDialog(title, message, flags)
+                                         const wxString & title,
+                                         const MessageTable & columns,
+                                         int flags /* = pdlgDefaultFlags */,
+                                         const wxString & sRemainingLabelText /* = {} */)
+: ProgressDialog(title, columns, flags, sRemainingLabelText)
 {
    mDuration = duration;
 }
 
-int TimerProgressDialog::Update(const wxString & message /*= wxEmptyString*/)
+ProgressResult TimerProgressDialog::UpdateProgress()
 {
    if (mCancel)
    {
       // for compatibility with old Update, that returned false on cancel
-      return eProgressCancelled;
+      return ProgressResult::Cancelled;
    }
    else if (mStop)
    {
-      return eProgressStopped;
+      return ProgressResult::Stopped;
    }
 
-   wxLongLong_t now = wxGetLocalTimeMillis().GetValue();
+   wxLongLong_t now = wxGetUTCTimeMillis().GetValue();
    wxLongLong_t elapsed = now - mStartTime;
 
    if (elapsed < 500)
    {
-      return eProgressSuccess;
+      return ProgressResult::Success;
    }
 
    if (mIsTransparent)
@@ -1548,8 +1669,6 @@ int TimerProgressDialog::Update(const wxString & message /*= wxEmptyString*/)
       mIsTransparent = false;
    }
 
-   SetMessage(message);
-
    wxLongLong_t remains = mStartTime + mDuration - now;
 
    int nGaugeValue = (1000 * elapsed) / mDuration; // range = [0,1000]
@@ -1558,7 +1677,14 @@ int TimerProgressDialog::Update(const wxString & message /*= wxEmptyString*/)
    // From testing, it's never shown bigger than 1009, but 
    // give it a little extra, to 1010. 
    //   wxASSERT((nGaugeValue >= 0) && (nGaugeValue <= 1000)); // This ought to work. 
-   wxASSERT((nGaugeValue >= 0) && (nGaugeValue <= 1010));
+   // wxASSERT((nGaugeValue >= 0) && (nGaugeValue <= 1010));
+   //
+   // stf. Update was being called after wxMilliSleep(<ms>), which could be up to <ms>
+   // beyond the completion time. My gusess is that the microsleep in RunWaitDialog was originally 10 ms
+   // (same as other uses of Update) but was updated to kTimerInterval = 50 ms, thus triggering
+   // the Assert (Bug 1367). By calling Update() before sleeping then I think nGaugeValue <= 1000 should work.
+   wxASSERT((nGaugeValue >= 0) && (nGaugeValue <= 1000));
+
    if (nGaugeValue != mLastValue)
    {
       mGauge->SetValue(nGaugeValue);
@@ -1569,12 +1695,20 @@ int TimerProgressDialog::Update(const wxString & message /*= wxEmptyString*/)
    // Only update if a full second has passed.
    if (now - mLastUpdate > 1000)
    {
-      wxTimeSpan tsElapsed(0, 0, 0, elapsed);
-      wxTimeSpan tsRemains(0, 0, 0, remains);
+      // Bug 1952:
+      // wxTimeSpan will assert on ridiculously large values.
+      // We silently wrap the displayed range at one day.
+      // You'll only see the remaining hours, mins and secs.
+      // With a + sign, if the time was wrapped.
+      const wxLongLong_t wrapTime = 24 * 60 * 60 * 1000;
+      if (m_bShowElapsedTime) {
+         wxTimeSpan tsElapsed(0, 0, 0, elapsed % wrapTime);
+         mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")) + ((elapsed >= wrapTime) ? " +":""));
+         mElapsed->Update();
+      }
 
-      mElapsed->SetLabel(tsElapsed.Format(wxT("%H:%M:%S")));
-      mElapsed->Update();
-      mRemaining->SetLabel(tsRemains.Format(wxT("%H:%M:%S")));
+      wxTimeSpan tsRemains(0, 0, 0, remains % wrapTime);
+      mRemaining->SetLabel(tsRemains.Format(wxT("%H:%M:%S")) + ((remains >= wrapTime) ? " +":""));
       mRemaining->Update();
 
       mLastUpdate = now;
@@ -1591,7 +1725,31 @@ int TimerProgressDialog::Update(const wxString & message /*= wxEmptyString*/)
    //      (and probably other things).  I do not yet know why this happens and
    //      I'm not too keen on having timer events processed here, but you do
    //      what you have to do.
-   wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI | wxEVT_CATEGORY_USER_INPUT | wxEVT_CATEGORY_TIMER);
+   // JKC: Added thread category, since blocking a thread message could cause things 
+   //      to gum up.  
+   //      See http://trac.wxwidgets.org/ticket/14027 for discussion of why
+   //      YieldFor is flaky.  
+   //      Conclusion...  use wxEVT_CATEGORY_ALL since with the list below, we 
+   //      are pretty much there already.
+   //           wxEVT_CATEGORY_UI |
+   //           wxEVT_CATEGORY_USER_INPUT |
+   //           wxEVT_CATEGORY_TIMER |
+   //           wxEVT_CATEGORY_THREAD
 
-   return eProgressSuccess;
+   wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_ALL );
+   // JKC: Yielding twice, because e.g. a timer can build up a lot of events, and we 
+   //      really want to make sure they are worked through.
+   wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_ALL );
+
+   // MY: Added this after the YieldFor to check we haven't changed the outcome based on buttons pressed...
+   auto iReturn = ProgressResult::Success;
+   if (mCancel)
+   {
+      iReturn = ProgressResult::Cancelled;
+   }
+   else if (mStop)
+   {
+      iReturn = ProgressResult::Stopped;
+   }
+   return iReturn;
 }
