@@ -27,7 +27,8 @@
 
 #include <wx/setup.h> // for wxUSE_* macros
 
-#include "ErrorDialog.h"
+#include "AudacityMessageBox.h"
+#include "../Internat.h"
 
 #ifdef __BORLANDC__
     #pragma hdrstop
@@ -44,7 +45,6 @@
 #include <wx/dataobj.h>
 
 #include "numformatter.h"
-#include "../Internat.h"
 
 // ============================================================================
 // NumValidatorBase implementation
